@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Thread Safe Resource Manager                                         |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1999, 2000, Andi Gutmans, Sascha Schumann, Zeev Suraski|
+   | Copyright (c) 1999-2009, Andi Gutmans, Sascha Schumann, Zeev Suraski |
    | This source file is subject to the TSRM license, that is bundled     |
    | with this package in the file LICENSE                                |
    +----------------------------------------------------------------------+
@@ -34,9 +34,7 @@ typedef struct {
 	size_t size;
 	ts_allocate_ctor ctor;
 	ts_allocate_dtor dtor;
-#ifdef MBO_0
 	int done;
-#endif
 } tsrm_resource_type;
 
 
@@ -94,12 +92,28 @@ static FILE *tsrm_error_file;
 #if defined(PTHREADS)
 /* Thread local storage */
 static pthread_key_t tls_key;
+# define tsrm_tls_set(what)		pthread_setspecific(tls_key, (void*)(what))
+# define tsrm_tls_get()			pthread_getspecific(tls_key)
+
 #elif defined(TSRM_ST)
 static int tls_key;
+# define tsrm_tls_set(what)		st_thread_setspecific(tls_key, (void*)(what))
+# define tsrm_tls_get()			st_thread_getspecific(tls_key)
+
 #elif defined(TSRM_WIN32)
 static DWORD tls_key;
+# define tsrm_tls_set(what)		TlsSetValue(tls_key, (void*)(what))
+# define tsrm_tls_get()			TlsGetValue(tls_key)
+
 #elif defined(BETHREADS)
 static int32 tls_key;
+# define tsrm_tls_set(what)		tls_set(tls_key, (void*)(what))
+# define tsrm_tls_get()			(tsrm_tls_entry*)tls_get(tls_key)
+
+#else
+# define tsrm_tls_set(what)
+# define tsrm_tls_get()			NULL
+# warning tsrm_set_interpreter_context is probably broken on this platform
 #endif
 
 /* Startup TSRM (call once for the entire process) */
@@ -162,11 +176,9 @@ TSRM_API void tsrm_shutdown(void)
 				next_p = p->next;
 				for (j=0; j<p->count; j++) {
 					if (p->storage[j]) {
-#ifdef MBO_0
 						if (resource_types_table && !resource_types_table[j].done && resource_types_table[j].dtor) {
 							resource_types_table[j].dtor(p->storage[j], &p->storage);
 						}
-#endif
 						free(p->storage[j]);
 					}
 				}
@@ -226,9 +238,7 @@ TSRM_API ts_rsrc_id ts_allocate_id(ts_rsrc_id *rsrc_id, size_t size, ts_allocate
 	resource_types_table[TSRM_UNSHUFFLE_RSRC_ID(*rsrc_id)].size = size;
 	resource_types_table[TSRM_UNSHUFFLE_RSRC_ID(*rsrc_id)].ctor = ctor;
 	resource_types_table[TSRM_UNSHUFFLE_RSRC_ID(*rsrc_id)].dtor = dtor;
-#ifdef MBO_0
 	resource_types_table[TSRM_UNSHUFFLE_RSRC_ID(*rsrc_id)].done = 0;
-#endif
 
 	/* enlarge the arrays for the already active threads */
 	for (i=0; i<tsrm_tls_table_size; i++) {
@@ -268,26 +278,16 @@ static void allocate_new_resource(tsrm_tls_entry **thread_resources_ptr, THREAD_
 	(*thread_resources_ptr)->thread_id = thread_id;
 	(*thread_resources_ptr)->next = NULL;
 
-#if defined(PTHREADS)
 	/* Set thread local storage to this new thread resources structure */
-	pthread_setspecific(tls_key, (void *) *thread_resources_ptr);
-#elif defined(TSRM_ST)
-	st_thread_setspecific(tls_key, (void *) *thread_resources_ptr);
-#elif defined(TSRM_WIN32)
-	TlsSetValue(tls_key, (void *) *thread_resources_ptr);
-#elif defined(BETHREADS)
-	tls_set(tls_key, (void*) *thread_resources_ptr);
-#endif
+	tsrm_tls_set(*thread_resources_ptr);
 
 	if (tsrm_new_thread_begin_handler) {
 		tsrm_new_thread_begin_handler(thread_id, &((*thread_resources_ptr)->storage));
 	}
 	for (i=0; i<id_count; i++) {
-#if MBO_0
 		if (resource_types_table[i].done) {
 			(*thread_resources_ptr)->storage[i] = NULL;
 		} else
-#endif
 		{
 			(*thread_resources_ptr)->storage[i] = (void *) malloc(resource_types_table[i].size);
 			if (resource_types_table[i].ctor) {
@@ -296,11 +296,11 @@ static void allocate_new_resource(tsrm_tls_entry **thread_resources_ptr, THREAD_
 		}
 	}
 
-	tsrm_mutex_unlock(tsmm_mutex);
-
 	if (tsrm_new_thread_end_handler) {
 		tsrm_new_thread_end_handler(thread_id, &((*thread_resources_ptr)->storage));
 	}
+
+	tsrm_mutex_unlock(tsmm_mutex);
 }
 
 
@@ -321,22 +321,13 @@ TSRM_API void *ts_resource_ex(ts_rsrc_id id, THREAD_T *th_id)
 	if(tsrm_tls_table) {
 #endif
 	if (!th_id) {
-#if defined(PTHREADS)
 		/* Fast path for looking up the resources for the current
 		 * thread. Its used by just about every call to
 		 * ts_resource_ex(). This avoids the need for a mutex lock
 		 * and our hashtable lookup.
 		 */
-		thread_resources = pthread_getspecific(tls_key);
-#elif defined(TSRM_ST)
-		thread_resources = st_thread_getspecific(tls_key);
-#elif defined(TSRM_WIN32)
-		thread_resources = TlsGetValue(tls_key);
-#elif defined(BETHREADS)
-		thread_resources = (tsrm_tls_entry*)tls_get(tls_key);
-#else
-		thread_resources = NULL;
-#endif
+		thread_resources = tsrm_tls_get();
+
 		if (thread_resources) {
 			TSRM_ERROR((TSRM_ERROR_LEVEL_INFO, "Fetching resource id %d for current thread %d", id, (long) thread_resources->thread_id));
 			/* Read a specific resource from the thread's resources.
@@ -387,6 +378,65 @@ TSRM_API void *ts_resource_ex(ts_rsrc_id id, THREAD_T *th_id)
 #endif
 }
 
+/* frees an interpreter context.  You are responsible for making sure that
+ * it is not linked into the TSRM hash, and not marked as the current interpreter */
+void tsrm_free_interpreter_context(void *context)
+{
+	tsrm_tls_entry *next, *thread_resources = (tsrm_tls_entry*)context;
+	int i;
+
+	while (thread_resources) {
+		next = thread_resources->next;
+
+		for (i=0; i<thread_resources->count; i++) {
+			if (resource_types_table[i].dtor) {
+				resource_types_table[i].dtor(thread_resources->storage[i], &thread_resources->storage);
+			}
+		}
+		for (i=0; i<thread_resources->count; i++) {
+			free(thread_resources->storage[i]);
+		}
+		free(thread_resources->storage);
+		free(thread_resources);
+		thread_resources = next;
+	}
+}
+
+void *tsrm_set_interpreter_context(void *new_ctx)
+{
+	tsrm_tls_entry *current;
+
+	current = tsrm_tls_get();
+
+	/* TODO: unlink current from the global linked list, and replace it
+	 * it with the new context, protected by mutex where/if appropriate */
+
+	/* Set thread local storage to this new thread resources structure */
+	tsrm_tls_set(new_ctx);
+
+	/* return old context, so caller can restore it when they're done */
+	return current;
+}
+
+
+/* allocates a new interpreter context */
+void *tsrm_new_interpreter_context(void)
+{
+	tsrm_tls_entry *new_ctx, *current;
+	THREAD_T thread_id;
+
+	thread_id = tsrm_thread_id();
+	tsrm_mutex_lock(tsmm_mutex);
+
+	current = tsrm_tls_get();
+
+	allocate_new_resource(&new_ctx, thread_id);
+	
+	/* switch back to the context that was in use prior to our creation
+	 * of the new one */
+	return tsrm_set_interpreter_context(current);
+}
+
 
 /* frees all resources allocated for the current thread */
 void ts_free_thread(void)
@@ -417,11 +467,7 @@ void ts_free_thread(void)
 			} else {
 				tsrm_tls_table[hash_value] = thread_resources->next;
 			}
-#if defined(PTHREADS)
-			pthread_setspecific(tls_key, 0);
-#elif defined(TSRM_WIN32)
-			TlsSetValue(tls_key, 0);
-#endif
+			tsrm_tls_set(0);
 			free(thread_resources);
 			break;
 		}
@@ -434,10 +480,55 @@ void ts_free_thread(void)
 }
 
 
+/* frees all resources allocated for all threads except current */
+void ts_free_worker_threads(void)
+{
+	tsrm_tls_entry *thread_resources;
+	int i;
+	THREAD_T thread_id = tsrm_thread_id();
+	int hash_value;
+	tsrm_tls_entry *last=NULL;
+
+	tsrm_mutex_lock(tsmm_mutex);
+	hash_value = THREAD_HASH_OF(thread_id, tsrm_tls_table_size);
+	thread_resources = tsrm_tls_table[hash_value];
+
+	while (thread_resources) {
+		if (thread_resources->thread_id != thread_id) {
+			for (i=0; i<thread_resources->count; i++) {
+				if (resource_types_table[i].dtor) {
+					resource_types_table[i].dtor(thread_resources->storage[i], &thread_resources->storage);
+				}
+			}
+			for (i=0; i<thread_resources->count; i++) {
+				free(thread_resources->storage[i]);
+			}
+			free(thread_resources->storage);
+			if (last) {
+				last->next = thread_resources->next;
+			} else {
+				tsrm_tls_table[hash_value] = thread_resources->next;
+			}
+			free(thread_resources);
+			if (last) {
+				thread_resources = last->next;
+			} else {
+				thread_resources = tsrm_tls_table[hash_value];
+			}
+		} else {
+			if (thread_resources->next) {
+				last = thread_resources;
+			}
+			thread_resources = thread_resources->next;
+		}
+	}
+	tsrm_mutex_unlock(tsmm_mutex);
+}
+
+
 /* deallocates all occurrences of a given id */
 void ts_free_id(ts_rsrc_id id)
 {
-#ifdef MBO_0
 	int i;
 	int j = TSRM_UNSHUFFLE_RSRC_ID(id);
 
@@ -466,7 +557,6 @@ void ts_free_id(ts_rsrc_id id)
 	tsrm_mutex_unlock(tsmm_mutex);
 
 	TSRM_ERROR((TSRM_ERROR_LEVEL_CORE, "Successfully freed resource id %d", id));
-#endif
 }
 
 
@@ -481,13 +571,6 @@ TSRM_API THREAD_T tsrm_thread_id(void)
 {
 #ifdef TSRM_WIN32
 	return GetCurrentThreadId();
-#elif defined(NETWARE)
-	/* There seems to be some problem with the LibC call: NXThreadGetId().
-	 * Due to this, the PHPMyAdmin application is abending in PHP calls.
-	 * Used the call, kCurrentThread instead and it works fine.
-	 */
-/*	return NXThreadGetId(); */
-	return kCurrentThread();
 #elif defined(GNUPTH)
 	return pth_self();
 #elif defined(PTHREADS)
@@ -508,24 +591,9 @@ TSRM_API THREAD_T tsrm_thread_id(void)
 TSRM_API MUTEX_T tsrm_mutex_alloc(void)
 {
 	MUTEX_T mutexp;
-#ifdef NETWARE
-#ifndef USE_MPK
-	/* To use the Recursive Mutex Locking of LibC */
-	long flags = NX_MUTEX_RECURSIVE;
-	NXHierarchy_t order = 0;
-	NX_LOCK_INFO_ALLOC (lockInfo, "PHP-TSRM", 0);
-#endif
-#endif
-
 #ifdef TSRM_WIN32
 	mutexp = malloc(sizeof(CRITICAL_SECTION));
 	InitializeCriticalSection(mutexp);
-#elif defined(NETWARE)
-#ifdef USE_MPK
-	mutexp = kMutexAlloc((BYTE*)"PHP-TSRM");
-#else
-	mutexp = NXMutexAlloc(flags, order, &lockInfo);
-#endif
 #elif defined(GNUPTH)
 	mutexp = (MUTEX_T) malloc(sizeof(*mutexp));
 	pth_mutex_init(mutexp);
@@ -557,12 +625,6 @@ TSRM_API void tsrm_mutex_free(MUTEX_T mutexp)
 #ifdef TSRM_WIN32
 		DeleteCriticalSection(mutexp);
 		free(mutexp);
-#elif defined(NETWARE)
-#ifdef USE_MPK
-		kMutexFree(mutexp);
-#else
-		NXMutexFree(mutexp);
-#endif
 #elif defined(GNUPTH)
 		free(mutexp);
 #elif defined(PTHREADS)
@@ -585,25 +647,26 @@ TSRM_API void tsrm_mutex_free(MUTEX_T mutexp)
 }
 
 
-/* Lock a mutex */
+/*
+  Lock a mutex.
+  A return value of 0 indicates success
+*/
 TSRM_API int tsrm_mutex_lock(MUTEX_T mutexp)
 {
 	TSRM_ERROR((TSRM_ERROR_LEVEL_INFO, "Mutex locked thread: %ld", tsrm_thread_id()));
 #ifdef TSRM_WIN32
 	EnterCriticalSection(mutexp);
-	return 1;
-#elif defined(NETWARE)
-#ifdef USE_MPK
-	return kMutexLock(mutexp);
-#else
-	return NXLock(mutexp);
-#endif
+	return 0;
 #elif defined(GNUPTH)
-	return pth_mutex_acquire(mutexp, 0, NULL);
+	if (pth_mutex_acquire(mutexp, 0, NULL)) {
+		return 0;
+	}
+	return -1;
 #elif defined(PTHREADS)
 	return pthread_mutex_lock(mutexp);
 #elif defined(NSAPI)
-	return crit_enter(mutexp);
+	crit_enter(mutexp);
+	return 0;
 #elif defined(PI3WEB)
 	return PISync_lock(mutexp);
 #elif defined(TSRM_ST)
@@ -616,25 +679,26 @@ TSRM_API int tsrm_mutex_lock(MUTEX_T mutexp)
 }
 
 
-/* Unlock a mutex */
+/*
+  Unlock a mutex.
+  A return value of 0 indicates success
+*/
 TSRM_API int tsrm_mutex_unlock(MUTEX_T mutexp)
 {
 	TSRM_ERROR((TSRM_ERROR_LEVEL_INFO, "Mutex unlocked thread: %ld", tsrm_thread_id()));
 #ifdef TSRM_WIN32
 	LeaveCriticalSection(mutexp);
-	return 1;
-#elif defined(NETWARE)
-#ifdef USE_MPK
-	return kMutexUnlock(mutexp);
-#else
-	return NXUnlock(mutexp);
-#endif
+	return 0;
 #elif defined(GNUPTH)
-	return pth_mutex_release(mutexp);
+	if (pth_mutex_release(mutexp)) {
+		return 0;
+	}
+	return -1;
 #elif defined(PTHREADS)
 	return pthread_mutex_unlock(mutexp);
 #elif defined(NSAPI)
-	return crit_exit(mutexp);
+	crit_exit(mutexp);
+	return 0;
 #elif defined(PI3WEB)
 	return PISync_unlock(mutexp);
 #elif defined(TSRM_ST)
