@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2004 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2009 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        | 
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: zend_highlight.c,v 1.43.2.4 2005/01/02 23:54:59 iliaa Exp $ */
+/* $Id: zend_highlight.c,v 1.49.2.3.2.2.2.7 2009/05/05 01:35:44 mattwil Exp $ */
 
 #include "zend.h"
 #include <zend_language_parser.h>
@@ -68,20 +68,12 @@ ZEND_API void zend_html_puts(const char *s, uint len TSRMLS_DC)
 		end = filtered + filtered_len;
 	}
 #endif /* ZEND_MULTIBYTE */
-	
+
 	while (ptr<end) {
 		if (*ptr==' ') {
-			/* Series of spaces should be displayed as &nbsp;'s
-			 * whereas single spaces should be displayed as a space
-			 */
-			if ((ptr+1) < end && *(ptr+1)==' ') {
-				do {
-					zend_html_putc(*ptr);
-				} while ((++ptr < end) && (*ptr==' '));
-			} else {
-				ZEND_PUTC(*ptr);
-				ptr++;
-			}
+			do {
+				zend_html_putc(*ptr);
+			} while ((++ptr < end) && (*ptr==' '));
 		} else {
 			zend_html_putc(*ptr++);
 		}
@@ -95,14 +87,12 @@ ZEND_API void zend_html_puts(const char *s, uint len TSRMLS_DC)
 }
 
 
-
 ZEND_API void zend_highlight(zend_syntax_highlighter_ini *syntax_highlighter_ini TSRMLS_DC)
 {
 	zval token;
 	int token_type;
 	char *last_color = syntax_highlighter_ini->highlight_html;
 	char *next_color;
-	int in_string=0, post_heredoc = 0;
 
 	zend_printf("<code>");
 	zend_printf("<span style=\"color: %s\">\n", last_color);
@@ -124,22 +114,18 @@ ZEND_API void zend_highlight(zend_syntax_highlighter_ini *syntax_highlighter_ini
 			case T_CLOSE_TAG:
 				next_color = syntax_highlighter_ini->highlight_default;
 				break;
+			case '"':
+			case T_ENCAPSED_AND_WHITESPACE:
 			case T_CONSTANT_ENCAPSED_STRING:
 				next_color = syntax_highlighter_ini->highlight_string;
 				break;
-			case '"':
-				next_color = syntax_highlighter_ini->highlight_string;
-				in_string = !in_string;
-				break;				
 			case T_WHITESPACE:
 				zend_html_puts(LANG_SCNG(yy_text), LANG_SCNG(yy_leng) TSRMLS_CC);  /* no color needed */
 				token.type = 0;
 				continue;
 				break;
 			default:
-				if (in_string) {
-					next_color = syntax_highlighter_ini->highlight_string;
-				} else if (token.type == 0) {
+				if (token.type == 0) {
 					next_color = syntax_highlighter_ini->highlight_keyword;
 				} else {
 					next_color = syntax_highlighter_ini->highlight_default;
@@ -156,19 +142,8 @@ ZEND_API void zend_highlight(zend_syntax_highlighter_ini *syntax_highlighter_ini
 				zend_printf("<span style=\"color: %s\">", last_color);
 			}
 		}
-		switch (token_type) {
-			case T_END_HEREDOC:
-				zend_html_puts(token.value.str.val, token.value.str.len TSRMLS_CC);
-				post_heredoc = 1;
-				break;
-			default:
-				zend_html_puts(LANG_SCNG(yy_text), LANG_SCNG(yy_leng) TSRMLS_CC);
-				if (post_heredoc) {
-					 zend_html_putc('\n');
-					 post_heredoc = 0;
-				}
-				break;
-		}
+
+		zend_html_puts(LANG_SCNG(yy_text), LANG_SCNG(yy_leng) TSRMLS_CC);
 
 		if (token.type == IS_STRING) {
 			switch (token_type) {
@@ -188,6 +163,7 @@ ZEND_API void zend_highlight(zend_syntax_highlighter_ini *syntax_highlighter_ini
 		}
 		token.type = 0;
 	}
+
 	if (last_color != syntax_highlighter_ini->highlight_html) {
 		zend_printf("</span>\n");
 	}
@@ -195,8 +171,6 @@ ZEND_API void zend_highlight(zend_syntax_highlighter_ini *syntax_highlighter_ini
 	zend_printf("</code>");
 }
 
-
-
 ZEND_API void zend_strip(TSRMLS_D)
 {
 	zval token;
@@ -217,19 +191,18 @@ ZEND_API void zend_strip(TSRMLS_D)
 				token.type = 0;
 				continue;
 			
-			case T_END_HEREDOC: {
-					char *ptr = LANG_SCNG(yy_text);
-
-					zend_write(ptr, LANG_SCNG(yy_leng) - 1);
-					/* The ensure that we only write one ; and that it followed by the required newline */
-					zend_write("\n", sizeof("\n") - 1);
-					if (ptr[LANG_SCNG(yy_leng) - 1] == ';') {
-						lex_scan(&token TSRMLS_CC);
-					}
-					efree(token.value.str.val);
+			case T_END_HEREDOC:
+				zend_write(LANG_SCNG(yy_text), LANG_SCNG(yy_leng));
+				efree(token.value.str.val);
+				/* read the following character, either newline or ; */
+				if (lex_scan(&token TSRMLS_CC) != T_WHITESPACE) {
+					zend_write(LANG_SCNG(yy_text), LANG_SCNG(yy_leng));
 				}
-				break;
-			
+				zend_write("\n", sizeof("\n") - 1);
+				prev_space = 1;
+				token.type = 0;
+				continue;
+
 			default:
 				zend_write(LANG_SCNG(yy_text), LANG_SCNG(yy_leng));
 				break;
