@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2003 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2009 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        | 
@@ -17,35 +17,57 @@
    +----------------------------------------------------------------------+
 */
 
+/* $Id: zend_language_scanner.h,v 1.19.2.1.2.1.2.5 2008/12/31 11:15:32 sebastian Exp $ */
 
 #ifndef ZEND_SCANNER_H
 #define ZEND_SCANNER_H
 
 typedef struct _zend_lex_state {
-	YY_BUFFER_STATE buffer_state;
-	int state;
+	unsigned int yy_leng;
+	unsigned char *yy_start;
+	unsigned char *yy_text;
+	unsigned char *yy_cursor;
+	unsigned char *yy_marker;
+	unsigned char *yy_limit;
+	int yy_state;
+	zend_stack state_stack;
+
 	zend_file_handle *in;
 	uint lineno;
 	char *filename;
+
 #ifdef ZEND_MULTIBYTE
-	char *code;
- 	int code_size;
-	char *current_code;
-	int current_code_size;
-	zend_multibyte_filter input_filter;
-	zend_multibyte_filter output_filter;
+	/* original (unfiltered) script */
+	unsigned char *script_org;
+	size_t script_org_size;
+
+	/* filtered script */
+	unsigned char *script_filtered;
+	size_t script_filtered_size;
+
+	/* input/ouput filters */
+	zend_encoding_filter input_filter;
+	zend_encoding_filter output_filter;
 	zend_encoding *script_encoding;
 	zend_encoding *internal_encoding;
 #endif /* ZEND_MULTIBYTE */
 } zend_lex_state;
 
 
-void zend_fatal_scanner_error(char *);
 BEGIN_EXTERN_C()
 int zend_compare_file_handles(zend_file_handle *fh1, zend_file_handle *fh2);
 ZEND_API void zend_save_lexical_state(zend_lex_state *lex_state TSRMLS_DC);
 ZEND_API void zend_restore_lexical_state(zend_lex_state *lex_state TSRMLS_DC);
 ZEND_API int zend_prepare_string_for_scanning(zval *str, char *filename TSRMLS_DC);
+
 END_EXTERN_C()
 
 #endif
+
+/*
+ * Local variables:
+ * tab-width: 4
+ * c-basic-offset: 4
+ * indent-tabs-mode: t
+ * End:
+ */
