@@ -89,8 +89,9 @@
  *	directly -- and assumed always to succeed.
  */
 
-/* $Id: zend_strtod.c,v 1.17.2.2.2.10 2007/04/26 23:13:25 tony2001 Exp $ */
+/* $Id: zend_strtod.c,v 1.17.2.2.2.13.2.5 2009/03/18 10:18:10 dmitry Exp $ */
 
+#include <zend_operators.h>
 #include <zend_strtod.h>
 
 #ifdef ZTS
@@ -135,6 +136,16 @@ typedef unsigned long int uint32_t;
 # endif
 #endif
 
+#if (defined(__APPLE__) || defined(__APPLE_CC__)) && (defined(__BIG_ENDIAN__) || defined(__LITTLE_ENDIAN__))
+# if defined(__LITTLE_ENDIAN__)
+#  undef WORDS_BIGENDIAN
+# else 
+#  if defined(__BIG_ENDIAN__)
+#   define WORDS_BIGENDIAN
+#  endif
+# endif
+#endif
+
 #ifdef WORDS_BIGENDIAN
 #define IEEE_BIG_ENDIAN
 #else
@@ -273,9 +284,9 @@ BEGIN_EXTERN_C()
 #endif
 
 /* The following definition of Storeinc is appropriate for MIPS processors.
- *  * An alternative that might be better on some machines is
- *   * #define Storeinc(a,b,c) (*a++ = b << 16 | c & 0xffff)
- *    */
+ * An alternative that might be better on some machines is
+ * #define Storeinc(a,b,c) (*a++ = b << 16 | c & 0xffff)
+ */
 #if defined(IEEE_LITTLE_ENDIAN) + defined(VAX) + defined(__arm__)
 #define Storeinc(a,b,c) (((unsigned short *)a)[1] = (unsigned short)b, \
 		((unsigned short *)a)[0] = (unsigned short)c, a++)
@@ -973,9 +984,9 @@ static Bigint * diff(Bigint *a, Bigint *b)
 
 static double ulp (double _x)
 {
-	_double x;
+	volatile _double x;
 	register Long L;
-	_double a;
+	volatile _double a;
 
 	value(x) = _x;
 	L = (word0(x) & Exp_mask) - (P-1)*Exp_msk1;
@@ -1015,7 +1026,7 @@ b2d
 {
 	ULong *xa, *xa0, w, y, z;
 	int k;
-	_double d;
+	volatile _double d;
 #ifdef VAX
 	ULong d0, d1;
 #else
@@ -1081,7 +1092,7 @@ static Bigint * d2b(double _d, int *e, int *bits)
 	Bigint *b;
 	int de, i, k;
 	ULong *x, y, z;
-	_double d;
+	volatile _double d;
 #ifdef VAX
 	ULong d0, d1;
 #endif
@@ -1202,7 +1213,7 @@ static Bigint * d2b(double _d, int *e, int *bits)
 
 static double ratio (Bigint *a, Bigint *b)
 {
-	_double da, db;
+	volatile _double da, db;
 	int k, ka, kb;
 
 	value(da) = b2d(a, &ka);
@@ -1469,7 +1480,7 @@ ZEND_API char * zend_dtoa(double _d, int mode, int ndigits, int *decpt, int *sig
 	Bigint *b, *b1, *delta, *mlo, *mhi, *S, *tmp;
 	double ds;
 	char *s, *s0;
-	_double d, d2, eps;
+	volatile _double d, d2, eps;
 
 	value(d) = _d;
 
@@ -1709,14 +1720,7 @@ ZEND_API char * zend_dtoa(double _d, int mode, int ndigits, int *decpt, int *sig
 					if (value(d) > 0.5 + value(eps))
 						goto bump_up;
 					else if (value(d) < 0.5 - value(eps)) {
-						/* cut ALL traling zeros only if the number of chars is greater than precision 
-						 * otherwise cut only extra zeros
-						 */
-						if (k < ndigits) {
-							while(*--s == '0' && (s - s0) > k);
-						} else {
-							while(*--s == '0');
-						}
+						while(*--s == '0');
 						s++;
 						goto ret1;
 					}
@@ -2032,7 +2036,7 @@ ZEND_API double zend_strtod (CONST char *s00, char **se)
 		e, e1, esign, i, j, k, nd, nd0, nf, nz, nz0, sign;
 	CONST char *s, *s0, *s1;
 	double aadj, aadj1, adj;
-	_double rv, rv0;
+	volatile _double rv, rv0;
 	Long L;
 	ULong y, z;
 	Bigint *bb, *bb1, *bd, *bd0, *bs, *delta, *tmp;
@@ -2617,7 +2621,7 @@ ZEND_API double zend_oct_strtod(const char *str, char **endptr)
 	s++;
 
 	while ((c = *s++)) {
-		if (c > '7') {
+		if (c < '0' || c > '7') {
 			/* break and return the current value if the number is not well-formed
 			 * that's what Linux strtol() does 
 			 */
