@@ -2,12 +2,12 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2004 The PHP Group                                |
+  | Copyright (c) 1997-2009 The PHP Group                                |
   +----------------------------------------------------------------------+
-  | This source file is subject to version 3.0 of the PHP license,       |
+  | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
-  | http://www.php.net/license/3_0.txt.                                  |
+  | http://www.php.net/license/3_01.txt                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
  
-/* $Id: bz2.c,v 1.6 2004/01/08 08:14:18 andi Exp $ */
+/* $Id: bz2.c,v 1.14.2.3.2.12.2.12 2009/05/15 17:28:08 kalle Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -40,20 +40,64 @@
 #define PHP_BZ_ERRSTR  1
 #define PHP_BZ_ERRBOTH 2
 
-/* Blocksize of the decompression buffer */
-#define PHP_BZ_DECOMPRESS_SIZE 4096
+static PHP_MINIT_FUNCTION(bz2);
+static PHP_MSHUTDOWN_FUNCTION(bz2);
+static PHP_MINFO_FUNCTION(bz2);
+static PHP_FUNCTION(bzopen);
+static PHP_FUNCTION(bzread);
+static PHP_FUNCTION(bzerrno);
+static PHP_FUNCTION(bzerrstr);
+static PHP_FUNCTION(bzerror);
+static PHP_FUNCTION(bzcompress);
+static PHP_FUNCTION(bzdecompress);
 
-function_entry bz2_functions[] = {
-	PHP_FE(bzopen,       NULL)
-	PHP_FE(bzread,       NULL)
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO_EX(arginfo_bzread, 0, 0, 1)
+	ZEND_ARG_INFO(0, bz)
+	ZEND_ARG_INFO(0, length)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_bzopen, 0)
+	ZEND_ARG_INFO(0, file)
+	ZEND_ARG_INFO(0, mode)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_bzerrno, 0)
+	ZEND_ARG_INFO(0, bz)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_bzerrstr, 0)
+	ZEND_ARG_INFO(0, bz)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_bzerror, 0)
+	ZEND_ARG_INFO(0, bz)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_bzcompress, 0, 0, 2)
+	ZEND_ARG_INFO(0, source)
+	ZEND_ARG_INFO(0, blocksize)
+	ZEND_ARG_INFO(0, workfactor)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_bzdecompress, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+	ZEND_ARG_INFO(0, small)
+ZEND_END_ARG_INFO()
+
+/* }}} */
+
+static const zend_function_entry bz2_functions[] = {
+	PHP_FE(bzopen,       arginfo_bzopen)
+	PHP_FE(bzread,       arginfo_bzread)
 	PHP_FALIAS(bzwrite,   fwrite,		NULL)
 	PHP_FALIAS(bzflush,   fflush,		NULL)
 	PHP_FALIAS(bzclose,   fclose,		NULL)
-	PHP_FE(bzerrno,      NULL)
-	PHP_FE(bzerrstr,     NULL)
-	PHP_FE(bzerror,      NULL)
-	PHP_FE(bzcompress,   NULL)
-	PHP_FE(bzdecompress, NULL)
+	PHP_FE(bzerrno,      arginfo_bzerrno)
+	PHP_FE(bzerrstr,     arginfo_bzerrstr)
+	PHP_FE(bzerror,      arginfo_bzerror)
+	PHP_FE(bzcompress,   arginfo_bzcompress)
+	PHP_FE(bzdecompress, arginfo_bzdecompress)
 	{NULL, NULL, NULL}
 };
 
@@ -165,7 +209,7 @@ PHP_BZ2_API php_stream *_php_stream_bz2open(php_stream_wrapper *wrapper,
 	if (strncasecmp("compress.bzip2://", path, 17) == 0) {
 		path += 17;
 	}
-	if (mode[0] != 'w' && mode[0] != 'r' && mode[1] != '\0') {
+	if (mode[0] == '\0' || (mode[0] != 'w' && mode[0] != 'r' && mode[1] != '\0')) {
 		return NULL;
 	}
 
@@ -174,6 +218,10 @@ PHP_BZ2_API php_stream *_php_stream_bz2open(php_stream_wrapper *wrapper,
 #else
 	path_copy = path;
 #endif  
+
+	if ((PG(safe_mode) && (!php_checkuid(path_copy, NULL, CHECKUID_CHECK_FILE_AND_DIR))) || php_check_open_basedir(path_copy TSRMLS_CC)) {
+		return NULL;
+	}
 	
 	/* try and open it directly first */
 	bz_file = BZ2_bzopen(path_copy, mode);
@@ -185,7 +233,7 @@ PHP_BZ2_API php_stream *_php_stream_bz2open(php_stream_wrapper *wrapper,
 	
 	if (bz_file == NULL) {
 		/* that didn't work, so try and get something from the network/wrapper */
-		stream = php_stream_open_wrapper(path, mode, options, opened_path);
+		stream = php_stream_open_wrapper(path, mode, options | STREAM_WILL_CAST | ENFORCE_SAFE_MODE, opened_path);
 	
 		if (stream) {
 			int fd;
@@ -193,10 +241,11 @@ PHP_BZ2_API php_stream *_php_stream_bz2open(php_stream_wrapper *wrapper,
 				bz_file = BZ2_bzdopen(fd, mode);
 			}
 		}
+
 		/* remove the file created by php_stream_open_wrapper(), it is not needed since BZ2 functions
 		 * failed.
 		 */
-		if (!bz_file && mode[0] == 'w') {
+		if (opened_path && !bz_file && mode[0] == 'w') {
 			VCWD_UNLINK(*opened_path);
 		}
 	}
@@ -232,7 +281,7 @@ static php_stream_wrapper_ops bzip2_stream_wops = {
 	NULL  /* rmdir */
 };
 
-php_stream_wrapper php_stream_bzip2_wrapper = {
+static php_stream_wrapper php_stream_bzip2_wrapper = {
 	&bzip2_stream_wops,
 	NULL,
 	0 /* is_url */
@@ -240,31 +289,34 @@ php_stream_wrapper php_stream_bzip2_wrapper = {
 
 static void php_bz2_error(INTERNAL_FUNCTION_PARAMETERS, int);
 
-PHP_MINIT_FUNCTION(bz2)
+static PHP_MINIT_FUNCTION(bz2)
 {
 	php_register_url_stream_wrapper("compress.bzip2", &php_stream_bzip2_wrapper TSRMLS_CC);
-
+	php_stream_filter_register_factory("bzip2.*", &php_bz2_filter_factory TSRMLS_CC);
 	return SUCCESS;
 }
 
-PHP_MSHUTDOWN_FUNCTION(bz2)
+static PHP_MSHUTDOWN_FUNCTION(bz2)
 {
 	php_unregister_url_stream_wrapper("compress.bzip2" TSRMLS_CC);
+	php_stream_filter_unregister_factory("bzip2.*" TSRMLS_CC);
 
 	return SUCCESS;
 }
 
-PHP_MINFO_FUNCTION(bz2)
+static PHP_MINFO_FUNCTION(bz2)
 {
 	php_info_print_table_start();
 	php_info_print_table_row(2, "BZip2 Support", "Enabled");
+	php_info_print_table_row(2, "Stream Wrapper support", "compress.bz2://");
+	php_info_print_table_row(2, "Stream Filter support", "bzip2.decompress, bzip2.compress");
 	php_info_print_table_row(2, "BZip2 Version", (char *) BZ2_bzlibVersion());
 	php_info_print_table_end();
 }
 
-/* {{{ proto string bzread(int bz[, int length])
+/* {{{ proto string bzread(resource bz[, int length])
    Reads up to length bytes from a BZip2 stream, or 1024 bytes if length is not specified */
-PHP_FUNCTION(bzread)
+static PHP_FUNCTION(bzread)
 {
 	zval *bz;
 	long len = 1024;
@@ -276,7 +328,7 @@ PHP_FUNCTION(bzread)
 	
 	php_stream_from_zval(stream, &bz);
 
-	if (len < 0) {
+	if ((len + 1) < 1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "length may not be negative");
 		RETURN_FALSE;
 	}
@@ -304,44 +356,86 @@ PHP_FUNCTION(bzread)
 
 /* {{{ proto resource bzopen(string|int file|fp, string mode)
    Opens a new BZip2 stream */
-PHP_FUNCTION(bzopen)
+static PHP_FUNCTION(bzopen)
 {
-	zval    **file,   /* The file to open */
-	        **mode;   /* The mode to open the stream with */
+	zval    **file;   /* The file to open */
+	char     *mode;   /* The mode to open the stream with */
+	int      mode_len;
+
 	BZFILE   *bz;     /* The compressed file stream */
 	php_stream *stream = NULL;
 	
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &file, &mode) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Zs", &file, &mode, &mode_len) == FAILURE) {
+		return;
 	}
-	convert_to_string_ex(mode);
 
-	if (Z_STRVAL_PP(mode)[0] != 'r' && Z_STRVAL_PP(mode)[0] != 'w' && Z_STRVAL_PP(mode)[1] != '\0') {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "'%s' is not a valid mode for bzopen(). Only 'w' and 'r' are supported.", Z_STRVAL_PP(mode));
+	if (mode_len != 1 || (mode[0] != 'r' && mode[0] != 'w')) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "'%s' is not a valid mode for bzopen(). Only 'w' and 'r' are supported.", mode);
 		RETURN_FALSE;
 	}
 
 	/* If it's not a resource its a string containing the filename to open */
-	if (Z_TYPE_PP(file) != IS_RESOURCE) {
+	if (Z_TYPE_PP(file) == IS_STRING) {
 		convert_to_string_ex(file);
+
+		if (Z_STRLEN_PP(file) == 0) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "filename cannot be empty");
+			RETURN_FALSE;
+		}
+
 		stream = php_stream_bz2open(NULL,
 									Z_STRVAL_PP(file), 
-									Z_STRVAL_PP(mode), 
+									mode, 
 									ENFORCE_SAFE_MODE | REPORT_ERRORS, 
 									NULL);
-	} else {
+	} else if (Z_TYPE_PP(file) == IS_RESOURCE) {
 		/* If it is a resource, than its a stream resource */
 		int fd;
+		int stream_mode_len;
 
 		php_stream_from_zval(stream, file);
+		stream_mode_len = strlen(stream->mode);
+		
+		if (stream_mode_len != 1 && !(stream_mode_len == 2 && memchr(stream->mode, 'b', 2))) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "cannot use stream opened in mode '%s'", stream->mode);
+			RETURN_FALSE;
+		} else if (stream_mode_len == 1 && stream->mode[0] != 'r' && stream->mode[0] != 'w' && stream->mode[0] != 'a' && stream->mode[0] != 'x') {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "cannot use stream opened in mode '%s'", stream->mode);
+			RETURN_FALSE;
+		}
+
+		switch(mode[0]) {
+			case 'r':
+				/* only "r" and "rb" are supported */
+				if (stream->mode[0] != mode[0] && !(stream_mode_len == 2 && stream->mode[1] != mode[0])) {
+					php_error_docref(NULL TSRMLS_CC, E_WARNING, "cannot read from a stream opened in write only mode");
+					RETURN_FALSE;
+				}
+				break;
+			case 'w':
+				/* support only "w"(b), "a"(b), "x"(b) */
+				if (stream->mode[0] != mode[0] && !(stream_mode_len == 2 && stream->mode[1] != mode[0])
+					&& stream->mode[0] != 'a' && !(stream_mode_len == 2 && stream->mode[1] != 'a')
+					&& stream->mode[0] != 'x' && !(stream_mode_len == 2 && stream->mode[1] != 'x')) {
+					php_error_docref(NULL TSRMLS_CC, E_WARNING, "cannot write to a stream opened in read only mode");
+					RETURN_FALSE;
+				}
+				break;
+			default:
+				/* not reachable */
+				break;
+		}
 
 		if (FAILURE == php_stream_cast(stream, PHP_STREAM_AS_FD, (void *) &fd, REPORT_ERRORS)) {
 			RETURN_FALSE;
 		}
 		
-		bz = BZ2_bzdopen(fd, Z_STRVAL_PP(mode));
+		bz = BZ2_bzdopen(fd, mode);
 
-		stream = php_stream_bz2open_from_BZFILE(bz, Z_STRVAL_PP(mode), stream);
+		stream = php_stream_bz2open_from_BZFILE(bz, mode, stream);
+	} else {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "first parameter has to be string or file-resource");
+		RETURN_FALSE;
 	}
 
 	if (stream) {
@@ -354,7 +448,7 @@ PHP_FUNCTION(bzopen)
 
 /* {{{ proto int bzerrno(resource bz)
    Returns the error number */
-PHP_FUNCTION(bzerrno)
+static PHP_FUNCTION(bzerrno)
 {
 	php_bz2_error(INTERNAL_FUNCTION_PARAM_PASSTHRU, PHP_BZ_ERRNO);
 }
@@ -362,7 +456,7 @@ PHP_FUNCTION(bzerrno)
 
 /* {{{ proto string bzerrstr(resource bz)
    Returns the error string */
-PHP_FUNCTION(bzerrstr)
+static PHP_FUNCTION(bzerrstr)
 {
 	php_bz2_error(INTERNAL_FUNCTION_PARAM_PASSTHRU, PHP_BZ_ERRSTR);
 }
@@ -370,7 +464,7 @@ PHP_FUNCTION(bzerrstr)
 
 /* {{{ proto array bzerror(resource bz)
    Returns the error number and error string in an associative array */
-PHP_FUNCTION(bzerror)
+static PHP_FUNCTION(bzerror)
 {
 	php_bz2_error(INTERNAL_FUNCTION_PARAM_PASSTHRU, PHP_BZ_ERRBOTH);
 }
@@ -378,49 +472,44 @@ PHP_FUNCTION(bzerror)
 
 /* {{{ proto string bzcompress(string source [, int blocksize100k [, int workfactor]])
    Compresses a string into BZip2 encoded data */
-PHP_FUNCTION(bzcompress)
+static PHP_FUNCTION(bzcompress)
 {
-	zval            **source,          /* Source data to compress */
-	                **zblock_size,     /* Optional block size to use */
-					**zwork_factor;    /* Optional work factor to use */
+	char             *source;          /* Source data to compress */
+	long              zblock_size = 0; /* Optional block size to use */
+	long              zwork_factor = 0;/* Optional work factor to use */
 	char             *dest = NULL;     /* Destination to place the compressed data into */
 	int               error,           /* Error Container */
 					  block_size  = 4, /* Block size for compression algorithm */
 					  work_factor = 0, /* Work factor for compression algorithm */
 					  argc;            /* Argument count */
-	unsigned int      source_len,      /* Length of the source data */
-					  dest_len;        /* Length of the destination buffer */ 
-	
+	int               source_len;      /* Length of the source data */
+	unsigned int      dest_len;        /* Length of the destination buffer */ 
+
 	argc = ZEND_NUM_ARGS();
 
-	if (argc < 1 || argc > 3 || zend_get_parameters_ex(argc, &source, &zblock_size, &zwork_factor) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|ll", &source, &source_len, &zblock_size, &zwork_factor) == FAILURE) {
+		return;
 	}
 
-	convert_to_string_ex(source);
-	
 	/* Assign them to easy to use variables, dest_len is initially the length of the data
 	   + .01 x length of data + 600 which is the largest size the results of the compression 
 	   could possibly be, at least that's what the libbz2 docs say (thanks to jeremy@nirvani.net 
 	   for pointing this out).  */
-	source_len = Z_STRLEN_PP(source);
-	dest_len   = Z_STRLEN_PP(source) + (0.01 * Z_STRLEN_PP(source)) + 600;
+	dest_len   = (unsigned int) (source_len + (0.01 * source_len) + 600);
 	
 	/* Allocate the destination buffer */
 	dest = emalloc(dest_len + 1);
 	
 	/* Handle the optional arguments */
 	if (argc > 1) {
-		convert_to_long_ex(zblock_size);
-		block_size = Z_LVAL_PP(zblock_size);
+		block_size = zblock_size;
 	}
 	
 	if (argc > 2) {
-		convert_to_long_ex(zwork_factor);
-		work_factor = Z_LVAL_PP(zwork_factor);
+		work_factor = zwork_factor;
 	}
 
-	error = BZ2_bzBuffToBuffCompress(dest, &dest_len, Z_STRVAL_PP(source), source_len, block_size, 0, work_factor);
+	error = BZ2_bzBuffToBuffCompress(dest, &dest_len, source, source_len, block_size, 0, work_factor);
 	if (error != BZ_OK) {
 		efree(dest);
 		RETURN_LONG(error);
@@ -436,58 +525,55 @@ PHP_FUNCTION(bzcompress)
 
 /* {{{ proto string bzdecompress(string source [, int small])
    Decompresses BZip2 compressed data */
-PHP_FUNCTION(bzdecompress)
+static PHP_FUNCTION(bzdecompress)
 {
-	zval    **source,                             /* Source data to decompress */
-	        **zsmall;                             /* (Optional) user specified small */
-	char     *dest;                               /* Destination buffer, initially allocated */
-	int       error,                              /* Error container */
-	          iter = 1,                           /* Iteration count for the compression loop */
-			  size,                               /* Current size to realloc the dest buffer to */
-			  dest_len = PHP_BZ_DECOMPRESS_SIZE,  /* Size of the destination length */
-			  small    = 0,                       /* The actual small */
-			  argc     = ZEND_NUM_ARGS();         /* Argument count */
-	
-	if (argc < 1 || argc > 2 || zend_get_parameters_ex(argc, &source, &zsmall) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	char *source, *dest;
+	int source_len, error;
+	long small = 0;
+#if defined(PHP_WIN32)
+	unsigned __int64 size = 0;
+#else
+	unsigned long long size = 0;
+#endif
+	bz_stream bzs;
+
+	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|l", &source, &source_len, &small)) {
+		RETURN_FALSE;
 	}
 
-	convert_to_string_ex(source);
-	
-	/* optional small argument handling */
-	if (argc > 1) {
-		convert_to_long_ex(zsmall);
-		small = Z_LVAL_PP(zsmall);
+	bzs.bzalloc = NULL;
+	bzs.bzfree = NULL;
+
+	if (BZ2_bzDecompressInit(&bzs, 0, small) != BZ_OK) {
+		RETURN_FALSE;
 	}
 
-	/* Depending on the size of the source buffer, either allocate
-	  the length of the source buffer or the a default decompression
-	  size */
-	dest = emalloc(PHP_BZ_DECOMPRESS_SIZE > Z_STRLEN_PP(source) ? PHP_BZ_DECOMPRESS_SIZE : Z_STRLEN_PP(source));
+	bzs.next_in = source;
+	bzs.avail_in = source_len;
 
-	/* (de)Compression Loop */	
-	do {
-		/* Handle the (re)allocation of the buffer */
-		size = dest_len * iter;
-		if (iter > 1) {
-			dest = erealloc(dest, size);
-		}
-		++iter;
-
-		/* Perform the decompression */
-		error = BZ2_bzBuffToBuffDecompress(dest, &size, Z_STRVAL_PP(source), Z_STRLEN_PP(source), small, 0);
-	} while (error == BZ_OUTBUFF_FULL);
+	/* in most cases bz2 offers at least 2:1 compression, so we use that as our base */
+	bzs.avail_out = source_len * 2;
+	bzs.next_out = dest = emalloc(bzs.avail_out + 1);
 	
-	if (error != BZ_OK) {
+	while ((error = BZ2_bzDecompress(&bzs)) == BZ_OK && bzs.avail_in > 0) {
+		/* compression is better then 2:1, need to allocate more memory */
+		bzs.avail_out = source_len;
+		size = (bzs.total_out_hi32 * (unsigned int) -1) + bzs.total_out_lo32;
+		dest = safe_erealloc(dest, 1, bzs.avail_out+1, (size_t) size );
+		bzs.next_out = dest + size;
+	}
+
+	if (error == BZ_STREAM_END || error == BZ_OK) {
+		size = (bzs.total_out_hi32 * (unsigned int) -1) + bzs.total_out_lo32;
+		dest = safe_erealloc(dest, 1, (size_t) size, 1);
+		dest[size] = '\0';
+		RETVAL_STRINGL(dest, (int) size, 0);
+	} else { /* real error */
 		efree(dest);
-		RETURN_LONG(error);
-	} else {
-		/* we might have allocated a little to much, so erealloc the buffer 
-		 down to size, before returning it */
-		dest = erealloc(dest, size + 1);
-		dest[size] = 0;
-		RETURN_STRINGL(dest, size, 0);
+		RETVAL_LONG(error);
 	}
+
+	BZ2_bzDecompressEnd(&bzs);
 }
 /* }}} */
 
@@ -495,17 +581,17 @@ PHP_FUNCTION(bzdecompress)
    The central error handling interface, does the work for bzerrno, bzerrstr and bzerror */
 static void php_bz2_error(INTERNAL_FUNCTION_PARAMETERS, int opt)
 { 
-	zval        **bzp;     /* BZip2 Resource Pointer */
+	zval         *bzp;     /* BZip2 Resource Pointer */
 	php_stream   *stream;
 	const char   *errstr;  /* Error string */
 	int           errnum;  /* Error number */
 	struct php_bz2_stream_data_t *self;
 	
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &bzp) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &bzp) == FAILURE) {
+		return;
 	}
 
-	php_stream_from_zval(stream, bzp);
+	php_stream_from_zval(stream, &bzp);
 
 	if (!php_stream_is(stream, PHP_STREAM_IS_BZIP2)) {
 		RETURN_FALSE;
