@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2008 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -14,7 +14,7 @@
    +----------------------------------------------------------------------+
    | Authors: Shane Caraveo             <shane@caraveo.com>               | 
    |          Colin Viebrock            <colin@easydns.com>               |
-   |          Hartmut Holzgraefe        <hartmut@six.de>                  |
+   |          Hartmut Holzgraefe        <hholzgra@php.net>                |
    +----------------------------------------------------------------------+
  */
 /* $Id: */
@@ -28,31 +28,24 @@
    Convert UNIX timestamp to Julian Day */
 PHP_FUNCTION(unixtojd)
 {
-  pval *timestamp;
-  long jdate; 
-  time_t t;
-  struct tm *ta, tmbuf;
-  int myargc=ZEND_NUM_ARGS();
-	
-  if ((myargc > 1) || (zend_get_parameters(ht, myargc, &timestamp) != SUCCESS)) {
-    WRONG_PARAM_COUNT;
-  }
+	time_t ts = 0;
+	struct tm *ta, tmbuf;
 
-  if(myargc==1) {
-    convert_to_long(timestamp);
-    t = Z_LVAL_P(timestamp);
-  } else {
-    t = time(NULL);
-  }
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &ts) == FAILURE) {
+		return;
+	}	
 
-  if(t < 0) {
-	RETURN_FALSE;
-  }
+	if (!ts) {
+		ts = time(NULL);
+	} else if (ts < 0) {
+		RETURN_FALSE;
+	}
 
-  ta = php_localtime_r(&t, &tmbuf);
-  jdate = GregorianToSdn(ta->tm_year+1900, ta->tm_mon+1, ta->tm_mday);
-  
-  RETURN_LONG(jdate);
+	if (!(ta = php_localtime_r(&ts, &tmbuf))) {
+		RETURN_FALSE;
+	}
+
+	RETURN_LONG(GregorianToSdn(ta->tm_year+1900, ta->tm_mon+1, ta->tm_mday));
 }
 /* }}} */
 
@@ -60,21 +53,18 @@ PHP_FUNCTION(unixtojd)
    Convert Julian Day to UNIX timestamp */
 PHP_FUNCTION(jdtounix)
 {
-  pval *jday;
-  long uday;
+	long uday;
 
-  if ((ZEND_NUM_ARGS()!= 1) || (zend_get_parameters(ht, 1, &jday) != SUCCESS)) {
-    WRONG_PARAM_COUNT;
-  }
-  
-  convert_to_long(jday);
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &uday) == FAILURE) {
+		return;
+	}	
+	uday -= 2440588 /* J.D. of 1.1.1970 */;
 
-  uday = Z_LVAL_P(jday) - 2440588 /* J.D. of 1.1.1970 */;
-  
-  if(uday<0)     RETURN_FALSE; /* before beginning of unix epoch */ 
-  if(uday>24755) RETURN_FALSE; /* behind end of unix epoch */
+	if (uday < 0 || uday > 24755) { /* before beginning of unix epoch or behind end of unix epoch */ 
+		RETURN_FALSE;
+	}
 
-  RETURN_LONG(uday*24*3600);
+	RETURN_LONG(uday * 24 * 3600);
 }
 /* }}} */
 
