@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: document.c,v 1.68.2.3.2.5 2007/01/01 09:36:00 sebastian Exp $ */
+/* $Id: document.c,v 1.68.2.3.2.5.2.17 2009/06/14 13:13:35 iliaa Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -41,6 +41,149 @@ struct _idsIterator {
 #define DOM_LOAD_STRING 0
 #define DOM_LOAD_FILE 1
 
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_element, 0, 0, 1)
+	ZEND_ARG_INFO(0, tagName)
+	ZEND_ARG_INFO(0, value)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_document_fragment, 0, 0, 0)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_text_node, 0, 0, 1)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_comment, 0, 0, 1)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_cdatasection, 0, 0, 1)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_processing_instruction, 0, 0, 2)
+	ZEND_ARG_INFO(0, target)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_attribute, 0, 0, 1)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_entity_reference, 0, 0, 1)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_get_elements_by_tag_name, 0, 0, 1)
+	ZEND_ARG_INFO(0, tagName)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_import_node, 0, 0, 2)
+	ZEND_ARG_OBJ_INFO(0, importedNode, DOMNode, 0)
+	ZEND_ARG_INFO(0, deep)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_element_ns, 0, 0, 2)
+	ZEND_ARG_INFO(0, namespaceURI)
+	ZEND_ARG_INFO(0, qualifiedName)
+	ZEND_ARG_INFO(0, value)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_attribute_ns, 0, 0, 2)
+	ZEND_ARG_INFO(0, namespaceURI)
+	ZEND_ARG_INFO(0, qualifiedName)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_get_elements_by_tag_name_ns, 0, 0, 2)
+	ZEND_ARG_INFO(0, namespaceURI)
+	ZEND_ARG_INFO(0, localName)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_get_element_by_id, 0, 0, 1)
+	ZEND_ARG_INFO(0, elementId)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_adopt_node, 0, 0, 1)
+	ZEND_ARG_OBJ_INFO(0, source, DOMNode, 0)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_normalize_document, 0, 0, 0)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_rename_node, 0, 0, 3)
+	ZEND_ARG_OBJ_INFO(0, node, DOMNode, 0)
+	ZEND_ARG_INFO(0, namespaceURI)
+	ZEND_ARG_INFO(0, qualifiedName)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_load, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+	ZEND_ARG_INFO(0, options)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_save, 0, 0, 1)
+	ZEND_ARG_INFO(0, file)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_loadxml, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+	ZEND_ARG_INFO(0, options)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_savexml, 0, 0, 0)
+	ZEND_ARG_OBJ_INFO(0, node, DOMNode, 1)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_construct, 0, 0, 0)
+	ZEND_ARG_INFO(0, version)
+	ZEND_ARG_INFO(0, encoding)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_validate, 0, 0, 0)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_xinclude, 0, 0, 0)
+	ZEND_ARG_INFO(0, options)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_loadhtml, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_loadhtmlfile, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_savehtml, 0, 0, 0)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_savehtmlfile, 0, 0, 1)
+	ZEND_ARG_INFO(0, file)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_schema_validate_file, 0, 0, 1)
+	ZEND_ARG_INFO(0, filename)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_schema_validate_xml, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_relaxNG_validate_file, 0, 0, 1)
+	ZEND_ARG_INFO(0, filename)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_relaxNG_validate_xml, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+ZEND_END_ARG_INFO();
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_registernodeclass, 0, 0, 2)
+	ZEND_ARG_INFO(0, baseClass)
+	ZEND_ARG_INFO(0, extendedClass)
+ZEND_END_ARG_INFO();
+/* }}} */
+
 /*
 * class DOMDocument extends DOMNode 
 *
@@ -48,46 +191,47 @@ struct _idsIterator {
 * Since: 
 */
 
-zend_function_entry php_dom_document_class_functions[] = {
-	PHP_FALIAS(createElement, dom_document_create_element, NULL)
-	PHP_FALIAS(createDocumentFragment, dom_document_create_document_fragment, NULL)
-	PHP_FALIAS(createTextNode, dom_document_create_text_node, NULL)
-	PHP_FALIAS(createComment, dom_document_create_comment, NULL)
-	PHP_FALIAS(createCDATASection, dom_document_create_cdatasection, NULL)
-	PHP_FALIAS(createProcessingInstruction, dom_document_create_processing_instruction, NULL)
-	PHP_FALIAS(createAttribute, dom_document_create_attribute, NULL)
-	PHP_FALIAS(createEntityReference, dom_document_create_entity_reference, NULL)
-	PHP_FALIAS(getElementsByTagName, dom_document_get_elements_by_tag_name, NULL)
-	PHP_FALIAS(importNode, dom_document_import_node, NULL)
-	PHP_FALIAS(createElementNS, dom_document_create_element_ns, NULL)
-	PHP_FALIAS(createAttributeNS, dom_document_create_attribute_ns, NULL)
-	PHP_FALIAS(getElementsByTagNameNS, dom_document_get_elements_by_tag_name_ns, NULL)
-	PHP_FALIAS(getElementById, dom_document_get_element_by_id, NULL)
-	PHP_FALIAS(adoptNode, dom_document_adopt_node, NULL)
-	PHP_FALIAS(normalizeDocument, dom_document_normalize_document, NULL)
-	PHP_FALIAS(renameNode, dom_document_rename_node, NULL)
-	PHP_ME(domdocument, load, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_FALIAS(save, dom_document_save, NULL)
-	PHP_ME(domdocument, loadXML, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_FALIAS(saveXML, dom_document_savexml, NULL)
-	PHP_ME(domdocument, __construct, NULL, ZEND_ACC_PUBLIC)
-	PHP_FALIAS(validate, dom_document_validate, NULL)
-	PHP_FALIAS(xinclude, dom_document_xinclude, NULL)
+const zend_function_entry php_dom_document_class_functions[] = { /* {{{ */
+	PHP_FALIAS(createElement, dom_document_create_element, arginfo_dom_document_create_element)
+	PHP_FALIAS(createDocumentFragment, dom_document_create_document_fragment, arginfo_dom_document_create_document_fragment)
+	PHP_FALIAS(createTextNode, dom_document_create_text_node, arginfo_dom_document_create_text_node)
+	PHP_FALIAS(createComment, dom_document_create_comment, arginfo_dom_document_create_comment)
+	PHP_FALIAS(createCDATASection, dom_document_create_cdatasection, arginfo_dom_document_create_cdatasection)
+	PHP_FALIAS(createProcessingInstruction, dom_document_create_processing_instruction, arginfo_dom_document_create_processing_instruction)
+	PHP_FALIAS(createAttribute, dom_document_create_attribute, arginfo_dom_document_create_attribute)
+	PHP_FALIAS(createEntityReference, dom_document_create_entity_reference, arginfo_dom_document_create_entity_reference)
+	PHP_FALIAS(getElementsByTagName, dom_document_get_elements_by_tag_name, arginfo_dom_document_get_elements_by_tag_name)
+	PHP_FALIAS(importNode, dom_document_import_node, arginfo_dom_document_import_node)
+	PHP_FALIAS(createElementNS, dom_document_create_element_ns, arginfo_dom_document_create_element_ns)
+	PHP_FALIAS(createAttributeNS, dom_document_create_attribute_ns, arginfo_dom_document_create_attribute_ns)
+	PHP_FALIAS(getElementsByTagNameNS, dom_document_get_elements_by_tag_name_ns, arginfo_dom_document_get_elements_by_tag_name_ns)
+	PHP_FALIAS(getElementById, dom_document_get_element_by_id, arginfo_dom_document_get_element_by_id)
+	PHP_FALIAS(adoptNode, dom_document_adopt_node, arginfo_dom_document_adopt_node)
+	PHP_FALIAS(normalizeDocument, dom_document_normalize_document, arginfo_dom_document_normalize_document)
+	PHP_FALIAS(renameNode, dom_document_rename_node, arginfo_dom_document_rename_node)
+	PHP_ME(domdocument, load, arginfo_dom_document_load, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_FALIAS(save, dom_document_save, arginfo_dom_document_save)
+	PHP_ME(domdocument, loadXML, arginfo_dom_document_loadxml, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_FALIAS(saveXML, dom_document_savexml, arginfo_dom_document_savexml)
+	PHP_ME(domdocument, __construct, arginfo_dom_document_construct, ZEND_ACC_PUBLIC)
+	PHP_FALIAS(validate, dom_document_validate, arginfo_dom_document_validate)
+	PHP_FALIAS(xinclude, dom_document_xinclude, arginfo_dom_document_xinclude)
 #if defined(LIBXML_HTML_ENABLED)
-	PHP_ME(domdocument, loadHTML, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_ME(domdocument, loadHTMLFile, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_FALIAS(saveHTML, dom_document_save_html, NULL)
-	PHP_FALIAS(saveHTMLFile, dom_document_save_html_file, NULL)
+	PHP_ME(domdocument, loadHTML, arginfo_dom_document_loadhtml, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_ME(domdocument, loadHTMLFile, arginfo_dom_document_loadhtmlfile, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_FALIAS(saveHTML, dom_document_save_html, arginfo_dom_document_savehtml)
+	PHP_FALIAS(saveHTMLFile, dom_document_save_html_file, arginfo_dom_document_savehtmlfile)
 #endif  /* defined(LIBXML_HTML_ENABLED) */
 #if defined(LIBXML_SCHEMAS_ENABLED)
-	PHP_FALIAS(schemaValidate, dom_document_schema_validate_file, NULL)
-	PHP_FALIAS(schemaValidateSource, dom_document_schema_validate_xml, NULL)
-	PHP_FALIAS(relaxNGValidate, dom_document_relaxNG_validate_file, NULL)
-	PHP_FALIAS(relaxNGValidateSource, dom_document_relaxNG_validate_xml, NULL)
+	PHP_FALIAS(schemaValidate, dom_document_schema_validate_file, arginfo_dom_document_schema_validate_file)
+	PHP_FALIAS(schemaValidateSource, dom_document_schema_validate_xml, arginfo_dom_document_schema_validate_xml)
+	PHP_FALIAS(relaxNGValidate, dom_document_relaxNG_validate_file, arginfo_dom_document_relaxNG_validate_file)
+	PHP_FALIAS(relaxNGValidateSource, dom_document_relaxNG_validate_xml, arginfo_dom_document_relaxNG_validate_xml)
 #endif
-	PHP_ME(domdocument, registerNodeClass, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(domdocument, registerNodeClass, arginfo_dom_document_registernodeclass, ZEND_ACC_PUBLIC)
 	{NULL, NULL, NULL}
 };
+/* }}} */
 
 /* {{{ docType	DOMDocumentType	
 readonly=yes 
@@ -107,12 +251,14 @@ int dom_document_doctype_read(dom_object *obj, zval **retval TSRMLS_DC)
 		return FAILURE;
 	}
 
+	ALLOC_ZVAL(*retval);
+
 	dtdptr = xmlGetIntSubset(docp);
 	if (!dtdptr) {
-		return FAILURE;
+		ZVAL_NULL(*retval);
+		return SUCCESS;
 	}
 
-	ALLOC_ZVAL(*retval);
 	if (NULL == (*retval = php_dom_create_object((xmlNodePtr) dtdptr, &ret, NULL, *retval, obj TSRMLS_CC))) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create required DOM object");
 		return FAILURE;
@@ -123,8 +269,6 @@ int dom_document_doctype_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ implementation	DOMImplementation	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1B793EBA
@@ -139,8 +283,6 @@ int dom_document_implementation_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ documentElement	DOMElement	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-87CD092
@@ -159,12 +301,14 @@ int dom_document_document_element_read(dom_object *obj, zval **retval TSRMLS_DC)
 		return FAILURE;
 	}
 
+	ALLOC_ZVAL(*retval);
+
 	root = xmlDocGetRootElement(docp);
 	if (!root) {
-		return FAILURE;
+		ZVAL_NULL(*retval);
+		return SUCCESS;
 	}
 
-	ALLOC_ZVAL(*retval);
 	if (NULL == (*retval = php_dom_create_object(root, &ret, NULL, *retval, obj TSRMLS_CC))) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create required DOM object");
 		return FAILURE;
@@ -216,7 +360,7 @@ int dom_document_encoding_write(dom_object *obj, zval *newval TSRMLS_DC)
 	}
 
 	if (newval->type != IS_STRING) {
-		if(newval->refcount > 1) {
+		if(Z_REFCOUNT_P(newval) > 1) {
 			value_copy = *newval;
 			zval_copy_ctor(&value_copy);
 			newval = &value_copy;
@@ -245,8 +389,6 @@ int dom_document_encoding_write(dom_object *obj, zval *newval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ standalone	boolean	
 readonly=no 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-Document3-standalone
@@ -284,7 +426,7 @@ int dom_document_standalone_write(dom_object *obj, zval *newval TSRMLS_DC)
 		return FAILURE;
 	}
 
-	if(newval->refcount > 1) {
+	if(Z_REFCOUNT_P(newval) > 1) {
 		value_copy = *newval;
 		zval_copy_ctor(&value_copy);
 		newval = &value_copy;
@@ -311,8 +453,6 @@ int dom_document_standalone_write(dom_object *obj, zval *newval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ version	string	
 readonly=no
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-Document3-version
@@ -359,7 +499,7 @@ int dom_document_version_write(dom_object *obj, zval *newval TSRMLS_DC)
 	}
 
 	if (newval->type != IS_STRING) {
-		if(newval->refcount > 1) {
+		if(Z_REFCOUNT_P(newval) > 1) {
 			value_copy = *newval;
 			zval_copy_ctor(&value_copy);
 			newval = &value_copy;
@@ -402,7 +542,7 @@ int dom_document_strict_error_checking_write(dom_object *obj, zval *newval TSRML
 	zval value_copy;
 	dom_doc_propsptr doc_prop;
 
-	if(newval->refcount > 1) {
+	if(Z_REFCOUNT_P(newval) > 1) {
 		value_copy = *newval;
 		zval_copy_ctor(&value_copy);
 		newval = &value_copy;
@@ -445,7 +585,7 @@ int dom_document_format_output_write(dom_object *obj, zval *newval TSRMLS_DC)
 	zval value_copy;
 	dom_doc_propsptr doc_prop;
 
-	if(newval->refcount > 1) {
+	if(Z_REFCOUNT_P(newval) > 1) {
 		value_copy = *newval;
 		zval_copy_ctor(&value_copy);
 		newval = &value_copy;
@@ -487,7 +627,7 @@ int dom_document_validate_on_parse_write(dom_object *obj, zval *newval TSRMLS_DC
 	zval value_copy;
 	dom_doc_propsptr doc_prop;
 
-	if(newval->refcount > 1) {
+	if(Z_REFCOUNT_P(newval) > 1) {
 		value_copy = *newval;
 		zval_copy_ctor(&value_copy);
 		newval = &value_copy;
@@ -507,7 +647,6 @@ int dom_document_validate_on_parse_write(dom_object *obj, zval *newval TSRMLS_DC
 }
 /* }}} */
 
-
 /* {{{ resolveExternals	boolean	
 readonly=no
 */
@@ -530,7 +669,7 @@ int dom_document_resolve_externals_write(dom_object *obj, zval *newval TSRMLS_DC
 	zval value_copy;
 	dom_doc_propsptr doc_prop;
 
-	if(newval->refcount > 1) {
+	if(Z_REFCOUNT_P(newval) > 1) {
 		value_copy = *newval;
 		zval_copy_ctor(&value_copy);
 		newval = &value_copy;
@@ -550,7 +689,6 @@ int dom_document_resolve_externals_write(dom_object *obj, zval *newval TSRMLS_DC
 }
 /* }}} */
 
-
 /* {{{ preserveWhiteSpace	boolean	
 readonly=no
 */
@@ -573,7 +711,7 @@ int dom_document_preserve_whitespace_write(dom_object *obj, zval *newval TSRMLS_
 	zval value_copy;
 	dom_doc_propsptr doc_prop;
 
-	if(newval->refcount > 1) {
+	if(Z_REFCOUNT_P(newval) > 1) {
 		value_copy = *newval;
 		zval_copy_ctor(&value_copy);
 		newval = &value_copy;
@@ -615,7 +753,7 @@ int dom_document_recover_write(dom_object *obj, zval *newval TSRMLS_DC)
 	zval value_copy;
 	dom_doc_propsptr doc_prop;
 
-	if(newval->refcount > 1) {
+	if(Z_REFCOUNT_P(newval) > 1) {
 		value_copy = *newval;
 		zval_copy_ctor(&value_copy);
 		newval = &value_copy;
@@ -635,7 +773,6 @@ int dom_document_recover_write(dom_object *obj, zval *newval TSRMLS_DC)
 }
 /* }}} */
 
-
 /* {{{ substituteEntities	boolean	
 readonly=no
 */
@@ -658,7 +795,7 @@ int dom_document_substitue_entities_write(dom_object *obj, zval *newval TSRMLS_D
 	zval value_copy;
 	dom_doc_propsptr doc_prop;
 
-	if(newval->refcount > 1) {
+	if(Z_REFCOUNT_P(newval) > 1) {
 		value_copy = *newval;
 		zval_copy_ctor(&value_copy);
 		newval = &value_copy;
@@ -678,7 +815,6 @@ int dom_document_substitue_entities_write(dom_object *obj, zval *newval TSRMLS_D
 }
 /* }}} */
 
-
 /* {{{ documentURI	string	
 readonly=no 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-Document3-documentURI
@@ -724,7 +860,7 @@ int dom_document_document_uri_write(dom_object *obj, zval *newval TSRMLS_DC)
 	}
 
 	if (newval->type != IS_STRING) {
-		if(newval->refcount > 1) {
+		if(Z_REFCOUNT_P(newval) > 1) {
 			value_copy = *newval;
 			zval_copy_ctor(&value_copy);
 			newval = &value_copy;
@@ -743,8 +879,6 @@ int dom_document_document_uri_write(dom_object *obj, zval *newval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ config	DOMConfiguration	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-Document3-config
@@ -759,8 +893,6 @@ int dom_document_config_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ proto DOMElement dom_document_create_element(string tagName [, string value]);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-2141741547
 Since: 
@@ -794,7 +926,6 @@ PHP_FUNCTION(dom_document_create_element)
 }
 /* }}} end dom_document_create_element */
 
-
 /* {{{ proto DOMDocumentFragment dom_document_create_document_fragment();
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-35CB04B5
 Since: 
@@ -822,7 +953,6 @@ PHP_FUNCTION(dom_document_create_document_fragment)
 }
 /* }}} end dom_document_create_document_fragment */
 
-
 /* {{{ proto DOMText dom_document_create_text_node(string data);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1975348127
 Since: 
@@ -851,7 +981,6 @@ PHP_FUNCTION(dom_document_create_text_node)
 }
 /* }}} end dom_document_create_text_node */
 
-
 /* {{{ proto DOMComment dom_document_create_comment(string data);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1334481328
 Since: 
@@ -880,7 +1009,6 @@ PHP_FUNCTION(dom_document_create_comment)
 }
 /* }}} end dom_document_create_comment */
 
-
 /* {{{ proto DOMCdataSection dom_document_create_cdatasection(string data);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-D26C0AF8
 Since: 
@@ -909,7 +1037,6 @@ PHP_FUNCTION(dom_document_create_cdatasection)
 }
 /* }}} end dom_document_create_cdatasection */
 
-
 /* {{{ proto DOMProcessingInstruction dom_document_create_processing_instruction(string target, string data);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-135944439
 Since: 
@@ -945,7 +1072,6 @@ PHP_FUNCTION(dom_document_create_processing_instruction)
 }
 /* }}} end dom_document_create_processing_instruction */
 
-
 /* {{{ proto DOMAttr dom_document_create_attribute(string name);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1084891198
 Since: 
@@ -980,7 +1106,6 @@ PHP_FUNCTION(dom_document_create_attribute)
 }
 /* }}} end dom_document_create_attribute */
 
-
 /* {{{ proto DOMEntityReference dom_document_create_entity_reference(string name);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-392B75AE
 Since: 
@@ -1014,7 +1139,6 @@ PHP_FUNCTION(dom_document_create_entity_reference)
 }
 /* }}} end dom_document_create_entity_reference */
 
-
 /* {{{ proto DOMNodeList dom_document_get_elements_by_tag_name(string tagname);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-A6C9094
 Since: 
@@ -1041,7 +1165,6 @@ PHP_FUNCTION(dom_document_get_elements_by_tag_name)
 }
 /* }}} end dom_document_get_elements_by_tag_name */
 
-
 /* {{{ proto DOMNode dom_document_import_node(DOMNode importedNode, boolean deep);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Core-Document-importNode
 Since: DOM Level 2
@@ -1073,6 +1196,9 @@ PHP_FUNCTION(dom_document_import_node)
 	if (nodep->doc == docp) {
 		retnodep = nodep;
 	} else {
+		if ((recursive == 0) && (nodep->type == XML_ELEMENT_NODE)) {
+			recursive = 2;
+		}
 		retnodep = xmlDocCopyNode(nodep, docp, recursive);
 		if (!retnodep) {
 			RETURN_FALSE;
@@ -1084,7 +1210,6 @@ PHP_FUNCTION(dom_document_import_node)
 }
 /* }}} end dom_document_import_node */
 
-
 /* {{{ proto DOMElement dom_document_create_element_ns(string namespaceURI, string qualifiedName [,string value]);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-DocCrElNS
 Since: DOM Level 2
@@ -1148,7 +1273,6 @@ PHP_FUNCTION(dom_document_create_element_ns)
 }
 /* }}} end dom_document_create_element_ns */
 
-
 /* {{{ proto DOMAttr dom_document_create_attribute_ns(string namespaceURI, string qualifiedName);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-DocCrAttrNS
 Since: DOM Level 2
@@ -1214,7 +1338,6 @@ PHP_FUNCTION(dom_document_create_attribute_ns)
 }
 /* }}} end dom_document_create_attribute_ns */
 
-
 /* {{{ proto DOMNodeList dom_document_get_elements_by_tag_name_ns(string namespaceURI, string localName);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-getElBTNNS
 Since: DOM Level 2
@@ -1242,7 +1365,6 @@ PHP_FUNCTION(dom_document_get_elements_by_tag_name_ns)
 }
 /* }}} end dom_document_get_elements_by_tag_name_ns */
 
-
 /* {{{ proto DOMElement dom_document_get_element_by_id(string elementId);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-getElBId
 Since: DOM Level 2
@@ -1273,7 +1395,6 @@ PHP_FUNCTION(dom_document_get_element_by_id)
 }
 /* }}} end dom_document_get_element_by_id */
 
-
 /* {{{ proto DOMNode dom_document_adopt_node(DOMNode source);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-Document3-adoptNode
 Since: DOM Level 3
@@ -1284,7 +1405,6 @@ PHP_FUNCTION(dom_document_adopt_node)
 }
 /* }}} end dom_document_adopt_node */
 
-
 /* {{{ proto void dom_document_normalize_document();
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-Document3-normalizeDocument
 Since: DOM Level 3
@@ -1305,7 +1425,6 @@ PHP_FUNCTION(dom_document_normalize_document)
 }
 /* }}} end dom_document_normalize_document */
 
-
 /* {{{ proto DOMNode dom_document_rename_node(node n, string namespaceURI, string qualifiedName);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-Document3-renameNode
 Since: DOM Level 3
@@ -1325,14 +1444,15 @@ PHP_METHOD(domdocument, __construct)
 	dom_object *intern;
 	char *encoding, *version = NULL;
 	int encoding_len = 0, version_len = 0, refcount;
+	zend_error_handling error_handling;
 
-	php_set_error_handling(EH_THROW, dom_domexception_class_entry TSRMLS_CC);
+	zend_replace_error_handling(EH_THROW, dom_domexception_class_entry, &error_handling TSRMLS_CC);
 	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "O|ss", &id, dom_document_class_entry, &version, &version_len, &encoding, &encoding_len) == FAILURE) {
-		php_std_error_handling();
+		zend_restore_error_handling(&error_handling TSRMLS_CC);
 		return;
 	}
 
-	php_std_error_handling();
+	zend_restore_error_handling(&error_handling TSRMLS_CC);
 	docp = xmlNewDoc(version);
 
 	if (!docp) {
@@ -1363,7 +1483,8 @@ PHP_METHOD(domdocument, __construct)
 }
 /* }}} end DOMDocument::__construct */
 
-char *_dom_get_valid_file_path(char *source, char *resolved_path, int resolved_path_len  TSRMLS_DC) {
+char *_dom_get_valid_file_path(char *source, char *resolved_path, int resolved_path_len  TSRMLS_DC) /* {{{ */
+{
 	xmlURI *uri;
 	xmlChar *escsource;
 	char *file_dest;
@@ -1408,10 +1529,10 @@ char *_dom_get_valid_file_path(char *source, char *resolved_path, int resolved_p
 
 	return file_dest;
 }
+/* }}} */
 
-
-/* {{{ */
-static xmlDocPtr dom_document_parser(zval *id, int mode, char *source, int options TSRMLS_DC) {
+static xmlDocPtr dom_document_parser(zval *id, int mode, char *source, int options TSRMLS_DC) /* {{{ */
+{
     xmlDocPtr ret;
     xmlParserCtxtPtr ctxt = NULL;
 	dom_doc_propsptr doc_props;
@@ -1511,7 +1632,7 @@ static xmlDocPtr dom_document_parser(zval *id, int mode, char *source, int optio
 			EG(error_reporting) = old_error_reporting;
 		}
 		/* If loading from memory, set the base reference uri for the document */
-		if (ret->URL == NULL && ctxt->directory != NULL) {
+		if (ret && ret->URL == NULL && ctxt->directory != NULL) {
 			ret->URL = xmlStrdup(ctxt->directory);
 		}
 	} else {
@@ -1719,7 +1840,8 @@ PHP_FUNCTION(dom_document_savexml)
 }
 /* }}} end dom_document_savexml */
 
-static xmlNodePtr php_dom_free_xinclude_node(xmlNodePtr cur TSRMLS_DC) {
+static xmlNodePtr php_dom_free_xinclude_node(xmlNodePtr cur TSRMLS_DC) /* {{{ */
+{
 	xmlNodePtr xincnode;
 
 	xincnode = cur;
@@ -1729,14 +1851,20 @@ static xmlNodePtr php_dom_free_xinclude_node(xmlNodePtr cur TSRMLS_DC) {
 
 	return cur;
 }
+/* }}} */
 
-static void php_dom_remove_xinclude_nodes(xmlNodePtr cur TSRMLS_DC) {
+static void php_dom_remove_xinclude_nodes(xmlNodePtr cur TSRMLS_DC) /* {{{ */
+{
 	while(cur) {
 		if (cur->type == XML_XINCLUDE_START) {
 			cur = php_dom_free_xinclude_node(cur TSRMLS_CC);
 
 			/* XML_XINCLUDE_END node will be a sibling of XML_XINCLUDE_START */
 			while(cur && cur->type != XML_XINCLUDE_END) {
+				/* remove xinclude processing nodes from recursive xincludes */
+				if (cur->type == XML_ELEMENT_NODE) {
+					   php_dom_remove_xinclude_nodes(cur->children TSRMLS_CC);
+				}
 				cur = cur->next;
 			}
 
@@ -1751,6 +1879,7 @@ static void php_dom_remove_xinclude_nodes(xmlNodePtr cur TSRMLS_DC) {
 		}
 	}
 }
+/* }}} */
 
 /* {{{ proto int dom_document_xinclude([int options])
    Substitutues xincludes in a DomDocument */
@@ -1790,7 +1919,7 @@ PHP_FUNCTION(dom_document_xinclude)
 	}
     
 }
-
+/* }}} */
 
 /* {{{ proto boolean dom_document_validate();
 Since: DOM extended
@@ -1807,11 +1936,7 @@ PHP_FUNCTION(dom_document_validate)
 	}
 
 	DOM_GET_OBJ(docp, id, xmlDocPtr, intern);
-	
-	if (docp->intSubset == NULL) {
-		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "No DTD given in XML-Document");
-	}
-	
+
 	cvp = xmlNewValidCtxt();
 	
 	cvp->userData = NULL;
@@ -1827,11 +1952,10 @@ PHP_FUNCTION(dom_document_validate)
 	xmlFreeValidCtxt(cvp);
 	
 }
- 
+/* }}} */
 
 #if defined(LIBXML_SCHEMAS_ENABLED)
-static void
-_dom_document_schema_validate(INTERNAL_FUNCTION_PARAMETERS, int type)
+static void _dom_document_schema_validate(INTERNAL_FUNCTION_PARAMETERS, int type) /* {{{ */
 {
 	zval *id;
 	xmlDoc *docp;
@@ -1904,6 +2028,7 @@ _dom_document_schema_validate(INTERNAL_FUNCTION_PARAMETERS, int type)
 		RETURN_FALSE;
 	}
 }
+/* }}} */
 
 /* {{{ proto boolean dom_document_schema_validate_file(string filename); */
 PHP_FUNCTION(dom_document_schema_validate_file)
@@ -1919,9 +2044,7 @@ PHP_FUNCTION(dom_document_schema_validate_xml)
 }
 /* }}} end dom_document_schema_validate */
 
-
-static void
-_dom_document_relaxNG_validate(INTERNAL_FUNCTION_PARAMETERS, int type)
+static void _dom_document_relaxNG_validate(INTERNAL_FUNCTION_PARAMETERS, int type) /* {{{ */
 {
 	zval *id;
 	xmlDoc *docp;
@@ -1994,6 +2117,7 @@ _dom_document_relaxNG_validate(INTERNAL_FUNCTION_PARAMETERS, int type)
 		RETURN_FALSE;
 	}
 }
+/* }}} */
 
 /* {{{ proto boolean dom_document_relaxNG_validate_file(string filename); */
 PHP_FUNCTION(dom_document_relaxNG_validate_file)
@@ -2013,7 +2137,7 @@ PHP_FUNCTION(dom_document_relaxNG_validate_xml)
 
 #if defined(LIBXML_HTML_ENABLED)
 
-static void dom_load_html(INTERNAL_FUNCTION_PARAMETERS, int mode)
+static void dom_load_html(INTERNAL_FUNCTION_PARAMETERS, int mode) /* {{{ */
 {
 	zval *id, *rv = NULL;
 	xmlDoc *docp = NULL, *newdoc;
@@ -2086,6 +2210,7 @@ static void dom_load_html(INTERNAL_FUNCTION_PARAMETERS, int mode)
 		DOM_RET_OBJ(rv, (xmlNodePtr) newdoc, &ret, NULL);
 	}
 }
+/* }}} */
 
 /* {{{ proto DOMNode dom_document_load_html_file(string source);
 Since: DOM extended
@@ -2225,3 +2350,12 @@ PHP_METHOD(domdocument, registerNodeClass)
 /* }}} */
 
 #endif  /* HAVE_LIBXML && HAVE_DOM */
+
+/*
+ * Local variables:
+ * tab-width: 4
+ * c-basic-offset: 4
+ * End:
+ * vim600: noet sw=4 ts=4 fdm=marker
+ * vim<600: noet sw=4 ts=4
+ */
