@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: node.c,v 1.37.2.3.2.13 2008/12/31 11:17:37 sebastian Exp $ */
+/* $Id: node.c,v 1.37.2.3.2.8.2.14 2009/03/13 13:43:29 rrichards Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -28,104 +28,87 @@
 #include "php_dom.h"
 
 /* {{{ arginfo */
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_insert_before, 0, 0, 1)
 	ZEND_ARG_OBJ_INFO(0, newChild, DOMNode, 0)
 	ZEND_ARG_OBJ_INFO(0, refChild, DOMNode, 1)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_replace_child, 0, 0, 2)
 	ZEND_ARG_OBJ_INFO(0, newChild, DOMNode, 0)
 	ZEND_ARG_OBJ_INFO(0, oldChild, DOMNode, 0)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_remove_child, 0, 0, 1)
 	ZEND_ARG_OBJ_INFO(0, oldChild, DOMNode, 0)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_append_child, 0, 0, 1)
 	ZEND_ARG_OBJ_INFO(0, newChild, DOMNode, 0)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_has_child_nodes, 0, 0, 0)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_clone_node, 0, 0, 1)
 	ZEND_ARG_INFO(0, deep)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_normalize, 0, 0, 0)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_is_supported, 0, 0, 2)
 	ZEND_ARG_INFO(0, feature)
 	ZEND_ARG_INFO(0, version)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_has_attributes, 0, 0, 0)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_compare_document_position, 0, 0, 1)
 	ZEND_ARG_OBJ_INFO(0, other, DOMNode, 0)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_is_same_node, 0, 0, 1)
 	ZEND_ARG_OBJ_INFO(0, other, DOMNode, 0)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_lookup_prefix, 0, 0, 1)
 	ZEND_ARG_INFO(0, namespaceURI)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_is_default_namespace, 0, 0, 1)
 	ZEND_ARG_INFO(0, namespaceURI)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_lookup_namespace_uri, 0, 0, 1)
 	ZEND_ARG_INFO(0, prefix)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_is_equal_node, 0, 0, 1)
 	ZEND_ARG_OBJ_INFO(0, arg, DOMNode, 0)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_get_feature, 0, 0, 2)
 	ZEND_ARG_INFO(0, feature)
 	ZEND_ARG_INFO(0, version)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_set_user_data, 0, 0, 3)
 	ZEND_ARG_INFO(0, key)
 	ZEND_ARG_INFO(0, data)
 	ZEND_ARG_INFO(0, handler)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_get_user_data, 0, 0, 1)
 	ZEND_ARG_INFO(0, key)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_getNodePath, 0, 0, 0)
 ZEND_END_ARG_INFO();
 
-static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_getLineNo, 0, 0, 0)
+ZEND_END_ARG_INFO();
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_C14N, 0, 0, 0)
 	ZEND_ARG_INFO(0, exclusive)
 	ZEND_ARG_INFO(0, with_comments)
@@ -133,7 +116,6 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_C14N, 0, 0, 0)
 	ZEND_ARG_ARRAY_INFO(0, ns_prefixes, 1)
 ZEND_END_ARG_INFO();
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_node_C14NFile, 0, 0, 1)
 	ZEND_ARG_INFO(0, uri)
 	ZEND_ARG_INFO(0, exclusive)
@@ -150,7 +132,7 @@ ZEND_END_ARG_INFO();
 * Since: 
 */
 
-zend_function_entry php_dom_node_class_functions[] = {
+const zend_function_entry php_dom_node_class_functions[] = { /* {{{ */
 	PHP_FALIAS(insertBefore, dom_node_insert_before, arginfo_dom_node_insert_before)
 	PHP_FALIAS(replaceChild, dom_node_replace_child, arginfo_dom_node_replace_child)
 	PHP_FALIAS(removeChild, dom_node_remove_child, arginfo_dom_node_remove_child)
@@ -170,26 +152,44 @@ zend_function_entry php_dom_node_class_functions[] = {
 	PHP_FALIAS(setUserData, dom_node_set_user_data, arginfo_dom_node_set_user_data)
 	PHP_FALIAS(getUserData, dom_node_get_user_data, arginfo_dom_node_get_user_data)
 	PHP_ME(domnode, getNodePath, arginfo_dom_node_getNodePath, ZEND_ACC_PUBLIC)
+	PHP_ME(domnode, getLineNo, arginfo_dom_node_getLineNo, ZEND_ACC_PUBLIC)
 	PHP_ME(domnode, C14N, arginfo_dom_node_C14N, ZEND_ACC_PUBLIC)
 	PHP_ME(domnode, C14NFile, arginfo_dom_node_C14NFile, ZEND_ACC_PUBLIC)
 	{NULL, NULL, NULL}
 };
+/* }}} */
 
-static void dom_reconcile_ns(xmlDocPtr doc, xmlNodePtr nodep) {
-	xmlNsPtr nsptr;
+static void dom_reconcile_ns(xmlDocPtr doc, xmlNodePtr nodep) /* {{{ */
+{
+	xmlNsPtr nsptr, nsdftptr, curns, prevns = NULL;
 
 	if (nodep->type == XML_ELEMENT_NODE) {
 		/* Following if block primarily used for inserting nodes created via createElementNS */
-		if (nodep->nsDef != NULL && nodep->nsDef->href != NULL) {
-			if((nsptr = xmlSearchNsByHref(doc, nodep->parent, nodep->nsDef->href)) && 
-				(nodep->nsDef->prefix == NULL || xmlStrEqual(nsptr->prefix, nodep->nsDef->prefix))) {
-				dom_set_old_ns(doc, nodep->nsDef);
-				nodep->nsDef = NULL;
+		if (nodep->nsDef != NULL) {
+			curns = nodep->nsDef;
+			while (curns) {
+				nsdftptr = curns->next;
+				if (curns->href != NULL) {
+					if((nsptr = xmlSearchNsByHref(doc, nodep->parent, curns->href)) && 
+						(curns->prefix == NULL || xmlStrEqual(nsptr->prefix, curns->prefix))) {
+						curns->next = NULL;
+						if (prevns == NULL) {
+							nodep->nsDef = nsdftptr;
+						} else {
+							prevns->next = nsdftptr;
+						}
+						dom_set_old_ns(doc, curns);
+						curns = prevns;
+					}
+				}
+				prevns = curns;
+				curns = nsdftptr;
 			}
 		}
 		xmlReconciliateNs(doc, nodep);
 	}
 }
+/* }}} */
 
 /* {{{ nodeName	string	
 readonly=yes 
@@ -260,7 +260,6 @@ int dom_node_node_name_read(dom_object *obj, zval **retval TSRMLS_DC)
 			break;
 		default:
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid Node Type");
-			return FAILURE;
 	}
 
 	ALLOC_ZVAL(*retval);
@@ -281,8 +280,6 @@ int dom_node_node_name_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ nodeValue	string	
 readonly=no 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-F68D080
@@ -356,7 +353,7 @@ int dom_node_node_value_write(dom_object *obj, zval *newval TSRMLS_DC)
 		case XML_CDATA_SECTION_NODE:
 		case XML_PI_NODE:
 			if (newval->type != IS_STRING) {
-				if(newval->refcount > 1) {
+				if(Z_REFCOUNT_P(newval) > 1) {
 					value_copy = *newval;
 					zval_copy_ctor(&value_copy);
 					newval = &value_copy;
@@ -377,8 +374,6 @@ int dom_node_node_value_write(dom_object *obj, zval *newval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ nodeType	int
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-111237558
@@ -409,8 +404,6 @@ int dom_node_node_type_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ parentNode	DomNode	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1060184317
@@ -428,13 +421,14 @@ int dom_node_parent_node_read(dom_object *obj, zval **retval TSRMLS_DC)
 		return FAILURE;
 	}
 
+	ALLOC_ZVAL(*retval);
+
 	nodeparent = nodep->parent;
 	if (!nodeparent) {
-		return FAILURE;
+		ZVAL_NULL(*retval);
+		return SUCCESS;
 	}
 
-	ALLOC_ZVAL(*retval);
-
 	if (NULL == (*retval = php_dom_create_object(nodeparent, &ret, NULL, *retval, obj TSRMLS_CC))) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create required DOM object");
 		return FAILURE;
@@ -444,8 +438,6 @@ int dom_node_parent_node_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ childNodes	DomNodeList	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1451460987
@@ -478,8 +470,6 @@ int dom_node_child_nodes_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ firstChild DomNode	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-169727388
@@ -501,12 +491,13 @@ int dom_node_first_child_read(dom_object *obj, zval **retval TSRMLS_DC)
 		first = nodep->children;
 	}
 
-	if (!first) {
-		return FAILURE;
-	}
-
 	ALLOC_ZVAL(*retval);
 
+	if (!first) {
+		ZVAL_NULL(*retval);
+		return SUCCESS;
+	}
+
 	if (NULL == (*retval = php_dom_create_object(first, &ret, NULL, *retval, obj TSRMLS_CC))) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create required DOM object");
 		return FAILURE;
@@ -516,8 +507,6 @@ int dom_node_first_child_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ lastChild	DomNode	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-61AD09FB
@@ -539,12 +528,13 @@ int dom_node_last_child_read(dom_object *obj, zval **retval TSRMLS_DC)
 		last = nodep->last;
 	}
 
-	if (!last) {
-		return FAILURE;
-	}
-
 	ALLOC_ZVAL(*retval);
 
+	if (!last) {
+		ZVAL_NULL(*retval);
+		return SUCCESS;
+	}
+
 	if (NULL == (*retval = php_dom_create_object(last, &ret, NULL, *retval, obj TSRMLS_CC))) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create required DOM object");
 		return FAILURE;
@@ -554,8 +544,6 @@ int dom_node_last_child_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ previousSibling	DomNode	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-640FB3C8
@@ -573,13 +561,14 @@ int dom_node_previous_sibling_read(dom_object *obj, zval **retval TSRMLS_DC)
 		return FAILURE;
 	}
 
+	ALLOC_ZVAL(*retval);
+
 	prevsib = nodep->prev;
 	if (!prevsib) {
-		return FAILURE;
+		ZVAL_NULL(*retval);
+		return SUCCESS;
 	}
 
-	ALLOC_ZVAL(*retval);
-
 	if (NULL == (*retval = php_dom_create_object(prevsib, &ret, NULL, *retval, obj TSRMLS_CC))) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create required DOM object");
 		return FAILURE;
@@ -589,8 +578,6 @@ int dom_node_previous_sibling_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ nextSibling	DomNode
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-6AC54C2F
@@ -624,8 +611,6 @@ int dom_node_next_sibling_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ attributes	DomNamedNodeMap	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-84CF096
@@ -658,8 +643,6 @@ int dom_node_attributes_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ ownerDocument	DomDocument	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-node-ownerDoc
@@ -700,8 +683,6 @@ int dom_node_owner_document_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ namespaceUri	string	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-NodeNSname
@@ -745,8 +726,6 @@ int dom_node_namespace_uri_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ prefix	string	
 readonly=no 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-NodeNSPrefix
@@ -816,7 +795,7 @@ int dom_node_prefix_write(dom_object *obj, zval *newval TSRMLS_DC)
 				}
 			}
 			if (newval->type != IS_STRING) {
-				if(newval->refcount > 1) {
+				if(Z_REFCOUNT_P(newval) > 1) {
 					value_copy = *newval;
 					zval_copy_ctor(&value_copy);
 					newval = &value_copy;
@@ -869,8 +848,6 @@ int dom_node_prefix_write(dom_object *obj, zval *newval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ localName	string	
 readonly=yes 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-NodeNSLocalN
@@ -900,8 +877,6 @@ int dom_node_local_name_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ baseURI	string
 readonly=yes
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-baseURI
@@ -934,8 +909,6 @@ int dom_node_base_uri_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-
-
 /* {{{ textContent	string	
 readonly=no 
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-textContent
@@ -974,10 +947,7 @@ int dom_node_text_content_write(dom_object *obj, zval *newval TSRMLS_DC)
 
 /* }}} */
 
-
-static xmlNodePtr _php_dom_insert_fragment(xmlNodePtr nodep, xmlNodePtr prevsib,
-					xmlNodePtr nextsib, xmlNodePtr fragment, 
-					dom_object *intern, dom_object *childobj TSRMLS_DC)
+static xmlNodePtr _php_dom_insert_fragment(xmlNodePtr nodep, xmlNodePtr prevsib, xmlNodePtr nextsib, xmlNodePtr fragment, dom_object *intern, dom_object *childobj TSRMLS_DC) /* {{{ */
 {
 	xmlNodePtr newchild, node;
 
@@ -1020,6 +990,7 @@ static xmlNodePtr _php_dom_insert_fragment(xmlNodePtr nodep, xmlNodePtr prevsib,
 
 	return newchild;
 }
+/* }}} */
 
 /* {{{ proto domnode dom_node_insert_before(DomNode newChild, DomNode refChild);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-952280727
@@ -1182,7 +1153,6 @@ PHP_FUNCTION(dom_node_insert_before)
 }
 /* }}} end dom_node_insert_before */
 
-
 /* {{{ proto DomNode dom_node_replace_child(DomNode newChild, DomNode oldChild);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-785887307
 Since: 
@@ -1274,7 +1244,6 @@ PHP_FUNCTION(dom_node_replace_child)
 }
 /* }}} end dom_node_replace_child */
 
-
 /* {{{ proto DomNode dom_node_remove_child(DomNode oldChild);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1734834066
 Since: 
@@ -1327,7 +1296,6 @@ PHP_FUNCTION(dom_node_remove_child)
 }
 /* }}} end dom_node_remove_child */
 
-
 /* {{{ proto DomNode dom_node_append_child(DomNode newChild);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-184E7107
 Since: 
@@ -1429,7 +1397,6 @@ PHP_FUNCTION(dom_node_append_child)
 }
 /* }}} end dom_node_append_child */
 
-
 /* {{{ proto boolean dom_node_has_child_nodes();
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-810594187
 Since: 
@@ -1458,7 +1425,6 @@ PHP_FUNCTION(dom_node_has_child_nodes)
 }
 /* }}} end dom_node_has_child_nodes */
 
-
 /* {{{ proto DomNode dom_node_clone_node(boolean deep);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-3A0ED0A4
 Since: 
@@ -1521,8 +1487,6 @@ PHP_FUNCTION(dom_node_clone_node)
 }
 /* }}} end dom_node_clone_node */
 
-
-
 /* {{{ proto void dom_node_normalize();
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-normalize
 Since: 
@@ -1544,7 +1508,6 @@ PHP_FUNCTION(dom_node_normalize)
 }
 /* }}} end dom_node_normalize */
 
-
 /* {{{ proto boolean dom_node_is_supported(string feature, string version);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-Level-2-Core-Node-supports
 Since: DOM Level 2
@@ -1567,7 +1530,6 @@ PHP_FUNCTION(dom_node_is_supported)
 }
 /* }}} end dom_node_is_supported */
 
-
 /* {{{ proto boolean dom_node_has_attributes();
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-NodeHasAttrs
 Since: DOM Level 2
@@ -1605,7 +1567,6 @@ PHP_FUNCTION(dom_node_compare_document_position)
 }
 /* }}} end dom_node_compare_document_position */
 
-
 /* {{{ proto boolean dom_node_is_same_node(DomNode other);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-isSameNode
 Since: DOM Level 3
@@ -1632,7 +1593,6 @@ PHP_FUNCTION(dom_node_is_same_node)
 }
 /* }}} end dom_node_is_same_node */
 
-
 /* {{{ proto string dom_node_lookup_prefix(string namespaceURI);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-lookupNamespacePrefix
 Since: DOM Level 3
@@ -1683,7 +1643,6 @@ PHP_FUNCTION(dom_node_lookup_prefix)
 }
 /* }}} end dom_node_lookup_prefix */
 
-
 /* {{{ proto boolean dom_node_is_default_namespace(string namespaceURI);
 URL: http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-isDefaultNamespace
 Since: DOM Level 3
@@ -1717,7 +1676,6 @@ PHP_FUNCTION(dom_node_is_default_namespace)
 }
 /* }}} end dom_node_is_default_namespace */
 
-
 /* {{{ proto string dom_node_lookup_namespace_uri(string prefix);
 URL: http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-lookupNamespaceURI
 Since: DOM Level 3
@@ -1752,7 +1710,6 @@ PHP_FUNCTION(dom_node_lookup_namespace_uri)
 }
 /* }}} end dom_node_lookup_namespace_uri */
 
-
 /* {{{ proto boolean dom_node_is_equal_node(DomNode arg);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-isEqualNode
 Since: DOM Level 3
@@ -1763,7 +1720,6 @@ PHP_FUNCTION(dom_node_is_equal_node)
 }
 /* }}} end dom_node_is_equal_node */
 
-
 /* {{{ proto DomNode dom_node_get_feature(string feature, string version);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-getFeature
 Since: DOM Level 3
@@ -1774,7 +1730,6 @@ PHP_FUNCTION(dom_node_get_feature)
 }
 /* }}} end dom_node_get_feature */
 
-
 /* {{{ proto DomUserData dom_node_set_user_data(string key, DomUserData data, userdatahandler handler);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-setUserData
 Since: DOM Level 3
@@ -1785,7 +1740,6 @@ PHP_FUNCTION(dom_node_set_user_data)
 }
 /* }}} end dom_node_set_user_data */
 
-
 /* {{{ proto DomUserData dom_node_get_user_data(string key);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Node3-getUserData
 Since: DOM Level 3
@@ -1796,8 +1750,7 @@ PHP_FUNCTION(dom_node_get_user_data)
 }
 /* }}} end dom_node_get_user_data */
 
-
-static void dom_canonicalization(INTERNAL_FUNCTION_PARAMETERS, int mode)
+static void dom_canonicalization(INTERNAL_FUNCTION_PARAMETERS, int mode) /* {{{ */
 {
 	zval *id;
 	zval *xpath_array=NULL, *ns_prefixes=NULL;
@@ -1965,6 +1918,7 @@ static void dom_canonicalization(INTERNAL_FUNCTION_PARAMETERS, int mode)
 		}
 	}
 }
+/* }}} */
 
 /* {{{ proto string DOMNode::C14N([bool exclusive [, bool with_comments [, array xpath [, array ns_prefixes]]]])
    Canonicalize nodes to a string */
@@ -1972,6 +1926,7 @@ PHP_METHOD(domnode, C14N)
 {
 	dom_canonicalization(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
 }
+/* }}} */
 
 /* {{{ proto int DOMNode::C14NFile(string uri [, bool exclusive [, bool with_comments [, array xpath [, array ns_prefixes]]]])
    Canonicalize nodes to a file */
@@ -1979,12 +1934,10 @@ PHP_METHOD(domnode, C14NFile)
 {
 	dom_canonicalization(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
 }
-
-#endif
+/* }}} */
 
 /* {{{ proto int DOMNode::getNodePath()
    Gets an xpath for a node */
-
 PHP_METHOD(domnode, getNodePath)
 {
 	zval *id;
@@ -1992,8 +1945,6 @@ PHP_METHOD(domnode, getNodePath)
 	dom_object *intern;
 	char *value;
 	
-
-	
 	DOM_GET_THIS_OBJ(nodep, id, xmlNodePtr, intern);
 
 	value = xmlGetNodePath(nodep);
@@ -2003,7 +1954,34 @@ PHP_METHOD(domnode, getNodePath)
 		RETVAL_STRING(value, 1);
 		xmlFree(value);
 	}
-
-	
 }
+/* }}} */
 
+/* {{{ proto int DOMNode::getLineNo()
+   Gets line number for a node */
+PHP_METHOD(domnode, getLineNo)
+{
+	zval *id;
+	xmlNode *nodep;
+	dom_object *intern;
+	
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	DOM_GET_THIS_OBJ(nodep, id, xmlNodePtr, intern);
+
+	RETURN_LONG(xmlGetLineNo(nodep));
+}
+/* }}} */
+
+#endif
+
+/*
+ * Local variables:
+ * tab-width: 4
+ * c-basic-offset: 4
+ * End:
+ * vim600: noet sw=4 ts=4 fdm=marker
+ * vim<600: noet sw=4 ts=4
+ */
