@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -38,229 +38,195 @@
 static int le_gmp;
 
 /* {{{ arginfo */
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_init, 0, 0, 1)
 	ZEND_ARG_INFO(0, number)
 	ZEND_ARG_INFO(0, base)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_intval, 0)
 	ZEND_ARG_INFO(0, gmpnumber)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_strval, 0, 0, 1)
 	ZEND_ARG_INFO(0, gmpnumber)
 	ZEND_ARG_INFO(0, base)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_add, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_sub, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_mul, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_div_qr, 0, 0, 2)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 	ZEND_ARG_INFO(0, round)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_div_r, 0, 0, 2)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 	ZEND_ARG_INFO(0, round)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_div_q, 0, 0, 2)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 	ZEND_ARG_INFO(0, round)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_mod, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_divexact, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_neg, 0)
 	ZEND_ARG_INFO(0, a)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_abs, 0)
 	ZEND_ARG_INFO(0, a)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_fact, 0)
 	ZEND_ARG_INFO(0, a)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_pow, 0)
 	ZEND_ARG_INFO(0, base)
 	ZEND_ARG_INFO(0, exp)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_powm, 0)
 	ZEND_ARG_INFO(0, base)
 	ZEND_ARG_INFO(0, exp)
 	ZEND_ARG_INFO(0, mod)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_sqrt, 0)
 	ZEND_ARG_INFO(0, a)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_sqrtrem, 0)
 	ZEND_ARG_INFO(0, a)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_perfect_square, 0)
 	ZEND_ARG_INFO(0, a)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_prob_prime, 0, 0, 1)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, reps)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_gcd, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_gcdext, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_invert, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_jacobi, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_legendre, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_cmp, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_sign, 0)
 	ZEND_ARG_INFO(0, a)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_random, 0, 0, 0)
 	ZEND_ARG_INFO(0, limiter)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_and, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_or, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_com, 0)
 	ZEND_ARG_INFO(0, a)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_xor, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_setbit, 0, 0, 2)
 	ZEND_ARG_INFO(1, a)
 	ZEND_ARG_INFO(0, index)
 	ZEND_ARG_INFO(0, set_clear)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_clrbit, 0)
 	ZEND_ARG_INFO(1, a)
 	ZEND_ARG_INFO(0, index)
 ZEND_END_ARG_INFO()
 
-static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_testbit, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, index)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_popcount, 0)
 	ZEND_ARG_INFO(0, a)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_hamdist, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, b)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_scan0, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, start)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_scan1, 0)
 	ZEND_ARG_INFO(0, a)
 	ZEND_ARG_INFO(0, start)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gmp_nextprime, 0)
 	ZEND_ARG_INFO(0, a)
 ZEND_END_ARG_INFO()
@@ -272,7 +238,7 @@ static ZEND_GINIT_FUNCTION(gmp);
 
 /* {{{ gmp_functions[]
  */
-zend_function_entry gmp_functions[] = {
+const zend_function_entry gmp_functions[] = {
 	ZEND_FE(gmp_init,	arginfo_gmp_init)
 	ZEND_FE(gmp_intval,	arginfo_gmp_intval)
 	ZEND_FE(gmp_strval,	arginfo_gmp_strval)
@@ -308,8 +274,9 @@ zend_function_entry gmp_functions[] = {
 	ZEND_FE(gmp_xor,	arginfo_gmp_xor)
 	ZEND_FE(gmp_setbit,	arginfo_gmp_setbit)
 	ZEND_FE(gmp_clrbit,	arginfo_gmp_clrbit)
-	ZEND_FE(gmp_scan0, arginfo_gmp_scan0)
-	ZEND_FE(gmp_scan1, arginfo_gmp_scan1)
+	ZEND_FE(gmp_scan0,  arginfo_gmp_scan0)
+	ZEND_FE(gmp_scan1,  arginfo_gmp_scan1)
+	ZEND_FE(gmp_testbit,arginfo_gmp_testbit)
 	ZEND_FE(gmp_popcount, arginfo_gmp_popcount)
 	ZEND_FE(gmp_hamdist, arginfo_gmp_hamdist)
 	ZEND_FE(gmp_nextprime, arginfo_gmp_nextprime)
@@ -423,16 +390,23 @@ ZEND_MODULE_INFO_D(gmp)
 
 /* Fetch zval to be GMP number.
    Initially, zval can be also number or string */
-#define FETCH_GMP_ZVAL(gmpnumber, zval) \
-if (Z_TYPE_PP(zval) == IS_RESOURCE) { \
-	ZEND_FETCH_RESOURCE(gmpnumber, mpz_t *, zval, -1, GMP_RESOURCE_NAME, le_gmp);\
-} else {\
-	if (convert_to_gmp(&gmpnumber, zval, 0 TSRMLS_CC) == FAILURE) {\
-		RETURN_FALSE;\
-	}\
-	ZEND_REGISTER_RESOURCE(NULL, gmpnumber, le_gmp);\
+#define FETCH_GMP_ZVAL(gmpnumber, zval, tmp_resource)								\
+if (Z_TYPE_PP(zval) == IS_RESOURCE) {												\
+	ZEND_FETCH_RESOURCE(gmpnumber, mpz_t *, zval, -1, GMP_RESOURCE_NAME, le_gmp);	\
+	tmp_resource = 0;																\
+} else {																			\
+	if (convert_to_gmp(&gmpnumber, zval, 0 TSRMLS_CC) == FAILURE) {					\
+		RETURN_FALSE;																\
+	}																				\
+	tmp_resource = ZEND_REGISTER_RESOURCE(NULL, gmpnumber, le_gmp);					\
 }
 
+#define FREE_GMP_TEMP(tmp_resource)			\
+	if(tmp_resource) {						\
+		zend_list_delete(tmp_resource);		\
+	}
+
+
 /* create a new initialized GMP number */
 #define INIT_GMP_NUM(gmpnumber) { gmpnumber=emalloc(sizeof(mpz_t)); mpz_init(*gmpnumber); }
 #define FREE_GMP_NUM(gmpnumber) { mpz_clear(*gmpnumber); efree(gmpnumber); }
@@ -524,13 +498,14 @@ static inline void gmp_zval_binary_ui_op_ex(zval *return_value, zval **a_arg, zv
 	mpz_t *gmpnum_a, *gmpnum_b, *gmpnum_result;
 	unsigned long long_result = 0;
 	int use_ui = 0;
+	int arga_tmp = 0, argb_tmp = 0;
 
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, arga_tmp);
 	
 	if (gmp_ui_op && Z_TYPE_PP(b_arg) == IS_LONG && Z_LVAL_PP(b_arg) >= 0) {
 		use_ui = 1;
 	} else {
-		FETCH_GMP_ZVAL(gmpnum_b, b_arg);
+		FETCH_GMP_ZVAL(gmpnum_b, b_arg, argb_tmp);
 	}
 
 	if(check_b_zero) {
@@ -543,6 +518,8 @@ static inline void gmp_zval_binary_ui_op_ex(zval *return_value, zval **a_arg, zv
 
 		if(b_is_zero) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Zero operand not allowed");
+			FREE_GMP_TEMP(arga_tmp);
+			FREE_GMP_TEMP(argb_tmp);
 			RETURN_FALSE;
 		}
 	}
@@ -552,6 +529,9 @@ static inline void gmp_zval_binary_ui_op_ex(zval *return_value, zval **a_arg, zv
 	if (use_ui && gmp_ui_op) {
 		if (allow_ui_return) {
 			long_result = gmp_ui_op(*gmpnum_result, *gmpnum_a, (unsigned long)Z_LVAL_PP(b_arg));
+			if (mpz_sgn(*gmpnum_a) == -1) {
+				long_result = -long_result;
+			}
 		} else {
 			gmp_ui_op(*gmpnum_result, *gmpnum_a, (unsigned long)Z_LVAL_PP(b_arg));
 		}
@@ -559,6 +539,9 @@ static inline void gmp_zval_binary_ui_op_ex(zval *return_value, zval **a_arg, zv
 		gmp_op(*gmpnum_result, *gmpnum_a, *gmpnum_b);
 	}
 
+	FREE_GMP_TEMP(arga_tmp);
+	FREE_GMP_TEMP(argb_tmp);
+
 	if (use_ui && allow_ui_return) {
 		FREE_GMP_NUM(gmpnum_result);
 		RETURN_LONG((long)long_result);
@@ -578,14 +561,15 @@ static inline void gmp_zval_binary_ui_op2_ex(zval *return_value, zval **a_arg, z
 	zval r;
 	int use_ui = 0;
 	unsigned long long_result = 0;
+	int arga_tmp = 0, argb_tmp = 0;
 
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, arga_tmp);
 
 	if (gmp_ui_op && Z_TYPE_PP(b_arg) == IS_LONG && Z_LVAL_PP(b_arg) >= 0) {
 		/* use _ui function */
 		use_ui = 1;
 	} else {
-		FETCH_GMP_ZVAL(gmpnum_b, b_arg);
+		FETCH_GMP_ZVAL(gmpnum_b, b_arg, argb_tmp);
 	}
 
 	if(check_b_zero) {
@@ -598,6 +582,8 @@ static inline void gmp_zval_binary_ui_op2_ex(zval *return_value, zval **a_arg, z
 
 		if(b_is_zero) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Zero operand not allowed");
+			FREE_GMP_TEMP(arga_tmp);
+			FREE_GMP_TEMP(argb_tmp);
 			RETURN_FALSE;
 		}
 	}
@@ -615,6 +601,9 @@ static inline void gmp_zval_binary_ui_op2_ex(zval *return_value, zval **a_arg, z
 		gmp_op(*gmpnum_result1, *gmpnum_result2, *gmpnum_a, *gmpnum_b);
 	}
 
+	FREE_GMP_TEMP(arga_tmp);
+	FREE_GMP_TEMP(argb_tmp);
+
 	array_init(return_value);
 	ZEND_REGISTER_RESOURCE(&r, gmpnum_result1, le_gmp);
 	add_index_resource(return_value, 0, Z_LVAL(r));
@@ -634,8 +623,8 @@ static inline void _gmp_binary_ui_op(INTERNAL_FUNCTION_PARAMETERS, gmp_binary_op
 {
 	zval **a_arg, **b_arg;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &b_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ", &a_arg, &b_arg) == FAILURE){
+		return;
 	}
 	
 	gmp_zval_binary_ui_op(return_value, a_arg, b_arg, gmp_op, gmp_ui_op);
@@ -649,12 +638,14 @@ static inline void _gmp_binary_ui_op(INTERNAL_FUNCTION_PARAMETERS, gmp_binary_op
 static inline void gmp_zval_unary_op(zval *return_value, zval **a_arg, gmp_unary_op_t gmp_op TSRMLS_DC) 
 {
 	mpz_t *gmpnum_a, *gmpnum_result;
+	int temp_a;
 	
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
 
 	INIT_GMP_NUM(gmpnum_result);
 	gmp_op(*gmpnum_result, *gmpnum_a);
 
+	FREE_GMP_TEMP(temp_a);
 	ZEND_REGISTER_RESOURCE(return_value, gmpnum_result, le_gmp);
 }
 /* }}} */
@@ -681,10 +672,10 @@ static inline void _gmp_unary_ui_op(INTERNAL_FUNCTION_PARAMETERS, gmp_unary_ui_o
 {
 	zval **a_arg;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &a_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &a_arg) == FAILURE){
+		return;
 	}
-	
+
 	gmp_zval_unary_ui_op(return_value, a_arg, gmp_op);
 }
 /* }}} */
@@ -695,8 +686,8 @@ static inline void _gmp_unary_op(INTERNAL_FUNCTION_PARAMETERS, gmp_unary_op_t gm
 {
 	zval **a_arg;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &a_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &a_arg) == FAILURE){
+		return;
 	}
 	
 	gmp_zval_unary_op(return_value, a_arg, gmp_op TSRMLS_CC);
@@ -709,14 +700,15 @@ static inline void _gmp_unary_opl(INTERNAL_FUNCTION_PARAMETERS, gmp_unary_opl_t
 {
 	zval **a_arg;
 	mpz_t *gmpnum_a;
+	int temp_a;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &a_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &a_arg) == FAILURE){
+		return;
 	}
 	
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
-
-	RETURN_LONG(gmp_op(*gmpnum_a));
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
+	RETVAL_LONG(gmp_op(*gmpnum_a));
+	FREE_GMP_TEMP(temp_a);
 }
 /* }}} */
 
@@ -726,15 +718,19 @@ static inline void _gmp_binary_opl(INTERNAL_FUNCTION_PARAMETERS, gmp_binary_opl_
 {
 	zval **a_arg, **b_arg;
 	mpz_t *gmpnum_a, *gmpnum_b;
+	int temp_a, temp_b;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &b_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ", &a_arg, &b_arg) == FAILURE){
+		return;
 	}
 
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
-	FETCH_GMP_ZVAL(gmpnum_b, a_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
+	FETCH_GMP_ZVAL(gmpnum_b, b_arg, temp_b);
 
-	RETURN_LONG(gmp_op(*gmpnum_a, *gmpnum_b));
+	RETVAL_LONG(gmp_op(*gmpnum_a, *gmpnum_b));
+
+	FREE_GMP_TEMP(temp_a);
+	FREE_GMP_TEMP(temp_b);
 }
 /* }}} */
 
@@ -742,23 +738,17 @@ static inline void _gmp_binary_opl(INTERNAL_FUNCTION_PARAMETERS, gmp_binary_opl_
    Initializes GMP number */
 ZEND_FUNCTION(gmp_init)
 {
-	zval **number_arg, **base_arg;
+	zval **number_arg;
 	mpz_t * gmpnumber;
-	int argc;
-	int base=0;
+	long base=0;
 
-	argc = ZEND_NUM_ARGS();
-	if (argc < 1 || argc > 2 || zend_get_parameters_ex(argc, &number_arg, &base_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z|l", &number_arg, &base) == FAILURE) {
+		return;
 	}
 
-	if (argc == 2) {
-			convert_to_long_ex(base_arg);
-			base = Z_LVAL_PP(base_arg);
-		if (base < 2 || base > 36) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Bad base for conversion: %d (should be between 2 and 36)", base);
-			RETURN_FALSE;
-		}
+	if (base && (base < 2 || base > 36)) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Bad base for conversion: %ld (should be between 2 and 36)", base);
+		RETURN_FALSE;
 	}
 
 	if (convert_to_gmp(&gmpnumber, number_arg, base TSRMLS_CC) == FAILURE) {
@@ -777,8 +767,8 @@ ZEND_FUNCTION(gmp_intval)
 	zval **gmpnumber_arg;
 	mpz_t * gmpnum;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &gmpnumber_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &gmpnumber_arg) == FAILURE){
+		return;
 	}
 	
 	if (Z_TYPE_PP(gmpnumber_arg) == IS_RESOURCE) {
@@ -795,40 +785,33 @@ ZEND_FUNCTION(gmp_intval)
    Gets string representation of GMP number  */
 ZEND_FUNCTION(gmp_strval)
 {
-	zval **gmpnumber_arg, **base_arg;
-	int base=10, num_len, argc;
+	zval **gmpnumber_arg;
+	int num_len;
+	long base = 10;
 	mpz_t * gmpnum;
 	char *out_string;
+	int temp_a;
 
-	argc = ZEND_NUM_ARGS();
-	if (argc < 1 || argc > 2 || zend_get_parameters_ex(argc, &gmpnumber_arg, &base_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
-	}
-
-	FETCH_GMP_ZVAL(gmpnum, gmpnumber_arg);
-
-	switch (argc) {
-		case 2:
-			convert_to_long_ex(base_arg);
-			base = Z_LVAL_PP(base_arg);
-			break;
-		case 1:
-			base = 10;
-			break;
+	if( zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "Z|l", &gmpnumber_arg, &base ) == FAILURE ) {
+		return;
 	}
 
 	if (base < 2 || base > 36) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Bad base for conversion: %d", base);
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Bad base for conversion: %ld", base);
 		RETURN_FALSE;
 	}
 
+	FETCH_GMP_ZVAL(gmpnum, gmpnumber_arg, temp_a);
+
 	num_len = mpz_sizeinbase(*gmpnum, base);
 	out_string = emalloc(num_len+2);
 	if (mpz_sgn(*gmpnum) < 0) {
 		num_len++;
 	}
 	mpz_get_str(out_string, base, *gmpnum);
-
+	
+	FREE_GMP_TEMP(temp_a);
+	
 	/* 
 	From GMP documentation for mpz_sizeinbase():
 	The returned value will be exact or 1 too big.  If base is a power of
@@ -874,22 +857,11 @@ ZEND_FUNCTION(gmp_mul)
    Divide a by b, returns quotient and reminder */
 ZEND_FUNCTION(gmp_div_qr)
 {
-	zval **a_arg, **b_arg, **round_arg;
-	int round = GMP_ROUND_ZERO, argc;
+	zval **a_arg, **b_arg;
+	long round = GMP_ROUND_ZERO;
 
-	argc = ZEND_NUM_ARGS();
-	if (argc < 2 || argc > 3 || zend_get_parameters_ex(argc, &a_arg, &b_arg, &round_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
-	}
-
-	switch (argc) {
-		case 3:
-			convert_to_long_ex(round_arg);
-			round = Z_LVAL_PP(round_arg);
-			break;
-		case 2:
-			round = GMP_ROUND_ZERO;
-			break;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ|l", &a_arg, &b_arg, &round) == FAILURE) {
+		return;
 	}
 
 	switch (round) {
@@ -911,22 +883,11 @@ ZEND_FUNCTION(gmp_div_qr)
    Divide a by b, returns reminder only */
 ZEND_FUNCTION(gmp_div_r)
 {
-	zval **a_arg, **b_arg, **round_arg;
-	int round = GMP_ROUND_ZERO, argc;
+	zval **a_arg, **b_arg;
+	long round = GMP_ROUND_ZERO;
 
-	argc = ZEND_NUM_ARGS();
-	if (argc < 2 || argc > 3 || zend_get_parameters_ex(argc, &a_arg, &b_arg, &round_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
-	}
-
-	switch (argc) {
-		case 3:
-			convert_to_long_ex(round_arg);
-			round = Z_LVAL_PP(round_arg);
-			break;
-		case 2:
-			round = GMP_ROUND_ZERO;
-			break;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ|l", &a_arg, &b_arg, &round) == FAILURE) {
+		return;
 	}
 
 	switch (round) {
@@ -947,22 +908,11 @@ ZEND_FUNCTION(gmp_div_r)
    Divide a by b, returns quotient only */
 ZEND_FUNCTION(gmp_div_q)
 {
-	zval **a_arg, **b_arg, **round_arg;
-	int round = GMP_ROUND_ZERO, argc;
+	zval **a_arg, **b_arg;
+	long round = GMP_ROUND_ZERO;
 
-	argc = ZEND_NUM_ARGS();
-	if (argc < 2 || argc > 3 || zend_get_parameters_ex(argc, &a_arg, &b_arg, &round_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
-	}
-
-	switch (argc) {
-		case 3:
-			convert_to_long_ex(round_arg);
-			round = Z_LVAL_PP(round_arg);
-			break;
-		case 2:
-			round = GMP_ROUND_ZERO;
-			break;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ|l", &a_arg, &b_arg, &round) == FAILURE) {
+		return;
 	}
 
 	switch (round) {
@@ -986,8 +936,8 @@ ZEND_FUNCTION(gmp_mod)
 {
 	zval **a_arg, **b_arg;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &b_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ", &a_arg, &b_arg) == FAILURE){
+		return;
 	}
 
 	gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_mod, (gmp_binary_ui_op_t)mpz_mod_ui, 1, 1 TSRMLS_CC);
@@ -1000,8 +950,8 @@ ZEND_FUNCTION(gmp_divexact)
 {
 	zval **a_arg, **b_arg;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &b_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ", &a_arg, &b_arg) == FAILURE){
+		return;
 	}
 	
 	gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_divexact, NULL, 0, 1 TSRMLS_CC);
@@ -1030,21 +980,22 @@ ZEND_FUNCTION(gmp_fact)
 {
 	zval **a_arg;
 	mpz_t *gmpnum_tmp;
+	int temp_a;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &a_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &a_arg) == FAILURE){
+		return;
 	}
 
 	if (Z_TYPE_PP(a_arg) == IS_RESOURCE) {
-		FETCH_GMP_ZVAL(gmpnum_tmp, a_arg);
+		FETCH_GMP_ZVAL(gmpnum_tmp, a_arg, temp_a);	/* no need to free this since it's IS_RESOURCE */
 		if (mpz_sgn(*gmpnum_tmp) < 0) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING,"Number has to be greater than or equal to 0");
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Number has to be greater than or equal to 0");
 			RETURN_FALSE;
 		}
 	} else {
 		convert_to_long_ex(a_arg);
 		if (Z_LVAL_PP(a_arg) < 0) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING,"Number has to be greater than or equal to 0");
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Number has to be greater than or equal to 0");
 			RETURN_FALSE;
 		}
 	}
@@ -1057,34 +1008,34 @@ ZEND_FUNCTION(gmp_fact)
    Raise base to power exp */
 ZEND_FUNCTION(gmp_pow)
 {
-	zval **base_arg, **exp_arg;
+	zval **base_arg;
 	mpz_t *gmpnum_result, *gmpnum_base;
 	int use_ui = 0;
+	int temp_base;
+	long exp;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &base_arg, &exp_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Zl", &base_arg, &exp) == FAILURE) {
+		return;
 	}
 
 	if (Z_TYPE_PP(base_arg) == IS_LONG && Z_LVAL_PP(base_arg) >= 0) {
 		use_ui = 1;
 	} else {
-		FETCH_GMP_ZVAL(gmpnum_base, base_arg);
+		FETCH_GMP_ZVAL(gmpnum_base, base_arg, temp_base);
 	}
 
-	convert_to_long_ex(exp_arg);
-
-	if (Z_LVAL_PP(exp_arg) < 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING,"Negative exponent not supported");
+	if (exp < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Negative exponent not supported");
 		RETURN_FALSE;
 	}
 	
 	INIT_GMP_NUM(gmpnum_result);
 	if (use_ui) {
-		mpz_ui_pow_ui(*gmpnum_result, Z_LVAL_PP(base_arg), Z_LVAL_PP(exp_arg));
+		mpz_ui_pow_ui(*gmpnum_result, Z_LVAL_PP(base_arg), exp);
 	} else {
-		mpz_pow_ui(*gmpnum_result, *gmpnum_base, Z_LVAL_PP(exp_arg));
+		mpz_pow_ui(*gmpnum_result, *gmpnum_base, exp);
+		FREE_GMP_TEMP(temp_base);
 	}
-
 	ZEND_REGISTER_RESOURCE(return_value, gmpnum_result, le_gmp);
 }
 /* }}} */
@@ -1096,25 +1047,31 @@ ZEND_FUNCTION(gmp_powm)
 	zval **base_arg, **exp_arg, **mod_arg;
 	mpz_t *gmpnum_base, *gmpnum_exp, *gmpnum_mod, *gmpnum_result;
 	int use_ui = 0;
+	int temp_base, temp_exp, temp_mod;
 
-	if (ZEND_NUM_ARGS() != 3 || zend_get_parameters_ex(3, &base_arg, &exp_arg, &mod_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZZ", &base_arg, &exp_arg, &mod_arg) == FAILURE){
+		return;
 	}
 
-	FETCH_GMP_ZVAL(gmpnum_base, base_arg);
+	FETCH_GMP_ZVAL(gmpnum_base, base_arg, temp_base);
 
 	if (Z_TYPE_PP(exp_arg) == IS_LONG && Z_LVAL_PP(exp_arg) >= 0) {
 		use_ui = 1;
 	} else {
-		FETCH_GMP_ZVAL(gmpnum_exp, exp_arg);
+		FETCH_GMP_ZVAL(gmpnum_exp, exp_arg, temp_exp);
 		if (mpz_sgn(*gmpnum_exp) < 0) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING,"Second parameter cannot be less than 0");
 			RETURN_FALSE;
 		}
 	}
-	FETCH_GMP_ZVAL(gmpnum_mod, mod_arg);
+	FETCH_GMP_ZVAL(gmpnum_mod, mod_arg, temp_mod);
 
 	if (!mpz_cmp_ui(*gmpnum_mod, 0)) {
+		FREE_GMP_TEMP(temp_base);
+		if (use_ui) {
+			FREE_GMP_TEMP(temp_exp);
+		}
+		FREE_GMP_TEMP(temp_mod);
 		RETURN_FALSE;
 	}
 
@@ -1123,8 +1080,12 @@ ZEND_FUNCTION(gmp_powm)
 		mpz_powm_ui(*gmpnum_result, *gmpnum_base, (unsigned long)Z_LVAL_PP(exp_arg), *gmpnum_mod);
 	} else {
 		mpz_powm(*gmpnum_result, *gmpnum_base, *gmpnum_exp, *gmpnum_mod);
+		FREE_GMP_TEMP(temp_exp);
 	}
 
+	FREE_GMP_TEMP(temp_base);
+	FREE_GMP_TEMP(temp_mod);
+
 	ZEND_REGISTER_RESOURCE(return_value, gmpnum_result, le_gmp);
 
 }
@@ -1136,20 +1097,23 @@ ZEND_FUNCTION(gmp_sqrt)
 {
 	zval **a_arg;
 	mpz_t *gmpnum_a, *gmpnum_result;
+	int temp_a;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &a_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &a_arg) == FAILURE){
+		return;
 	}
 	
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
 
 	if (mpz_sgn(*gmpnum_a) < 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING,"Number has to be greater than or equal to 0");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Number has to be greater than or equal to 0");
+		FREE_GMP_TEMP(temp_a);
 		RETURN_FALSE;
 	}	
 	
 	INIT_GMP_NUM(gmpnum_result);
 	mpz_sqrt(*gmpnum_result, *gmpnum_a);
+	FREE_GMP_TEMP(temp_a);
 
 	ZEND_REGISTER_RESOURCE(return_value, gmpnum_result, le_gmp);
 }
@@ -1162,12 +1126,13 @@ ZEND_FUNCTION(gmp_sqrtrem)
 	zval **a_arg;
 	mpz_t *gmpnum_a, *gmpnum_result1, *gmpnum_result2;
 	zval r;
+	int temp_a;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &a_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &a_arg) == FAILURE){
+		return;
 	}
 
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
 	
 	if (mpz_sgn(*gmpnum_a) < 0) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING,"Number has to be greater than or equal to 0");
@@ -1178,6 +1143,7 @@ ZEND_FUNCTION(gmp_sqrtrem)
 	INIT_GMP_NUM(gmpnum_result2);
 
 	mpz_sqrtrem(*gmpnum_result1, *gmpnum_result2, *gmpnum_a);
+	FREE_GMP_TEMP(temp_a);
 
 	array_init(return_value);
 	ZEND_REGISTER_RESOURCE(&r, gmpnum_result1, le_gmp);
@@ -1193,14 +1159,16 @@ ZEND_FUNCTION(gmp_perfect_square)
 {
 	zval **a_arg;
 	mpz_t *gmpnum_a;
+	int temp_a;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &a_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &a_arg) == FAILURE){
+		return;
 	}
 
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
 
-	RETURN_BOOL((mpz_perfect_square_p(*gmpnum_a)!=0));
+	RETVAL_BOOL((mpz_perfect_square_p(*gmpnum_a)!=0));
+	FREE_GMP_TEMP(temp_a);
 }
 /* }}} */
 
@@ -1208,28 +1176,19 @@ ZEND_FUNCTION(gmp_perfect_square)
    Checks if a is "probably prime" */
 ZEND_FUNCTION(gmp_prob_prime)
 {
-	zval **gmpnumber_arg, **reps_arg;
+	zval **gmpnumber_arg;
 	mpz_t *gmpnum_a;
-	int argc, reps = 10;
+	long reps = 10;
+	int temp_a;
 
-	argc = ZEND_NUM_ARGS();
-	if (argc < 1 || argc > 2 || zend_get_parameters_ex(argc, &gmpnumber_arg, &reps_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z|l", &gmpnumber_arg, &reps) == FAILURE) {
+		return;
 	}
 
-	FETCH_GMP_ZVAL(gmpnum_a, gmpnumber_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, gmpnumber_arg, temp_a);
 
-	switch (argc) {
-		case 2:
-			convert_to_long_ex(reps_arg);
-			reps = Z_LVAL_PP(reps_arg);
-			break;
-		case 1:
-			reps = 10;
-			break;
-	}
-
-	RETURN_LONG(mpz_probab_prime_p(*gmpnum_a, reps));
+	RETVAL_LONG(mpz_probab_prime_p(*gmpnum_a, reps));
+	FREE_GMP_TEMP(temp_a);
 }
 /* }}} */
 
@@ -1239,11 +1198,11 @@ ZEND_FUNCTION(gmp_gcd)
 {
 	zval **a_arg, **b_arg;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &b_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ", &a_arg, &b_arg) == FAILURE){
+		return;
 	}
 
-	gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_gcd, (gmp_binary_ui_op_t)mpz_gcd_ui, 1, 0 TSRMLS_CC);
+	gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_gcd, (gmp_binary_ui_op_t)mpz_gcd_ui, 0, 0 TSRMLS_CC);
 }
 /* }}} */
 
@@ -1254,19 +1213,22 @@ ZEND_FUNCTION(gmp_gcdext)
 	zval **a_arg, **b_arg;
 	mpz_t *gmpnum_a, *gmpnum_b, *gmpnum_t, *gmpnum_s, *gmpnum_g;
 	zval r;
+	int temp_a, temp_b;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &b_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ", &a_arg, &b_arg) == FAILURE){
+		return;
 	}
 
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
-	FETCH_GMP_ZVAL(gmpnum_b, b_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
+	FETCH_GMP_ZVAL(gmpnum_b, b_arg, temp_b);
 
 	INIT_GMP_NUM(gmpnum_g);
 	INIT_GMP_NUM(gmpnum_s);
 	INIT_GMP_NUM(gmpnum_t);
 
 	mpz_gcdext(*gmpnum_g, *gmpnum_s, *gmpnum_t, *gmpnum_a, *gmpnum_b);
+	FREE_GMP_TEMP(temp_a);
+	FREE_GMP_TEMP(temp_b);
 
 	array_init(return_value);
 
@@ -1285,16 +1247,21 @@ ZEND_FUNCTION(gmp_invert)
 {
 	zval **a_arg, **b_arg;
 	mpz_t *gmpnum_a, *gmpnum_b, *gmpnum_result;
+	int temp_a, temp_b;
+	int res;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &b_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ", &a_arg, &b_arg) == FAILURE){
+		return;
 	}
 
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
-	FETCH_GMP_ZVAL(gmpnum_b, b_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
+	FETCH_GMP_ZVAL(gmpnum_b, b_arg, temp_b);
 
 	INIT_GMP_NUM(gmpnum_result);
-	if (mpz_invert(*gmpnum_result, *gmpnum_a, *gmpnum_b)) {
+	res=mpz_invert(*gmpnum_result, *gmpnum_a, *gmpnum_b);
+	FREE_GMP_TEMP(temp_a);
+	FREE_GMP_TEMP(temp_b);
+	if (res) {
 		ZEND_REGISTER_RESOURCE(return_value, gmpnum_result, le_gmp);
 	} else {
 		FREE_GMP_NUM(gmpnum_result);
@@ -1326,17 +1293,18 @@ ZEND_FUNCTION(gmp_cmp)
 	zval **a_arg, **b_arg;
 	mpz_t *gmpnum_a, *gmpnum_b;
 	int use_si = 0, res;
+	int temp_a, temp_b;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &b_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ", &a_arg, &b_arg) == FAILURE){
+		return;
 	}
 
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
 
 	if (Z_TYPE_PP(b_arg) == IS_LONG) {
 		use_si = 1;
 	} else {
-		FETCH_GMP_ZVAL(gmpnum_b, b_arg);
+		FETCH_GMP_ZVAL(gmpnum_b, b_arg, temp_b);
 	}
 
 	if (use_si) {
@@ -1344,6 +1312,7 @@ ZEND_FUNCTION(gmp_cmp)
 	} else {
 		res = mpz_cmp(*gmpnum_a, *gmpnum_b);
 	}
+	FREE_GMP_TEMP(temp_a);
 	
 	RETURN_LONG(res);
 }
@@ -1355,14 +1324,16 @@ ZEND_FUNCTION(gmp_sign)
 {
 	zval **a_arg;
 	mpz_t *gmpnum_a;
+	int temp_a;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &a_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &a_arg) == FAILURE){
+		return;
 	}
 	
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
 
-	RETURN_LONG(mpz_sgn(*gmpnum_a));
+	RETVAL_LONG(mpz_sgn(*gmpnum_a));
+	FREE_GMP_TEMP(temp_a);
 }
 /* }}} */
 
@@ -1370,19 +1341,11 @@ ZEND_FUNCTION(gmp_sign)
    Gets random number */
 ZEND_FUNCTION(gmp_random)
 {
-	zval **limiter_arg;
-	int limiter, argc;
+	long limiter = 20;
 	mpz_t *gmpnum_result;
 
-	argc = ZEND_NUM_ARGS();
-
-	if (argc == 0) {
-		limiter = 20;
-	} else if (argc == 1 && zend_get_parameters_ex(1, &limiter_arg) == SUCCESS) {
-		convert_to_long_ex(limiter_arg);
-		limiter = Z_LVAL_PP(limiter_arg);
-	} else {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &limiter) == FAILURE) {
+		return;
 	}
 
 	INIT_GMP_NUM(gmpnum_result);
@@ -1396,7 +1359,7 @@ ZEND_FUNCTION(gmp_random)
 
 		GMPG(rand_initialized) = 1;
 	}
-	mpz_urandomb(*gmpnum_result, GMPG(rand_state), GMP_ABS (limiter) * GMP_LIMB_BITS);
+	mpz_urandomb(*gmpnum_result, GMPG(rand_state), GMP_ABS (limiter) * __GMP_BITS_PER_MP_LIMB);
 
 	ZEND_REGISTER_RESOURCE(return_value, gmpnum_result, le_gmp);
 }
@@ -1441,13 +1404,14 @@ ZEND_FUNCTION(gmp_xor)
 	/* use formula: a^b = (a|b)&^(a&b) */
 	zval **a_arg, **b_arg;
 	mpz_t *gmpnum_a, *gmpnum_b, *gmpnum_result, *gmpnum_t;
+	int temp_a, temp_b;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &b_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ", &a_arg, &b_arg) == FAILURE){
+		return;
 	}
 
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
-	FETCH_GMP_ZVAL(gmpnum_b, b_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
+	FETCH_GMP_ZVAL(gmpnum_b, b_arg, temp_b);
 
 	INIT_GMP_NUM(gmpnum_result);
 	INIT_GMP_NUM(gmpnum_t);
@@ -1460,6 +1424,8 @@ ZEND_FUNCTION(gmp_xor)
 
 	FREE_GMP_NUM(gmpnum_t);
 
+	FREE_GMP_TEMP(temp_a);
+	FREE_GMP_TEMP(temp_b);
 	ZEND_REGISTER_RESOURCE(return_value, gmpnum_result, le_gmp);
 }
 /* }}} */
@@ -1468,30 +1434,17 @@ ZEND_FUNCTION(gmp_xor)
    Sets or clear bit in a */
 ZEND_FUNCTION(gmp_setbit)
 {
-	zval **a_arg, **ind_arg, **set_c_arg;
-	int argc, index, set = 1;
+	zval **a_arg;
+	long index;
+	zend_bool set = 1;
 	mpz_t *gmpnum_a;
 
-	argc = ZEND_NUM_ARGS();
-	if (argc < 2 || argc > 3 || zend_get_parameters_ex(argc, &a_arg, &ind_arg, &set_c_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Zl|b", &a_arg, &index, &set) == FAILURE) {
+		return;
 	}
 
 	ZEND_FETCH_RESOURCE(gmpnum_a, mpz_t *, a_arg, -1, GMP_RESOURCE_NAME, le_gmp);
 
-	convert_to_long_ex(ind_arg);
-	index = Z_LVAL_PP(ind_arg);
-
-	switch (argc) {
-		case 3:
-			convert_to_long_ex(set_c_arg);
-			set = Z_LVAL_PP(set_c_arg);
-			break;
-		case 2:
-			set = 1;
-			break;
-	}
-
 	if (index < 0) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Index must be greater than or equal to zero");
 		return;
@@ -1509,19 +1462,16 @@ ZEND_FUNCTION(gmp_setbit)
    Clears bit in a */
 ZEND_FUNCTION(gmp_clrbit)
 {
-	zval **a_arg, **ind_arg;
-	int index;
+	zval **a_arg;
+	long index;
 	mpz_t *gmpnum_a;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &ind_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Zl", &a_arg, &index) == FAILURE){
+		return;
 	}
 
 	ZEND_FETCH_RESOURCE(gmpnum_a, mpz_t *, a_arg, -1, GMP_RESOURCE_NAME, le_gmp);
 
-	convert_to_long_ex(ind_arg);
-	index = Z_LVAL_PP(ind_arg);
-	
 	if (index < 0) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Index must be greater than or equal to zero");
 		return;
@@ -1531,20 +1481,48 @@ ZEND_FUNCTION(gmp_clrbit)
 }
 /* }}} */
 
+/* {{{ proto bool gmp_testbit(resource a, int index)
+   Tests if bit is set in a */
+ZEND_FUNCTION(gmp_testbit)
+{
+	zval **a_arg;
+	long index;
+	mpz_t *gmpnum_a;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Zl", &a_arg, &index) == FAILURE){
+		return;
+	}
+
+	ZEND_FETCH_RESOURCE(gmpnum_a, mpz_t *, a_arg, -1, GMP_RESOURCE_NAME, le_gmp);
+
+	if (index < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Index must be greater than or equal to zero");
+		RETURN_FALSE;
+	}
+
+	if (mpz_tstbit(*gmpnum_a, index)) {
+		RETURN_TRUE;
+	}
+	RETURN_FALSE;
+}
+/* }}} */
+
 /* {{{ proto int gmp_popcount(resource a)
    Calculates the population count of a */
 ZEND_FUNCTION(gmp_popcount)
 {
 	zval **a_arg;
 	mpz_t *gmpnum_a;
+	int temp_a;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &a_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Z", &a_arg) == FAILURE){
+		return;
 	}
 	
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
 
-	RETURN_LONG(mpz_popcount(*gmpnum_a));
+	RETVAL_LONG(mpz_popcount(*gmpnum_a));
+	FREE_GMP_TEMP(temp_a);
 }
 /* }}} */
 
@@ -1554,15 +1532,18 @@ ZEND_FUNCTION(gmp_hamdist)
 {
 	zval **a_arg, **b_arg;
 	mpz_t *gmpnum_a, *gmpnum_b;
+	int temp_a, temp_b;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &b_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ZZ", &a_arg, &b_arg) == FAILURE){
+		return;
 	}
-	
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
-	FETCH_GMP_ZVAL(gmpnum_b, b_arg);
 
-	RETURN_LONG(mpz_hamdist(*gmpnum_a, *gmpnum_b));
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
+	FETCH_GMP_ZVAL(gmpnum_b, b_arg, temp_b);
+
+	RETVAL_LONG(mpz_hamdist(*gmpnum_a, *gmpnum_b));
+	FREE_GMP_TEMP(temp_a);
+	FREE_GMP_TEMP(temp_b);
 }
 /* }}} */
 
@@ -1570,22 +1551,24 @@ ZEND_FUNCTION(gmp_hamdist)
    Finds first zero bit */
 ZEND_FUNCTION(gmp_scan0)
 {
-	zval **a_arg, **start_arg;
+	zval **a_arg;
 	mpz_t *gmpnum_a;
+	int temp_a;
+	long start;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &start_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Zl", &a_arg, &start) == FAILURE){
+		return;
 	}
 
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
-	convert_to_long_ex(start_arg);
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
 
-	if (Z_LVAL_PP(start_arg) < 0) {
+	if (start < 0) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Starting index must be greater than or equal to zero");
 		RETURN_FALSE;
 	}
 
-	RETURN_LONG(mpz_scan0(*gmpnum_a, Z_LVAL_PP(start_arg)));
+	RETVAL_LONG(mpz_scan0(*gmpnum_a, start));
+	FREE_GMP_TEMP(temp_a);
 }
 /* }}} */
 
@@ -1593,22 +1576,23 @@ ZEND_FUNCTION(gmp_scan0)
    Finds first non-zero bit */
 ZEND_FUNCTION(gmp_scan1)
 {
-	zval **a_arg, **start_arg;
+	zval **a_arg;
 	mpz_t *gmpnum_a;
+	int temp_a;
+	long start;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &start_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Zl", &a_arg, &start) == FAILURE){
+		return;
 	}
 
-	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
-	convert_to_long_ex(start_arg);
-	
-	if (Z_LVAL_PP(start_arg) < 0) {
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg, temp_a);
+	if (start < 0) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Starting index must be greater than or equal to zero");
 		RETURN_FALSE;
 	}
 
-	RETURN_LONG(mpz_scan1(*gmpnum_a, Z_LVAL_PP(start_arg)));
+	RETVAL_LONG(mpz_scan1(*gmpnum_a, start));
+	FREE_GMP_TEMP(temp_a);
 }
 /* }}} */
 
