@@ -17,7 +17,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: hash_crc32.c 272374 2008-12-31 11:17:49Z sebastian $ */
+/* $Id: hash_crc32.c,v 1.2.2.3.2.2.2.4 2008/12/31 11:15:37 sebastian Exp $ */
 
 #include "php_hash.h"
 #include "php_hash_crc32.h"
@@ -66,10 +66,17 @@ PHP_HASH_API void PHP_CRC32BFinal(unsigned char digest[4], PHP_CRC32_CTX *contex
 	context->state = 0;
 }
 
+PHP_HASH_API int PHP_CRC32Copy(const php_hash_ops *ops, PHP_CRC32_CTX *orig_context, PHP_CRC32_CTX *copy_context)
+{
+	copy_context->state = orig_context->state;
+	return SUCCESS;
+}
+
 const php_hash_ops php_hash_crc32_ops = {
 	(php_hash_init_func_t) PHP_CRC32Init,
 	(php_hash_update_func_t) PHP_CRC32Update,
 	(php_hash_final_func_t) PHP_CRC32Final,
+	(php_hash_copy_func_t) PHP_CRC32Copy,
 	4, /* what to say here? */
 	4,
 	sizeof(PHP_CRC32_CTX)
@@ -79,6 +86,7 @@ const php_hash_ops php_hash_crc32b_ops = {
 	(php_hash_init_func_t) PHP_CRC32Init,
 	(php_hash_update_func_t) PHP_CRC32BUpdate,
 	(php_hash_final_func_t) PHP_CRC32BFinal,
+	(php_hash_copy_func_t) PHP_CRC32Copy,
 	4, /* what to say here? */
 	4,
 	sizeof(PHP_CRC32_CTX)
