@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: interbase.c,v 1.223 2004/07/06 13:46:18 abies Exp $ */
+/* $Id: interbase.c,v 1.225.2.4.2.7.2.11 2008/12/31 11:15:38 sebastian Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -43,64 +43,340 @@
 #define COMMIT			1
 #define RETAIN			2
 
-/* {{{ extension definition structures */
-function_entry ibase_functions[] = {
-	PHP_FE(ibase_connect, NULL)
-	PHP_FE(ibase_pconnect, NULL)
-	PHP_FE(ibase_close, NULL)
-	PHP_FE(ibase_drop_db, NULL)
-	PHP_FE(ibase_query, NULL)
-	PHP_FE(ibase_fetch_row, NULL)
-	PHP_FE(ibase_fetch_assoc, NULL)
-	PHP_FE(ibase_fetch_object, NULL)
-	PHP_FE(ibase_free_result, NULL)
-	PHP_FE(ibase_name_result, NULL)
-	PHP_FE(ibase_prepare, NULL)
-	PHP_FE(ibase_execute, NULL)
-	PHP_FE(ibase_free_query, NULL)
-	PHP_FE(ibase_gen_id, NULL)
-	PHP_FE(ibase_num_fields, NULL)
-	PHP_FE(ibase_num_params, NULL)
+#define CHECK_LINK(link) { if (link==-1) { php_error_docref(NULL TSRMLS_CC, E_WARNING, "A link to the server could not be established"); RETURN_FALSE; } }
+
+ZEND_DECLARE_MODULE_GLOBALS(ibase)
+static PHP_GINIT_FUNCTION(ibase);
+
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO(arginfo_ibase_errmsg, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_ibase_errcode, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_connect, 0, 0, 1)
+	ZEND_ARG_INFO(0, database)
+	ZEND_ARG_INFO(0, username)
+	ZEND_ARG_INFO(0, password)
+	ZEND_ARG_INFO(0, charset)
+	ZEND_ARG_INFO(0, buffers)
+	ZEND_ARG_INFO(0, dialect)
+	ZEND_ARG_INFO(0, role)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_pconnect, 0, 0, 1)
+	ZEND_ARG_INFO(0, database)
+	ZEND_ARG_INFO(0, username)
+	ZEND_ARG_INFO(0, password)
+	ZEND_ARG_INFO(0, charset)
+	ZEND_ARG_INFO(0, buffers)
+	ZEND_ARG_INFO(0, dialect)
+	ZEND_ARG_INFO(0, role)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_close, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_drop_db, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_trans, 0, 0, 0)
+	ZEND_ARG_INFO(0, trans_args)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, trans_args)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_commit, 0, 0, 1)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_rollback, 0, 0, 1)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_commit_ret, 0, 0, 1)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_rollback_ret, 0, 0, 1)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_gen_id, 0, 0, 1)
+	ZEND_ARG_INFO(0, generator)
+	ZEND_ARG_INFO(0, increment)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_create, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_open, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, blob_id)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_add, 0, 0, 2)
+	ZEND_ARG_INFO(0, blob_handle)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_get, 0, 0, 2)
+	ZEND_ARG_INFO(0, blob_handle)
+	ZEND_ARG_INFO(0, len)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_close, 0, 0, 1)
+	ZEND_ARG_INFO(0, blob_handle)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_cancel, 0, 0, 1)
+	ZEND_ARG_INFO(0, blob_handle)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_info, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, blob_id)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_echo, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, blob_id)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_blob_import, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, file)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_query, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, query)
+	ZEND_ARG_INFO(0, bind_arg)
+	ZEND_ARG_INFO(0, bind_arg)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_affected_rows, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+ZEND_END_ARG_INFO()
+
 #if abies_0
-	PHP_FE(ibase_num_rows, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_num_rows, 0, 0, 1)
+	ZEND_ARG_INFO(0, result_identifier)
+ZEND_END_ARG_INFO()
 #endif
-	PHP_FE(ibase_affected_rows, NULL)
-	PHP_FE(ibase_field_info, NULL)
-	PHP_FE(ibase_param_info, NULL)
 
-	PHP_FE(ibase_trans, NULL)
-	PHP_FE(ibase_commit, NULL)
-	PHP_FE(ibase_rollback, NULL)
-	PHP_FE(ibase_commit_ret, NULL)
-	PHP_FE(ibase_rollback_ret, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_fetch_row, 0, 0, 1)
+	ZEND_ARG_INFO(0, result)
+	ZEND_ARG_INFO(0, fetch_flags)
+ZEND_END_ARG_INFO()
 
-	PHP_FE(ibase_blob_info, NULL)
-	PHP_FE(ibase_blob_create, NULL)
-	PHP_FE(ibase_blob_add, NULL)
-	PHP_FE(ibase_blob_cancel, NULL)
-	PHP_FE(ibase_blob_close, NULL)
-	PHP_FE(ibase_blob_open, NULL)
-	PHP_FE(ibase_blob_get, NULL)
-	PHP_FE(ibase_blob_echo, NULL)
-	PHP_FE(ibase_blob_import, NULL)
-	PHP_FE(ibase_errmsg, NULL)
-	PHP_FE(ibase_errcode, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_fetch_assoc, 0, 0, 1)
+	ZEND_ARG_INFO(0, result)
+	ZEND_ARG_INFO(0, fetch_flags)
+ZEND_END_ARG_INFO()
 
-	PHP_FE(ibase_add_user, NULL)
-	PHP_FE(ibase_modify_user, NULL)
-	PHP_FE(ibase_delete_user, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_fetch_object, 0, 0, 1)
+	ZEND_ARG_INFO(0, result)
+	ZEND_ARG_INFO(0, fetch_flags)
+ZEND_END_ARG_INFO()
 
-	PHP_FE(ibase_service_attach, NULL)
-	PHP_FE(ibase_service_detach, NULL)
-	PHP_FE(ibase_backup, NULL)
-	PHP_FE(ibase_restore, NULL)
-	PHP_FE(ibase_maintain_db, NULL)
-	PHP_FE(ibase_db_info, NULL)
-	PHP_FE(ibase_server_info, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_name_result, 0, 0, 2)
+	ZEND_ARG_INFO(0, result)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO()
 
-	PHP_FE(ibase_wait_event, NULL)
-	PHP_FE(ibase_set_event_handler, NULL)
-	PHP_FE(ibase_free_event_handler, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_free_result, 0, 0, 1)
+	ZEND_ARG_INFO(0, result)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_prepare, 0, 0, 0)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, query)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_execute, 0, 0, 1)
+	ZEND_ARG_INFO(0, query)
+	ZEND_ARG_INFO(0, bind_arg)
+	ZEND_ARG_INFO(0, bind_arg)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_free_query, 0, 0, 1)
+	ZEND_ARG_INFO(0, query)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_num_fields, 0, 0, 1)
+	ZEND_ARG_INFO(0, query_result)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_field_info, 0, 0, 2)
+	ZEND_ARG_INFO(0, query_result)
+	ZEND_ARG_INFO(0, field_number)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_num_params, 0, 0, 1)
+	ZEND_ARG_INFO(0, query)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_param_info, 0, 0, 2)
+	ZEND_ARG_INFO(0, query)
+	ZEND_ARG_INFO(0, field_number)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_add_user, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, user_name)
+	ZEND_ARG_INFO(0, password)
+	ZEND_ARG_INFO(0, first_name)
+	ZEND_ARG_INFO(0, middle_name)
+	ZEND_ARG_INFO(0, last_name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_modify_user, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, user_name)
+	ZEND_ARG_INFO(0, password)
+	ZEND_ARG_INFO(0, first_name)
+	ZEND_ARG_INFO(0, middle_name)
+	ZEND_ARG_INFO(0, last_name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_delete_user, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, user_name)
+	ZEND_ARG_INFO(0, password)
+	ZEND_ARG_INFO(0, first_name)
+	ZEND_ARG_INFO(0, middle_name)
+	ZEND_ARG_INFO(0, last_name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_service_attach, 0, 0, 3)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, dba_username)
+	ZEND_ARG_INFO(0, dba_password)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_service_detach, 0, 0, 1)
+	ZEND_ARG_INFO(0, service_handle)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_backup, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, source_db)
+	ZEND_ARG_INFO(0, dest_file)
+	ZEND_ARG_INFO(0, options)
+	ZEND_ARG_INFO(0, verbose)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_restore, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, source_file)
+	ZEND_ARG_INFO(0, dest_db)
+	ZEND_ARG_INFO(0, options)
+	ZEND_ARG_INFO(0, verbose)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_maintain_db, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, db)
+	ZEND_ARG_INFO(0, action)
+	ZEND_ARG_INFO(0, argument)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_db_info, 0, 0, 3)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, db)
+	ZEND_ARG_INFO(0, action)
+	ZEND_ARG_INFO(0, argument)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_server_info, 0, 0, 2)
+	ZEND_ARG_INFO(0, service_handle)
+	ZEND_ARG_INFO(0, action)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_wait_event, 0, 0, 1)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, event)
+	ZEND_ARG_INFO(0, event2)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_set_event_handler, 0, 0, 2)
+	ZEND_ARG_INFO(0, link_identifier)
+	ZEND_ARG_INFO(0, handler)
+	ZEND_ARG_INFO(0, event)
+	ZEND_ARG_INFO(0, event2)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ibase_free_event_handler, 0, 0, 1)
+	ZEND_ARG_INFO(0, event)
+ZEND_END_ARG_INFO()
+/* }}} */
+
+/* {{{ extension definition structures */
+const zend_function_entry ibase_functions[] = {
+	PHP_FE(ibase_connect, 		arginfo_ibase_connect)
+	PHP_FE(ibase_pconnect, 		arginfo_ibase_pconnect)
+	PHP_FE(ibase_close, 		arginfo_ibase_close)
+	PHP_FE(ibase_drop_db, 		arginfo_ibase_drop_db)
+	PHP_FE(ibase_query, 		arginfo_ibase_query)
+	PHP_FE(ibase_fetch_row, 	arginfo_ibase_fetch_row)
+	PHP_FE(ibase_fetch_assoc, 	arginfo_ibase_fetch_assoc)
+	PHP_FE(ibase_fetch_object, 	arginfo_ibase_fetch_object)
+	PHP_FE(ibase_free_result, 	arginfo_ibase_free_result)
+	PHP_FE(ibase_name_result, 	arginfo_ibase_name_result)
+	PHP_FE(ibase_prepare, 		arginfo_ibase_prepare)
+	PHP_FE(ibase_execute, 		arginfo_ibase_execute)
+	PHP_FE(ibase_free_query, 	arginfo_ibase_free_query)
+	PHP_FE(ibase_gen_id, 		arginfo_ibase_gen_id)
+	PHP_FE(ibase_num_fields, 	arginfo_ibase_num_fields)
+	PHP_FE(ibase_num_params, 	arginfo_ibase_num_params)
+#if abies_0
+	PHP_FE(ibase_num_rows, 		arginfo_ibase_num_rows)
+#endif
+	PHP_FE(ibase_affected_rows, arginfo_ibase_affected_rows)
+	PHP_FE(ibase_field_info, 	arginfo_ibase_field_info)
+	PHP_FE(ibase_param_info, 	arginfo_ibase_param_info)
+
+	PHP_FE(ibase_trans, 		arginfo_ibase_trans)
+	PHP_FE(ibase_commit, 		arginfo_ibase_commit)
+	PHP_FE(ibase_rollback, 		arginfo_ibase_rollback)
+	PHP_FE(ibase_commit_ret, 	arginfo_ibase_commit_ret)
+	PHP_FE(ibase_rollback_ret, 	arginfo_ibase_rollback_ret)
+
+	PHP_FE(ibase_blob_info, 	arginfo_ibase_blob_info)
+	PHP_FE(ibase_blob_create, 	arginfo_ibase_blob_create)
+	PHP_FE(ibase_blob_add, 		arginfo_ibase_blob_add)
+	PHP_FE(ibase_blob_cancel, 	arginfo_ibase_blob_cancel)
+	PHP_FE(ibase_blob_close, 	arginfo_ibase_blob_close)
+	PHP_FE(ibase_blob_open, 	arginfo_ibase_blob_open)
+	PHP_FE(ibase_blob_get, 		arginfo_ibase_blob_get)
+	PHP_FE(ibase_blob_echo, 	arginfo_ibase_blob_echo)
+	PHP_FE(ibase_blob_import, 	arginfo_ibase_blob_import)
+	PHP_FE(ibase_errmsg, 		arginfo_ibase_errmsg)
+	PHP_FE(ibase_errcode, 		arginfo_ibase_errcode)
+
+	PHP_FE(ibase_add_user, 		arginfo_ibase_add_user)
+	PHP_FE(ibase_modify_user, 	arginfo_ibase_modify_user)
+	PHP_FE(ibase_delete_user, 	arginfo_ibase_delete_user)
+
+	PHP_FE(ibase_service_attach, arginfo_ibase_service_attach)
+	PHP_FE(ibase_service_detach, arginfo_ibase_service_detach)
+	PHP_FE(ibase_backup, 		arginfo_ibase_backup)
+	PHP_FE(ibase_restore, 		arginfo_ibase_restore)
+	PHP_FE(ibase_maintain_db, 	arginfo_ibase_maintain_db)
+	PHP_FE(ibase_db_info, 		arginfo_ibase_db_info)
+	PHP_FE(ibase_server_info, 	arginfo_ibase_server_info)
+
+	PHP_FE(ibase_wait_event, 			arginfo_ibase_wait_event)
+	PHP_FE(ibase_set_event_handler, 	arginfo_ibase_set_event_handler)
+	PHP_FE(ibase_free_event_handler, 	arginfo_ibase_free_event_handler)
 
 	/**
 	* These aliases are provided in order to maintain forward compatibility. As Firebird
@@ -178,7 +454,11 @@ zend_module_entry ibase_module_entry = {
 	PHP_RSHUTDOWN(ibase),
 	PHP_MINFO(ibase),
 	NO_VERSION_YET,
-	STANDARD_MODULE_PROPERTIES
+	PHP_MODULE_GLOBALS(ibase),
+	PHP_GINIT(ibase),
+	NULL,
+	NULL,
+	STANDARD_MODULE_PROPERTIES_EX
 };
 
 #ifdef COMPILE_DL_INTERBASE
@@ -188,8 +468,6 @@ ZEND_GET_MODULE(ibase)
 /* True globals, no need for thread safety */
 int le_link, le_plink, le_trans;
 
-ZEND_DECLARE_MODULE_GLOBALS(ibase)
-
 /* }}} */
 
 /* error handling ---------------------------- */
@@ -198,8 +476,8 @@ ZEND_DECLARE_MODULE_GLOBALS(ibase)
    Return error message */
 PHP_FUNCTION(ibase_errmsg)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 
 	if (IBG(sql_code) != 0) {
@@ -214,8 +492,8 @@ PHP_FUNCTION(ibase_errmsg)
    Return error code */
 PHP_FUNCTION(ibase_errcode)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 
 	if (IBG(sql_code) != 0) {
@@ -448,7 +726,7 @@ PHP_INI_BEGIN()
 	PHP_INI_ENTRY("ibase.timeformat", IB_DEF_TIME_FMT, PHP_INI_ALL, NULL)
 PHP_INI_END()
 
-static void php_ibase_init_globals(zend_ibase_globals *ibase_globals)
+static PHP_GINIT_FUNCTION(ibase)
 {
 	ibase_globals->num_persistent = ibase_globals->num_links = 0;
 	ibase_globals->sql_code = *ibase_globals->errmsg = 0;
@@ -457,8 +735,6 @@ static void php_ibase_init_globals(zend_ibase_globals *ibase_globals)
 
 PHP_MINIT_FUNCTION(ibase)
 {
-	ZEND_INIT_MODULE_GLOBALS(ibase, php_ibase_init_globals, NULL);
-
 	REGISTER_INI_ENTRIES();
 
 	le_link = zend_register_list_destructors_ex(_php_ibase_close_link, NULL, LE_LINK, module_number);
@@ -539,7 +815,7 @@ PHP_MINFO_FUNCTION(ibase)
 #endif
 
 #ifdef FB_API_VER
-	sprintf( (s = tmp), "Firebird API version %d", FB_API_VER);
+	snprintf( (s = tmp), sizeof(tmp), "Firebird API version %d", FB_API_VER);
 #elif (SQLDA_CURRENT_VERSION > 1)
 	s =  "Interbase 7.0 and up";
 #elif !defined(DSC_null)
@@ -585,22 +861,28 @@ static char const dpb_args[] = {
 	
 int _php_ibase_attach_db(char **args, int *len, long *largs, isc_db_handle *db TSRMLS_DC)
 {
-	short i;
+	short i, dpb_len, buf_len = 256;
 	char dpb_buffer[256] = { isc_dpb_version1 }, *dpb;
 
 	dpb = dpb_buffer + 1;
 
 	for (i = 0; i < sizeof(dpb_args); ++i) {
-		if (dpb_args[i] && args[i]) {
-			dpb += sprintf(dpb, "%c%c%s", dpb_args[i],(unsigned char)len[i],args[i]);
+		if (dpb_args[i] && args[i] && len[i] && buf_len > 0) {
+			dpb_len = slprintf(dpb, buf_len, "%c%c%s", dpb_args[i],(unsigned char)len[i],args[i]);
+			dpb += dpb_len;
+			buf_len -= dpb_len;
 		}
 	}
-	if (largs[BUF]) {
-		dpb += sprintf(dpb, "%c\2%c%c", isc_dpb_num_buffers, 
+	if (largs[BUF] && buf_len > 0) {
+		dpb_len = slprintf(dpb, buf_len, "%c\2%c%c", isc_dpb_num_buffers, 
 			(char)(largs[BUF] >> 8), (char)(largs[BUF] & 0xff));
+		dpb += dpb_len;
+		buf_len -= dpb_len;
 	}
-	if (largs[SYNC]) {
-		dpb += sprintf(dpb, "%c\1%c", isc_dpb_force_write, largs[SYNC] == isc_spb_prp_wm_sync ? 1 : 0);
+	if (largs[SYNC] && buf_len > 0) {
+		dpb_len = slprintf(dpb, buf_len, "%c\1%c", isc_dpb_force_write, largs[SYNC] == isc_spb_prp_wm_sync ? 1 : 0);
+		dpb += dpb_len;
+		buf_len -= dpb_len;
 	}
 	if (isc_attach_database(IB_STATUS, (short)len[DB], args[DB], db, (short)(dpb-dpb_buffer), dpb_buffer)) {
 		_php_ibase_error(TSRMLS_C);
@@ -616,7 +898,7 @@ static void _php_ibase_connect(INTERNAL_FUNCTION_PARAMETERS, int persistent) /*
 	int i, len[] = { 0, 0, 0, 0, 0 };
 	long largs[] = { 0, 0, 0 };
 	PHP_MD5_CTX hash_context;
-	list_entry new_index_ptr, *le;
+	zend_rsrc_list_entry new_index_ptr, *le;
 	isc_db_handle db_handle = NULL;
 	ibase_db_link *ib_link;
 
@@ -715,7 +997,7 @@ static void _php_ibase_connect(INTERNAL_FUNCTION_PARAMETERS, int persistent) /*
 			ib_link = (ibase_db_link *) emalloc(sizeof(ibase_db_link));
 			ZEND_REGISTER_RESOURCE(return_value, ib_link, le_link);
 		} else {
-			list_entry new_le;
+			zend_rsrc_list_entry new_le;
 			
 			ib_link = (ibase_db_link *) malloc(sizeof(ibase_db_link));
 	
@@ -723,7 +1005,7 @@ static void _php_ibase_connect(INTERNAL_FUNCTION_PARAMETERS, int persistent) /*
 			Z_TYPE(new_le) = le_plink;
 			new_le.ptr = ib_link;
 			if (FAILURE == zend_hash_update(&EG(persistent_list), hash, sizeof(hash),
-					(void *) &new_le, sizeof(list_entry), NULL)) {
+					(void *) &new_le, sizeof(zend_rsrc_list_entry), NULL)) {
 				free(ib_link);
 				RETURN_FALSE;
 			}
@@ -742,7 +1024,7 @@ static void _php_ibase_connect(INTERNAL_FUNCTION_PARAMETERS, int persistent) /*
 	new_index_ptr.ptr = (void *) Z_LVAL_P(return_value);
 	Z_TYPE(new_index_ptr) = le_index_ptr;
 	if (FAILURE == zend_hash_update(&EG(regular_list), hash, sizeof(hash),
-			(void *) &new_index_ptr, sizeof(list_entry), NULL)) {
+			(void *) &new_index_ptr, sizeof(zend_rsrc_list_entry), NULL)) {
 		RETURN_FALSE;
 	}
 	zend_list_addref(IBG(default_link) = Z_LVAL_P(return_value));
@@ -769,30 +1051,25 @@ PHP_FUNCTION(ibase_pconnect)
    Close an InterBase connection */
 PHP_FUNCTION(ibase_close)
 {
-	zval **link_arg = NULL;
+	zval *link_arg = NULL;
 	ibase_db_link *ib_link;
 	int link_id;
 
 	RESET_ERRMSG;
 	
-	switch (ZEND_NUM_ARGS()) {
-		case 0:
-			link_id = IBG(default_link);
-			IBG(default_link) = -1;
-			break;
-		case 1:
-			if (zend_get_parameters_ex(1, &link_arg) == FAILURE) {
-				RETURN_FALSE;
-			}
-			convert_to_long_ex(link_arg);
-			link_id = Z_LVAL_PP(link_arg);
-			break;
-		default:
-			WRONG_PARAM_COUNT;
-			break;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|r", &link_arg) == FAILURE) {
+		return;
+	}
+	
+	if (ZEND_NUM_ARGS() == 0) {
+		link_id = IBG(default_link);
+		CHECK_LINK(link_id);
+		IBG(default_link) = -1;
+	} else {
+		link_id = Z_RESVAL_P(link_arg);
 	}
 
-	ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, link_arg, link_id, LE_LINK, le_link, le_plink);
+	ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, &link_arg, link_id, LE_LINK, le_link, le_plink);
 	zend_list_delete(link_id);
 	RETURN_TRUE;
 }
@@ -802,31 +1079,26 @@ PHP_FUNCTION(ibase_close)
    Drop an InterBase database */
 PHP_FUNCTION(ibase_drop_db)
 {
-	zval **link_arg = NULL;
+	zval *link_arg = NULL;
 	ibase_db_link *ib_link;
 	ibase_tr_list *l;
 	int link_id;
 
 	RESET_ERRMSG;
-
-	switch (ZEND_NUM_ARGS()) {
-		case 0:
-			link_id = IBG(default_link);
-			IBG(default_link) = -1;
-			break;
-		case 1:
-			if (zend_get_parameters_ex(1, &link_arg) == FAILURE) {
-				RETURN_FALSE;
-			}
-			convert_to_long_ex(link_arg);
-			link_id = Z_LVAL_PP(link_arg);
-			break;
-		default:
-			WRONG_PARAM_COUNT;
-			break;
+	
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|r", &link_arg) == FAILURE) {
+		return;
 	}
 	
-	ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, link_arg, link_id, LE_LINK, le_link, le_plink);
+	if (ZEND_NUM_ARGS() == 0) {
+		link_id = IBG(default_link);
+		CHECK_LINK(link_id);
+		IBG(default_link) = -1;
+	} else {
+		link_id = Z_RESVAL_P(link_arg);
+	}
+	
+	ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, &link_arg, link_id, LE_LINK, le_link, le_plink);
 
 	if (isc_drop_database(IB_STATUS, &ib_link->handle)) {
 		_php_ibase_error(TSRMLS_C);
@@ -868,9 +1140,9 @@ PHP_FUNCTION(ibase_trans)
 		long trans_argl = 0;
 		char *tpb;
 		ISC_TEB *teb;
-		zval ***args = (zval ***) safe_emalloc(sizeof(zval **),argn,0);
+		zval ***args = NULL;
 
-		if (zend_get_parameters_array_ex(argn, args) == FAILURE) {
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "+", &args, &argn) == FAILURE) {
 			efree(args);
 			efree(ib_link);
 			RETURN_FALSE;
@@ -885,8 +1157,13 @@ PHP_FUNCTION(ibase_trans)
 			
 			if (Z_TYPE_PP(args[i]) == IS_RESOURCE) {
 				
-				ZEND_FETCH_RESOURCE2(ib_link[link_cnt], ibase_db_link *, args[i], -1, 
-					LE_LINK, le_link, le_plink);
+				if (!ZEND_FETCH_RESOURCE2_NO_RETURN(ib_link[link_cnt], ibase_db_link *, args[i], -1, LE_LINK, le_link, le_plink)) {
+					efree(teb);
+					efree(tpb);
+					efree(ib_link);
+					efree(args);
+					RETURN_FALSE;
+				}
 	
 				/* copy the most recent modifier string into tbp[] */
 				memcpy(&tpb[TPB_MAX_SIZE * link_cnt], last_tpb, TPB_MAX_SIZE);
@@ -950,8 +1227,10 @@ PHP_FUNCTION(ibase_trans)
 
 	if (link_cnt == 0) {
 		link_cnt = 1;
-		ZEND_FETCH_RESOURCE2(ib_link[0], ibase_db_link *, NULL, IBG(default_link), LE_LINK, 
-			le_link, le_plink);
+		if (!ZEND_FETCH_RESOURCE2_NO_RETURN(ib_link[0], ibase_db_link *, NULL, IBG(default_link), LE_LINK, le_link, le_plink)) {
+			efree(ib_link);
+			RETURN_FALSE;
+		}
 		result = isc_start_transaction(IB_STATUS, &tr_handle, 1, &ib_link[0]->handle, tpb_len, last_tpb);
 	}
 	
@@ -1031,55 +1310,42 @@ static void _php_ibase_trans_end(INTERNAL_FUNCTION_PARAMETERS, int commit) /* {{
 	ibase_trans *trans = NULL;
 	int res_id = 0;
 	ISC_STATUS result;
+	ibase_db_link *ib_link;
+	zval *arg = NULL;
+	int type;
 
 	RESET_ERRMSG;
+	
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|r", &arg) == FAILURE) {
+		return;
+	}
 
-	switch (ZEND_NUM_ARGS()) {
+	if (ZEND_NUM_ARGS() == 0) {
+		ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, NULL, IBG(default_link), LE_LINK, le_link, le_plink);
+		if (ib_link->tr_list == NULL || ib_link->tr_list->trans == NULL) {
+			/* this link doesn't have a default transaction */
+			_php_ibase_module_error("Default link has no default transaction" TSRMLS_CC);
+			RETURN_FALSE;
+		}
+		trans = ib_link->tr_list->trans;
+	} else {
+		/* one id was passed, could be db or trans id */
+		if (zend_list_find(Z_RESVAL_P(arg), &type) && type == le_trans) {
+			ZEND_FETCH_RESOURCE(trans, ibase_trans *, &arg, -1, LE_TRANS, le_trans);
+			res_id = Z_RESVAL_P(arg);
+		} else {
+			ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, &arg, -1, LE_LINK, le_link, le_plink);
 
-		ibase_db_link *ib_link;
-		zval **arg;
-		int type;
-
-		case 0:
-			ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, NULL, IBG(default_link), LE_LINK, 
-				le_link, le_plink);
 			if (ib_link->tr_list == NULL || ib_link->tr_list->trans == NULL) {
 				/* this link doesn't have a default transaction */
-				_php_ibase_module_error("Default link has no default transaction" TSRMLS_CC);
+				_php_ibase_module_error("Link has no default transaction" TSRMLS_CC);
 				RETURN_FALSE;
 			}
 			trans = ib_link->tr_list->trans;
-			break;
-
-		case 1: 
-			if (zend_get_parameters_ex(1, &arg) == FAILURE) {
-				RETURN_FALSE;
-			}
-			/* one id was passed, could be db or trans id */
-			if (zend_list_find(Z_LVAL_PP(arg), &type) && type == le_trans) {			
-				ZEND_FETCH_RESOURCE(trans, ibase_trans *, arg, -1, LE_TRANS, le_trans);
-
-				convert_to_long_ex(arg);
-				res_id = Z_LVAL_PP(arg);
-			} else {
-				ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, arg, -1, LE_LINK, le_link, le_plink);
-
-				if (ib_link->tr_list == NULL || ib_link->tr_list->trans == NULL) {
-					/* this link doesn't have a default transaction */
-					_php_ibase_module_error("Link has no default transaction" TSRMLS_CC);
-					RETURN_FALSE;
-				}
-				trans = ib_link->tr_list->trans;
-			}
-			break;
-
-		default:
-			WRONG_PARAM_COUNT;
-			break;
+		}
 	}
 
 	switch (commit) {
-		
 		default: /* == case ROLLBACK: */
 			result = isc_rollback_transaction(IB_STATUS, &trans->handle);
 			break;
@@ -1100,7 +1366,7 @@ static void _php_ibase_trans_end(INTERNAL_FUNCTION_PARAMETERS, int commit) /* {{
 	}
 
 	/* Don't try to destroy implicitly opened transaction from list... */
-	if ( (commit & RETAIN) == 0 && res_id != 0) {
+	if ((commit & RETAIN) == 0 && res_id != 0) {
 		zend_list_delete(res_id);
 	}
 	RETURN_TRUE;
@@ -1161,7 +1427,7 @@ PHP_FUNCTION(ibase_gen_id)
 
 	PHP_IBASE_LINK_TRANS(link, ib_link, trans);
 	
-	sprintf(query, "SELECT GEN_ID(%s,%ld) FROM rdb$database", generator, inc);
+	snprintf(query, sizeof(query), "SELECT GEN_ID(%s,%ld) FROM rdb$database", generator, inc);
 
 	/* allocate a minimal descriptor area */
 	out_sqlda.sqln = out_sqlda.sqld = 1;
@@ -1183,10 +1449,11 @@ PHP_FUNCTION(ibase_gen_id)
 	/* don't return the generator value as a string unless it doesn't fit in a long */
 #if SIZEOF_LONG < 8
 	if (result < LONG_MIN || result > LONG_MAX) {
-		char res[24];
+		char *res;
+		int l;
 
-		sprintf(res, "%" LL_MASK "d", result);
-		RETURN_STRING(res,1);
+		l = spprintf(&res, 0, "%" LL_MASK "d", result);
+		RETURN_STRINGL(res, l, 0);
 	}
 #endif
 	RETURN_LONG((long)result);
