@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: libxml.c,v 1.18.2.5 2004/11/09 08:13:04 dmitry Exp $ */
+/* $Id: libxml.c,v 1.32.2.7.2.15.2.13 2009/06/23 13:44:24 bjori Exp $ */
 
 #define IS_EXT_MODULE
 
@@ -40,6 +40,7 @@
 #include <libxml/tree.h>
 #include <libxml/uri.h>
 #include <libxml/xmlerror.h>
+#include <libxml/xmlsave.h>
 #ifdef LIBXML_SCHEMAS_ENABLED
 #include <libxml/relaxng.h>
 #endif
@@ -51,7 +52,7 @@
 #define PHP_LIBXML_CTX_WARNING 2
 
 /* a true global for initialization */
-int _php_libxml_initialized = 0;
+static int _php_libxml_initialized = 0;
 
 typedef struct _php_libxml_func_handler {
 	php_libxml_export_node export_func;
@@ -59,38 +60,70 @@ typedef struct _php_libxml_func_handler {
 
 static HashTable php_libxml_exports;
 
-#ifdef ZTS
-int libxml_globals_id;
-#else
-PHP_LIBXML_API php_libxml_globals libxml_globals;
-#endif
+static ZEND_DECLARE_MODULE_GLOBALS(libxml)
+static PHP_GINIT_FUNCTION(libxml);
+
+static PHP_FUNCTION(libxml_set_streams_context);
+static PHP_FUNCTION(libxml_use_internal_errors);
+static PHP_FUNCTION(libxml_get_last_error);
+static PHP_FUNCTION(libxml_clear_errors);
+static PHP_FUNCTION(libxml_get_errors);
+static PHP_FUNCTION(libxml_disable_entity_loader);
+
+static zend_class_entry *libxmlerror_class_entry;
 
 /* {{{ dynamically loadable module stuff */
 #ifdef COMPILE_DL_LIBXML
 ZEND_GET_MODULE(libxml)
-# ifdef PHP_WIN32
-# include "zend_arg_defs.c"
-# endif
 #endif /* COMPILE_DL_LIBXML */
 /* }}} */
 
 /* {{{ function prototypes */
-PHP_MINIT_FUNCTION(libxml);
-PHP_RINIT_FUNCTION(libxml);
-PHP_MSHUTDOWN_FUNCTION(libxml);
-PHP_RSHUTDOWN_FUNCTION(libxml);
-PHP_MINFO_FUNCTION(libxml);
+static PHP_MINIT_FUNCTION(libxml);
+static PHP_RINIT_FUNCTION(libxml);
+static PHP_MSHUTDOWN_FUNCTION(libxml);
+static PHP_RSHUTDOWN_FUNCTION(libxml);
+static PHP_MINFO_FUNCTION(libxml);
+
+/* }}} */
+
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO(arginfo_libxml_set_streams_context, 0)
+	ZEND_ARG_INFO(0, context)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_libxml_use_internal_errors, 0, 0, 0)
+	ZEND_ARG_INFO(0, use_errors)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_libxml_get_last_error, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_libxml_get_errors, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_libxml_clear_errors, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_libxml_disable_entity_loader, 0, 0, 0)
+	ZEND_ARG_INFO(0, disable)
+ZEND_END_ARG_INFO()
 
 /* }}} */
 
 /* {{{ extension definition structures */
-function_entry libxml_functions[] = {
-	PHP_FE(libxml_set_streams_context, NULL)
+static const zend_function_entry libxml_functions[] = {
+	PHP_FE(libxml_set_streams_context, arginfo_libxml_set_streams_context)
+	PHP_FE(libxml_use_internal_errors, arginfo_libxml_use_internal_errors)
+	PHP_FE(libxml_get_last_error, arginfo_libxml_get_last_error)
+	PHP_FE(libxml_clear_errors, arginfo_libxml_clear_errors)
+	PHP_FE(libxml_get_errors, arginfo_libxml_get_errors)
+	PHP_FE(libxml_disable_entity_loader, arginfo_libxml_disable_entity_loader)
 	{NULL, NULL, NULL}
 };
 
 zend_module_entry libxml_module_entry = {
-    STANDARD_MODULE_HEADER,
+	STANDARD_MODULE_HEADER,
 	"libxml",                /* extension name */
 	libxml_functions,        /* extension function list */
 	PHP_MINIT(libxml),       /* extension-wide startup function */
@@ -98,31 +131,17 @@ zend_module_entry libxml_module_entry = {
 	PHP_RINIT(libxml),       /* per-request startup function */
 	PHP_RSHUTDOWN(libxml),   /* per-request shutdown function */
 	PHP_MINFO(libxml),       /* information function */
-    NO_VERSION_YET,
-	STANDARD_MODULE_PROPERTIES
+	NO_VERSION_YET,
+	PHP_MODULE_GLOBALS(libxml), /* globals descriptor */
+	PHP_GINIT(libxml),          /* globals ctor */
+	NULL,                       /* globals dtor */
+	NULL,                       /* post deactivate */
+	STANDARD_MODULE_PROPERTIES_EX
 };
 
 /* }}} */
 
 /* {{{ internal functions for interoperability */
-static int php_libxml_dec_node(php_libxml_node_ptr *nodeptr)
-{
-	int ret_refcount;
-
-	ret_refcount = --nodeptr->refcount;
-	if (ret_refcount == 0) {
-		if (nodeptr->node != NULL && nodeptr->node->type != XML_DOCUMENT_NODE) {
-			nodeptr->node->_private = NULL;
-		}
-		/* node is destroyed by another object. reset ret_refcount to 1 and node to NULL
-		so the php_libxml_node_ptr is detroyed when the object is destroyed */
-		nodeptr->refcount = 1;
-		nodeptr->node = NULL;
-	}
-
-	return ret_refcount;
-}
-
 static int php_libxml_clear_object(php_libxml_node_object *object TSRMLS_DC)
 {
 	if (object->properties) {
@@ -143,7 +162,10 @@ static int php_libxml_unregister_node(xmlNodePtr nodep TSRMLS_DC)
 		if (wrapper) {
 			php_libxml_clear_object(wrapper TSRMLS_CC);
 		} else {
-			php_libxml_dec_node(nodeptr);
+			if (nodeptr->node != NULL && nodeptr->node->type != XML_DOCUMENT_NODE) {
+				nodeptr->node->_private = NULL;
+			}
+			nodeptr->node = NULL;
 		}
 	}
 
@@ -204,12 +226,16 @@ static void php_libxml_node_free_list(xmlNodePtr node TSRMLS_DC)
 				case XML_ENTITY_REF_NODE:
 					php_libxml_node_free_list((xmlNodePtr) node->properties TSRMLS_CC);
 					break;
+				case XML_ATTRIBUTE_NODE:
+    					if ((node->doc != NULL) && (((xmlAttrPtr) node)->atype == XML_ATTRIBUTE_ID)) {
+	    					xmlRemoveID(node->doc, (xmlAttrPtr) node);
+    					}
 				case XML_ATTRIBUTE_DECL:
 				case XML_DTD_NODE:
 				case XML_DOCUMENT_TYPE_NODE:
 				case XML_ENTITY_DECL:
-				case XML_ATTRIBUTE_NODE:
 				case XML_NAMESPACE_DECL:
+				case XML_TEXT_NODE:
 					php_libxml_node_free_list(node->children TSRMLS_CC);
 					break;
 				default:
@@ -230,45 +256,39 @@ static void php_libxml_node_free_list(xmlNodePtr node TSRMLS_DC)
 /* }}} */
 
 /* {{{ startup, shutdown and info functions */
-#ifdef ZTS
-static void php_libxml_init_globals(php_libxml_globals *libxml_globals_p TSRMLS_DC)
+static PHP_GINIT_FUNCTION(libxml)
 {
-	LIBXML(stream_context) = NULL;
-	LIBXML(error_buffer).c = NULL;
+	libxml_globals->stream_context = NULL;
+	libxml_globals->error_buffer.c = NULL;
+	libxml_globals->error_list = NULL;
 }
-#endif
 
 /* Channel libxml file io layer through the PHP streams subsystem.
  * This allows use of ftps:// and https:// urls */
 
-int php_libxml_streams_IO_match_wrapper(const char *filename)
-{
-	char *resolved_path;
-	int retval;
-
-	TSRMLS_FETCH();
-
-	if (zend_is_executing(TSRMLS_C)) {
-		resolved_path = xmlURIUnescapeString(filename, 0, NULL);
-		retval = php_stream_locate_url_wrapper(resolved_path, NULL, 0 TSRMLS_CC) ? 1 : 0;
-		if (resolved_path) {
-			xmlFree(resolved_path);
-		}
-		return retval;
-	}
-	return 0;
-}
-
-void *php_libxml_streams_IO_open_wrapper(const char *filename, const char *mode, const int read_only)
+static void *php_libxml_streams_IO_open_wrapper(const char *filename, const char *mode, const int read_only)
 {
 	php_stream_statbuf ssbuf;
 	php_stream_context *context = NULL;
 	php_stream_wrapper *wrapper = NULL;
 	char *resolved_path, *path_to_open = NULL;
 	void *ret_val = NULL;
+	int isescaped=0;
+	xmlURI *uri;
 
 	TSRMLS_FETCH();
-	resolved_path = xmlURIUnescapeString(filename, 0, NULL);
+
+	uri = xmlParseURI((xmlChar *)filename);
+	if (uri && (uri->scheme == NULL || (xmlStrncmp(uri->scheme, "file", 4) == 0))) {
+		resolved_path = xmlURIUnescapeString(filename, 0, NULL);
+		isescaped = 1;
+	} else {
+		resolved_path = (char *)filename;
+	}
+
+	if (uri) {
+		xmlFreeURI(uri);
+	}
 
 	if (resolved_path == NULL) {
 		return NULL;
@@ -283,7 +303,9 @@ void *php_libxml_streams_IO_open_wrapper(const char *filename, const char *mode,
 	wrapper = php_stream_locate_url_wrapper(resolved_path, &path_to_open, ENFORCE_SAFE_MODE TSRMLS_CC);
 	if (wrapper && read_only && wrapper->wops->url_stat) {
 		if (wrapper->wops->url_stat(wrapper, path_to_open, PHP_STREAM_URL_STAT_QUIET, &ssbuf, NULL TSRMLS_CC) == -1) {
-			xmlFree(resolved_path);
+			if (isescaped) {
+				xmlFree(resolved_path);
+			}
 			return NULL;
 		}
 	}
@@ -293,38 +315,158 @@ void *php_libxml_streams_IO_open_wrapper(const char *filename, const char *mode,
 	}
 
 	ret_val = php_stream_open_wrapper_ex(path_to_open, (char *)mode, ENFORCE_SAFE_MODE|REPORT_ERRORS, NULL, context);
-	xmlFree(resolved_path);
+	if (isescaped) {
+		xmlFree(resolved_path);
+	}
 	return ret_val;
 }
 
-void *php_libxml_streams_IO_open_read_wrapper(const char *filename)
+static void *php_libxml_streams_IO_open_read_wrapper(const char *filename)
 {
 	return php_libxml_streams_IO_open_wrapper(filename, "rb", 1);
 }
 
-void *php_libxml_streams_IO_open_write_wrapper(const char *filename)
+static void *php_libxml_streams_IO_open_write_wrapper(const char *filename)
 {
 	return php_libxml_streams_IO_open_wrapper(filename, "wb", 0);
 }
 
-int php_libxml_streams_IO_read(void *context, char *buffer, int len)
+static int php_libxml_streams_IO_read(void *context, char *buffer, int len)
 {
 	TSRMLS_FETCH();
 	return php_stream_read((php_stream*)context, buffer, len);
 }
 
-int php_libxml_streams_IO_write(void *context, const char *buffer, int len)
+static int php_libxml_streams_IO_write(void *context, const char *buffer, int len)
 {
 	TSRMLS_FETCH();
 	return php_stream_write((php_stream*)context, buffer, len);
 }
 
-int php_libxml_streams_IO_close(void *context)
+static int php_libxml_streams_IO_close(void *context)
 {
 	TSRMLS_FETCH();
 	return php_stream_close((php_stream*)context);
 }
 
+static xmlParserInputBufferPtr
+php_libxml_input_buffer_noload(const char *URI, xmlCharEncoding enc)
+{
+	return NULL;
+}
+
+static xmlParserInputBufferPtr
+php_libxml_input_buffer_create_filename(const char *URI, xmlCharEncoding enc)
+{
+	xmlParserInputBufferPtr ret;
+	void *context = NULL;
+
+	if (URI == NULL)
+		return(NULL);
+
+	context = php_libxml_streams_IO_open_read_wrapper(URI);
+
+	if (context == NULL) {
+		return(NULL);
+	}
+
+	/* Allocate the Input buffer front-end. */
+	ret = xmlAllocParserInputBuffer(enc);
+	if (ret != NULL) {
+		ret->context = context;
+		ret->readcallback = php_libxml_streams_IO_read;
+		ret->closecallback = php_libxml_streams_IO_close;
+	} else
+		php_libxml_streams_IO_close(context);
+
+	return(ret);
+}
+
+static xmlOutputBufferPtr
+php_libxml_output_buffer_create_filename(const char *URI,
+                              xmlCharEncodingHandlerPtr encoder,
+                              int compression ATTRIBUTE_UNUSED)
+{
+	xmlOutputBufferPtr ret;
+	xmlURIPtr puri;
+	void *context = NULL;
+	char *unescaped = NULL;
+
+	if (URI == NULL)
+		return(NULL);
+
+	puri = xmlParseURI(URI);
+	if (puri != NULL) {
+		if (puri->scheme != NULL)
+			unescaped = xmlURIUnescapeString(URI, 0, NULL);
+		xmlFreeURI(puri);
+	}
+
+	if (unescaped != NULL) {
+		context = php_libxml_streams_IO_open_write_wrapper(unescaped);
+		xmlFree(unescaped);
+	}
+
+	/* try with a non-escaped URI this may be a strange filename */
+	if (context == NULL) {
+		context = php_libxml_streams_IO_open_write_wrapper(URI);
+	}
+
+	if (context == NULL) {
+		return(NULL);
+	}
+
+	/* Allocate the Output buffer front-end. */
+	ret = xmlAllocOutputBuffer(encoder);
+	if (ret != NULL) {
+		ret->context = context;
+		ret->writecallback = php_libxml_streams_IO_write;
+		ret->closecallback = php_libxml_streams_IO_close;
+	}
+
+	return(ret);
+}
+
+static int _php_libxml_free_error(xmlErrorPtr error)
+{
+	/* This will free the libxml alloc'd memory */
+	xmlResetError(error);
+	return 1;
+}
+
+static void _php_list_set_error_structure(xmlErrorPtr error, const char *msg)
+{
+	xmlError error_copy;
+	int ret;
+
+	TSRMLS_FETCH();
+
+	memset(&error_copy, 0, sizeof(xmlError));
+
+	if (error) {
+		ret = xmlCopyError(error, &error_copy);
+	} else {
+		error_copy.domain = 0;
+		error_copy.code = XML_ERR_INTERNAL_ERROR;
+		error_copy.level = XML_ERR_ERROR;
+		error_copy.line = 0;
+		error_copy.node = NULL;
+		error_copy.int1 = 0;
+		error_copy.int2 = 0;
+		error_copy.ctxt = NULL;
+		error_copy.message = xmlStrdup(msg);
+		error_copy.file = NULL;
+		error_copy.str1 = NULL;
+		error_copy.str2 = NULL;
+		error_copy.str3 = NULL;
+		ret = 0;
+	}
+
+	if (ret == 0) {
+		zend_llist_add_element(LIBXML(error_list), &error_copy);
+	}
+}
+
 static void php_libxml_ctx_error_level(int level, void *ctx, const char *msg TSRMLS_DC)
 {
 	xmlParserCtxtPtr parser;
@@ -340,6 +482,15 @@ static void php_libxml_ctx_error_level(int level, void *ctx, const char *msg TSR
 	}
 }
 
+void php_libxml_issue_error(int level, const char *msg TSRMLS_DC)
+{
+	if (LIBXML(error_list)) {
+		_php_list_set_error_structure(NULL, msg);
+	} else {
+		php_error_docref(NULL TSRMLS_CC, level, "%s", msg);
+	}
+}
+
 static void php_libxml_internal_error_handler(int error_type, void *ctx, const char **msg, va_list ap)
 {
 	char *buf;
@@ -361,21 +512,25 @@ static void php_libxml_internal_error_handler(int error_type, void *ctx, const c
 	efree(buf);
 
 	if (output == 1) {
-		switch (error_type) {
-			case PHP_LIBXML_CTX_ERROR:
-				php_libxml_ctx_error_level(E_WARNING, ctx, LIBXML(error_buffer).c TSRMLS_CC);
-				break;
-			case PHP_LIBXML_CTX_WARNING:
-				php_libxml_ctx_error_level(E_NOTICE, ctx, LIBXML(error_buffer).c TSRMLS_CC);
-				break;
-			default:
-				php_error(E_WARNING, "%s", LIBXML(error_buffer).c);
+		if (LIBXML(error_list)) {
+			_php_list_set_error_structure(NULL, LIBXML(error_buffer).c);
+		} else {
+			switch (error_type) {
+				case PHP_LIBXML_CTX_ERROR:
+					php_libxml_ctx_error_level(E_WARNING, ctx, LIBXML(error_buffer).c TSRMLS_CC);
+					break;
+				case PHP_LIBXML_CTX_WARNING:
+					php_libxml_ctx_error_level(E_NOTICE, ctx, LIBXML(error_buffer).c TSRMLS_CC);
+					break;
+				default:
+					php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", LIBXML(error_buffer).c);
+			}
 		}
 		smart_str_free(&LIBXML(error_buffer));
 	}
 }
 
-void php_libxml_ctx_error(void *ctx, const char *msg, ...)
+PHP_LIBXML_API void php_libxml_ctx_error(void *ctx, const char *msg, ...)
 {
 	va_list args;
 	va_start(args, msg);
@@ -383,7 +538,7 @@ void php_libxml_ctx_error(void *ctx, const char *msg, ...)
 	va_end(args);
 }
 
-void php_libxml_ctx_warning(void *ctx, const char *msg, ...)
+PHP_LIBXML_API void php_libxml_ctx_warning(void *ctx, const char *msg, ...)
 {
 	va_list args;
 	va_start(args, msg);
@@ -391,6 +546,13 @@ void php_libxml_ctx_warning(void *ctx, const char *msg, ...)
 	va_end(args);
 }
 
+PHP_LIBXML_API void php_libxml_structured_error_handler(void *userData, xmlErrorPtr error)
+{
+	_php_list_set_error_structure(error, NULL);
+
+	return;
+}
+
 PHP_LIBXML_API void php_libxml_error_handler(void *ctx, const char *msg, ...)
 {
 	va_list args;
@@ -400,34 +562,20 @@ PHP_LIBXML_API void php_libxml_error_handler(void *ctx, const char *msg, ...)
 }
 
 
-PHP_LIBXML_API void php_libxml_initialize() {
+PHP_LIBXML_API void php_libxml_initialize(void)
+{
 	if (!_php_libxml_initialized) {
 		/* we should be the only one's to ever init!! */
 		xmlInitParser();
 
-		/* Enable php stream/wrapper support for libxml 
-		   we only use php streams, so we do not enable
-		   the default io handlers in libxml.
-		*/
-		xmlRegisterInputCallbacks(
-			php_libxml_streams_IO_match_wrapper, 
-			php_libxml_streams_IO_open_read_wrapper,
-			php_libxml_streams_IO_read, 
-			php_libxml_streams_IO_close);
-
-		xmlRegisterOutputCallbacks(
-			php_libxml_streams_IO_match_wrapper, 
-			php_libxml_streams_IO_open_write_wrapper,
-			php_libxml_streams_IO_write, 
-			php_libxml_streams_IO_close);
-
 		zend_hash_init(&php_libxml_exports, 0, NULL, NULL, 1);
 
 		_php_libxml_initialized = 1;
 	}
 }
 
-PHP_LIBXML_API void php_libxml_shutdown() {
+PHP_LIBXML_API void php_libxml_shutdown(void)
+{
 	if (_php_libxml_initialized) {
 #if defined(LIBXML_SCHEMAS_ENABLED)
 		xmlRelaxNGCleanupTypes();
@@ -438,7 +586,8 @@ PHP_LIBXML_API void php_libxml_shutdown() {
 	}
 }
 
-PHP_LIBXML_API zval *php_libxml_switch_context(zval *context TSRMLS_DC) {
+PHP_LIBXML_API zval *php_libxml_switch_context(zval *context TSRMLS_DC)
+{
 	zval *oldcontext;
 
 	oldcontext = LIBXML(stream_context);
@@ -447,31 +596,58 @@ PHP_LIBXML_API zval *php_libxml_switch_context(zval *context TSRMLS_DC) {
 
 }
 
-PHP_MINIT_FUNCTION(libxml)
+static PHP_MINIT_FUNCTION(libxml)
 {
+	zend_class_entry ce;
+
 	php_libxml_initialize();
 
-#ifdef ZTS
-	ts_allocate_id(&libxml_globals_id, sizeof(php_libxml_globals), (ts_allocate_ctor) php_libxml_init_globals, NULL);
-#else
-	LIBXML(stream_context) = NULL;
-	LIBXML(error_buffer).c = NULL;
+	REGISTER_LONG_CONSTANT("LIBXML_VERSION",			LIBXML_VERSION,			CONST_CS | CONST_PERSISTENT);
+	REGISTER_STRING_CONSTANT("LIBXML_DOTTED_VERSION",	LIBXML_DOTTED_VERSION,	CONST_CS | CONST_PERSISTENT);
+	REGISTER_STRING_CONSTANT("LIBXML_LOADED_VERSION",	(char *)xmlParserVersion,		CONST_CS | CONST_PERSISTENT);
+
+	/* For use with loading xml */
+	REGISTER_LONG_CONSTANT("LIBXML_NOENT",		XML_PARSE_NOENT,		CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_DTDLOAD",	XML_PARSE_DTDLOAD,		CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_DTDATTR",	XML_PARSE_DTDATTR,		CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_DTDVALID",	XML_PARSE_DTDVALID,		CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_NOERROR",	XML_PARSE_NOERROR,		CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_NOWARNING",	XML_PARSE_NOWARNING,	CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_NOBLANKS",	XML_PARSE_NOBLANKS,		CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_XINCLUDE",	XML_PARSE_XINCLUDE,		CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_NSCLEAN",	XML_PARSE_NSCLEAN,		CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_NOCDATA",	XML_PARSE_NOCDATA,		CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_NONET",		XML_PARSE_NONET,		CONST_CS | CONST_PERSISTENT);
+#if LIBXML_VERSION >= 20621
+	REGISTER_LONG_CONSTANT("LIBXML_COMPACT",	XML_PARSE_COMPACT,		CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_NOXMLDECL",	XML_SAVE_NO_DECL,		CONST_CS | CONST_PERSISTENT);
 #endif
+	REGISTER_LONG_CONSTANT("LIBXML_NOEMPTYTAG",	LIBXML_SAVE_NOEMPTYTAG,	CONST_CS | CONST_PERSISTENT);
+
+	/* Error levels */
+	REGISTER_LONG_CONSTANT("LIBXML_ERR_NONE",		XML_ERR_NONE,		CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_ERR_WARNING",	XML_ERR_WARNING,	CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_ERR_ERROR",		XML_ERR_ERROR,		CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("LIBXML_ERR_FATAL",		XML_ERR_FATAL,		CONST_CS | CONST_PERSISTENT);
+
+	INIT_CLASS_ENTRY(ce, "LibXMLError", NULL);
+	libxmlerror_class_entry = zend_register_internal_class(&ce TSRMLS_CC);
 
 	return SUCCESS;
 }
 
 
-PHP_RINIT_FUNCTION(libxml)
+static PHP_RINIT_FUNCTION(libxml)
 {
 	/* report errors via handler rather than stderr */
 	xmlSetGenericErrorFunc(NULL, php_libxml_error_handler);
-
-    return SUCCESS;
+	xmlParserInputBufferCreateFilenameDefault(php_libxml_input_buffer_create_filename);
+	xmlOutputBufferCreateFilenameDefault(php_libxml_output_buffer_create_filename);
+	return SUCCESS;
 }
 
 
-PHP_MSHUTDOWN_FUNCTION(libxml)
+static PHP_MSHUTDOWN_FUNCTION(libxml)
 {
 	php_libxml_shutdown();
 
@@ -479,30 +655,45 @@ PHP_MSHUTDOWN_FUNCTION(libxml)
 }
 
 
-PHP_RSHUTDOWN_FUNCTION(libxml)
+static PHP_RSHUTDOWN_FUNCTION(libxml)
 {
 	/* reset libxml generic error handling */
 	xmlSetGenericErrorFunc(NULL, NULL);
+	xmlSetStructuredErrorFunc(NULL, NULL);
 
+	xmlParserInputBufferCreateFilenameDefault(NULL);
+	xmlOutputBufferCreateFilenameDefault(NULL);
+
+	if (LIBXML(stream_context)) {
+		zval_ptr_dtor(&LIBXML(stream_context));
+		LIBXML(stream_context) = NULL;
+	}
 	smart_str_free(&LIBXML(error_buffer));
+	if (LIBXML(error_list)) {
+		zend_llist_destroy(LIBXML(error_list));
+		efree(LIBXML(error_list));
+		LIBXML(error_list) = NULL;
+	}
+	xmlResetLastError();
+
 	return SUCCESS;
 }
 
 
-PHP_MINFO_FUNCTION(libxml)
+static PHP_MINFO_FUNCTION(libxml)
 {
 	php_info_print_table_start();
 	php_info_print_table_row(2, "libXML support", "active");
-	php_info_print_table_row(2, "libXML Version", LIBXML_DOTTED_VERSION);
+	php_info_print_table_row(2, "libXML Compiled Version", LIBXML_DOTTED_VERSION);
+	php_info_print_table_row(2, "libXML Loaded Version", (char *)xmlParserVersion);
 	php_info_print_table_row(2, "libXML streams", "enabled");
 	php_info_print_table_end();
 }
 /* }}} */
 
-
 /* {{{ proto void libxml_set_streams_context(resource streams_context) 
    Set the streams context for the next libxml document load or write */
-PHP_FUNCTION(libxml_set_streams_context)
+static PHP_FUNCTION(libxml_set_streams_context)
 {
 	zval *arg;
 
@@ -513,11 +704,159 @@ PHP_FUNCTION(libxml_set_streams_context)
 		zval_ptr_dtor(&LIBXML(stream_context));
 		LIBXML(stream_context) = NULL;
 	}
-	ZVAL_ADDREF(arg);
+	Z_ADDREF_P(arg);
 	LIBXML(stream_context) = arg;
 }
 /* }}} */
 
+/* {{{ proto bool libxml_use_internal_errors([boolean use_errors]) 
+   Disable libxml errors and allow user to fetch error information as needed */
+static PHP_FUNCTION(libxml_use_internal_errors)
+{
+	xmlStructuredErrorFunc current_handler;
+	zend_bool use_errors=0, retval;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &use_errors) == FAILURE) {
+		return;
+	}
+
+	current_handler = xmlStructuredError;
+	if (current_handler && current_handler == php_libxml_structured_error_handler) {
+		retval = 1;
+	} else {
+		retval = 0;
+	}
+
+	if (ZEND_NUM_ARGS() == 0) {
+		RETURN_BOOL(retval);
+	}
+
+	if (use_errors == 0) {
+		xmlSetStructuredErrorFunc(NULL, NULL);
+		if (LIBXML(error_list)) {
+			zend_llist_destroy(LIBXML(error_list));
+			efree(LIBXML(error_list));
+			LIBXML(error_list) = NULL;
+		}
+	} else {
+		xmlSetStructuredErrorFunc(NULL, php_libxml_structured_error_handler);
+		if (LIBXML(error_list) == NULL) {
+			LIBXML(error_list) = (zend_llist *) emalloc(sizeof(zend_llist));
+			zend_llist_init(LIBXML(error_list), sizeof(xmlError), (llist_dtor_func_t) _php_libxml_free_error, 0);
+		}
+	}
+	RETURN_BOOL(retval);
+}
+/* }}} */
+
+/* {{{ proto object libxml_get_last_error() 
+   Retrieve last error from libxml */
+static PHP_FUNCTION(libxml_get_last_error)
+{
+	xmlErrorPtr error;
+
+	error = xmlGetLastError();
+	
+	if (error) {
+		object_init_ex(return_value, libxmlerror_class_entry);
+		add_property_long(return_value, "level", error->level);
+		add_property_long(return_value, "code", error->code);
+		add_property_long(return_value, "column", error->int2);
+		if (error->message) {
+			add_property_string(return_value, "message", error->message, 1);
+		} else {
+			add_property_stringl(return_value, "message", "", 0, 1);
+		}
+		if (error->file) {
+			add_property_string(return_value, "file", error->file, 1);
+		} else {
+			add_property_stringl(return_value, "file", "", 0, 1);
+		}
+		add_property_long(return_value, "line", error->line);
+	} else {
+		RETURN_FALSE;
+	}
+}
+/* }}} */
+
+/* {{{ proto object libxml_get_errors()
+   Retrieve array of errors */
+static PHP_FUNCTION(libxml_get_errors)
+{
+	
+	xmlErrorPtr error;
+
+	if (array_init(return_value) == FAILURE) {
+		RETURN_FALSE;
+	}
+
+	if (LIBXML(error_list)) {
+
+		error = zend_llist_get_first(LIBXML(error_list));
+
+		while (error != NULL) {
+			zval *z_error;
+			MAKE_STD_ZVAL(z_error);
+
+			object_init_ex(z_error, libxmlerror_class_entry);
+			add_property_long(z_error, "level", error->level);
+			add_property_long(z_error, "code", error->code);
+			add_property_long(z_error, "column", error->int2);
+			if (error->message) {
+				add_property_string(z_error, "message", error->message, 1);
+			} else {
+				add_property_stringl(z_error, "message", "", 0, 1);
+			}
+			if (error->file) {
+				add_property_string(z_error, "file", error->file, 1);
+			} else {
+				add_property_stringl(z_error, "file", "", 0, 1);
+			}
+			add_property_long(z_error, "line", error->line);
+			add_next_index_zval(return_value, z_error);
+
+			error = zend_llist_get_next(LIBXML(error_list));
+		}
+	}
+}
+/* }}} */
+
+/* {{{ proto void libxml_clear_errors() 
+   Clear last error from libxml */
+static PHP_FUNCTION(libxml_clear_errors)
+{
+	xmlResetLastError();
+	if (LIBXML(error_list)) {
+		zend_llist_clean(LIBXML(error_list));
+	}
+}
+/* }}} */
+
+/* {{{ proto bool libxml_disable_entity_loader([boolean disable]) 
+   Disable/Enable ability to load external entities */
+static PHP_FUNCTION(libxml_disable_entity_loader)
+{
+	zend_bool disable = 1;
+	xmlParserInputBufferCreateFilenameFunc old;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &disable) == FAILURE) {
+		return;
+	}
+
+	if (disable == 0) {
+		old = xmlParserInputBufferCreateFilenameDefault(php_libxml_input_buffer_create_filename);
+	} else {
+		old = xmlParserInputBufferCreateFilenameDefault(php_libxml_input_buffer_noload);
+	}
+
+	if (old == php_libxml_input_buffer_noload) {
+		RETURN_TRUE;
+	}
+
+	RETURN_FALSE;
+}
+/* }}} */
+
 /* {{{ Common functions shared by extensions */
 int php_libxml_xmlCheckUTF8(const unsigned char *s)
 {
@@ -553,7 +892,7 @@ int php_libxml_register_export(zend_class_entry *ce, php_libxml_export_node expo
 	php_libxml_initialize();
 	export_hnd.export_func = export_function;
 
-    return zend_hash_add(&php_libxml_exports, ce->name, ce->name_length + 1, &export_hnd, sizeof(export_hnd), NULL);
+	return zend_hash_add(&php_libxml_exports, ce->name, ce->name_length + 1, &export_hnd, sizeof(export_hnd), NULL);
 }
 
 PHP_LIBXML_API xmlNodePtr php_libxml_import_node(zval *object TSRMLS_DC)
@@ -570,13 +909,11 @@ PHP_LIBXML_API xmlNodePtr php_libxml_import_node(zval *object TSRMLS_DC)
 		if (zend_hash_find(&php_libxml_exports, ce->name, ce->name_length + 1, (void **) &export_hnd)  == SUCCESS) {
 			node = export_hnd->export_func(object TSRMLS_CC);
 		}
-    }
-
+	}
 	return node;
-
 }
 
-int php_libxml_increment_node_ptr(php_libxml_node_object *object, xmlNodePtr node, void *private_data TSRMLS_DC)
+PHP_LIBXML_API int php_libxml_increment_node_ptr(php_libxml_node_object *object, xmlNodePtr node, void *private_data TSRMLS_DC)
 {
 	int ret_refcount = -1;
 
@@ -608,7 +945,8 @@ int php_libxml_increment_node_ptr(php_libxml_node_object *object, xmlNodePtr nod
 	return ret_refcount;
 }
 
-int php_libxml_decrement_node_ptr(php_libxml_node_object *object TSRMLS_DC) {
+PHP_LIBXML_API int php_libxml_decrement_node_ptr(php_libxml_node_object *object TSRMLS_DC)
+{
 	int ret_refcount = -1;
 	php_libxml_node_ptr *obj_node;
 
@@ -627,7 +965,8 @@ int php_libxml_decrement_node_ptr(php_libxml_node_object *object TSRMLS_DC) {
 	return ret_refcount;
 }
 
-int php_libxml_increment_doc_ref(php_libxml_node_object *object, xmlDocPtr docp TSRMLS_DC) {
+PHP_LIBXML_API int php_libxml_increment_doc_ref(php_libxml_node_object *object, xmlDocPtr docp TSRMLS_DC)
+{
 	int ret_refcount = -1;
 
 	if (object->document != NULL) {
@@ -644,7 +983,8 @@ int php_libxml_increment_doc_ref(php_libxml_node_object *object, xmlDocPtr docp
 	return ret_refcount;
 }
 
-int php_libxml_decrement_doc_ref(php_libxml_node_object *object TSRMLS_DC) {
+PHP_LIBXML_API int php_libxml_decrement_doc_ref(php_libxml_node_object *object TSRMLS_DC)
+{
 	int ret_refcount = -1;
 
 	if (object != NULL && object->document != NULL) {
@@ -654,17 +994,21 @@ int php_libxml_decrement_doc_ref(php_libxml_node_object *object TSRMLS_DC) {
 				xmlFreeDoc((xmlDoc *) object->document->ptr);
 			}
 			if (object->document->doc_props != NULL) {
+				if (object->document->doc_props->classmap) {
+					zend_hash_destroy(object->document->doc_props->classmap);
+					FREE_HASHTABLE(object->document->doc_props->classmap);
+				}
 				efree(object->document->doc_props);
 			}
 			efree(object->document);
+			object->document = NULL;
 		}
-		object->document = NULL;
 	}
 
 	return ret_refcount;
 }
 
-void php_libxml_node_free_resource(xmlNodePtr node TSRMLS_DC)
+PHP_LIBXML_API void php_libxml_node_free_resource(xmlNodePtr node TSRMLS_DC)
 {
 	if (!node) {
 		return;
@@ -685,6 +1029,7 @@ void php_libxml_node_free_resource(xmlNodePtr node TSRMLS_DC)
 					case XML_ENTITY_DECL:
 					case XML_ATTRIBUTE_NODE:
 					case XML_NAMESPACE_DECL:
+					case XML_TEXT_NODE:
 						break;
 					default:
 						php_libxml_node_free_list((xmlNodePtr) node->properties TSRMLS_CC);
@@ -699,7 +1044,7 @@ void php_libxml_node_free_resource(xmlNodePtr node TSRMLS_DC)
 	}
 }
 
-void php_libxml_node_decrement_resource(php_libxml_node_object *object TSRMLS_DC)
+PHP_LIBXML_API void php_libxml_node_decrement_resource(php_libxml_node_object *object TSRMLS_DC)
 {
 	int ret_refcount = -1;
 	xmlNodePtr nodep;
@@ -711,13 +1056,26 @@ void php_libxml_node_decrement_resource(php_libxml_node_object *object TSRMLS_DC
 		ret_refcount = php_libxml_decrement_node_ptr(object TSRMLS_CC);
 		if (ret_refcount == 0) {
 			php_libxml_node_free_resource(nodep TSRMLS_CC);
+		} else {
+			if (obj_node && object == obj_node->_private) {
+				obj_node->_private = NULL;
+			}
 		}
+	}
+	if (object != NULL && object->document != NULL) {
 		/* Safe to call as if the resource were freed then doc pointer is NULL */
 		php_libxml_decrement_doc_ref(object TSRMLS_CC);
 	}
 }
 /* }}} */
 
+#ifdef PHP_WIN32
+PHP_LIBXML_API BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
+{
+	return xmlDllMain(hinstDLL, fdwReason, lpvReserved);
+}
+#endif
+
 #endif
 
 /*
