@@ -1,3 +1,23 @@
+/*
+   +----------------------------------------------------------------------+
+   | PHP Version 5                                                        |
+   +----------------------------------------------------------------------+
+   | Copyright (c) 1997-2009 The PHP Group                                |
+   +----------------------------------------------------------------------+
+   | This source file is subject to version 3.01 of the PHP license,      |
+   | that is bundled with this package in the file LICENSE, and is        |
+   | available through the world-wide-web at the following url:           |
+   | http://www.php.net/license/3_01.txt                                  |
+   | If you did not receive a copy of the PHP license and are unable to   |
+   | obtain it through the world-wide-web, please send a note to          |
+   | license@php.net so we can mail you a copy immediately.               |
+   +----------------------------------------------------------------------+
+   | Authors: Sascha Schumann <sascha@schumann.cx>                        |
+   |          Derick Rethans <derick@derickrethans.nl>                    |
+   +----------------------------------------------------------------------+
+ */
+/* $Id: php_mcrypt.h,v 1.26.2.1.2.2.2.4 2008/12/31 11:15:38 sebastian Exp $ */
+
 #ifndef PHP_MCRYPT_H
 #define PHP_MCRYPT_H
 
@@ -7,14 +27,7 @@
 #include "TSRM.h"
 #endif
 
-#if PHP_API_VERSION < 19990421
-#define  zend_module_entry zend_module_entry
-#include "zend_modules.h"
-#include "internal_functions.h"
-#endif
-
 extern zend_module_entry mcrypt_module_entry;
-
 #define mcrypt_module_ptr &mcrypt_module_entry
 
 /* Functions for both old and new API */
@@ -27,7 +40,6 @@ PHP_FUNCTION(mcrypt_get_block_size);
 PHP_FUNCTION(mcrypt_get_key_size);
 PHP_FUNCTION(mcrypt_create_iv);
 
-#if HAVE_LIBMCRYPT24
 /* Support functions for old API */
 PHP_FUNCTION(mcrypt_list_algorithms);
 PHP_FUNCTION(mcrypt_list_modes);
@@ -40,7 +52,8 @@ PHP_FUNCTION(mcrypt_module_open);
 PHP_FUNCTION(mcrypt_generic_init);
 PHP_FUNCTION(mcrypt_generic);
 PHP_FUNCTION(mdecrypt_generic);
-PHP_FUNCTION(mcrypt_generic_end);
+PHP_FUNCTION(mcrypt_generic_deinit);
+
 PHP_FUNCTION(mcrypt_enc_self_test);
 PHP_FUNCTION(mcrypt_enc_is_block_algorithm_mode);
 PHP_FUNCTION(mcrypt_enc_is_block_algorithm);
@@ -67,21 +80,9 @@ ZEND_BEGIN_MODULE_GLOBALS(mcrypt)
 ZEND_END_MODULE_GLOBALS(mcrypt)
 
 #ifdef ZTS
-# define MCLS_D        zend_mcrypt_globals *mcrypt_globals
-# define MCLS_DC       , MCLS_D
-# define MCLS_C        mcrypt_globals
-# define MCLS_CC       , MCLS_C
-# define MCG(v)        (mcrypt_globals->v)
-# define MCLS_FETCH()  zend_mcrypt_globals *mcrypt_globals = ts_resource(mcrypt_globals_id)
+# define MCG(v)    TSRMG(mcrypt_globals_id, zend_mcrypt_globals *, v)
 #else
-# define MCLS_D
-# define MCLS_DC
-# define MCLS_C
-# define MCLS_CC
-# define MCG(v)        (mcrypt_globals.v)
-# define MCLS_FETCH()
-#endif
-
+# define MCG(v)    (mcrypt_globals.v)
 #endif
 
 #else
