@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -25,24 +25,32 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: php_oci8.h,v 1.36.2.2 2006/01/01 12:50:10 sniper Exp $ */
+/* $Id: php_oci8.h,v 1.36.2.2.2.1.2.13 2009/03/16 05:44:49 sixd Exp $ */
 
 #if HAVE_OCI8
 # ifndef PHP_OCI8_H
 #  define PHP_OCI8_H
 
-#ifdef PHP_WIN32
-#define PHP_OCI_API __declspec(dllexport)
-#else
-#define PHP_OCI_API
-#endif                                   
-
 #ifdef ZTS
 # include "TSRM.h"
 #endif
 
+
+/*
+ * The version of the OCI8 extension.
+ */
+#ifdef PHP_OCI8_VERSION
+/* The definition of PHP_OCI8_VERSION changed in PHP 5.3 and building
+ * this code with PHP 5.2 and earlier (e.g. when using OCI8 from PECL)
+ * will conflict.
+ */
+#undef PHP_OCI8_VERSION
+#endif
+#define PHP_OCI8_VERSION "1.3.5"
+
 extern zend_module_entry oci8_module_entry;
 #define phpext_oci8_ptr &oci8_module_entry
+#define phpext_oci8_11g_ptr &oci8_module_entry
 
 
 PHP_MINIT_FUNCTION(oci);
@@ -64,5 +72,3 @@ PHP_MINFO_FUNCTION(oci);
  * c-basic-offset: 4
  * End:
  */
-
-
