@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: birdstep.c,v 1.13.2.2 2006/01/01 12:50:10 sniper Exp $ */
+/* $Id: birdstep.c,v 1.13.2.2.2.2.2.9 2009/06/06 02:40:48 mattwil Exp $ */
 
 /*
  * TODO:
@@ -47,38 +47,94 @@
 #ifdef HAVE_BIRDSTEP
 #include "php_birdstep.h"
 #include "ext/standard/info.h"
+#include "php_ini.h"
 
-zend_function_entry birdstep_functions[] = {
-	PHP_FE(birdstep_connect,        NULL)
-	PHP_FE(birdstep_close,          NULL)
-	PHP_FE(birdstep_exec,           NULL)
-	PHP_FE(birdstep_fetch,          NULL)
-	PHP_FE(birdstep_result,         NULL)
-	PHP_FE(birdstep_freeresult,     NULL)
-	PHP_FE(birdstep_autocommit,     NULL)
-	PHP_FE(birdstep_off_autocommit, NULL)
-	PHP_FE(birdstep_commit,         NULL)
-	PHP_FE(birdstep_rollback,       NULL)
-	PHP_FE(birdstep_fieldnum,       NULL)
-	PHP_FE(birdstep_fieldname,      NULL)
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO_EX(arginfo_birdstep_connect, 0, 0, 3)
+	ZEND_ARG_INFO(0, server)
+	ZEND_ARG_INFO(0, user)
+	ZEND_ARG_INFO(0, pass)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_birdstep_close, 0, 0, 1)
+	ZEND_ARG_INFO(0, id)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_birdstep_exec, 0, 0, 2)
+	ZEND_ARG_INFO(0, index)
+	ZEND_ARG_INFO(0, exec_str)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_birdstep_fetch, 0, 0, 1)
+	ZEND_ARG_INFO(0, index)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_birdstep_result, 0, 0, 2)
+	ZEND_ARG_INFO(0, index)
+	ZEND_ARG_INFO(0, col)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_birdstep_freeresult, 0, 0, 1)
+	ZEND_ARG_INFO(0, index)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_birdstep_autocommit, 0, 0, 1)
+	ZEND_ARG_INFO(0, index)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_birdstep_off_autocommit, 0, 0, 1)
+	ZEND_ARG_INFO(0, index)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_birdstep_commit, 0, 0, 1)
+	ZEND_ARG_INFO(0, index)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_birdstep_rollback, 0, 0, 1)
+	ZEND_ARG_INFO(0, index)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_birdstep_fieldname, 0, 0, 2)
+	ZEND_ARG_INFO(0, index)
+	ZEND_ARG_INFO(0, col)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_birdstep_fieldnum, 0, 0, 1)
+	ZEND_ARG_INFO(0, index)
+ZEND_END_ARG_INFO()
+/* }}} */
+
+const zend_function_entry birdstep_functions[] = {
+	PHP_FE(birdstep_connect,        arginfo_birdstep_connect)
+	PHP_FE(birdstep_close,          arginfo_birdstep_close)
+	PHP_FE(birdstep_exec,           arginfo_birdstep_exec)
+	PHP_FE(birdstep_fetch,          arginfo_birdstep_fetch)
+	PHP_FE(birdstep_result,         arginfo_birdstep_result)
+	PHP_FE(birdstep_freeresult,     arginfo_birdstep_freeresult)
+	PHP_FE(birdstep_autocommit,     arginfo_birdstep_autocommit)
+	PHP_FE(birdstep_off_autocommit, arginfo_birdstep_off_autocommit)
+	PHP_FE(birdstep_commit,         arginfo_birdstep_commit)
+	PHP_FE(birdstep_rollback,       arginfo_birdstep_rollback)
+	PHP_FE(birdstep_fieldnum,       arginfo_birdstep_fieldnum)
+	PHP_FE(birdstep_fieldname,      arginfo_birdstep_fieldname)
 /*
  * Temporary Function aliases until the next major upgrade to PHP.  
  * These should allow users to continue to use their current scripts, 
  * but should in reality warn the user that this functionality is 
  * deprecated.
  */
-	PHP_FALIAS(velocis_connect,        birdstep_connect,        NULL)
-	PHP_FALIAS(velocis_close,          birdstep_close,          NULL)
-	PHP_FALIAS(velocis_exec,           birdstep_exec,           NULL)
-	PHP_FALIAS(velocis_fetch,          birdstep_fetch,          NULL)
-	PHP_FALIAS(velocis_result,         birdstep_result,         NULL)
-	PHP_FALIAS(velocis_freeresult,     birdstep_freeresult,     NULL)
-	PHP_FALIAS(velocis_autocommit,     birdstep_autocommit,     NULL)
-	PHP_FALIAS(velocis_off_autocommit, birdstep_off_autocommit, NULL)
-	PHP_FALIAS(velocis_commit,         birdstep_commit,         NULL)
-	PHP_FALIAS(velocis_rollback,       birdstep_rollback,       NULL)
-	PHP_FALIAS(velocis_fieldnum,       birdstep_fieldnum,       NULL)
-	PHP_FALIAS(velocis_fieldname,      birdstep_fieldname,      NULL)
+	PHP_FALIAS(velocis_connect,        birdstep_connect,        arginfo_birdstep_connect)
+	PHP_FALIAS(velocis_close,          birdstep_close,          arginfo_birdstep_close)
+	PHP_FALIAS(velocis_exec,           birdstep_exec,           arginfo_birdstep_exec)
+	PHP_FALIAS(velocis_fetch,          birdstep_fetch,          arginfo_birdstep_fetch)
+	PHP_FALIAS(velocis_result,         birdstep_result,         arginfo_birdstep_result)
+	PHP_FALIAS(velocis_freeresult,     birdstep_freeresult,     arginfo_birdstep_freeresult)
+	PHP_FALIAS(velocis_autocommit,     birdstep_autocommit,     arginfo_birdstep_autocommit)
+	PHP_FALIAS(velocis_off_autocommit, birdstep_off_autocommit, arginfo_birdstep_off_autocommit)
+	PHP_FALIAS(velocis_commit,         birdstep_commit,         arginfo_birdstep_commit)
+	PHP_FALIAS(velocis_rollback,       birdstep_rollback,       arginfo_birdstep_rollback)
+	PHP_FALIAS(velocis_fieldnum,       birdstep_fieldnum,       arginfo_birdstep_fieldnum)
+	PHP_FALIAS(velocis_fieldname,      birdstep_fieldname,      arginfo_birdstep_fieldname)
 /* End temporary aliases */
 	{NULL, NULL, NULL}
 };
@@ -103,8 +159,8 @@ ZEND_GET_MODULE(birdstep)
 THREAD_LS birdstep_module php_birdstep_module;
 THREAD_LS static HENV henv;
 
-#define PHP_GET_BIRDSTEP_RES_IDX(id) convert_to_long_ex(id); if (!(res = birdstep_find_result(list, Z_LVAL_PP(id)))) { php_error_docref(NULL TSRMLS_CC, E_WARNING, "Birdstep: Not result index (%ld)", Z_LVAL_PP(id)); RETURN_FALSE; } 
-#define PHP_BIRDSTEP_CHK_LNK(id) convert_to_long_ex(id); if (!(conn = birdstep_find_conn(list,Z_LVAL_PP(id)))) { php_error_docref(NULL TSRMLS_CC, E_WARNING, "Birdstep: Not connection index (%ld)", Z_LVAL_PP(id)); RETURN_FALSE; }
+#define PHP_GET_BIRDSTEP_RES_IDX(id) if (!(res = birdstep_find_result(list, id))) { php_error_docref(NULL TSRMLS_CC, E_WARNING, "Birdstep: Not result index (%ld)", id); RETURN_FALSE; } 
+#define PHP_BIRDSTEP_CHK_LNK(id) if (!(conn = birdstep_find_conn(list, id))) { php_error_docref(NULL TSRMLS_CC, E_WARNING, "Birdstep: Not connection index (%ld)", id); RETURN_FALSE; }
                                                         
 
 static void _close_birdstep_link(zend_rsrc_list_entry *rsrc TSRMLS_DC)
@@ -230,36 +286,30 @@ static void birdstep_del_result(HashTable *list,int ind)
  */
 PHP_FUNCTION(birdstep_connect)
 {
-	zval **serv,**user,**pass;
-	char *Serv = NULL;
-	char *User = NULL;
-	char *Pass = NULL;
+	char *serv, *user, *pass;
+	int serv_len, user_len, pass_len;
 	RETCODE stat;
 	HDBC hdbc;
 	VConn *new;
 	long ind;
 
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sss", &serv, &serv_len, &user, &user_len, &pass, &pass_len) == FAILURE) {
+		return;
+	}
+	
 	if ( php_birdstep_module.max_links != -1 && php_birdstep_module.num_links == php_birdstep_module.max_links ) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Birdstep: Too many open connections (%d)",php_birdstep_module.num_links);
 		RETURN_FALSE;
 	}
-	if (ZEND_NUM_ARGS() != 3 || zend_get_parameters_ex(3, &serv, &user, &pass) == FAILURE ) {
-		WRONG_PARAM_COUNT;
-	}
-	convert_to_string_ex(serv);
-	convert_to_string_ex(user);
-	convert_to_string_ex(pass);
-	Serv = Z_STRVAL_PP(serv);
-	User = Z_STRVAL_PP(user);
-	Pass = Z_STRVAL_PP(pass);
+
 	stat = SQLAllocConnect(henv,&hdbc);
 	if ( stat != SQL_SUCCESS ) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Birdstep: Could not allocate connection handle");
 		RETURN_FALSE;
 	}
-	stat = SQLConnect(hdbc,Serv,SQL_NTS,User,SQL_NTS,Pass,SQL_NTS);
+	stat = SQLConnect(hdbc, serv, SQL_NTS, user, SQL_NTS, pass, SQL_NTS);
 	if ( stat != SQL_SUCCESS && stat != SQL_SUCCESS_WITH_INFO ) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Birdstep: Could not connect to server \"%s\" for %s",Serv,User);
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Birdstep: Could not connect to server \"%s\" for %s", serv, user);
 		SQLFreeConnect(hdbc);
 		RETURN_FALSE;
 	}
@@ -274,17 +324,18 @@ PHP_FUNCTION(birdstep_connect)
  */
 PHP_FUNCTION(birdstep_close)
 {
-	zval **id;
+	long id;
 	VConn *conn;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &id) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &id) == FAILURE) {
+		return;
 	}
+
 	PHP_BIRDSTEP_CHK_LNK(id);
 
 	SQLDisconnect(conn->hdbc);
 	SQLFreeConnect(conn->hdbc);
-	birdstep_del_conn(list,Z_LVAL_PP(id));
+	birdstep_del_conn(list, id);
 	php_birdstep_module.num_links--;
 	RETURN_TRUE;
 }
@@ -294,22 +345,20 @@ PHP_FUNCTION(birdstep_close)
  */
 PHP_FUNCTION(birdstep_exec)
 {
-	zval **ind, **exec_str;
-	char *query = NULL;
-	int indx;
+	char *query;
+	long ind;
+	int query_len, indx;
 	VConn *conn;
 	Vresult *res;
 	RETCODE stat;
 	SWORD cols,i,colnamelen;
 	SDWORD rows,coldesc;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &ind, &exec_str) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ls", &ind, &query, &query_len) == FAILURE) {
+		return;
 	}
-	PHP_BIRDSTEP_CHK_LNK(ind);
 
-	convert_to_string_ex(exec_str);
-	query = Z_STRVAL_PP(exec_str);
+	PHP_BIRDSTEP_CHK_LNK(ind);
 
 	res = (Vresult *)emalloc(sizeof(Vresult));
 	stat = SQLAllocStmt(conn->hdbc,&res->hstmt);
@@ -320,7 +369,7 @@ PHP_FUNCTION(birdstep_exec)
 	}
 	stat = SQLExecDirect(res->hstmt,query,SQL_NTS);
 	if ( stat != SQL_SUCCESS && stat != SQL_SUCCESS_WITH_INFO ) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Birdstep: Can not execute \"%s\" query",query);
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Birdstep: Cannot execute \"%s\" query",query);
 		SQLFreeStmt(res->hstmt,SQL_DROP);
 		efree(res);
 		RETURN_FALSE;
@@ -377,15 +426,16 @@ PHP_FUNCTION(birdstep_exec)
  */
 PHP_FUNCTION(birdstep_fetch)
 {
-	zval **ind;
+	long ind;
 	Vresult *res;
 	RETCODE stat;
 	UDWORD  row;
 	UWORD   RowStat[1];
 
-	if ( ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &ind) == FAILURE ) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &ind) == FAILURE) {
+		return;
 	}
+
 	PHP_GET_BIRDSTEP_RES_IDX(ind);
 
 	stat = SQLExtendedFetch(res->hstmt,SQL_FETCH_NEXT,1,&row,RowStat);
@@ -405,11 +455,12 @@ PHP_FUNCTION(birdstep_fetch)
 }
 /* }}} */
 
-/* {{{ proto mixed birdstep_result(int index, int col)
+/* {{{ proto mixed birdstep_result(int index, mixed col)
  */
 PHP_FUNCTION(birdstep_result)
 {
-	zval **ind, **col;
+	zval **col;
+	long ind;
 	Vresult *res;
 	RETCODE stat;
 	int i,sql_c_type;
@@ -418,9 +469,10 @@ PHP_FUNCTION(birdstep_result)
 	SWORD indx = -1;
 	char *field = NULL;
 
-	if ( ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &ind, &col) == FAILURE ) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lZ", &ind, &col) == FAILURE) {
+		return;
 	}
+
 	PHP_GET_BIRDSTEP_RES_IDX(ind);
 
 	if ( Z_TYPE_PP(col) == IS_STRING ) {
@@ -501,16 +553,17 @@ l1:
  */
 PHP_FUNCTION(birdstep_freeresult)
 {
-	zval **ind;
+	long ind;
 	Vresult *res;
 
-	if ( ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &ind) == FAILURE ) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &ind) == FAILURE) {
+		return;
 	}
+
 	PHP_GET_BIRDSTEP_RES_IDX(ind);
 
 	SQLFreeStmt(res->hstmt,SQL_DROP);
-	birdstep_del_result(list,Z_LVAL_PP(ind));
+	birdstep_del_result(list, ind);
 	RETURN_TRUE;
 }
 /* }}} */
@@ -519,13 +572,14 @@ PHP_FUNCTION(birdstep_freeresult)
  */
 PHP_FUNCTION(birdstep_autocommit)
 {
-	zval **id;
+	long id;
 	RETCODE stat;
 	VConn *conn;
 
-	if ( ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &id) == FAILURE ) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &id) == FAILURE) {
+		return;
 	}
+
 	PHP_BIRDSTEP_CHK_LNK(id);
 
 	stat = SQLSetConnectOption(conn->hdbc,SQL_AUTOCOMMIT,SQL_AUTOCOMMIT_ON);
@@ -541,13 +595,14 @@ PHP_FUNCTION(birdstep_autocommit)
  */
 PHP_FUNCTION(birdstep_off_autocommit)
 {
-	zval **id;
+	long id;
 	RETCODE stat;
 	VConn *conn;
 
-	if ( ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &id) == FAILURE ) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &id) == FAILURE) {
+		return;
 	}
+
 	PHP_BIRDSTEP_CHK_LNK(id);
 
 	stat = SQLSetConnectOption(conn->hdbc,SQL_AUTOCOMMIT,SQL_AUTOCOMMIT_OFF);
@@ -563,13 +618,14 @@ PHP_FUNCTION(birdstep_off_autocommit)
  */
 PHP_FUNCTION(birdstep_commit)
 {
-	zval **id;
+	long id;
 	RETCODE stat;
 	VConn *conn;
 
-	if ( ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &id) == FAILURE ) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &id) == FAILURE) {
+		return;
 	}
+
 	PHP_BIRDSTEP_CHK_LNK(id)
 
 	stat = SQLTransact(NULL,conn->hdbc,SQL_COMMIT);
@@ -585,13 +641,14 @@ PHP_FUNCTION(birdstep_commit)
  */
 PHP_FUNCTION(birdstep_rollback)
 {
-	zval **id;
+	long id;
 	RETCODE stat;
 	VConn *conn;
 
-	if ( ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &id) == FAILURE ) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &id) == FAILURE) {
+		return;
 	}
+
 	PHP_BIRDSTEP_CHK_LNK(id);
 
 	stat = SQLTransact(NULL,conn->hdbc,SQL_ROLLBACK);
@@ -607,17 +664,17 @@ PHP_FUNCTION(birdstep_rollback)
  */
 PHP_FUNCTION(birdstep_fieldname)
 {
-	zval **ind, **col;
+	long ind, col;
 	Vresult *res;
 	SWORD indx;
 
-	if ( ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &ind, &col) == FAILURE ) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &ind, &col) == FAILURE) {
+		return;
 	}
+
 	PHP_GET_BIRDSTEP_RES_IDX(ind);
 
-	convert_to_long_ex(col);
-	indx = Z_LVAL_PP(col);
+	indx = col;
 	if ( indx < 0 || indx >= res->numcols ) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Birdstep: Field index not in range");
 		RETURN_FALSE;
@@ -630,12 +687,13 @@ PHP_FUNCTION(birdstep_fieldname)
  */
 PHP_FUNCTION(birdstep_fieldnum)
 {
-	zval **ind;
+	long ind;
 	Vresult *res;
 
-	if ( ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &ind) == FAILURE ) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &ind) == FAILURE) {
+		return;
 	}
+
 	PHP_GET_BIRDSTEP_RES_IDX(ind);
 
 	RETURN_LONG(res->numcols);
