@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2007 The PHP Group                                |
+  | Copyright (c) 1997-2009 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: pdo_oci.c,v 1.5.2.6.2.1 2007/01/01 09:36:05 sebastian Exp $ */
+/* $Id: pdo_oci.c,v 1.5.2.6.2.1.2.3 2008/12/31 11:15:41 sebastian Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -31,7 +31,7 @@
 #include "php_pdo_oci_int.h"
 
 /* {{{ pdo_oci_functions[] */
-zend_function_entry pdo_oci_functions[] = {
+const zend_function_entry pdo_oci_functions[] = {
 	{NULL, NULL, NULL}
 };
 /* }}} */
@@ -39,7 +39,7 @@ zend_function_entry pdo_oci_functions[] = {
 /* {{{ pdo_oci_module_entry */
 
 #if ZEND_MODULE_API_NO >= 20050922
-static zend_module_dep pdo_oci_deps[] = {
+static const zend_module_dep pdo_oci_deps[] = {
 	ZEND_MOD_REQUIRED("pdo")
 	{NULL, NULL, NULL}
 };
