@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2008 The PHP Group                                |
+  | Copyright (c) 1997-2009 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.0 of the PHP license,       |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: php_pdo_odbc_int.h,v 1.9.2.1.2.4 2008/11/30 23:56:36 pajoye Exp $ */
+/* $Id: php_pdo_odbc_int.h,v 1.9.2.1.2.2.2.3 2008/12/31 11:15:41 sebastian Exp $ */
 
 #ifdef PHP_WIN32
 # define PDO_ODBC_TYPE	"Win32"
@@ -129,6 +129,8 @@ typedef struct {
 	PDO_ODBC_HENV	env;
 	PDO_ODBC_HDBC	dbc;
 	pdo_odbc_errinfo einfo;
+	unsigned assume_utf8:1;
+	unsigned _spare:31;
 } pdo_odbc_db_handle;
 
 typedef struct {
@@ -138,6 +140,8 @@ typedef struct {
 	SWORD	coltype;
 	char colname[128];
 	unsigned is_long;
+	unsigned is_unicode:1;
+	unsigned _spare:31;
 } pdo_odbc_column;
 
 typedef struct {
@@ -145,14 +149,19 @@ typedef struct {
 	pdo_odbc_column *cols;
 	pdo_odbc_db_handle *H;
 	pdo_odbc_errinfo einfo;
+	char *convbuf;
+	unsigned long convbufsize;
 	unsigned going_long:1;
-	unsigned _spare:31;
+	unsigned assume_utf8:1;
+	unsigned _spare:30;
 } pdo_odbc_stmt;
 
 typedef struct {
 	SQLINTEGER len;
 	SQLSMALLINT paramtype;
 	char *outbuf;
+	unsigned is_unicode:1;
+	unsigned _spare:31;
 } pdo_odbc_param;
 	
 extern pdo_driver_t pdo_odbc_driver;
@@ -173,6 +182,7 @@ extern SQLUINTEGER pdo_odbc_pool_mode;
 
 enum {
 	PDO_ODBC_ATTR_USE_CURSOR_LIBRARY = PDO_ATTR_DRIVER_SPECIFIC,
+	PDO_ODBC_ATTR_ASSUME_UTF8 /* assume that input strings are UTF-8 when feeding data to unicode columns */
 };
 
 /*
