@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: posix.c,v 1.70.2.3.2.12 2007/01/12 01:46:11 iliaa Exp $ */
+/* $Id: posix.c,v 1.70.2.3.2.16.2.15 2009/06/06 02:40:48 mattwil Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -36,6 +36,11 @@
 #endif
 
 #include <sys/resource.h>
+
+#if defined(_GNU_SOURCE) && !defined(__USE_GNU)
+# define __USE_GNU
+#endif
+
 #include <sys/utsname.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -51,91 +56,249 @@
 ZEND_DECLARE_MODULE_GLOBALS(posix)
 static PHP_MINFO_FUNCTION(posix);
 
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_kill, 0, 0, 2)
+	ZEND_ARG_INFO(0, pid)
+	ZEND_ARG_INFO(0, sig)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_posix_getpid, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_posix_getppid, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_posix_getuid, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_setuid, 0, 0, 1)
+	ZEND_ARG_INFO(0, uid)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_posix_geteuid, 0)
+ZEND_END_ARG_INFO()
+
+#ifdef HAVE_SETEUID
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_seteuid, 0, 0, 1)
+	ZEND_ARG_INFO(0, uid)
+ZEND_END_ARG_INFO()
+#endif
+
+ZEND_BEGIN_ARG_INFO(arginfo_posix_getgid, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_setgid, 0, 0, 1)
+	ZEND_ARG_INFO(0, gid)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_posix_getegid, 0)
+ZEND_END_ARG_INFO()
+
+#ifdef HAVE_SETEGID
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_setegid, 0, 0, 1)
+	ZEND_ARG_INFO(0, gid)
+ZEND_END_ARG_INFO()
+#endif
+
+#ifdef HAVE_GETGROUPS
+ZEND_BEGIN_ARG_INFO(arginfo_posix_getgroups, 0)
+ZEND_END_ARG_INFO()
+#endif
+
+#ifdef HAVE_GETLOGIN
+ZEND_BEGIN_ARG_INFO(arginfo_posix_getlogin, 0)
+ZEND_END_ARG_INFO()
+#endif
+
+ZEND_BEGIN_ARG_INFO(arginfo_posix_getpgrp, 0)
+ZEND_END_ARG_INFO()
+
+#ifdef HAVE_SETSID
+ZEND_BEGIN_ARG_INFO(arginfo_posix_setsid, 0)
+ZEND_END_ARG_INFO()
+#endif
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_setpgid, 0, 0, 2)
+	ZEND_ARG_INFO(0, pid)
+	ZEND_ARG_INFO(0, pgid)
+ZEND_END_ARG_INFO()
+
+#ifdef HAVE_GETPGID
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_getpgid, 0, 0, 1)
+	ZEND_ARG_INFO(0, pid)
+ZEND_END_ARG_INFO()
+#endif
+
+#ifdef HAVE_GETSID
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_getsid, 0, 0, 1)
+	ZEND_ARG_INFO(0, pid)
+ZEND_END_ARG_INFO()
+#endif
+
+ZEND_BEGIN_ARG_INFO(arginfo_posix_uname, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_posix_times, 0)
+ZEND_END_ARG_INFO()
+
+#ifdef HAVE_CTERMID
+ZEND_BEGIN_ARG_INFO(arginfo_posix_ctermid, 0)
+ZEND_END_ARG_INFO()
+#endif
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_ttyname, 0, 0, 1)
+	ZEND_ARG_INFO(0, fd)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_isatty, 0, 0, 1)
+	ZEND_ARG_INFO(0, fd)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_posix_getcwd, 0)
+ZEND_END_ARG_INFO()
+
+#ifdef HAVE_MKFIFO
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_mkfifo, 0, 0, 2)
+	ZEND_ARG_INFO(0, pathname)
+	ZEND_ARG_INFO(0, mode)
+ZEND_END_ARG_INFO()
+#endif
+
+#ifdef HAVE_MKNOD
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_mknod, 0, 0, 2)
+	ZEND_ARG_INFO(0, pathname)
+	ZEND_ARG_INFO(0, mode)
+	ZEND_ARG_INFO(0, major)
+	ZEND_ARG_INFO(0, minor)
+ZEND_END_ARG_INFO()
+#endif
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_access, 0, 0, 1)
+	ZEND_ARG_INFO(0, file)
+	ZEND_ARG_INFO(0, mode)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_getgrnam, 0, 0, 1)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_getgrgid, 0, 0, 1)
+	ZEND_ARG_INFO(0, gid)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_getpwnam, 0, 0, 1)
+	ZEND_ARG_INFO(0, username)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_getpwuid, 0, 0, 1)
+	ZEND_ARG_INFO(0, uid)
+ZEND_END_ARG_INFO()
+
+#ifdef HAVE_GETRLIMIT
+ZEND_BEGIN_ARG_INFO(arginfo_posix_getrlimit, 0)
+ZEND_END_ARG_INFO()
+#endif
+
+ZEND_BEGIN_ARG_INFO(arginfo_posix_get_last_error, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_strerror, 0, 0, 1)
+	ZEND_ARG_INFO(0, errno)
+ZEND_END_ARG_INFO()
+
+#ifdef HAVE_INITGROUPS
+ZEND_BEGIN_ARG_INFO_EX(arginfo_posix_initgroups, 0, 0, 2)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, base_group_id)
+ZEND_END_ARG_INFO()
+#endif
+/* }}} */
+
 /* {{{ posix_functions[]
  */
-zend_function_entry posix_functions[] = {
+const zend_function_entry posix_functions[] = {
     /* POSIX.1, 3.3 */
-	PHP_FE(posix_kill,		NULL)
+	PHP_FE(posix_kill,		arginfo_posix_kill)
 
 	/* POSIX.1, 4.1 */
-	PHP_FE(posix_getpid,	NULL)
-	PHP_FE(posix_getppid,	NULL)
+	PHP_FE(posix_getpid,	arginfo_posix_getpid)
+	PHP_FE(posix_getppid,	arginfo_posix_getppid)
 
 	/* POSIX.1,  4.2 */
-	PHP_FE(posix_getuid,	NULL)
-	PHP_FE(posix_setuid,	NULL)
-	PHP_FE(posix_geteuid,	NULL)
+	PHP_FE(posix_getuid,	arginfo_posix_getuid)
+	PHP_FE(posix_setuid,	arginfo_posix_setuid)
+	PHP_FE(posix_geteuid,	arginfo_posix_geteuid)
 #ifdef HAVE_SETEUID
-	PHP_FE(posix_seteuid,	NULL)
+	PHP_FE(posix_seteuid,	arginfo_posix_seteuid)
 #endif
-	PHP_FE(posix_getgid,	NULL)
-	PHP_FE(posix_setgid,	NULL)
-	PHP_FE(posix_getegid,	NULL)
+	PHP_FE(posix_getgid,	arginfo_posix_getgid)
+	PHP_FE(posix_setgid,	arginfo_posix_setgid)
+	PHP_FE(posix_getegid,	arginfo_posix_getegid)
 #ifdef HAVE_SETEGID
-	PHP_FE(posix_setegid,	NULL)
+	PHP_FE(posix_setegid,	arginfo_posix_setegid)
 #endif
 #ifdef HAVE_GETGROUPS
-	PHP_FE(posix_getgroups,	NULL)
+	PHP_FE(posix_getgroups,	arginfo_posix_getgroups)
 #endif
 #ifdef HAVE_GETLOGIN
-	PHP_FE(posix_getlogin,	NULL)
+	PHP_FE(posix_getlogin,	arginfo_posix_getlogin)
 #endif
 
 	/* POSIX.1, 4.3 */
-	PHP_FE(posix_getpgrp,	NULL)
+	PHP_FE(posix_getpgrp,	arginfo_posix_getpgrp)
 #ifdef HAVE_SETSID
-	PHP_FE(posix_setsid,	NULL)
+	PHP_FE(posix_setsid,	arginfo_posix_setsid)
 #endif
-	PHP_FE(posix_setpgid,	NULL)
+	PHP_FE(posix_setpgid,	arginfo_posix_setpgid)
 	/* Non-Posix functions which are common */
 #ifdef HAVE_GETPGID
-	PHP_FE(posix_getpgid,	NULL)
+	PHP_FE(posix_getpgid,	arginfo_posix_getpgid)
 #endif /* HAVE_GETPGID */
 #ifdef HAVE_GETSID
-	PHP_FE(posix_getsid,	NULL)
+	PHP_FE(posix_getsid,	arginfo_posix_getsid)
 #endif /* HAVE_GETSID */
 
 	/* POSIX.1, 4.4 */
-	PHP_FE(posix_uname,		NULL)
+	PHP_FE(posix_uname,		arginfo_posix_uname)
 
 	/* POSIX.1, 4.5 */
-	PHP_FE(posix_times,		NULL)
+	PHP_FE(posix_times,		arginfo_posix_times)
 
 	/* POSIX.1, 4.7 */
 #ifdef HAVE_CTERMID
-	PHP_FE(posix_ctermid,	NULL)
+	PHP_FE(posix_ctermid,	arginfo_posix_ctermid)
 #endif
-	PHP_FE(posix_ttyname,	NULL)
-	PHP_FE(posix_isatty,	NULL)
+	PHP_FE(posix_ttyname,	arginfo_posix_ttyname)
+	PHP_FE(posix_isatty,	arginfo_posix_isatty)
 
     /* POSIX.1, 5.2 */
-	PHP_FE(posix_getcwd,	NULL)
+	PHP_FE(posix_getcwd,	arginfo_posix_getcwd)
 
 	/* POSIX.1, 5.4 */
 #ifdef HAVE_MKFIFO
-	PHP_FE(posix_mkfifo,	NULL)
+	PHP_FE(posix_mkfifo,	arginfo_posix_mkfifo)
 #endif
 #ifdef HAVE_MKNOD
-	PHP_FE(posix_mknod,	NULL)
+	PHP_FE(posix_mknod,		arginfo_posix_mknod)
 #endif
 
 	/* POSIX.1, 5.6 */
-	PHP_FE(posix_access,	NULL)
+	PHP_FE(posix_access,	arginfo_posix_access)
 	/* POSIX.1, 9.2 */
-	PHP_FE(posix_getgrnam,	NULL)
-	PHP_FE(posix_getgrgid,	NULL)
-	PHP_FE(posix_getpwnam,	NULL)
-	PHP_FE(posix_getpwuid,	NULL)
+	PHP_FE(posix_getgrnam,	arginfo_posix_getgrnam)
+	PHP_FE(posix_getgrgid,	arginfo_posix_getgrgid)
+	PHP_FE(posix_getpwnam,	arginfo_posix_getpwnam)
+	PHP_FE(posix_getpwuid,	arginfo_posix_getpwuid)
 
 #ifdef HAVE_GETRLIMIT
-	PHP_FE(posix_getrlimit,	NULL)
+	PHP_FE(posix_getrlimit,	arginfo_posix_getrlimit)
 #endif
 
-	PHP_FE(posix_get_last_error,					NULL)
+	PHP_FE(posix_get_last_error,					arginfo_posix_get_last_error)
 	PHP_FALIAS(posix_errno, posix_get_last_error,	NULL)
-	PHP_FE(posix_strerror,							NULL)
+	PHP_FE(posix_strerror,							arginfo_posix_strerror)
 #ifdef HAVE_INITGROUPS
-	PHP_FE(posix_initgroups,	NULL)
+	PHP_FE(posix_initgroups,	arginfo_posix_initgroups)
 #endif
 
 	{NULL, NULL, NULL}
@@ -147,15 +310,16 @@ zend_function_entry posix_functions[] = {
 static PHP_MINFO_FUNCTION(posix)
 {
 	php_info_print_table_start();
-	php_info_print_table_row(2, "Revision", "$Revision: 1.70.2.3.2.12 $");
+	php_info_print_table_row(2, "Revision", "$Revision: 1.70.2.3.2.16.2.15 $");
 	php_info_print_table_end();
 }
 /* }}} */
 
-static PHP_GINIT_FUNCTION(posix)
+static PHP_GINIT_FUNCTION(posix) /* {{{ */
 {
 	posix_globals->last_error = 0;
 }
+/* }}} */
 
 /* {{{ PHP_MINIT_FUNCTION(posix)
  */
@@ -209,7 +373,7 @@ zend_module_entry posix_module_entry = {
 ZEND_GET_MODULE(posix)
 #endif
 
-#define PHP_POSIX_NO_ARGS	if (ZEND_NUM_ARGS()) return;
+#define PHP_POSIX_NO_ARGS	if (zend_parse_parameters_none() == FAILURE) return;
 
 #define PHP_POSIX_RETURN_LONG_FUNC(func_name)	\
 	PHP_POSIX_NO_ARGS	\
@@ -467,7 +631,8 @@ PHP_FUNCTION(posix_uname)
 	add_assoc_string(return_value, "release",  u.release,  1);
 	add_assoc_string(return_value, "version",  u.version,  1);
 	add_assoc_string(return_value, "machine",  u.machine,  1);
-#ifdef _GNU_SOURCE
+
+#if defined(_GNU_SOURCE) && !defined(DARWIN) && defined(HAVE_UTSNAME_DOMAINNAME)
 	add_assoc_string(return_value, "domainname", u.domainname, 1);
 #endif
 }
@@ -528,7 +693,7 @@ PHP_FUNCTION(posix_ctermid)
 /* }}} */
 
 /* Checks if the provides resource is a stream and if it provides a file descriptor */
-static int php_posix_stream_get_fd(zval *zfp, int *fd TSRMLS_DC)
+static int php_posix_stream_get_fd(zval *zfp, int *fd TSRMLS_DC) /* {{{ */
 {
 	php_stream *stream;
 
@@ -547,6 +712,7 @@ static int php_posix_stream_get_fd(zval *zfp, int *fd TSRMLS_DC)
 	}
 	return 1;
 }
+/* }}} */
 
 /* {{{ proto string posix_ttyname(int fd)
    Determine terminal device name (POSIX.1, 4.7.2) */
@@ -726,7 +892,7 @@ PHP_FUNCTION(posix_mknod)
 #if defined(HAVE_MAKEDEV) || defined(makedev)
 			php_dev = makedev(major, minor);
 #else
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Can not create a block or character device, creating a normal file instead");
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create a block or character device, creating a normal file instead");
 #endif
 		}
 	}
@@ -744,7 +910,8 @@ PHP_FUNCTION(posix_mknod)
 
 /* Takes a pointer to posix group and a pointer to an already initialized ZVAL
  * array container and fills the array with the posix group member data. */
-int php_posix_group_to_array(struct group *g, zval *array_group) {
+int php_posix_group_to_array(struct group *g, zval *array_group) /* {{{ */
+{
 	zval *array_members;
 	int count;
 
@@ -766,6 +933,7 @@ int php_posix_group_to_array(struct group *g, zval *array_group) {
 	add_assoc_long(array_group, "gid", g->gr_gid);
 	return 1;
 }
+/* }}} */
 
 /*
 	POSIX.1, 5.5.1 unlink()
@@ -872,10 +1040,10 @@ PHP_FUNCTION(posix_getgrnam)
 PHP_FUNCTION(posix_getgrgid)
 {
 	long gid;
-#ifdef HAVE_GETGRGID_R
+#if defined(ZTS) && defined(HAVE_GETGRGID_R) && defined(_SC_GETGR_R_SIZE_MAX)
 	int ret;
 	struct group _g;
-	struct group *retgrptr;
+	struct group *retgrptr = NULL;
 	long grbuflen;
 	char *grbuf;
 #endif
@@ -884,13 +1052,17 @@ PHP_FUNCTION(posix_getgrgid)
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &gid) == FAILURE) {
 		RETURN_FALSE;
 	}
-#ifdef HAVE_GETGRGID_R
+#if defined(ZTS) && defined(HAVE_GETGRGID_R) && defined(_SC_GETGR_R_SIZE_MAX)
 	
 	grbuflen = sysconf(_SC_GETGR_R_SIZE_MAX);
+	if (grbuflen < 1) {
+		RETURN_FALSE;
+	}
+
 	grbuf = emalloc(grbuflen);
 
 	ret = getgrgid_r(gid, &_g, grbuf, grbuflen, &retgrptr);
-	if (ret) {
+	if (ret || retgrptr == NULL) {
 		POSIX_G(last_error) = ret;
 		efree(grbuf);
 		RETURN_FALSE;
@@ -909,13 +1081,14 @@ PHP_FUNCTION(posix_getgrgid)
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "unable to convert posix group struct to array");
 		RETVAL_FALSE;
 	}
-#ifdef HAVE_GETGRGID_R
+#if defined(ZTS) && defined(HAVE_GETGRGID_R) && defined(_SC_GETGR_R_SIZE_MAX)
 	efree(grbuf);
 #endif
 }
 /* }}} */
 
-int php_posix_passwd_to_array(struct passwd *pw, zval *return_value) {
+int php_posix_passwd_to_array(struct passwd *pw, zval *return_value) /* {{{ */
+{
 	if (NULL == pw)
 		return 0;
 	if (NULL == return_value || Z_TYPE_P(return_value) != IS_ARRAY)
@@ -930,6 +1103,7 @@ int php_posix_passwd_to_array(struct passwd *pw, zval *return_value) {
 	add_assoc_string(return_value, "shell",     pw->pw_shell, 1);
 	return 1;
 }
+/* }}} */
 
 /* {{{ proto array posix_getpwnam(string groupname) 
    User database access (POSIX.1, 9.2.2) */
@@ -1005,7 +1179,7 @@ PHP_FUNCTION(posix_getpwuid)
 	pwbuf = emalloc(pwbuflen);
 
 	ret = getpwuid_r(uid, &_pw, pwbuf, pwbuflen, &retpwptr);
-	if (ret) {
+	if (ret || retpwptr == NULL) {
 		POSIX_G(last_error) = ret;
 		efree(pwbuf);
 		RETURN_FALSE;
