@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -20,7 +20,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: snmp.c,v 1.92.2.11 2005/06/28 13:43:29 hyanantha Exp $ */
+/* $Id: snmp.c,v 1.106.2.2.2.5.2.13 2009/06/01 13:10:18 iliaa Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -107,45 +107,240 @@
 #define SNMP_VALUE_OBJECT	2
 
 ZEND_DECLARE_MODULE_GLOBALS(snmp)
+static PHP_GINIT_FUNCTION(snmp);
 
 /* constant - can be shared among threads */
 static oid objid_mib[] = {1, 3, 6, 1, 2, 1};
 
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmpget, 0, 0, 3)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, community)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmpgetnext, 0, 0, 3)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, community)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmpwalk, 0, 0, 3)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, community)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmprealwalk, 0, 0, 3)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, community)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp_get_quick_print, 0, 0, 1)
+	ZEND_ARG_INFO(0, d)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp_set_quick_print, 0, 0, 1)
+	ZEND_ARG_INFO(0, quick_print)
+ZEND_END_ARG_INFO()
+
+#ifdef HAVE_NET_SNMP
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp_set_enum_print, 0, 0, 1)
+	ZEND_ARG_INFO(0, enum_print)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp_set_oid_output_format, 0, 0, 1)
+	ZEND_ARG_INFO(0, oid_format)
+ZEND_END_ARG_INFO()
+#endif
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmpset, 0, 0, 5)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, community)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, type)
+	ZEND_ARG_INFO(0, value)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp2_get, 0, 0, 3)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, community)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp2_getnext, 0, 0, 3)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, community)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp2_walk, 0, 0, 3)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, community)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp2_real_walk, 0, 0, 3)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, community)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp2_set, 0, 0, 5)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, community)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, type)
+	ZEND_ARG_INFO(0, value)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_php_snmpv3, 0, 0, 2)
+	ZEND_ARG_INFO(0, s)
+	ZEND_ARG_INFO(0, st)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp3_get, 0, 0, 8)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, sec_name)
+	ZEND_ARG_INFO(0, sec_level)
+	ZEND_ARG_INFO(0, auth_protocol)
+	ZEND_ARG_INFO(0, auth_passphrase)
+	ZEND_ARG_INFO(0, priv_protocol)
+	ZEND_ARG_INFO(0, priv_passphrase)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp3_getnext, 0, 0, 8)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, sec_name)
+	ZEND_ARG_INFO(0, sec_level)
+	ZEND_ARG_INFO(0, auth_protocol)
+	ZEND_ARG_INFO(0, auth_passphrase)
+	ZEND_ARG_INFO(0, priv_protocol)
+	ZEND_ARG_INFO(0, priv_passphrase)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp3_walk, 0, 0, 8)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, sec_name)
+	ZEND_ARG_INFO(0, sec_level)
+	ZEND_ARG_INFO(0, auth_protocol)
+	ZEND_ARG_INFO(0, auth_passphrase)
+	ZEND_ARG_INFO(0, priv_protocol)
+	ZEND_ARG_INFO(0, priv_passphrase)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp3_real_walk, 0, 0, 8)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, sec_name)
+	ZEND_ARG_INFO(0, sec_level)
+	ZEND_ARG_INFO(0, auth_protocol)
+	ZEND_ARG_INFO(0, auth_passphrase)
+	ZEND_ARG_INFO(0, priv_protocol)
+	ZEND_ARG_INFO(0, priv_passphrase)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp3_set, 0, 0, 10)
+	ZEND_ARG_INFO(0, host)
+	ZEND_ARG_INFO(0, sec_name)
+	ZEND_ARG_INFO(0, sec_level)
+	ZEND_ARG_INFO(0, auth_protocol)
+	ZEND_ARG_INFO(0, auth_passphrase)
+	ZEND_ARG_INFO(0, priv_protocol)
+	ZEND_ARG_INFO(0, priv_passphrase)
+	ZEND_ARG_INFO(0, object_id)
+	ZEND_ARG_INFO(0, type)
+	ZEND_ARG_INFO(0, value)
+	ZEND_ARG_INFO(0, timeout)
+	ZEND_ARG_INFO(0, retries)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp_set_valueretrieval, 0, 0, 1)
+	ZEND_ARG_INFO(0, method)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_snmp_get_valueretrieval, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_snmp_read_mib, 0, 0, 1)
+	ZEND_ARG_INFO(0, filename)
+ZEND_END_ARG_INFO()
+/* }}} */
+
 /* {{{ snmp_functions[]
  */
-function_entry snmp_functions[] = {
-	PHP_FE(snmpget, NULL)
-	PHP_FE(snmpgetnext, NULL)
-	PHP_FE(snmpwalk, NULL)
-	PHP_FE(snmprealwalk, NULL)
-	PHP_FALIAS(snmpwalkoid, snmprealwalk, NULL)
-	PHP_FE(snmp_get_quick_print, NULL)
-	PHP_FE(snmp_set_quick_print, NULL)
+const zend_function_entry snmp_functions[] = {
+	PHP_FE(snmpget, 						arginfo_snmpget)
+	PHP_FE(snmpgetnext, 					arginfo_snmpgetnext)
+	PHP_FE(snmpwalk, 						arginfo_snmpwalk)
+	PHP_FE(snmprealwalk, 					arginfo_snmprealwalk)
+	PHP_FALIAS(snmpwalkoid, snmprealwalk, 	arginfo_snmprealwalk)
+	PHP_FE(snmp_get_quick_print, 			arginfo_snmp_get_quick_print)
+	PHP_FE(snmp_set_quick_print, 			arginfo_snmp_set_quick_print)
 #ifdef HAVE_NET_SNMP
-	PHP_FE(snmp_set_enum_print, NULL)
-	PHP_FE(snmp_set_oid_numeric_print, NULL)
+	PHP_FE(snmp_set_enum_print, 			arginfo_snmp_set_enum_print)
+	PHP_FE(snmp_set_oid_output_format, 		arginfo_snmp_set_oid_output_format)
+	PHP_FALIAS(snmp_set_oid_numeric_print, snmp_set_oid_output_format, arginfo_snmp_set_oid_output_format)
 #endif
-	PHP_FE(snmpset, NULL)
+	PHP_FE(snmpset, 				arginfo_snmpset)
 
-	PHP_FE(snmp2_get, NULL)
-	PHP_FE(snmp2_getnext, NULL)
-	PHP_FE(snmp2_walk, NULL)
-	PHP_FE(snmp2_real_walk, NULL)
-	PHP_FE(snmp2_set, NULL)
+	PHP_FE(snmp2_get, 				arginfo_snmp2_get)
+	PHP_FE(snmp2_getnext, 			arginfo_snmp2_getnext)
+	PHP_FE(snmp2_walk, 				arginfo_snmp2_walk)
+	PHP_FE(snmp2_real_walk, 		arginfo_snmp2_real_walk)
+	PHP_FE(snmp2_set, 				arginfo_snmp2_set)
 
-	PHP_FE(snmp3_get, NULL)
-	PHP_FE(snmp3_getnext, NULL)
-	PHP_FE(snmp3_walk, NULL)
-	PHP_FE(snmp3_real_walk, NULL)
-	PHP_FE(snmp3_set, NULL)
-	PHP_FE(snmp_set_valueretrieval, NULL)
-	PHP_FE(snmp_get_valueretrieval, NULL)
+	PHP_FE(snmp3_get, 				arginfo_snmp3_get)
+	PHP_FE(snmp3_getnext, 			arginfo_snmp3_getnext)
+	PHP_FE(snmp3_walk, 				arginfo_snmp3_walk)
+	PHP_FE(snmp3_real_walk, 		arginfo_snmp3_real_walk)
+	PHP_FE(snmp3_set, 				arginfo_snmp3_set)
+	PHP_FE(snmp_set_valueretrieval, arginfo_snmp_set_valueretrieval)
+	PHP_FE(snmp_get_valueretrieval, arginfo_snmp_get_valueretrieval)
 
-	PHP_FE(snmp_read_mib, NULL)
+	PHP_FE(snmp_read_mib, 			arginfo_snmp_read_mib)
 	{NULL,NULL,NULL}
 };
 /* }}} */
 
+#define SNMP_CMD_GET		1
+#define SNMP_CMD_GETNEXT	2
+#define SNMP_CMD_WALK		3
+#define SNMP_CMD_REALWALK	4
+#define SNMP_CMD_SET		11
+
 /* {{{ snmp_module_entry
  */
 zend_module_entry snmp_module_entry = {
@@ -158,7 +353,11 @@ zend_module_entry snmp_module_entry = {
 	NULL,
 	PHP_MINFO(snmp),
 	NO_VERSION_YET,
-	STANDARD_MODULE_PROPERTIES
+	PHP_MODULE_GLOBALS(snmp),
+	PHP_GINIT(snmp),
+	NULL,
+	NULL,
+	STANDARD_MODULE_PROPERTIES_EX
 };
 /* }}} */
 
@@ -168,11 +367,11 @@ ZEND_GET_MODULE(snmp)
 
 /* THREAD_LS snmp_module php_snmp_module; - may need one of these at some point */
 
-/* {{{ php_snmp_init_globals
+/* {{{ PHP_GINIT_FUNCTION
  */
-static void php_snmp_init_globals(zend_snmp_globals *snmp_globals)
+static PHP_GINIT_FUNCTION(snmp)
 {
-	snmp_globals->valueretrieval = 0;
+	snmp_globals->valueretrieval = SNMP_VALUE_LIBRARY;
 }
 /* }}} */
 
@@ -187,7 +386,10 @@ PHP_MINIT_FUNCTION(snmp)
 	netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_DONT_PERSIST_STATE, 1);
 #endif
 
-	ZEND_INIT_MODULE_GLOBALS(snmp, php_snmp_init_globals, NULL);
+#ifdef HAVE_NET_SNMP
+	REGISTER_LONG_CONSTANT("SNMP_OID_OUTPUT_FULL", NETSNMP_OID_OUTPUT_FULL, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("SNMP_OID_OUTPUT_NUMERIC", NETSNMP_OID_OUTPUT_NUMERIC, CONST_CS | CONST_PERSISTENT);
+#endif
 
 	REGISTER_LONG_CONSTANT("SNMP_VALUE_LIBRARY", SNMP_VALUE_LIBRARY, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("SNMP_VALUE_PLAIN", SNMP_VALUE_PLAIN, CONST_CS | CONST_PERSISTENT);
@@ -247,7 +449,7 @@ static void php_snmp_getvalue(struct variable_list *vars, zval *snmpval TSRMLS_D
 
 	buf[0] = 0;
 
-	if (SNMP_G(valueretrieval) == 0) {
+	if (SNMP_G(valueretrieval) == SNMP_VALUE_LIBRARY) {
 #ifdef HAVE_NET_SNMP
 		snprint_value(buf, sizeof(buf), vars->name, vars->name_length, vars);
 #else
@@ -317,7 +519,7 @@ static void php_snmp_getvalue(struct variable_list *vars, zval *snmpval TSRMLS_D
 		break;
 	}
 
-	if (SNMP_G(valueretrieval) == 1) {
+	if (SNMP_G(valueretrieval) == SNMP_VALUE_PLAIN) {
 		*snmpval = *val;
 		zval_copy_ctor(snmpval);
 	} else {
@@ -331,14 +533,13 @@ static void php_snmp_getvalue(struct variable_list *vars, zval *snmpval TSRMLS_D
 *
 * Generic SNMP object fetcher (for all SNMP versions)
 *
-* st=1   snmpget()  - query an agent and return a single value.
-* st=2   snmpget()  - query an agent and return the next single value.
-* st=3   snmpwalk() - walk the mib and return a single dimensional array 
-*                     containing the values.
-* st=4   snmprealwalk() and snmpwalkoid() - walk the mib and return an 
-*                                           array of oid,value pairs.
-* st=5-8 ** Reserved **
-* st=11  snmpset()  - query an agent and set a single value
+* st=SNMP_CMD_GET   get - query an agent with SNMP-GET.
+* st=SNMP_CMD_GETNEXT   getnext - query an agent with SNMP-GETNEXT.
+* st=SNMP_CMD_WALK   walk - walk the mib and return a single dimensional array 
+*          containing the values.
+* st=SNMP_CMD_REALWALK   realwalk() and walkoid() - walk the mib and return an 
+*          array of oid,value pairs.
+* st=SNMP_CMD_SET  set() - query an agent and set a single value
 *
 */
 static void php_snmp_internal(INTERNAL_FUNCTION_PARAMETERS, int st, 
@@ -362,7 +563,7 @@ static void php_snmp_internal(INTERNAL_FUNCTION_PARAMETERS, int st,
 	char *err;
 	zval *snmpval = NULL;
 
-	if (st >= 3) { /* walk */
+	if (st >= SNMP_CMD_WALK) { /* walk */
 		rootlen = MAX_NAME_LEN;
 		if (strlen(objid)) { /* on a walk, an empty string means top of tree - no error */
 			if (snmp_parse_oid(objid, root, &rootlen)) {
@@ -386,12 +587,12 @@ static void php_snmp_internal(INTERNAL_FUNCTION_PARAMETERS, int st,
 		RETURN_FALSE;
 	}
 
-	if (st >= 3) {
+	if (st >= SNMP_CMD_WALK) {
 		memmove((char *)name, (char *)root, rootlen * sizeof(oid));
 		name_length = rootlen;
 		switch(st) {
-			case 3:
-			case 4:
+			case SNMP_CMD_WALK:
+			case SNMP_CMD_REALWALK:
 				array_init(return_value);
 				break;
 			default:
@@ -402,16 +603,16 @@ static void php_snmp_internal(INTERNAL_FUNCTION_PARAMETERS, int st,
 
 	while (keepwalking) {
 		keepwalking = 0;
-		if ((st == 1) || (st == 2)) {
-			pdu = snmp_pdu_create((st == 1) ? SNMP_MSG_GET : SNMP_MSG_GETNEXT);
+		if ((st == SNMP_CMD_GET) || (st == SNMP_CMD_GETNEXT)) {
 			name_length = MAX_OID_LEN;
 			if (!snmp_parse_oid(objid, name, &name_length)) {
 				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid object identifier: %s", objid);
 				snmp_close(ss);
 				RETURN_FALSE;
 			}
+			pdu = snmp_pdu_create((st == SNMP_CMD_GET) ? SNMP_MSG_GET : SNMP_MSG_GETNEXT);
 			snmp_add_null_var(pdu, name, name_length);
-		} else if (st == 11) {
+		} else if (st == SNMP_CMD_SET) {
 			pdu = snmp_pdu_create(SNMP_MSG_SET);
 			if (snmp_add_var(pdu, name, name_length, type, value)) {
 #ifdef HAVE_NET_SNMP
@@ -420,11 +621,18 @@ static void php_snmp_internal(INTERNAL_FUNCTION_PARAMETERS, int st,
 				sprint_objid(buf, name, name_length);
 #endif
 				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not add variable: %s %c %s", buf, type, value);
+				snmp_free_pdu(pdu);
 				snmp_close(ss);
 				RETURN_FALSE;
 			}
-		} else if (st >= 3) {
-			pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
+		} else if (st >= SNMP_CMD_WALK) {
+			if (session->version == SNMP_VERSION_1) {
+				pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
+			} else {
+				pdu = snmp_pdu_create(SNMP_MSG_GETBULK);
+				pdu->non_repeaters = 0;
+				pdu->max_repetitions = 20;
+			}
 			snmp_add_null_var(pdu, name, name_length);
 		}
 
@@ -433,30 +641,32 @@ retry:
 		if (status == STAT_SUCCESS) {
 			if (response->errstat == SNMP_ERR_NOERROR) {
 				for (vars = response->variables; vars; vars = vars->next_variable) {
-					if (st >= 3 && st != 11 && 
+					if (st >= SNMP_CMD_WALK && st != SNMP_CMD_SET && 
 						(vars->name_length < rootlen || memcmp(root, vars->name, rootlen * sizeof(oid)))) {
 						continue;       /* not part of this subtree */
 					}
 
-					if (st != 11) {
+					if (st != SNMP_CMD_SET) {
 						MAKE_STD_ZVAL(snmpval);
 						php_snmp_getvalue(vars, snmpval TSRMLS_CC);
 					}
 
-					if (st == 1) {
+					if (st == SNMP_CMD_GET) {
 						*return_value = *snmpval;
 						zval_copy_ctor(return_value);
 						zval_ptr_dtor(&snmpval);
+						snmp_free_pdu(response);
 						snmp_close(ss);
 						return;
-					} else if (st == 2) {
+					} else if (st == SNMP_CMD_GETNEXT) {
 						*return_value = *snmpval;
 						zval_copy_ctor(return_value);
+						snmp_free_pdu(response);
 						snmp_close(ss);
 						return;
-					} else if (st == 3) {
+					} else if (st == SNMP_CMD_WALK) {
 						add_next_index_zval(return_value,snmpval); /* Add to returned array */
-					} else if (st == 4)  {
+					} else if (st == SNMP_CMD_REALWALK && vars->type != SNMP_ENDOFMIBVIEW && vars->type != SNMP_NOSUCHOBJECT && vars->type != SNMP_NOSUCHINSTANCE) {
 #ifdef HAVE_NET_SNMP
 						snprint_objid(buf2, sizeof(buf2), vars->name, vars->name_length);
 #else
@@ -464,17 +674,22 @@ retry:
 #endif
 						add_assoc_zval(return_value,buf2,snmpval);
 					}
-					if (st >= 3 && st != 11) {
+					if (st >= SNMP_CMD_WALK && st != SNMP_CMD_SET) {
 						if (vars->type != SNMP_ENDOFMIBVIEW && 
 							vars->type != SNMP_NOSUCHOBJECT && vars->type != SNMP_NOSUCHINSTANCE) {
-							memmove((char *)name, (char *)vars->name,vars->name_length * sizeof(oid));
-							name_length = vars->name_length;
-							keepwalking = 1;
+							if (snmp_oid_compare(name, name_length, vars->name, vars->name_length) >= 0) {
+								php_error_docref(NULL TSRMLS_CC, E_WARNING, "Error: OID not increasing: %s",name);
+								keepwalking = 0;
+							} else {
+								memmove((char *)name, (char *)vars->name,vars->name_length * sizeof(oid));
+								name_length = vars->name_length;
+								keepwalking = 1;
+							}
 						}
 					}
 				}	
 			} else {
-				if (st != 3 || response->errstat != SNMP_ERR_NOSUCHNAME) {
+				if (st != SNMP_CMD_WALK || response->errstat != SNMP_ERR_NOSUCHNAME) {
 					php_error_docref(NULL TSRMLS_CC, E_WARNING, "Error in packet: %s", snmp_errstring(response->errstat));
 					if (response->errstat == SNMP_ERR_NOSUCHNAME) {
 						for (count=1, vars = response->variables; vars && count != response->errindex;
@@ -488,33 +703,47 @@ retry:
 						}
 						php_error_docref(NULL TSRMLS_CC, E_WARNING, "This name does not exist: %s",buf);
 					}
-					if (st == 1) {
+					if (st == SNMP_CMD_GET) {
 						if ((pdu = snmp_fix_pdu(response, SNMP_MSG_GET)) != NULL) {
+							snmp_free_pdu(response);
 							goto retry;
 						}
-					} else if (st == 11) {
+					} else if (st == SNMP_CMD_SET) {
 						if ((pdu = snmp_fix_pdu(response, SNMP_MSG_SET)) != NULL) {
+							snmp_free_pdu(response);
 							goto retry;
 						}
-					} else if (st >= 2) { /* Here we do both getnext and walks. */
+					} else if (st == SNMP_CMD_GETNEXT) {
 						if ((pdu = snmp_fix_pdu(response, SNMP_MSG_GETNEXT)) != NULL) {
+							snmp_free_pdu(response);
+							goto retry;
+						}
+					} else if (st >= SNMP_CMD_WALK) { /* Here we do walks. */
+						if ((pdu = snmp_fix_pdu(response, ((session->version == SNMP_VERSION_1)
+										? SNMP_MSG_GETNEXT
+										: SNMP_MSG_GETBULK))) != NULL) {
+							snmp_free_pdu(response);
 							goto retry;
 						}
 					}
+					snmp_free_pdu(response);
 					snmp_close(ss);
+					if (st == SNMP_CMD_WALK || st == SNMP_CMD_REALWALK) {
+						zval_dtor(return_value);
+					}
 					RETURN_FALSE;
 				}
 			}
 		} else if (status == STAT_TIMEOUT) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "No response from %s", session->peername);
-			if (st == 3 || st == 4) {
+			if (st == SNMP_CMD_WALK || st == SNMP_CMD_REALWALK) {
 				zval_dtor(return_value);
 			}
 			snmp_close(ss);
 			RETURN_FALSE;
 		} else {    /* status == STAT_ERROR */
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "An error occurred, quitting");
-			if (st == 3 || st == 4) {
+			if (st == SNMP_CMD_WALK || st == SNMP_CMD_REALWALK) {
 				zval_dtor(return_value);
 			}
 			snmp_close(ss);
@@ -534,72 +763,75 @@ retry:
 * This function makes use of the internal SNMP object fetcher.
 * The object fetcher is shared with SNMPv3.
 *
-* st=1   snmpget() - query an agent and return a single value.
-* st=2   snmpgetnext() - query an agent and return the next single value.
-* st=3   snmpwalk() - walk the mib and return a single dimensional array 
+* st=SNMP_CMD_GET   get - query an agent with SNMP-GET.
+* st=SNMP_CMD_GETNEXT   getnext - query an agent with SNMP-GETNEXT.
+* st=SNMP_CMD_WALK   walk - walk the mib and return a single dimensional array 
 *          containing the values.
-* st=4 snmprealwalk() and snmpwalkoid() - walk the mib and return an 
+* st=SNMP_CMD_REALWALK   realwalk() and walkoid() - walk the mib and return an 
 *          array of oid,value pairs.
 * st=5-8 ** Reserved **
-* st=11  snmpset() - query an agent and set a single value
+* st=SNMP_CMD_SET  set() - query an agent and set a single value
 *
 */
 static void php_snmp(INTERNAL_FUNCTION_PARAMETERS, int st, int version) 
 {
-	zval **a1, **a2, **a3, **a4, **a5, **a6, **a7;
+	char *a1, **a2, **a3;
+	int a1_len, a2_len, a3_len;
+	zval **a4 = NULL, **a5 = NULL;
+	long a6 = 0, a7 = 0;
 	struct snmp_session session;
-	long timeout=SNMP_DEFAULT_TIMEOUT;
-	long retries=SNMP_DEFAULT_RETRIES;
-	int myargc = ZEND_NUM_ARGS();
+	long timeout = SNMP_DEFAULT_TIMEOUT;
+	long retries = SNMP_DEFAULT_RETRIES;
 	char type = (char) 0;
 	char *value = (char *) 0;
 	char hostname[MAX_NAME_LEN];
 	int remote_port = 161;
 	char *pptr;
+	int argc = ZEND_NUM_ARGS();
 
-	if (myargc < 3 || myargc > 7 ||
-		zend_get_parameters_ex(myargc, &a1, &a2, &a3, &a4, &a5, &a6, &a7) == FAILURE) {
-		WRONG_PARAM_COUNT;
-	}
-
-	convert_to_string_ex(a1);
-	convert_to_string_ex(a2);
-	convert_to_string_ex(a3);
-	
-	if (st == 11) {
-		if (myargc < 5) {
-			WRONG_PARAM_COUNT;
+	if (st == SNMP_CMD_SET) {
+		if (zend_parse_parameters(argc TSRMLS_CC, "sssZZ|ll", &a1, &a1_len, &a2, &a2_len, &a3, &a3_len, &a4, &a5, &a6, &a7) == FAILURE) {
+			return;
 		}
-
+	} else {
+		/* SNMP_CMD_GET
+		 * SNMP_CMD_GETNEXT
+		 * SNMP_CMD_WALK
+		 * SNMP_CMD_REALWALK
+		 */
+		if (zend_parse_parameters(argc TSRMLS_CC, "sss|ZZ", &a1, &a1_len, &a2, &a2_len, &a3, &a3_len, &a4, &a5) == FAILURE) {
+			return;
+		}
+	}
+	
+	if (st == SNMP_CMD_SET) {
 		convert_to_string_ex(a4);
 		convert_to_string_ex(a5);
 
-		if(myargc > 5) {
-			convert_to_long_ex(a6);
-			timeout = Z_LVAL_PP(a6);
+		if (argc > 5) {
+			timeout = a6;
 		}
 
-		if(myargc > 6) {
-			convert_to_long_ex(a7);
-			retries = Z_LVAL_PP(a7);
+		if (argc > 6) {
+			retries = a7;
 		}
 
 		type = Z_STRVAL_PP(a4)[0];
 		value = Z_STRVAL_PP(a5);
 	} else {
-		if(myargc > 3) {
+		if (argc > 3) {
 			convert_to_long_ex(a4);
 			timeout = Z_LVAL_PP(a4);
 		}
 
-		if(myargc > 4) {
+		if (argc > 4) {
 			convert_to_long_ex(a5);
 			retries = Z_LVAL_PP(a5);
 		}
 	}
 
 	snmp_sess_init(&session);
-	strlcpy(hostname, Z_STRVAL_PP(a1), sizeof(hostname));
+	strlcpy(hostname, a1, sizeof(hostname));
 	if ((pptr = strchr (hostname, ':'))) {
 		remote_port = strtol (pptr + 1, NULL, 0);
 	}
@@ -614,17 +846,17 @@ static void php_snmp(INTERNAL_FUNCTION_PARAMETERS, int st, int version)
 	* memory it did not allocate
 	*/
 #ifdef UCD_SNMP_HACK
-	session.community = (u_char *)strdup(Z_STRVAL_PP(a2)); /* memory freed by SNMP library, strdup NOT estrdup */
+	session.community = (u_char *)strdup(a2); /* memory freed by SNMP library, strdup NOT estrdup */
 #else
-	session.community = (u_char *)Z_STRVAL_PP(a2);
+	session.community = (u_char *)a2;
 #endif
-	session.community_len = Z_STRLEN_PP(a2);
+	session.community_len = a2_len;
 	session.retries = retries;
 	session.timeout = timeout;
 	
 	session.authenticator = NULL;
 
-	php_snmp_internal(INTERNAL_FUNCTION_PARAM_PASSTHRU, st, &session, Z_STRVAL_PP(a3), type, value);
+	php_snmp_internal(INTERNAL_FUNCTION_PARAM_PASSTHRU, st, &session, a3, type, value);
 }
 /* }}} */
 
@@ -632,7 +864,7 @@ static void php_snmp(INTERNAL_FUNCTION_PARAMETERS, int st, int version)
    Fetch a SNMP object */
 PHP_FUNCTION(snmpget)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,1, SNMP_VERSION_1);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,SNMP_CMD_GET, SNMP_VERSION_1);
 }
 /* }}} */
 
@@ -640,7 +872,7 @@ PHP_FUNCTION(snmpget)
    Fetch a SNMP object */
 PHP_FUNCTION(snmpgetnext)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,2, SNMP_VERSION_1);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,SNMP_CMD_GETNEXT, SNMP_VERSION_1);
 }
 /* }}} */
 
@@ -648,7 +880,7 @@ PHP_FUNCTION(snmpgetnext)
    Return all objects under the specified object id */
 PHP_FUNCTION(snmpwalk)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,3, SNMP_VERSION_1);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,SNMP_CMD_WALK, SNMP_VERSION_1);
 }
 /* }}} */
 
@@ -656,7 +888,7 @@ PHP_FUNCTION(snmpwalk)
    Return all objects including their respective object id withing the specified one */
 PHP_FUNCTION(snmprealwalk)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,4, SNMP_VERSION_1);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,SNMP_CMD_REALWALK, SNMP_VERSION_1);
 }
 /* }}} */
 
@@ -664,8 +896,8 @@ PHP_FUNCTION(snmprealwalk)
    Return the current status of quick_print */
 PHP_FUNCTION(snmp_get_quick_print)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 
 #ifdef HAVE_NET_SNMP
@@ -680,10 +912,9 @@ PHP_FUNCTION(snmp_get_quick_print)
    Return all objects including their respective object id withing the specified one */
 PHP_FUNCTION(snmp_set_quick_print)
 {
-	int argc = ZEND_NUM_ARGS();
 	long a1;
 
-	if (zend_parse_parameters(argc TSRMLS_CC, "l", &a1) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &a1) == FAILURE) {
 		return;
 	}
 
@@ -700,10 +931,9 @@ PHP_FUNCTION(snmp_set_quick_print)
    Return all values that are enums with their enum value instead of the raw integer */
 PHP_FUNCTION(snmp_set_enum_print)
 {
-	int argc = ZEND_NUM_ARGS();
 	long a1;
 
-	if (zend_parse_parameters(argc TSRMLS_CC, "l", &a1) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &a1) == FAILURE) {
 		return;
 	}
 
@@ -711,21 +941,29 @@ PHP_FUNCTION(snmp_set_enum_print)
 } 
 /* }}} */
 
-/* {{{ proto void snmp_set_oid_numeric_print(int oid_numeric_print)
-   Return all objects including their respective object id withing the specified one */
-PHP_FUNCTION(snmp_set_oid_numeric_print)
+/* {{{ proto void snmp_set_oid_output_format(int oid_format)
+   Set the OID output format. */
+PHP_FUNCTION(snmp_set_oid_output_format)
 {
-	int argc = ZEND_NUM_ARGS();
 	long a1;
 
-	if (zend_parse_parameters(argc TSRMLS_CC, "l", &a1) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &a1) == FAILURE) {
 		return;
 	}
-	if ((int) a1 != 0) {
-		netsnmp_ds_set_int(NETSNMP_DS_LIBRARY_ID,
-			NETSNMP_DS_LIB_OID_OUTPUT_FORMAT,
-			NETSNMP_OID_OUTPUT_NUMERIC);
+
+	switch ((int) a1) {
+		case 0:
+		case NETSNMP_OID_OUTPUT_FULL:
+			a1 = NETSNMP_OID_OUTPUT_FULL;
+			break;
+
+		default:
+		case NETSNMP_OID_OUTPUT_NUMERIC:
+			a1 = NETSNMP_OID_OUTPUT_NUMERIC;
+			break;
 	}
+
+	netsnmp_ds_set_int(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_OID_OUTPUT_FORMAT, a1);
 } 
 /* }}} */
 #endif
@@ -734,7 +972,7 @@ PHP_FUNCTION(snmp_set_oid_numeric_print)
    Set the value of a SNMP object */
 PHP_FUNCTION(snmpset)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,11, SNMP_VERSION_1);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,SNMP_CMD_SET, SNMP_VERSION_1);
 }
 /* }}} */
 
@@ -814,10 +1052,14 @@ static int netsnmp_session_set_sec_protocol(struct snmp_session *s, char *prot T
 * 
 * As we want this extension to compile on both versions, we use the latter
 * symbol on purpose, as it's defined to be the same as the former.
+*
+* However, in 5.2 the type of usmAES128PrivProtocol is a pointer, not an
+* array, so we cannot use the OIDSIZE macro because it uses sizeof().
+*
 */
 			|| !strcasecmp(prot, "AES")) {
 			s->securityPrivProto = usmAES128PrivProtocol;
-			s->securityPrivProtoLen = OIDSIZE(usmAES128PrivProtocol);
+			s->securityPrivProtoLen = USM_PRIV_PROTO_AES128_LEN;
 			return (0);
 #else			
 		) {
@@ -907,7 +1149,7 @@ static int netsnmp_session_gen_sec_key(struct snmp_session *s, u_char *pass TSRM
    Fetch a SNMP object */
 PHP_FUNCTION(snmp2_get)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,1, SNMP_VERSION_2c);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,SNMP_CMD_GET, SNMP_VERSION_2c);
 }
 /* }}} */
 
@@ -915,7 +1157,7 @@ PHP_FUNCTION(snmp2_get)
    Fetch a SNMP object */
 PHP_FUNCTION(snmp2_getnext)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,2, SNMP_VERSION_2c);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,SNMP_CMD_GETNEXT, SNMP_VERSION_2c);
 }
 /* }}} */
 
@@ -923,7 +1165,7 @@ PHP_FUNCTION(snmp2_getnext)
    Return all objects under the specified object id */
 PHP_FUNCTION(snmp2_walk)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,3, SNMP_VERSION_2c);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,SNMP_CMD_WALK, SNMP_VERSION_2c);
 }
 /* }}} */
 
@@ -931,7 +1173,7 @@ PHP_FUNCTION(snmp2_walk)
    Return all objects including their respective object id withing the specified one */
 PHP_FUNCTION(snmp2_real_walk)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,4, SNMP_VERSION_2c);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,SNMP_CMD_REALWALK, SNMP_VERSION_2c);
 }
 /* }}} */
 
@@ -939,7 +1181,7 @@ PHP_FUNCTION(snmp2_real_walk)
    Set the value of a SNMP object */
 PHP_FUNCTION(snmp2_set)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,11, SNMP_VERSION_2c);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,SNMP_CMD_SET, SNMP_VERSION_2c);
 }
 /* }}} */
 
@@ -948,31 +1190,46 @@ PHP_FUNCTION(snmp2_set)
 * Generic SNMPv3 object fetcher
 * From here is passed on the the common internal object fetcher.
 *
-* st=1   snmp3_get() - query an agent and return a single value.
-* st=2   snmp3_getnext() - query an agent and return the next single value.
-* st=3   snmp3_walk() - walk the mib and return a single dimensional array 
+* st=SNMP_CMD_GET   snmp3_get() - query an agent and return a single value.
+* st=SNMP_CMD_GETNEXT   snmp3_getnext() - query an agent and return the next single value.
+* st=SNMP_CMD_WALK   snmp3_walk() - walk the mib and return a single dimensional array 
 *                       containing the values.
-* st=4   snmp3_real_walk() - walk the mib and return an 
+* st=SNMP_CMD_REALWALK   snmp3_real_walk() - walk the mib and return an 
 *                            array of oid,value pairs.
-* st=11  snmp3_set() - query an agent and set a single value
+* st=SNMP_CMD_SET  snmp3_set() - query an agent and set a single value
 *
 */
 static void php_snmpv3(INTERNAL_FUNCTION_PARAMETERS, int st)
 {
-	zval **a1, **a2, **a3, **a4, **a5, **a6, **a7, **a8, **a9, **a10, **a11, **a12;
+	zval **a9 = NULL, **a10 = NULL;
+	char *a1, *a2, *a3, *a4, *a5, *a6, *a7, *a8;
+	int a1_len, a2_len, a3_len, a4_len, a5_len, a6_len, a7_len, a8_len;
+	long a11 = 0, a12 = 0;
 	struct snmp_session session;
-	long timeout=SNMP_DEFAULT_TIMEOUT;
-	long retries=SNMP_DEFAULT_RETRIES;
-	int myargc = ZEND_NUM_ARGS();
+	long timeout = SNMP_DEFAULT_TIMEOUT;
+	long retries = SNMP_DEFAULT_RETRIES;
 	char type = (char) 0;
 	char *value = (char *) 0;
 	char hostname[MAX_NAME_LEN];
 	int remote_port = 161;
 	char *pptr;
-
-	if (myargc < 8 || myargc > 12 ||
-		zend_get_parameters_ex(myargc, &a1, &a2, &a3, &a4, &a5, &a6, &a7, &a8, &a9, &a10, &a11, &a12) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	int argc = ZEND_NUM_ARGS();
+	
+	if (st == SNMP_CMD_SET) {	
+		if (zend_parse_parameters(argc TSRMLS_CC, "ssssssssZZ|ll", &a1, &a1_len, &a2, &a2_len, &a3, &a3_len,
+			&a4, &a4_len, &a5, &a5_len, &a6, &a6_len, &a7, &a7_len, &a8, &a8_len, &a9, &a10, &a11, &a12) == FAILURE) {
+			return;
+		}
+	} else {
+		/* SNMP_CMD_GET
+		 * SNMP_CMD_GETNEXT
+		 * SNMP_CMD_WALK
+		 * SNMP_CMD_REALWALK
+		 */
+		if (zend_parse_parameters(argc TSRMLS_CC, "ssssssss|ZZ", &a1, &a1_len, &a2, &a2_len, &a3, &a3_len,
+			&a4, &a4_len, &a5, &a5_len, &a6, &a6_len, &a7, &a7_len, &a8, &a8_len, &a9, &a10) == FAILURE) {
+			return;
+		}
 	}
 
 	snmp_sess_init(&session);
@@ -980,80 +1237,66 @@ static void php_snmpv3(INTERNAL_FUNCTION_PARAMETERS, int st)
 	session.version = SNMP_VERSION_3;
 
 	/* Reading the hostname and its optional non-default port number */
-	convert_to_string_ex(a1);
-	strlcpy(hostname, Z_STRVAL_PP(a1), sizeof(hostname));
-	if ((pptr = strchr (hostname, ':'))) {
-		remote_port = strtol (pptr + 1, NULL, 0);
+	strlcpy(hostname, a1, sizeof(hostname));
+	if ((pptr = strchr(hostname, ':'))) {
+		remote_port = strtol(pptr + 1, NULL, 0);
 	}
 	session.peername = hostname;
 	session.remote_port = remote_port;
 
 	/* Setting the security name. */
-	convert_to_string_ex(a2);
-	if (netsnmp_session_set_sec_name(&session, Z_STRVAL_PP(a2))) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could net set security name: %s", Z_STRVAL_PP(a2));
+	if (netsnmp_session_set_sec_name(&session, a2)) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could net set security name: %s", a2);
 		RETURN_FALSE;
 	}
 
 	/* Setting the security level. */
-	convert_to_string_ex(a3);
-	if (netsnmp_session_set_sec_level(&session, Z_STRVAL_PP(a3) TSRMLS_CC)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid security level: %s", Z_STRVAL_PP(a3));
+	if (netsnmp_session_set_sec_level(&session, a3 TSRMLS_CC)) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid security level: %s", a3);
 		RETURN_FALSE;
 	}
 
 	/* Setting the authentication protocol. */
-	convert_to_string_ex(a4);
-	if (netsnmp_session_set_auth_protocol(&session, Z_STRVAL_PP(a4) TSRMLS_CC)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid authentication protocol: %s", Z_STRVAL_PP(a4));
+	if (netsnmp_session_set_auth_protocol(&session, a4 TSRMLS_CC)) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid authentication protocol: %s", a4);
 		RETURN_FALSE;
 	}
 
 	/* Setting the authentication passphrase. */
-	convert_to_string_ex(a5);
-	if (netsnmp_session_gen_auth_key(&session, Z_STRVAL_PP(a5) TSRMLS_CC)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not generate key for authentication pass phrase: %s", Z_STRVAL_PP(a4));
+	if (netsnmp_session_gen_auth_key(&session, a5 TSRMLS_CC)) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not generate key for authentication pass phrase: %s", a5);
 		RETURN_FALSE;
 	}
 
 	/* Setting the security protocol. */
-	convert_to_string_ex(a6);
-	if (netsnmp_session_set_sec_protocol(&session, Z_STRVAL_PP(a6) TSRMLS_CC) &&
-			(0 != strlen(Z_STRVAL_PP(a6)))) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid security protocol: %s", Z_STRVAL_PP(a6));
+	if (netsnmp_session_set_sec_protocol(&session, a6 TSRMLS_CC) && a6_len) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid security protocol: %s", a6);
 		RETURN_FALSE;
 	}
 
 	/* Setting the security protocol passphrase. */
-	convert_to_string_ex(a7);
-	if (netsnmp_session_gen_sec_key(&session, Z_STRVAL_PP(a7) TSRMLS_CC) &&
-							(0 != strlen(Z_STRVAL_PP(a7)))) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not generate key for security pass phrase: %s", Z_STRVAL_PP(a7));
+	if (netsnmp_session_gen_sec_key(&session, a7 TSRMLS_CC) && a7_len) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not generate key for security pass phrase: %s", a7);
 		RETURN_FALSE;
 	}
 
-	if (st == 11) {
-		if (myargc < 10) {
-			WRONG_PARAM_COUNT;
+	if (st == SNMP_CMD_SET) {
+		if (argc > 10) {
+			timeout = a11;
 		}
-		if (myargc > 10) {
-			convert_to_long_ex(a11);
-			timeout = Z_LVAL_PP(a11);
-		}
-		if (myargc > 11) {
-			convert_to_long_ex(a12);
-			retries = Z_LVAL_PP(a12);
+		if (argc > 11) {
+			retries = a12;
 		}
 		convert_to_string_ex(a9);
 		convert_to_string_ex(a10);
 		type = Z_STRVAL_PP(a9)[0];
 		value = Z_STRVAL_PP(a10);
 	} else {
-		if (myargc > 8) {
+		if (argc > 8) {
 			convert_to_long_ex(a9);
 			timeout = Z_LVAL_PP(a9);
 		}
-		if (myargc > 9) {
+		if (argc > 9) {
 			convert_to_long_ex(a10);
 			retries = Z_LVAL_PP(a10);
 		}
@@ -1062,7 +1305,7 @@ static void php_snmpv3(INTERNAL_FUNCTION_PARAMETERS, int st)
 	session.retries = retries;
 	session.timeout = timeout;
 
-	php_snmp_internal(INTERNAL_FUNCTION_PARAM_PASSTHRU, st, &session, Z_STRVAL_PP(a8), type, value);
+	php_snmp_internal(INTERNAL_FUNCTION_PARAM_PASSTHRU, st, &session, a8, type, value);
 }
 /* }}} */
 
@@ -1070,7 +1313,7 @@ static void php_snmpv3(INTERNAL_FUNCTION_PARAMETERS, int st)
    Fetch the value of a SNMP object */
 PHP_FUNCTION(snmp3_get)
 {
-	php_snmpv3(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
+	php_snmpv3(INTERNAL_FUNCTION_PARAM_PASSTHRU, SNMP_CMD_GET);
 }
 /* }}} */
 
@@ -1078,7 +1321,7 @@ PHP_FUNCTION(snmp3_get)
    Fetch the value of a SNMP object */
 PHP_FUNCTION(snmp3_getnext)
 {
-	php_snmpv3(INTERNAL_FUNCTION_PARAM_PASSTHRU, 2);
+	php_snmpv3(INTERNAL_FUNCTION_PARAM_PASSTHRU, SNMP_CMD_GETNEXT);
 }
 /* }}} */
 
@@ -1086,7 +1329,7 @@ PHP_FUNCTION(snmp3_getnext)
    Fetch the value of a SNMP object */
 PHP_FUNCTION(snmp3_walk)
 {
-	php_snmpv3(INTERNAL_FUNCTION_PARAM_PASSTHRU, 3);
+	php_snmpv3(INTERNAL_FUNCTION_PARAM_PASSTHRU, SNMP_CMD_WALK);
 }
 /* }}} */
 
@@ -1094,7 +1337,7 @@ PHP_FUNCTION(snmp3_walk)
    Fetch the value of a SNMP object */
 PHP_FUNCTION(snmp3_real_walk)
 {
-	php_snmpv3(INTERNAL_FUNCTION_PARAM_PASSTHRU, 4);
+	php_snmpv3(INTERNAL_FUNCTION_PARAM_PASSTHRU, SNMP_CMD_REALWALK);
 }
 /* }}} */
 
@@ -1102,27 +1345,22 @@ PHP_FUNCTION(snmp3_real_walk)
    Fetch the value of a SNMP object */
 PHP_FUNCTION(snmp3_set)
 {
-	php_snmpv3(INTERNAL_FUNCTION_PARAM_PASSTHRU, 11);
+	php_snmpv3(INTERNAL_FUNCTION_PARAM_PASSTHRU, SNMP_CMD_SET);
 }
 /* }}} */
 
-/* {{{ proto int snmp_set_valueretrieval(int method)
+/* {{{ proto void snmp_set_valueretrieval(int method)
    Specify the method how the SNMP values will be returned */
 PHP_FUNCTION(snmp_set_valueretrieval)
 {
-	zval **method;
+	long method;
 
-	if (ZEND_NUM_ARGS() != 1 ||
-		zend_get_parameters_ex(ZEND_NUM_ARGS(), &method) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &method) == FAILURE) {
+		return;
 	}
 
-	convert_to_long_ex(method);
-
-	if ((Z_LVAL_PP(method) == SNMP_VALUE_LIBRARY) ||
-	    (Z_LVAL_PP(method) == SNMP_VALUE_PLAIN) ||
-	    (Z_LVAL_PP(method) == SNMP_VALUE_OBJECT)) {
-		SNMP_G(valueretrieval) = Z_LVAL_PP(method);
+	if ((method == SNMP_VALUE_LIBRARY) || (method == SNMP_VALUE_PLAIN) || (method == SNMP_VALUE_OBJECT)) {
+		SNMP_G(valueretrieval) = method;
 	}
 }
 /* }}} */
@@ -1139,21 +1377,19 @@ PHP_FUNCTION(snmp_get_valueretrieval)
    Reads and parses a MIB file into the active MIB tree. */
 PHP_FUNCTION(snmp_read_mib)
 {
-	zval **filename;
+	char *filename;
+	int filename_len;
 
-	if (ZEND_NUM_ARGS() != 1 ||
-		zend_get_parameters_ex(ZEND_NUM_ARGS(), &filename) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) {
+		return;
 	}
 
-	convert_to_string_ex(filename);
-
 	/* Prevent read_mib() from printing any errors. */
 	snmp_disable_stderrlog();
 	
-	if (!read_mib(Z_STRVAL_PP(filename))) {
+	if (!read_mib(filename)) {
 		char *error = strerror(errno);
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Error while reading MIB file '%s': %s", Z_STRVAL_PP(filename), error);
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Error while reading MIB file '%s': %s", filename, error);
 		RETURN_FALSE;
 	}
 	RETURN_TRUE;
