@@ -2,12 +2,12 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2004 The PHP Group                                |
+  | Copyright (c) 1997-2009 The PHP Group                                |
   +----------------------------------------------------------------------+
-  | This source file is subject to version 3.0 of the PHP license,       |
+  | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
-  | http://www.php.net/license/3_0.txt.                                  |
+  | http://www.php.net/license/3_01.txt                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
@@ -17,7 +17,7 @@
   |          Dmitry Stogov <dmitry@zend.com>                             |
   +----------------------------------------------------------------------+
 */
-/* $Id: php_packet_soap.c,v 1.36.2.5 2005/04/20 10:58:37 dmitry Exp $ */
+/* $Id: php_packet_soap.c,v 1.42.2.1.2.4.2.3 2008/12/31 11:15:43 sebastian Exp $ */
 
 #include "php_soap.h"
 
@@ -90,7 +90,7 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 				add_soap_fault(this_ptr, "Client", "encodingStyle cannot be specified on the Envelope", NULL, NULL TSRMLS_CC);
 				xmlFreeDoc(response);
 				return FALSE;
-			} else if (strcmp(attr->children->content,SOAP_1_1_ENC_NAMESPACE) != 0) {
+			} else if (strcmp((char*)attr->children->content, SOAP_1_1_ENC_NAMESPACE) != 0) {
 				add_soap_fault(this_ptr, "Client", "Unknown data encoding style", NULL, NULL TSRMLS_CC);
 				xmlFreeDoc(response);
 				return FALSE;
@@ -140,7 +140,7 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 				add_soap_fault(this_ptr, "Client", "encodingStyle cannot be specified on the Body", NULL, NULL TSRMLS_CC);
 				xmlFreeDoc(response);
 				return FALSE;
-			} else if (strcmp(attr->children->content,SOAP_1_1_ENC_NAMESPACE) != 0) {
+			} else if (strcmp((char*)attr->children->content, SOAP_1_1_ENC_NAMESPACE) != 0) {
 				add_soap_fault(this_ptr, "Client", "Unknown data encoding style", NULL, NULL TSRMLS_CC);
 				xmlFreeDoc(response);
 				return FALSE;
@@ -166,7 +166,7 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 					add_soap_fault(this_ptr, "Client", "encodingStyle cannot be specified on the Header", NULL, NULL TSRMLS_CC);
 					xmlFreeDoc(response);
 					return FALSE;
-				} else if (strcmp(attr->children->content,SOAP_1_1_ENC_NAMESPACE) != 0) {
+				} else if (strcmp((char*)attr->children->content, SOAP_1_1_ENC_NAMESPACE) != 0) {
 					add_soap_fault(this_ptr, "Client", "Unknown data encoding style", NULL, NULL TSRMLS_CC);
 					xmlFreeDoc(response);
 					return FALSE;
@@ -184,31 +184,35 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 		xmlNodePtr tmp;
 
 		if (soap_version == SOAP_1_1) {
-			tmp = get_node(fault->children,"faultcode");
+			tmp = get_node(fault->children, "faultcode");
 			if (tmp != NULL && tmp->children != NULL) {
-				faultcode = tmp->children->content;
+				faultcode = (char*)tmp->children->content;
 			}
 
-			tmp = get_node(fault->children,"faultstring");
+			tmp = get_node(fault->children, "faultstring");
 			if (tmp != NULL && tmp->children != NULL) {
-				faultstring = tmp->children->content;
+				zval *zv = master_to_zval(get_conversion(IS_STRING), tmp);
+				faultstring = Z_STRVAL_P(zv);
+				FREE_ZVAL(zv);
 			}
 
-			tmp = get_node(fault->children,"faultactor");
+			tmp = get_node(fault->children, "faultactor");
 			if (tmp != NULL && tmp->children != NULL) {
-				faultactor = tmp->children->content;
+				zval *zv = master_to_zval(get_conversion(IS_STRING), tmp);
+				faultactor = Z_STRVAL_P(zv);
+				FREE_ZVAL(zv);
 			}
 
-			tmp = get_node(fault->children,"detail");
+			tmp = get_node(fault->children, "detail");
 			if (tmp != NULL) {
 				details = master_to_zval(NULL, tmp);
 			}
 		} else {
-			tmp = get_node(fault->children,"Code");
+			tmp = get_node(fault->children, "Code");
 			if (tmp != NULL && tmp->children != NULL) {
-				tmp = get_node(tmp->children,"Value");
+				tmp = get_node(tmp->children, "Value");
 				if (tmp != NULL && tmp->children != NULL) {
-					faultcode = tmp->children->content;
+					faultcode = (char*)tmp->children->content;
 				}
 			}
 
@@ -217,7 +221,9 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 				/* TODO: lang attribute */
 				tmp = get_node(tmp->children,"Text");
 				if (tmp != NULL && tmp->children != NULL) {
-					faultstring = tmp->children->content;
+					zval *zv = master_to_zval(get_conversion(IS_STRING), tmp);
+					faultstring = Z_STRVAL_P(zv);
+					FREE_ZVAL(zv);
 				}
 			}
 
@@ -227,9 +233,15 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 			}
 		}
 		add_soap_fault(this_ptr, faultcode, faultstring, faultactor, details TSRMLS_CC);
+		if (faultstring) {
+			efree(faultstring);
+		}
+		if (faultactor) {
+			efree(faultactor);
+		}
 #ifdef ZEND_ENGINE_2
 		if (details) {
-			details->refcount--;
+			Z_DELREF_P(details);
 		}
 #endif
 		xmlFreeDoc(response);
@@ -338,10 +350,22 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 				if (val != NULL) {
 					if (!node_is_equal_ex(val,"result",RPC_SOAP12_NAMESPACE)) {
 						zval *tmp;
+						zval **arr;
 
 						tmp = master_to_zval(NULL, val);
 						if (val->name) {
-							add_assoc_zval(return_value, (char*)val->name, tmp);
+							if (zend_hash_find(Z_ARRVAL_P(return_value), (char*)val->name, strlen((char*)val->name)+1, (void**)&arr) == SUCCESS) {
+								add_next_index_zval(*arr, tmp);
+							} else if (val->next && get_node(val->next, (char*)val->name)) {
+								zval *arr;
+
+								MAKE_STD_ZVAL(arr);
+								array_init(arr);
+								add_next_index_zval(arr, tmp);
+								add_assoc_zval(return_value, (char*)val->name, arr);
+							} else {
+								add_assoc_zval(return_value, (char*)val->name, tmp);
+							}
 						} else {
 							add_next_index_zval(return_value, tmp);
 						}
@@ -363,7 +387,7 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 			zend_hash_internal_pointer_reset(Z_ARRVAL_P(return_value));
 			zend_hash_get_current_data(Z_ARRVAL_P(return_value), (void**)&tmp);
 			tmp = *(zval**)tmp;
-			tmp->refcount++;
+			Z_ADDREF_P(tmp);
 			zval_dtor(return_value);
 			*return_value = *tmp;
 			FREE_ZVAL(tmp);
@@ -382,10 +406,10 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 					sdlSoapBindingFunctionHeaderPtr *hdr;
 
 					if (trav->ns) {
-						smart_str_appends(&key,trav->ns->href);
+						smart_str_appends(&key, (char*)trav->ns->href);
 						smart_str_appendc(&key,':');
 					}
-					smart_str_appends(&key,trav->name);
+					smart_str_appends(&key, (char*)trav->name);
 					smart_str_0(&key);
 					if (zend_hash_find(hdrs, key.c, key.len+1, (void**)&hdr) == SUCCESS) {
 						enc = (*hdr)->encode;
