@@ -2,12 +2,12 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2004 The PHP Group                                |
+  | Copyright (c) 1997-2009 The PHP Group                                |
   +----------------------------------------------------------------------+
-  | This source file is subject to version 3.0 of the PHP license,       |
+  | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
-  | http://www.php.net/license/3_0.txt.                                  |
+  | http://www.php.net/license/3_01.txt                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
@@ -17,7 +17,7 @@
   |          Dmitry Stogov <dmitry@zend.com>                             |
   +----------------------------------------------------------------------+
 */
-/* $Id: php_sdl.h,v 1.31.2.2 2004/11/09 08:13:04 dmitry Exp $ */
+/* $Id: php_sdl.h,v 1.37.2.3.2.2.2.2 2008/12/31 11:15:43 sebastian Exp $ */
 
 #ifndef PHP_SDL_H
 #define PHP_SDL_H
@@ -61,6 +61,7 @@ struct _sdl {
 	HashTable *groups;           /* array of sdlTypesPtr */
 	char      *target_ns;
 	char      *source;
+	zend_bool  is_persistent;
 };
 
 typedef struct sdlCtx {
@@ -109,7 +110,6 @@ typedef struct _sdlSoapBindingFunctionFault {
 struct _sdlSoapBindingFunctionBody {
 	char                *ns;
 	sdlEncodingUse       use;
-	char                *parts;          /* not implemented yet */
 	sdlRpcEncodingStyle  encodingStyle;  /* not implemented yet */
 	HashTable           *headers;        /* array of sdlSoapBindingFunctionHeaderPtr */
 };
@@ -153,7 +153,8 @@ typedef enum _sdlContentKind {
 	XSD_CONTENT_ALL,
 	XSD_CONTENT_CHOICE,
 	XSD_CONTENT_GROUP_REF,
-	XSD_CONTENT_GROUP
+	XSD_CONTENT_GROUP,
+	XSD_CONTENT_ANY
 } sdlContentKind;
 
 
@@ -180,6 +181,19 @@ typedef enum _sdlTypeKind {
 	XSD_TYPEKIND_EXTENSION
 } sdlTypeKind;
 
+typedef enum _sdlUse {
+	XSD_USE_DEFAULT,
+	XSD_USE_OPTIONAL,
+	XSD_USE_PROHIBITED,
+	XSD_USE_REQUIRED
+} sdlUse;
+
+typedef enum _sdlForm {
+	XSD_FORM_DEFAULT,
+	XSD_FORM_QUALIFIED,
+	XSD_FORM_UNQUALIFIED
+} sdlForm;
+
 struct _sdlType {
 	sdlTypeKind         kind;
 	char               *name;
@@ -193,6 +207,7 @@ struct _sdlType {
 	char               *def;
 	char               *fixed;
 	char               *ref;
+	sdlForm             form;
 };
 
 struct _sdlParam {
@@ -219,19 +234,6 @@ struct _sdlFunction {
 	HashTable          *faults;             /* array of sdlFaultPtr */
 };
 
-typedef enum _sdlUse {
-	XSD_USE_DEFAULT,
-	XSD_USE_OPTIONAL,
-	XSD_USE_PROHIBITED,
-	XSD_USE_REQUIRED
-} sdlUse;
-
-typedef enum _sdlForm {
-	XSD_FORM_DEFAULT,
-	XSD_FORM_QUALIFIED,
-	XSD_FORM_UNQUALIFIED
-} sdlForm;
-
 typedef struct _sdlExtraAttribute {
 	char *ns;
 	char *val;
@@ -249,9 +251,10 @@ struct _sdlAttribute {
 	encodePtr  encode;
 };
 
-sdlPtr get_sdl(zval *this_ptr, char *uri TSRMLS_DC);
 
-encodePtr get_encoder_from_prefix(sdlPtr sdl, xmlNodePtr data, const char *type);
+sdlPtr get_sdl(zval *this_ptr, char *uri, long cache_wsdl TSRMLS_DC);
+
+encodePtr get_encoder_from_prefix(sdlPtr sdl, xmlNodePtr data, const xmlChar *type);
 encodePtr get_encoder(sdlPtr sdl, const char *ns, const char *type);
 encodePtr get_encoder_ex(sdlPtr sdl, const char *nscat, int len);
 
@@ -259,5 +262,6 @@ sdlBindingPtr get_binding_from_type(sdlPtr sdl, int type);
 sdlBindingPtr get_binding_from_name(sdlPtr sdl, char *name, char *ns);
 
 void delete_sdl(void *handle);
+void delete_sdl_impl(void *handle);
 
 #endif
