@@ -16,7 +16,7 @@
 ** sqliteRegisterDateTimeFunctions() found at the bottom of the file.
 ** All other code has file scope.
 **
-** $Id: date.c,v 1.3.4.1 2005/09/07 15:11:32 iliaa Exp $
+** $Id: date.c,v 1.3.4.1.2.3.2.2 2009/04/07 11:45:13 kalle Exp $
 **
 ** NOTES:
 **
@@ -53,6 +53,9 @@
 #include <stdlib.h>
 #include <assert.h>
 #include <time.h>
+#ifndef PHP_WIN32
+#include "main/php_reentrancy.h"
+#endif
 
 #ifndef SQLITE_OMIT_DATETIME_FUNCS
 
@@ -397,7 +400,7 @@ static void clearYMD_HMS_TZ(DateTime *p){
 static double localtimeOffset(DateTime *p){
   DateTime x, y;
   time_t t;
-  struct tm *pTm;
+  struct tm *pTm, tmbuf;
   x = *p;
   computeYMD_HMS(&x);
   if( x.Y<1971 || x.Y>=2038 ){
@@ -416,7 +419,10 @@ static double localtimeOffset(DateTime *p){
   computeJD(&x);
   t = (x.rJD-2440587.5)*86400.0 + 0.5;
   sqliteOsEnterMutex();
-  pTm = localtime(&t);
+  pTm = php_localtime_r(&t, &tmbuf);
+  if (!pTm) {
+	  return 0;
+  }
   y.Y = pTm->tm_year + 1900;
   y.M = pTm->tm_mon + 1;
   y.D = pTm->tm_mday;
