@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2008 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,13 +17,19 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: basic_functions.h,v 1.109.2.2.4.4 2007/12/31 07:22:52 sebastian Exp $ */
+/* $Id: basic_functions.h,v 1.139.2.4.2.6.2.11 2008/12/31 11:15:44 sebastian Exp $ */
 
 #ifndef BASIC_FUNCTIONS_H
 #define BASIC_FUNCTIONS_H
 
 #include <sys/stat.h>
 
+#ifdef HAVE_WCHAR_H
+#include <wchar.h>
+#endif
+
+#include "php_filestat.h"
+
 #include "zend_highlight.h"
 
 #include "url_scanner.h"
@@ -39,10 +45,19 @@ PHP_RSHUTDOWN_FUNCTION(basic);
 PHP_MINFO_FUNCTION(basic);
 
 PHP_FUNCTION(constant);
-PHP_FUNCTION(toggle_short_open_tag);
 PHP_FUNCTION(sleep);
 PHP_FUNCTION(usleep);
+#if HAVE_NANOSLEEP
+PHP_FUNCTION(time_nanosleep);
+PHP_FUNCTION(time_sleep_until);
+#endif
 PHP_FUNCTION(flush);
+#ifdef HAVE_INET_NTOP
+PHP_NAMED_FUNCTION(php_inet_ntop);
+#endif
+#ifdef HAVE_INET_PTON
+PHP_NAMED_FUNCTION(php_inet_pton);
+#endif
 PHP_FUNCTION(ip2long);
 PHP_FUNCTION(long2ip);
 
@@ -63,15 +78,19 @@ PHP_FUNCTION(get_magic_quotes_gpc);
 PHP_FUNCTION(import_request_variables);
 
 PHP_FUNCTION(error_log);
+PHP_FUNCTION(error_get_last);
 
 PHP_FUNCTION(call_user_func);
 PHP_FUNCTION(call_user_func_array);
 PHP_FUNCTION(call_user_method);
 PHP_FUNCTION(call_user_method_array);
+PHP_FUNCTION(forward_static_call);
+PHP_FUNCTION(forward_static_call_array);
 
 PHP_FUNCTION(register_shutdown_function);
 PHP_FUNCTION(highlight_file);
 PHP_FUNCTION(highlight_string);
+PHP_FUNCTION(php_strip_whitespace);
 ZEND_API void php_get_highlight_struct(zend_syntax_highlighter_ini *syntax_highlighter_ini);
 
 PHP_FUNCTION(ini_get);
@@ -83,6 +102,8 @@ PHP_FUNCTION(set_include_path);
 PHP_FUNCTION(restore_include_path);
 
 PHP_FUNCTION(print_r);
+PHP_FUNCTION(fprintf);
+PHP_FUNCTION(vfprintf);
 
 PHP_FUNCTION(connection_aborted);
 PHP_FUNCTION(connection_status);
@@ -97,22 +118,32 @@ PHP_NAMED_FUNCTION(php_if_crc32);
 
 PHP_FUNCTION(register_tick_function);
 PHP_FUNCTION(unregister_tick_function);
+#ifdef HAVE_GETLOADAVG
+PHP_FUNCTION(sys_getloadavg);
+#endif
 
 PHP_FUNCTION(is_uploaded_file);
 PHP_FUNCTION(move_uploaded_file);
 
 /* From the INI parser */
 PHP_FUNCTION(parse_ini_file);
-
-PHP_FUNCTION(str_rot13);
-
-#ifdef PHP_WIN32
-typedef unsigned int php_stat_len;
-#else
-typedef int php_stat_len;
+PHP_FUNCTION(parse_ini_string);
+#if ZEND_DEBUG
+PHP_FUNCTION(config_get_hash);
 #endif
 
+PHP_FUNCTION(str_rot13);
+PHP_FUNCTION(stream_get_filters);
+PHP_FUNCTION(stream_filter_register);
+PHP_FUNCTION(stream_bucket_make_writeable);
+PHP_FUNCTION(stream_bucket_prepend);
+PHP_FUNCTION(stream_bucket_append);
+PHP_FUNCTION(stream_bucket_new);
+PHP_MINIT_FUNCTION(user_filters);
+PHP_RSHUTDOWN_FUNCTION(user_filters);
+
 PHPAPI int _php_error_log(int opt_err, char *message, char *opt, char *headers TSRMLS_DC);
+PHPAPI int php_prefix_varname(zval *result, zval *prefix, char *var_name, int var_name_len, zend_bool add_underscore TSRMLS_DC);
 
 #if SIZEOF_INT == 4
 /* Most 32-bit and 64-bit systems have 32-bit ints */
@@ -128,7 +159,7 @@ typedef signed long php_int32;
 
 #define MT_N (624)
 
-typedef struct {
+typedef struct _php_basic_globals {
 	HashTable *user_shutdown_function_names;
 	HashTable putenv_ht;
 	zval *strtok_zval;
@@ -138,8 +169,10 @@ typedef struct {
 	char strtok_table[256];
 	ulong strtok_len;
 	char str_ebuf[40];
-	zval **array_walk_func_name;
-	zval **user_compare_func_name;
+	zend_fcall_info array_walk_fci;
+	zend_fcall_info_cache array_walk_fci_cache;
+	zend_fcall_info user_compare_fci;
+	zend_fcall_info_cache user_compare_fci_cache;
 	zend_llist *user_tick_functions;
 
 	zval *active_ini_file_section;
@@ -151,13 +184,11 @@ typedef struct {
 	long page_uid;
 	long page_gid;
 	long page_inode;
-	long page_mtime;
+	time_t page_mtime;
 
-	/* filestat.c */
-	char *CurrentStatFile;
-	php_stat_len CurrentStatLength;
-	struct stat sb;
-	struct stat lsb;
+	/* filestat.c && main/streams/streams.c */
+	char *CurrentStatFile, *CurrentLStatFile;
+	php_stream_statbuf ssb, lssb;
 
 	/* rand.c */
 	php_uint32   state[MT_N+1];  /* state vector + 1 extra to not violate ANSI C */
@@ -186,15 +217,22 @@ typedef struct {
 	size_t mmap_len;
 #endif
 
-	HashTable *aggregation_table;
+	HashTable *user_filter_map;
+
+	/* file.c */
+#if defined(_REENTRANT) && defined(HAVE_MBRLEN) && defined(HAVE_MBSTATE_T)
+	mbstate_t mblen_state;
+#endif
+
+	int umask;
 } php_basic_globals;
 
 #ifdef ZTS
 #define BG(v) TSRMG(basic_globals_id, php_basic_globals *, v)
-extern int basic_globals_id;
+PHPAPI extern int basic_globals_id;
 #else
 #define BG(v) (basic_globals.v)
-extern php_basic_globals basic_globals;
+PHPAPI extern php_basic_globals basic_globals;
 #endif
 
 #if HAVE_PUTENV
@@ -211,7 +249,7 @@ typedef struct {
 #define SAFE_MODE_PROTECTED_ENV_VARS	"LD_LIBRARY_PATH"
 #define SAFE_MODE_ALLOWED_ENV_VARS		"PHP_"
 
-PHPAPI double php_get_nan();
-PHPAPI double php_get_inf();
+PHPAPI double php_get_nan(void);
+PHPAPI double php_get_inf(void);
 
 #endif /* BASIC_FUNCTIONS_H */
