@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: crc32.c,v 1.12.8.1.8.1 2006/01/01 13:46:57 sniper Exp $ */
+/* $Id: crc32.c,v 1.16.2.4.2.1.2.2 2008/12/31 11:15:44 sebastian Exp $ */
 
 #include "php.h"
 #include "basic_functions.h"
@@ -26,19 +26,20 @@
    Calculate the crc32 polynomial of a string */
 PHP_NAMED_FUNCTION(php_if_crc32)
 {
-	unsigned int crc = ~0;
 	char *p;
 	int len, nr;
-	
+	php_uint32 crcinit = 0;
+	register php_uint32 crc;
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &p, &nr) == FAILURE) {
 		return;
 	}
+	crc = crcinit^0xFFFFFFFF;
 
-	len = 0 ;
-	for (len += nr; nr--; ++p) {
-	    CRC32(crc, *p);
+	for (len =+nr; nr--; ++p) {
+		crc = ((crc >> 8) & 0x00FFFFFF) ^ crc32tab[(crc ^ (*p)) & 0xFF ];
 	}
-	RETVAL_LONG(~crc);
+	RETVAL_LONG(crc^0xFFFFFFFF);
 }
 /* }}} */
 
