@@ -1,8 +1,8 @@
 /* 
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,16 +18,16 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: dl.h,v 1.16.8.1.8.2 2007/01/01 09:46:48 sebastian Exp $ */
+/* $Id: dl.h,v 1.23.2.1.2.1.2.5 2008/12/31 18:55:22 helly Exp $ */
 
 #ifndef DL_H
 #define DL_H
 
-void php_dl(pval *file,int type,pval *return_value TSRMLS_DC);
-
+PHPAPI int php_load_extension(char *filename, int type, int start_now TSRMLS_DC);
+PHPAPI void php_dl(char *file, int type, zval *return_value, int start_now TSRMLS_DC);
 
 /* dynamic loading functions */
-PHP_FUNCTION(dl);
+PHPAPI PHP_FUNCTION(dl);
 
 PHP_MINFO_FUNCTION(dl);
 
