@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: flock_compat.c,v 1.29.2.1 2006/01/01 12:50:14 sniper Exp $ */
+/* $Id: flock_compat.c,v 1.29.2.1.2.1.2.3 2008/12/31 11:15:45 sebastian Exp $ */
 
 #include "php.h"
 #include <errno.h>
@@ -30,6 +30,7 @@
 
 #ifdef PHP_WIN32
 #include <io.h>
+#include "config.w32.h"
 #endif
 
 #ifdef NETWARE
@@ -44,7 +45,7 @@ PHPAPI int flock(int fd, int operation)
 #endif /* !defined(HAVE_FLOCK) */
 
 PHPAPI int php_flock(int fd, int operation)
-#if HAVE_STRUCT_FLOCK
+#if HAVE_STRUCT_FLOCK /* {{{ */
 {
 	struct flock flck;
 	int ret;
@@ -73,7 +74,8 @@ PHPAPI int php_flock(int fd, int operation)
 
 	return ret;
 }
-#elif defined(PHP_WIN32)
+/* }}} */
+#elif defined(PHP_WIN32) /* {{{ */
 /*
  * Program:   Unix compatibility routines
  *
@@ -152,6 +154,7 @@ PHPAPI int php_flock(int fd, int operation)
 #endif
     return -1;
 }
+/* }}} */
 #else
 #warning no proper flock support for your site
 {
@@ -160,6 +163,7 @@ PHPAPI int php_flock(int fd, int operation)
 }
 #endif
 
+#ifndef PHP_WIN32
 #if !(HAVE_INET_ATON)
 /* {{{ inet_aton
  * Check whether "cp" is a valid ascii representation
@@ -223,6 +227,7 @@ int inet_aton(const char *cp, struct in_addr *ap)
 }
 /* }}} */
 #endif /* !HAVE_INET_ATON */
+#endif
 
 /*
  * Local variables:
