@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: iptc.c,v 1.50.2.2.2.4 2007/02/12 20:40:11 tony2001 Exp $ */
+/* $Id: iptc.c,v 1.50.2.2.2.5.2.2 2008/12/31 11:15:45 sebastian Exp $ */
 
 /*
  * Functions to parse & compse IPTC data.
@@ -212,16 +212,16 @@ PHP_FUNCTION(iptcembed)
 
 	if (php_iptc_get1(fp, spool, poi?&poi:0 TSRMLS_CC) != 0xFF) {
 		fclose(fp);
-		if (poi) {
-			efree(poi);
+		if (spoolbuf) {
+			efree(spoolbuf);
 		}
 		RETURN_FALSE;
 	}
 
 	if (php_iptc_get1(fp, spool, poi?&poi:0 TSRMLS_CC) != 0xD8) {
 		fclose(fp);
-		if (poi) {
-			efree(poi);
+		if (spoolbuf) {
+			efree(spoolbuf);
 		}
 		RETURN_FALSE;
 	}
