@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,13 +16,15 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: mail.c,v 1.66.2.12.4.6 2007/03/30 00:29:32 iliaa Exp $ */
+/* $Id: mail.c,v 1.87.2.1.2.7.2.11 2009/06/21 15:29:16 iliaa Exp $ */
 
 #include <stdlib.h>
 #include <ctype.h>
 #include <stdio.h>
 #include "php.h"
 #include "ext/standard/info.h"
+#include "ext/standard/php_string.h"
+#include "ext/standard/basic_functions.h"
 
 #if HAVE_SYSEXITS_H
 #include <sysexits.h>
@@ -31,12 +33,17 @@
 #include <sys/sysexits.h>
 #endif
 
+#if PHP_SIGCHILD
+#if HAVE_SIGNAL_H
+#include <signal.h>
+#endif
+#endif
+
 #include "php_mail.h"
 #include "php_ini.h"
 #include "safe_mode.h"
 #include "exec.h"
 
-#if HAVE_SENDMAIL
 #ifdef PHP_WIN32
 #include "win32/sendmail.h"
 #endif
@@ -46,42 +53,42 @@
 #define EX_TEMPFAIL     75      /* temp failure; user is invited to retry */
 #endif
 
-#define SKIP_LONG_HEADER_SEP(str, pos)										\
+#define SKIP_LONG_HEADER_SEP(str, pos)																	\
 	if (str[pos] == '\r' && str[pos + 1] == '\n' && (str[pos + 2] == ' ' || str[pos + 2] == '\t')) {	\
-		pos += 2;											\
-		while (str[pos + 1] == ' ' || str[pos + 1] == '\t') {							\
-			pos++;											\
-		}												\
-		continue;											\
-	}													\
+		pos += 2;																						\
+		while (str[pos + 1] == ' ' || str[pos + 1] == '\t') {											\
+			pos++;																						\
+		}																								\
+		continue;																						\
+	}																									\
 
-#define MAIL_ASCIIZ_CHECK(str, len)			\
-	p = str;					\
-	e = p + len;					\
+#define MAIL_ASCIIZ_CHECK(str, len)				\
+	p = str;									\
+	e = p + len;								\
 	while ((p = memchr(p, '\0', (e - p)))) {	\
-		*p = ' ';				\
-	}						\
+		*p = ' ';								\
+	}											\
 
+extern long php_getuid(void);
 
 /* {{{ proto int ezmlm_hash(string addr)
    Calculate EZMLM list hash value. */
 PHP_FUNCTION(ezmlm_hash)
 {
 	char *str = NULL;
-	unsigned long h = 5381L;
+	unsigned int h = 5381;
 	int j, str_len;
-	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s",
-							  &str, &str_len) == FAILURE) {
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &str, &str_len) == FAILURE) {
 		return;
 	}
 
 	for (j = 0; j < str_len; j++) {
 		h = (h + (h << 5)) ^ (unsigned long) (unsigned char) tolower(str[j]);
 	}
-	
+
 	h = (h % 53);
-	
+
 	RETURN_LONG((int) h);
 }
 /* }}} */
@@ -92,23 +99,20 @@ PHP_FUNCTION(mail)
 {
 	char *to=NULL, *message=NULL, *headers=NULL;
 	char *subject=NULL, *extra_cmd=NULL;
-	int to_len, message_len, headers_len;
-	int subject_len, extra_cmd_len, i;
+	int to_len, message_len, headers_len = 0;
+	int subject_len, extra_cmd_len = 0, i;
+	char *force_extra_parameters = INI_STR("mail.force_extra_parameters");
 	char *to_r, *subject_r;
 	char *p, *e;
 
 	if (PG(safe_mode) && (ZEND_NUM_ARGS() == 5)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "SAFE MODE Restriction in effect.  The fifth parameter is disabled in SAFE MODE.");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "SAFE MODE Restriction in effect.  The fifth parameter is disabled in SAFE MODE");
 		RETURN_FALSE;
-	}	
-	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sss|ss",
-							  &to, &to_len,
-							  &subject, &subject_len,
-							  &message, &message_len,
-							  &headers, &headers_len,
-							  &extra_cmd, &extra_cmd_len
-							  ) == FAILURE) {
+	}
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sss|ss",	&to, &to_len, &subject, &subject_len, &message, &message_len,
+																	&headers, &headers_len, &extra_cmd, &extra_cmd_len) == FAILURE
+	) {
 		return;
 	}
 
@@ -136,8 +140,7 @@ PHP_FUNCTION(mail)
 				/* According to RFC 822, section 3.1.1 long headers may be separated into
 				 * parts using CRLF followed at least one linear-white-space character ('\t' or ' ').
 				 * To prevent these separators from being replaced with a space, we use the
-				 * SKIP_LONG_HEADER_SEP to skip over them.
-				 */
+				 * SKIP_LONG_HEADER_SEP to skip over them. */
 				SKIP_LONG_HEADER_SEP(to_r, i);
 				to_r[i] = ' ';
 			}
@@ -154,7 +157,7 @@ PHP_FUNCTION(mail)
 			}
 			subject_r[subject_len - 1] = '\0';
 		}
-		for(i = 0; subject[i]; i++) {
+		for (i = 0; subject_r[i]; i++) {
 			if (iscntrl((unsigned char) subject_r[i])) {
 				SKIP_LONG_HEADER_SEP(subject_r, i);
 				subject_r[i] = ' ';
@@ -164,7 +167,9 @@ PHP_FUNCTION(mail)
 		subject_r = subject;
 	}
 
-	if (extra_cmd) {
+	if (force_extra_parameters) {
+		extra_cmd = php_escape_shell_cmd(force_extra_parameters);
+	} else if (extra_cmd) {
 		extra_cmd = php_escape_shell_cmd(extra_cmd);
 	}
 
@@ -198,33 +203,84 @@ PHPAPI int php_mail(char *to, char *subject, char *message, char *headers, char
 	int ret;
 	char *sendmail_path = INI_STR("sendmail_path");
 	char *sendmail_cmd = NULL;
+	char *mail_log = INI_STR("mail.log");
+	char *hdr = headers;
+#if PHP_SIGCHILD
+	void (*sig_handler)() = NULL;
+#endif
+
+#define MAIL_RET(val) \
+	if (hdr != headers) {	\
+		efree(hdr);	\
+	}	\
+	return val;	\
+
+	if (mail_log) {
+		char *tmp;
+		int l = spprintf(&tmp, 0, "mail() on [%s:%d]: To: %s -- Headers: %s\n", zend_get_executed_filename(TSRMLS_C), zend_get_executed_lineno(TSRMLS_C), to, hdr ? hdr : "");
+		php_stream *stream = php_stream_open_wrapper(mail_log, "a", IGNORE_URL_WIN | REPORT_ERRORS | STREAM_DISABLE_OPEN_BASEDIR, NULL);
+
+		if (hdr) { /* find all \r\n instances and replace them with spaces, so a log line is always one line long */ 
+			char *p = tmp;
+			while ((p = strpbrk(p, "\r\n"))) {
+				*p = ' ';
+			}
+			tmp[l - 1] = '\n';
+		}
+		if (stream) {
+			php_stream_write(stream, tmp, l);
+			php_stream_close(stream);
+		}
+		efree(tmp);
+	}
+	if (PG(mail_x_header)) {
+		char *tmp = zend_get_executed_filename(TSRMLS_C);
+		char *f;
+		size_t f_len;
+
+		php_basename(tmp, strlen(tmp), NULL, 0,&f, &f_len TSRMLS_CC);
+
+		if (headers != NULL) {
+			spprintf(&hdr, 0, "X-PHP-Originating-Script: %ld:%s\r\n%s", php_getuid(), f, headers);
+		} else {
+			spprintf(&hdr, 0, "X-PHP-Originating-Script: %ld:%s\n", php_getuid(), f);
+		}
+		efree(f);
+	}
 
 	if (!sendmail_path) {
 #if (defined PHP_WIN32 || defined NETWARE)
 		/* handle old style win smtp sending */
-		if (TSendMail(INI_STR("SMTP"), &tsm_err, &tsm_errmsg, headers, subject, to, message, NULL, NULL, NULL) == FAILURE) {
+		if (TSendMail(INI_STR("SMTP"), &tsm_err, &tsm_errmsg, hdr, subject, to, message, NULL, NULL, NULL TSRMLS_CC) == FAILURE) {
 			if (tsm_errmsg) {
 				php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", tsm_errmsg);
 				efree(tsm_errmsg);
 			} else {
 				php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", GetSMErrorText(tsm_err));
 			}
-			return 0;
+			MAIL_RET(0);
 		}
-		return 1;
+		MAIL_RET(1);
 #else
-		return 0;
+		MAIL_RET(0);
 #endif
 	}
 	if (extra_cmd != NULL) {
-		sendmail_cmd = emalloc (strlen (sendmail_path) + strlen (extra_cmd) + 2);
-		strcpy (sendmail_cmd, sendmail_path);
-		strcat (sendmail_cmd, " ");
-		strcat (sendmail_cmd, extra_cmd);
+		spprintf(&sendmail_cmd, 0, "%s %s", sendmail_path, extra_cmd);
 	} else {
 		sendmail_cmd = sendmail_path;
 	}
 
+#if PHP_SIGCHILD
+	/* Set signal handler of SIGCHLD to default to prevent other signal handlers
+	 * from being called and reaping the return code when our child exits.
+	 * The original handler needs to be restored after pclose() */
+	sig_handler = (void *)signal(SIGCHLD, SIG_DFL);
+	if (sig_handler == SIG_ERR) {
+		sig_handler = NULL;
+	}
+#endif
+
 #ifdef PHP_WIN32
 	sendmail = popen(sendmail_cmd, "wb");
 #else
@@ -234,24 +290,39 @@ PHPAPI int php_mail(char *to, char *subject, char *message, char *headers, char
 	errno = 0;
 	sendmail = popen(sendmail_cmd, "w");
 #endif
-	if (extra_cmd != NULL)
+	if (extra_cmd != NULL) {
 		efree (sendmail_cmd);
+	}
 
 	if (sendmail) {
 #ifndef PHP_WIN32
 		if (EACCES == errno) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Permission denied: unable to execute shell to run mail delivery binary '%s'", sendmail_path);
 			pclose(sendmail);
-			return 0;
+#if PHP_SIGCHILD
+			/* Restore handler in case of error on Windows
+			   Not sure if this applicable on Win but just in case. */
+			if (sig_handler) {
+				signal(SIGCHLD, sig_handler);
+			}
+#endif
+			MAIL_RET(0);
 		}
 #endif
 		fprintf(sendmail, "To: %s\n", to);
 		fprintf(sendmail, "Subject: %s\n", subject);
-		if (headers != NULL) {
-			fprintf(sendmail, "%s\n", headers);
+		if (hdr != NULL) {
+			fprintf(sendmail, "%s\n", hdr);
 		}
 		fprintf(sendmail, "\n%s\n", message);
 		ret = pclose(sendmail);
+
+#if PHP_SIGCHILD
+		if (sig_handler) {
+			signal(SIGCHLD, sig_handler);
+		}
+#endif
+
 #ifdef PHP_WIN32
 		if (ret == -1)
 #else
@@ -264,16 +335,21 @@ PHPAPI int php_mail(char *to, char *subject, char *message, char *headers, char
 #endif
 #endif
 		{
-			return 0;
+			MAIL_RET(0);
 		} else {
-			return 1;
+			MAIL_RET(1);
 		}
 	} else {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not execute mail delivery program '%s'", sendmail_path);
-		return 0;
+#if PHP_SIGCHILD
+		if (sig_handler) {
+			signal(SIGCHLD, sig_handler);						
+		}
+#endif
+		MAIL_RET(0);
 	}
 
-	return 1; /* never reached */
+	MAIL_RET(1); /* never reached */
 }
 /* }}} */
 
@@ -295,13 +371,6 @@ PHP_MINFO_FUNCTION(mail)
 }
 /* }}} */
 
-#else
-
-PHP_FUNCTION(mail) {}
-PHP_MINFO_FUNCTION(mail) {}
-
-#endif
-
 /*
  * Local variables:
  * tab-width: 4
