@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -15,7 +15,7 @@
    | Author: Chris Schneider <cschneid@relog.ch>                          |
    +----------------------------------------------------------------------+
  */
-/* $Id: pack.c,v 1.57.2.5.2.5 2007/06/06 22:04:04 iliaa Exp $ */
+/* $Id: pack.c,v 1.57.2.5.2.6.2.6 2009/05/12 12:34:55 iliaa Exp $ */
 
 #include "php.h"
 
@@ -56,6 +56,9 @@
 
 #define INC_OUTPUTPOS(a,b) \
 	if ((a) < 0 || ((INT_MAX - outputpos)/((int)b)) < (a)) { \
+		efree(argv);	\
+		efree(formatcodes);	\
+		efree(formatargs);	\
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Type %c: integer overflow in format string", code); \
 		RETURN_FALSE; \
 	} \
@@ -103,8 +106,8 @@ static void php_pack(zval **val, int size, int *map, char *output)
    Takes one or more arguments and packs them into a binary string according to the format argument */
 PHP_FUNCTION(pack)
 {
-	zval ***argv;
-	int argc, i;
+	zval ***argv = NULL;
+	int num_args, i;
 	int currentarg;
 	char *format;
 	int formatlen;
@@ -114,20 +117,12 @@ PHP_FUNCTION(pack)
 	int outputpos = 0, outputsize = 0;
 	char *output;
 
-	argc = ZEND_NUM_ARGS();
-
-	if (argc < 1) {
-		WRONG_PARAM_COUNT;
-	}
-
-	argv = safe_emalloc(argc, sizeof(zval **), 0);
-
-	if (zend_get_parameters_array_ex(argc, argv) == FAILURE) {
-		efree(argv);
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "+", &argv, &num_args) == FAILURE) {
+		return;
 	}
 
 	convert_to_string_ex(argv[0]);
+
 	format = Z_STRVAL_PP(argv[0]);
 	formatlen = Z_STRLEN_PP(argv[0]);
 
@@ -175,7 +170,7 @@ PHP_FUNCTION(pack)
 			case 'A': 
 			case 'h': 
 			case 'H':
-				if (currentarg >= argc) {
+				if (currentarg >= num_args) {
 					efree(argv);
 					efree(formatcodes);
 					efree(formatargs);
@@ -207,12 +202,12 @@ PHP_FUNCTION(pack)
 			case 'f': 
 			case 'd': 
 				if (arg < 0) {
-					arg = argc - currentarg;
+					arg = num_args - currentarg;
 				}
 
 				currentarg += arg;
 
-				if (currentarg > argc) {
+				if (currentarg > num_args) {
 					efree(argv);
 					efree(formatcodes);
 					efree(formatargs);
@@ -233,8 +228,8 @@ PHP_FUNCTION(pack)
 		formatargs[formatcount] = arg;
 	}
 
-	if (currentarg < argc) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%d arguments unused", (argc - currentarg));
+	if (currentarg < num_args) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%d arguments unused", (num_args - currentarg));
 	}
 
 	/* Calculate output length and upper bound while processing*/
@@ -511,26 +506,19 @@ static long php_unpack(char *data, int size, int issigned, int *map)
    Unpack binary string into named array elements according to format argument */
 PHP_FUNCTION(unpack)
 {
-	zval **formatarg;
-	zval **inputarg;
-	char *format;
-	char *input;
-	int formatlen;
-	int inputpos, inputlen;
-	int i;
+	char *format, *input, *formatarg, *inputarg;
+	int formatlen, formatarg_len, inputarg_len;
+	int inputpos, inputlen, i;
 
-	if (ZEND_NUM_ARGS() != 2 || 
-        zend_get_parameters_ex(2, &formatarg, &inputarg) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", &formatarg, &formatarg_len,
+		&inputarg, &inputarg_len) == FAILURE) {
+		return;
 	}
 
-	convert_to_string_ex(formatarg);
-	convert_to_string_ex(inputarg);
-
-	format = Z_STRVAL_PP(formatarg);
-	formatlen = Z_STRLEN_PP(formatarg);
-	input = Z_STRVAL_PP(inputarg);
-	inputlen = Z_STRLEN_PP(inputarg);
+	format = formatarg;
+	formatlen = formatarg_len;
+	input = inputarg;
+	inputlen = inputarg_len;
 	inputpos = 0;
 
 	array_init(return_value);
@@ -796,6 +784,13 @@ PHP_FUNCTION(unpack)
 						}
 
 						v |= php_unpack(&input[inputpos], 4, issigned, map);
+						if (sizeof(long) > 4) {
+ 							if (type == 'l') {
+								v = (signed int) v; 
+							} else {
+								v = (unsigned int) v;
+							}
+						}
 						add_assoc_long(return_value, n, v);
 						break;
 					}
