@@ -1,8 +1,8 @@
 /* 
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,11 +12,11 @@
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
-   | Author: Stig Sther Bakken <ssb@fast.no>                             |
+   | Author: Stig Sther Bakken <ssb@php.net>                             |
    +----------------------------------------------------------------------+
 */
 
-/* $Id: php_versioning.h,v 1.5.8.1.8.2 2007/01/01 09:46:48 sebastian Exp $ */
+/* $Id: php_versioning.h,v 1.10.2.1.2.1.2.2 2008/12/31 11:15:45 sebastian Exp $ */
 
 #ifndef PHP_VERSIONING_H
 #define PHP_VERSIONING_H
