@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2008 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,11 +12,11 @@
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
-   | Author: Stig Sther Bakken <ssb@fast.no>                             |
+   | Author: Stig Sther Bakken <ssb@php.net>                             |
    +----------------------------------------------------------------------+
  */
 
-/* $Id: versioning.c,v 1.12.4.2.4.3 2007/12/31 07:22:53 sebastian Exp $ */
+/* $Id: versioning.c,v 1.19.2.1.2.3.2.4 2008/12/31 11:15:46 sebastian Exp $ */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -34,7 +34,7 @@ PHPAPI char *
 php_canonicalize_version(const char *version)
 {
     int len = strlen(version);
-    char *buf = emalloc(len * 2 + 1), *q, lp, lq;
+    char *buf = safe_emalloc(len, 2, 1), *q, lp, lq;
     const char *p;
 
     if (len == 0) {
@@ -45,7 +45,7 @@ php_canonicalize_version(const char *version)
     p = version;
     q = buf;
     *q++ = lp = *p++;
-    lq = '\0';
+
     while (*p) {
 /*  s/[-_+]/./g;
  *  s/([^\d\.])([^\D\.])/$1.$2/g;
@@ -55,22 +55,22 @@ php_canonicalize_version(const char *version)
 #define isndig(x) (!isdigit(x)&&(x)!='.')
 #define isspecialver(x) ((x)=='-'||(x)=='_'||(x)=='+')
 
-        lq = *(q - 1);
+		lq = *(q - 1);
 		if (isspecialver(*p)) {
 			if (lq != '.') {
-				lq = *q++ = '.';
+				*q++ = '.';
 			}
 		} else if ((isndig(lp) && isdig(*p)) || (isdig(lp) && isndig(*p))) {
 			if (lq != '.') {
 				*q++ = '.';
 			}
-			lq = *q++ = *p;
+			*q++ = *p;
 		} else if (!isalnum(*p)) {
 			if (lq != '.') {
-				lq = *q++ = '.';
+				*q++ = '.';
 			}
 		} else {
-			lq = *q++ = *p;
+			*q++ = *p;
 		}
 		lp = *p++;
     }
@@ -90,13 +90,14 @@ static int
 compare_special_version_forms(char *form1, char *form2)
 {
 	int found1 = -1, found2 = -1;
-	special_forms_t special_forms[10] = {
+	special_forms_t special_forms[11] = {
 		{"dev", 0},
 		{"alpha", 1},
 		{"a", 1},
 		{"beta", 2},
 		{"b", 2},
 		{"RC", 3},
+		{"rc", 3},
 		{"#", 4},
 		{"pl", 5},
 		{"p", 5},
@@ -204,15 +205,13 @@ php_version_compare(const char *orig_ver1, const char *orig_ver2)
 }
 
 /* }}} */
-/* {{{ do_version_compare() */
-
 /* {{{ proto int version_compare(string ver1, string ver2 [, string oper])
   Compares two "PHP-standardized" version number strings */
 
 PHP_FUNCTION(version_compare)
 {
-	char *v1, *v2, *op;
-	int v1_len, v2_len, op_len;
+	char *v1, *v2, *op = NULL;
+	int v1_len, v2_len, op_len = 0;
 	int compare, argc;
 
 	argc = ZEND_NUM_ARGS();
