@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2006 The PHP Group                                |
+  | Copyright (c) 1997-2009 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: sysvmsg.c,v 1.20.2.3 2006/01/01 12:50:16 sniper Exp $ */
+/* $Id: sysvmsg.c,v 1.20.2.3.2.6.2.9 2008/12/31 11:15:46 sebastian Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -24,7 +24,6 @@
 
 #include "php.h"
 #include "php_globals.h"
-#include "php_ini.h"
 #include "ext/standard/info.h"
 #include "php_sysvmsg.h"
 #include "ext/standard/php_var.h"
@@ -40,39 +39,62 @@
 /* True global resources - no need for thread safety here */
 static int le_sysvmsg;
 
-static
-	ZEND_BEGIN_ARG_INFO(sixth_arg_force_ref, 0)
-		ZEND_ARG_PASS_INFO(0)
-		ZEND_ARG_PASS_INFO(0)
-		ZEND_ARG_PASS_INFO(0)
-		ZEND_ARG_PASS_INFO(0)
-		ZEND_ARG_PASS_INFO(0)
-		ZEND_ARG_PASS_INFO(1)
-	ZEND_END_ARG_INFO();
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO_EX(arginfo_msg_get_queue, 0, 0, 1)
+	ZEND_ARG_INFO(0, key)
+	ZEND_ARG_INFO(0, perms)
+ZEND_END_ARG_INFO()
 
-static
-	ZEND_BEGIN_ARG_INFO(msg_receive_args_force_ref, 0)
-		ZEND_ARG_PASS_INFO(0)
-		ZEND_ARG_PASS_INFO(0)
-		ZEND_ARG_PASS_INFO(1)
-		ZEND_ARG_PASS_INFO(0)
-		ZEND_ARG_PASS_INFO(1)
-		ZEND_ARG_PASS_INFO(0)
-		ZEND_ARG_PASS_INFO(0)
-		ZEND_ARG_PASS_INFO(1)
-	ZEND_END_ARG_INFO();
+ZEND_BEGIN_ARG_INFO_EX(arginfo_msg_send, 0, 0, 3)
+	ZEND_ARG_INFO(0, queue)
+	ZEND_ARG_INFO(0, msgtype)
+	ZEND_ARG_INFO(0, message)
+	ZEND_ARG_INFO(0, serialize)
+	ZEND_ARG_INFO(0, blocking)
+	ZEND_ARG_INFO(1, errorcode)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_msg_receive, 0, 0, 5)
+	ZEND_ARG_INFO(0, queue)
+	ZEND_ARG_INFO(0, desiredmsgtype)
+	ZEND_ARG_INFO(1, msgtype)
+	ZEND_ARG_INFO(0, maxsize)
+	ZEND_ARG_INFO(1, message)
+	ZEND_ARG_INFO(0, unserialize)
+	ZEND_ARG_INFO(0, flags)
+	ZEND_ARG_INFO(1, errorcode)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_msg_remove_queue, 0, 0, 1)
+	ZEND_ARG_INFO(0, queue)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_msg_stat_queue, 0, 0, 1)
+	ZEND_ARG_INFO(0, queue)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_msg_set_queue, 0, 0, 2)
+	ZEND_ARG_INFO(0, queue)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_msg_queue_exists, 0, 0, 1)
+	ZEND_ARG_INFO(0, key)
+ZEND_END_ARG_INFO()
+/* }}} */
 
 /* {{{ sysvmsg_functions[]
  *
  * Every user visible function must have an entry in sysvmsg_functions[].
  */
-zend_function_entry sysvmsg_functions[] = {
-	PHP_FE(msg_get_queue,				NULL)
-	PHP_FE(msg_send,					sixth_arg_force_ref)
-	PHP_FE(msg_receive,					msg_receive_args_force_ref)
-	PHP_FE(msg_remove_queue,			NULL)
-	PHP_FE(msg_stat_queue,				NULL)
-	PHP_FE(msg_set_queue,				NULL)
+const zend_function_entry sysvmsg_functions[] = {
+	PHP_FE(msg_get_queue,				arginfo_msg_get_queue)
+	PHP_FE(msg_send,					arginfo_msg_send)
+	PHP_FE(msg_receive,					arginfo_msg_receive)
+	PHP_FE(msg_remove_queue,			arginfo_msg_remove_queue)
+	PHP_FE(msg_stat_queue,				arginfo_msg_stat_queue)
+	PHP_FE(msg_set_queue,				arginfo_msg_set_queue)
+	PHP_FE(msg_queue_exists,			arginfo_msg_queue_exists)
 	{NULL, NULL, NULL}	/* Must be the last line in sysvmsg_functions[] */
 };
 /* }}} */
@@ -84,7 +106,7 @@ zend_module_entry sysvmsg_module_entry = {
 	"sysvmsg",
 	sysvmsg_functions,
 	PHP_MINIT(sysvmsg),
-	PHP_MSHUTDOWN(sysvmsg),
+	NULL,
 	NULL,
 	NULL,
 	PHP_MINFO(sysvmsg),
@@ -95,21 +117,8 @@ zend_module_entry sysvmsg_module_entry = {
 
 #ifdef COMPILE_DL_SYSVMSG
 ZEND_GET_MODULE(sysvmsg)
-# ifdef PHP_WIN32
-# include "zend_arg_defs.c"
-# endif
 #endif
 
-/* {{{ PHP_INI
- */
-/* Remove comments and fill if you need to have entries in php.ini
-PHP_INI_BEGIN()
-	STD_PHP_INI_ENTRY("sysvmsg.value",  "42",     PHP_INI_ALL, OnUpdateLong,    global_value,  zend_sysvmsg_globals, sysvmsg_globals)
-	STD_PHP_INI_ENTRY("sysvmsg.string", "foobar", PHP_INI_ALL, OnUpdateString, global_string, zend_sysvmsg_globals, sysvmsg_globals)
-PHP_INI_END()
-*/
-/* }}} */
-
 static void sysvmsg_release(zend_rsrc_list_entry *rsrc TSRMLS_DC)
 {
 	sysvmsg_queue_t * mq = (sysvmsg_queue_t *) rsrc->ptr;
@@ -122,27 +131,21 @@ PHP_MINIT_FUNCTION(sysvmsg)
 {
 	le_sysvmsg = zend_register_list_destructors_ex(sysvmsg_release, NULL, "sysvmsg queue", module_number);
 	REGISTER_LONG_CONSTANT("MSG_IPC_NOWAIT", PHP_MSG_IPC_NOWAIT, CONST_PERSISTENT|CONST_CS);
+	REGISTER_LONG_CONSTANT("MSG_EAGAIN",	 EAGAIN, 	     CONST_PERSISTENT|CONST_CS);
+	REGISTER_LONG_CONSTANT("MSG_ENOMSG",	 ENOMSG, 	     CONST_PERSISTENT|CONST_CS);
 	REGISTER_LONG_CONSTANT("MSG_NOERROR",    PHP_MSG_NOERROR,    CONST_PERSISTENT|CONST_CS);
 	REGISTER_LONG_CONSTANT("MSG_EXCEPT",     PHP_MSG_EXCEPT,     CONST_PERSISTENT|CONST_CS);
 	return SUCCESS;
 }
 /* }}} */
 
-/* {{{ PHP_MSHUTDOWN_FUNCTION
- */
-PHP_MSHUTDOWN_FUNCTION(sysvmsg)
-{
-	return SUCCESS;
-}
-/* }}} */
-
 /* {{{ PHP_MINFO_FUNCTION
  */
 PHP_MINFO_FUNCTION(sysvmsg)
 {
 	php_info_print_table_start();
 	php_info_print_table_row(2, "sysvmsg support", "enabled");
-	php_info_print_table_row(2, "Revision", "$Revision: 1.20.2.3 $");
+	php_info_print_table_row(2, "Revision", "$Revision: 1.20.2.3.2.6.2.9 $");
 	php_info_print_table_end();
 }
 /* }}} */
@@ -223,6 +226,26 @@ PHP_FUNCTION(msg_stat_queue)
 }
 /* }}} */
 
+
+/* {{{ proto bool msg_queue_exists(int key)
+   Check wether a message queue exists */
+PHP_FUNCTION(msg_queue_exists)
+{
+	long key;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &key) == FAILURE)	{
+		return;
+	}
+
+	if (msgget(key, 0) < 0) {
+		RETURN_FALSE;
+	}
+
+	RETURN_TRUE;
+}
+/* }}} */
+
+
 /* {{{ proto resource msg_get_queue(int key [, int perms])
    Attach to a message queue */
 PHP_FUNCTION(msg_get_queue)
@@ -292,7 +315,12 @@ PHP_FUNCTION(msg_receive)
 				&out_message, &do_unserialize, &flags, &zerrcode) == FAILURE) {
 		return;
 	}
-	
+
+	if (maxsize <= 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "maximum size of the message has to be greater than zero");
+		return;
+	}
+
 	if (flags != 0) {
 		if (flags & PHP_MSG_EXCEPT) {
 #ifndef MSG_EXCEPT
@@ -312,8 +340,8 @@ PHP_FUNCTION(msg_receive)
 	
 	ZEND_FETCH_RESOURCE(mq, sysvmsg_queue_t *, &queue, -1, "sysvmsg queue", le_sysvmsg);
 
-	messagebuffer = (struct php_msgbuf *) emalloc(sizeof(struct php_msgbuf) + maxsize);
-	
+	messagebuffer = (struct php_msgbuf *) safe_emalloc(maxsize, 1, sizeof(struct php_msgbuf));
+
 	result = msgrcv(mq->id, messagebuffer, maxsize, desiredmsgtype, realflags);
 		
 	zval_dtor(out_msgtype);
@@ -387,7 +415,7 @@ PHP_FUNCTION(msg_send)
 		
 		/* NB: php_msgbuf is 1 char bigger than a long, so there is no need to
 		 * allocate the extra byte. */
-		messagebuffer = emalloc(sizeof(struct php_msgbuf) + msg_var.len);
+		messagebuffer = safe_emalloc(msg_var.len, 1, sizeof(struct php_msgbuf));
 		memcpy(messagebuffer->mtext, msg_var.c, msg_var.len + 1);
 		message_len = msg_var.len;
 		smart_str_free(&msg_var);
@@ -405,7 +433,7 @@ PHP_FUNCTION(msg_send)
 				break;
 
 			case IS_DOUBLE:
-				message_len = spprintf(&p, 0, "%f", Z_DVAL_P(message));
+				message_len = spprintf(&p, 0, "%F", Z_DVAL_P(message));
 				break;
 
 			default:
@@ -413,7 +441,7 @@ PHP_FUNCTION(msg_send)
 				RETURN_FALSE;
 		}
 
-		messagebuffer = emalloc(sizeof(struct php_msgbuf) + message_len);
+		messagebuffer = safe_emalloc(message_len, 1, sizeof(struct php_msgbuf));
 		memcpy(messagebuffer->mtext, p, message_len + 1);
 
 		if (Z_TYPE_P(message) != IS_STRING) {
