@@ -1,8 +1,8 @@
 /* 
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: php_sysvshm.h,v 1.11.8.1.8.1 2006/01/01 13:46:58 sniper Exp $ */
+/* $Id: php_sysvshm.h,v 1.17.2.1.2.1.2.3 2008/12/31 11:15:46 sebastian Exp $ */
 
 #ifndef PHP_SYSVSHM_H
 #define PHP_SYSVSHM_H
@@ -30,12 +30,13 @@ extern zend_module_entry sysvshm_module_entry;
 #include <sys/ipc.h>
 #include <sys/shm.h>
 
+#define PHP_SHM_RSRC_NAME "sysvshm"
+
 typedef struct {
 	int le_shm;
 	long init_mem;
 } sysvshm_module;
 
-
 typedef struct {
 	long key;
 	long length;
@@ -43,7 +44,6 @@ typedef struct {
 	char mem;
 } sysvshm_chunk;
 
-
 typedef struct {
 	char magic[8];
 	long start;
@@ -52,25 +52,20 @@ typedef struct {
 	long total;
 } sysvshm_chunk_head;
 
-
 typedef struct {
-	key_t key;					/* Key set by user */
-	long id;					/* Returned by shmget. */
-	sysvshm_chunk_head *ptr;			/* memoryaddress of shared memory */ 
+	key_t key;               /* key set by user */
+	long id;                 /* returned by shmget */
+	sysvshm_chunk_head *ptr; /* memory address of shared memory */
 } sysvshm_shm;
 
-
-
 PHP_MINIT_FUNCTION(sysvshm);
 PHP_FUNCTION(shm_attach);
 PHP_FUNCTION(shm_detach);
 PHP_FUNCTION(shm_remove);
 PHP_FUNCTION(shm_put_var);
 PHP_FUNCTION(shm_get_var);
+PHP_FUNCTION(shm_has_var);
 PHP_FUNCTION(shm_remove_var);
-int php_put_shm_data(sysvshm_chunk_head *ptr,long key,char *data, long len);
-long php_check_shm_data(sysvshm_chunk_head *ptr, long key);
-int php_remove_shm_data(sysvshm_chunk_head *ptr, long shm_varpos);
 
 extern sysvshm_module php_sysvshm;
 
