@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php_wddx_api.h 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: php_wddx_api.h,v 1.23.2.3.2.1.2.3 2008/12/31 11:15:46 sebastian Exp $ */
 
 #ifndef PHP_WDDX_API_H
 #define PHP_WDDX_API_H
@@ -27,7 +27,8 @@
 #define WDDX_ARRAY_E			"</array>"
 #define WDDX_BINARY_S			"<binary>"
 #define WDDX_BINARY_E			"</binary>"
-#define WDDX_BOOLEAN			"<boolean value='%s'/>"
+#define WDDX_BOOLEAN_TRUE		"<boolean value='true'/>"
+#define WDDX_BOOLEAN_FALSE		"<boolean value='false'/>"
 #define WDDX_CHAR				"<char code='%02X'/>"
 #define WDDX_COMMENT_S			"<comment>"
 #define WDDX_COMMENT_E			"</comment>"
