@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: xml.c,v 1.151.2.4 2005/04/28 12:17:18 rrichards Exp $ */
+/* $Id: xml.c,v 1.157.2.4.2.5.2.16 2008/12/31 11:15:46 sebastian Exp $ */
 
 #define IS_EXT_MODULE
 
@@ -57,24 +57,18 @@
  * - Weird things happen with <![CDATA[]]> sections.
  */
 
-#ifdef ZTS
-int xml_globals_id;
-#else
-PHP_XML_API php_xml_globals xml_globals;
-#endif
+ZEND_DECLARE_MODULE_GLOBALS(xml)
 
 /* {{{ dynamically loadable module stuff */
 #ifdef COMPILE_DL_XML
 ZEND_GET_MODULE(xml)
-# ifdef PHP_WIN32
-# include "zend_arg_defs.c"
-# endif
 #endif /* COMPILE_DL_XML */
 /* }}} */
 
 /* {{{ function prototypes */
 PHP_MINIT_FUNCTION(xml);
 PHP_MINFO_FUNCTION(xml);
+static PHP_GINIT_FUNCTION(xml);
 
 static void xml_parser_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC);
 static void xml_set_handler(zval **, zval **);
@@ -82,7 +76,6 @@ inline static unsigned short xml_encode_iso_8859_1(unsigned char);
 inline static char xml_decode_iso_8859_1(unsigned short);
 inline static unsigned short xml_encode_us_ascii(unsigned char);
 inline static char xml_decode_us_ascii(unsigned short);
-static XML_Char *xml_utf8_encode(const char *, int, int *, const XML_Char *);
 static zval *xml_call_handler(xml_parser *, zval *, zend_function *, int, zval **);
 static zval *_xml_xmlchar_zval(const XML_Char *, int, const XML_Char *);
 static int _xml_xmlcharlen(const XML_Char *);
@@ -103,44 +96,164 @@ void _xml_endNamespaceDeclHandler(void *, const XML_Char *);
 /* }}} */
 
 /* {{{ extension definition structures */
-static
-	ZEND_BEGIN_ARG_INFO(third_and_fourth_args_force_ref, 0)
-		ZEND_ARG_PASS_INFO(0)
-		ZEND_ARG_PASS_INFO(0)
-		ZEND_ARG_PASS_INFO(1)
-		ZEND_ARG_PASS_INFO(1)
-	ZEND_END_ARG_INFO();
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parser_create, 0, 0, 0)
+	ZEND_ARG_INFO(0, encoding)
+ZEND_END_ARG_INFO()
 
-function_entry xml_functions[] = {
-	PHP_FE(xml_parser_create, NULL)
-	PHP_FE(xml_parser_create_ns, NULL)
-	PHP_FE(xml_set_object, second_arg_force_ref)
-	PHP_FE(xml_set_element_handler, NULL)
-	PHP_FE(xml_set_character_data_handler, NULL)
-	PHP_FE(xml_set_processing_instruction_handler, NULL)
-	PHP_FE(xml_set_default_handler, NULL)
-	PHP_FE(xml_set_unparsed_entity_decl_handler, NULL)
-	PHP_FE(xml_set_notation_decl_handler, NULL)
-	PHP_FE(xml_set_external_entity_ref_handler, NULL)
-	PHP_FE(xml_set_start_namespace_decl_handler, NULL)
-	PHP_FE(xml_set_end_namespace_decl_handler, NULL)
-	PHP_FE(xml_parse, NULL)
-	PHP_FE(xml_parse_into_struct, third_and_fourth_args_force_ref)
-	PHP_FE(xml_get_error_code, NULL)
-	PHP_FE(xml_error_string, NULL)
-	PHP_FE(xml_get_current_line_number, NULL)
-	PHP_FE(xml_get_current_column_number, NULL)
-	PHP_FE(xml_get_current_byte_index, NULL)
-	PHP_FE(xml_parser_free, NULL)
-	PHP_FE(xml_parser_set_option, NULL)
-	PHP_FE(xml_parser_get_option, NULL)
-	PHP_FE(utf8_encode, NULL)
-	PHP_FE(utf8_decode, NULL)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parser_create_ns, 0, 0, 0)
+	ZEND_ARG_INFO(0, encoding)
+	ZEND_ARG_INFO(0, sep)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_object, 0, 0, 2)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(1, obj)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_element_handler, 0, 0, 3)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, shdl)
+	ZEND_ARG_INFO(0, ehdl)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_character_data_handler, 0, 0, 2)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, hdl)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_processing_instruction_handler, 0, 0, 2)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, hdl)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_default_handler, 0, 0, 2)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, hdl)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_unparsed_entity_decl_handler, 0, 0, 2)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, hdl)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_notation_decl_handler, 0, 0, 2)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, hdl)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_external_entity_ref_handler, 0, 0, 2)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, hdl)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_start_namespace_decl_handler, 0, 0, 2)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, hdl)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_set_end_namespace_decl_handler, 0, 0, 2)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, hdl)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parse, 0, 0, 2)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, data)
+	ZEND_ARG_INFO(0, isfinal)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parse_into_struct, 0, 0, 3)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, data)
+	ZEND_ARG_INFO(1, values)
+	ZEND_ARG_INFO(1, index)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_get_error_code, 0, 0, 1)
+	ZEND_ARG_INFO(0, parser)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_error_string, 0, 0, 1)
+	ZEND_ARG_INFO(0, code)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_get_current_line_number, 0, 0, 1)
+	ZEND_ARG_INFO(0, parser)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_get_current_column_number, 0, 0, 1)
+	ZEND_ARG_INFO(0, parser)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_get_current_byte_index, 0, 0, 1)
+	ZEND_ARG_INFO(0, parser)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parser_free, 0, 0, 1)
+	ZEND_ARG_INFO(0, parser)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parser_set_option, 0, 0, 3)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, option)
+	ZEND_ARG_INFO(0, value)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xml_parser_get_option, 0, 0, 2)
+	ZEND_ARG_INFO(0, parser)
+	ZEND_ARG_INFO(0, option)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_utf8_encode, 0, 0, 1)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_utf8_decode, 0, 0, 1)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO()
+
+const zend_function_entry xml_functions[] = {
+	PHP_FE(xml_parser_create,					arginfo_xml_parser_create)
+	PHP_FE(xml_parser_create_ns,				arginfo_xml_parser_create_ns)
+	PHP_FE(xml_set_object, 						arginfo_xml_set_object)
+	PHP_FE(xml_set_element_handler,				arginfo_xml_set_element_handler)
+	PHP_FE(xml_set_character_data_handler,		arginfo_xml_set_character_data_handler)
+	PHP_FE(xml_set_processing_instruction_handler, 	arginfo_xml_set_processing_instruction_handler)
+	PHP_FE(xml_set_default_handler, 				arginfo_xml_set_default_handler)
+	PHP_FE(xml_set_unparsed_entity_decl_handler,arginfo_xml_set_unparsed_entity_decl_handler)
+	PHP_FE(xml_set_notation_decl_handler,		arginfo_xml_set_notation_decl_handler)
+	PHP_FE(xml_set_external_entity_ref_handler,	arginfo_xml_set_external_entity_ref_handler)
+	PHP_FE(xml_set_start_namespace_decl_handler,arginfo_xml_set_start_namespace_decl_handler)
+	PHP_FE(xml_set_end_namespace_decl_handler,	arginfo_xml_set_end_namespace_decl_handler)
+	PHP_FE(xml_parse,							arginfo_xml_parse)
+	PHP_FE(xml_parse_into_struct, 				arginfo_xml_parse_into_struct)
+	PHP_FE(xml_get_error_code,					arginfo_xml_get_error_code)
+	PHP_FE(xml_error_string,					arginfo_xml_error_string)
+	PHP_FE(xml_get_current_line_number,			arginfo_xml_get_current_line_number)
+	PHP_FE(xml_get_current_column_number,		arginfo_xml_get_current_column_number)
+	PHP_FE(xml_get_current_byte_index,			arginfo_xml_get_current_byte_index)
+	PHP_FE(xml_parser_free, 					arginfo_xml_parser_free)
+	PHP_FE(xml_parser_set_option, 				arginfo_xml_parser_set_option)
+	PHP_FE(xml_parser_get_option,				arginfo_xml_parser_get_option)
+	PHP_FE(utf8_encode, 						arginfo_utf8_encode)
+	PHP_FE(utf8_decode, 						arginfo_utf8_decode)
 	{NULL, NULL, NULL}
 };
 
+#ifdef LIBXML_EXPAT_COMPAT
+static const zend_module_dep xml_deps[] = {
+	ZEND_MOD_REQUIRED("libxml")
+	{NULL, NULL, NULL}
+};
+#endif
+
 zend_module_entry xml_module_entry = {
+#ifdef LIBXML_EXPAT_COMPAT
+    STANDARD_MODULE_HEADER_EX, NULL,
+	xml_deps,
+#else
     STANDARD_MODULE_HEADER,
+#endif
 	"xml",                /* extension name */
 	xml_functions,        /* extension function list */
 	PHP_MINIT(xml),       /* extension-wide startup function */
@@ -149,7 +262,11 @@ zend_module_entry xml_module_entry = {
 	NULL,                 /* per-request shutdown function */
 	PHP_MINFO(xml),       /* information function */
     NO_VERSION_YET,
-	STANDARD_MODULE_PROPERTIES
+    PHP_MODULE_GLOBALS(xml), /* globals descriptor */
+    PHP_GINIT(xml),          /* globals ctor */
+    NULL,                    /* globals dtor */
+    NULL,                    /* post deactivate */
+	STANDARD_MODULE_PROPERTIES_EX
 };
 
 /* All the encoding functions are set to NULL right now, since all
@@ -170,12 +287,10 @@ static int le_xml_parser;
 /* }}} */
 
 /* {{{ startup, shutdown and info functions */
-#ifdef ZTS
-static void php_xml_init_globals(php_xml_globals *xml_globals_p TSRMLS_DC)
+static PHP_GINIT_FUNCTION(xml)
 {
-	XML(default_encoding) = "UTF-8";
+	xml_globals->default_encoding = "UTF-8";
 }
-#endif
 
 static void *php_xml_malloc_wrapper(size_t sz)
 {
@@ -198,12 +313,6 @@ PHP_MINIT_FUNCTION(xml)
 {
 	le_xml_parser =	zend_register_list_destructors_ex(xml_parser_dtor, NULL, "xml", module_number);
 
-#ifdef ZTS
-	ts_allocate_id(&xml_globals_id, sizeof(php_xml_globals), (ts_allocate_ctor) php_xml_init_globals, NULL);
-#else
-	XML(default_encoding) = "UTF-8";
-#endif
-
 	REGISTER_LONG_CONSTANT("XML_ERROR_NONE", XML_ERROR_NONE, CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("XML_ERROR_NO_MEMORY", XML_ERROR_NO_MEMORY, CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("XML_ERROR_SYNTAX", XML_ERROR_SYNTAX, CONST_CS|CONST_PERSISTENT);
@@ -375,8 +484,13 @@ static void xml_set_handler(zval **handler, zval **data)
 	}
 
 	/* IS_ARRAY might indicate that we're using array($obj, 'method') syntax */
-	if (Z_TYPE_PP(data) != IS_ARRAY) {
+	if (Z_TYPE_PP(data) != IS_ARRAY && Z_TYPE_PP(data) != IS_OBJECT) {
+
 		convert_to_string_ex(data);
+		if (Z_STRLEN_PP(data) == 0) {
+			*handler = NULL;
+			return;
+		}
 	}
 
 	zval_add_ref(data);
@@ -406,7 +520,7 @@ static zval *xml_call_handler(xml_parser *parser, zval *handler, zend_function *
 		fci.function_table = EG(function_table);
 		fci.function_name = handler;
 		fci.symbol_table = NULL;
-		fci.object_pp = &parser->object;
+		fci.object_ptr = parser->object;
 		fci.retval_ptr_ptr = &retval;
 		fci.param_count = argc;
 		fci.params = args;
@@ -442,8 +556,8 @@ static zval *xml_call_handler(xml_parser *parser, zval *handler, zend_function *
 	} else {
 		for (i = 0; i < argc; i++) {
 			zval_ptr_dtor(&argv[i]);
-	}
-	return NULL;
+		}
+		return NULL;
 	}
 }
 /* }}} */
@@ -492,7 +606,7 @@ static xml_encoding *xml_get_encoding(const XML_Char *name)
 /* }}} */
 
 /* {{{ xml_utf8_encode */
-static XML_Char *xml_utf8_encode(const char *s, int len, int *newlen, const XML_Char *encoding)
+PHPAPI char *xml_utf8_encode(const char *s, int len, int *newlen, const XML_Char *encoding)
 {
 	int pos = len;
 	char *newbuf;
@@ -570,15 +684,27 @@ PHPAPI char *xml_utf8_decode(const XML_Char *s, int len, int *newlen, const XML_
 	while (pos > 0) {
 		c = (unsigned char)(*s);
 		if (c >= 0xf0) { /* four bytes encoded, 21 bits */
-			c = ((s[0]&7)<<18) | ((s[1]&63)<<12) | ((s[2]&63)<<6) | (s[3]&63);
+			if(pos-4 >= 0) {
+				c = ((s[0]&7)<<18) | ((s[1]&63)<<12) | ((s[2]&63)<<6) | (s[3]&63);
+			} else {
+				c = '?';	
+			}
 			s += 4;
 			pos -= 4;
 		} else if (c >= 0xe0) { /* three bytes encoded, 16 bits */
-			c = ((s[0]&63)<<12) | ((s[1]&63)<<6) | (s[2]&63);
+			if(pos-3 >= 0) {
+				c = ((s[0]&63)<<12) | ((s[1]&63)<<6) | (s[2]&63);
+			} else {
+				c = '?';
+			}
 			s += 3;
 			pos -= 3;
 		} else if (c >= 0xc0) { /* two bytes encoded, 11 bits */
-			c = ((s[0]&63)<<6) | (s[1]&63);
+			if(pos-2 >= 0) {
+				c = ((s[0]&63)<<6) | (s[1]&63);
+			} else {
+				c = '?';
+			}
 			s += 2;
 			pos -= 2;
 		} else {
@@ -853,6 +979,25 @@ void _xml_characterDataHandler(void *userData, const XML_Char *s, int len)
 					
 				} else {
 					zval *tag;
+					zval **curtag, **mytype, **myval;
+					HashPosition hpos=NULL;
+
+					zend_hash_internal_pointer_end_ex(Z_ARRVAL_P(parser->data), &hpos);
+
+					if (hpos && (zend_hash_get_current_data_ex(Z_ARRVAL_P(parser->data), (void **) &curtag, &hpos) == SUCCESS)) {
+						if (zend_hash_find(Z_ARRVAL_PP(curtag),"type",sizeof("type"),(void **) &mytype) == SUCCESS) {
+							if (!strcmp(Z_STRVAL_PP(mytype), "cdata")) {
+								if (zend_hash_find(Z_ARRVAL_PP(curtag),"value",sizeof("value"),(void **) &myval) == SUCCESS) {
+									int newlen = Z_STRLEN_PP(myval) + decoded_len;
+									Z_STRVAL_PP(myval) = erealloc(Z_STRVAL_PP(myval),newlen+1);
+									strcpy(Z_STRVAL_PP(myval) + Z_STRLEN_PP(myval),decoded_value);
+									Z_STRLEN_PP(myval) += decoded_len;
+									efree(decoded_value);
+									return;
+								}
+							}
+						}
+					}
 
 					MAKE_STD_ZVAL(tag);
 					
@@ -1027,7 +1172,7 @@ void _xml_endNamespaceDeclHandler(void *userData, const XML_Char *prefix)
 
 /************************* EXTENSION FUNCTIONS *************************/
 
-static void php_xml_parser_create_impl(INTERNAL_FUNCTION_PARAMETERS, int ns_support)
+static void php_xml_parser_create_impl(INTERNAL_FUNCTION_PARAMETERS, int ns_support) /* {{{ */
 {
 	xml_parser *parser;
 	int auto_detect = 0;
@@ -1069,7 +1214,7 @@ static void php_xml_parser_create_impl(INTERNAL_FUNCTION_PARAMETERS, int ns_supp
 		ns_param = ":";
 	}
 
-	parser = ecalloc(sizeof(xml_parser), 1);
+	parser = ecalloc(1, sizeof(xml_parser));
 	parser->parser = XML_ParserCreate_MM((auto_detect ? NULL : encoding),
                                          &php_xml_mem_hdlrs, ns_param);
 
@@ -1083,6 +1228,7 @@ static void php_xml_parser_create_impl(INTERNAL_FUNCTION_PARAMETERS, int ns_supp
 	ZEND_REGISTER_RESOURCE(return_value, parser,le_xml_parser);
 	parser->index = Z_LVAL_P(return_value);
 }
+/* }}} */
 
 /* {{{ proto resource xml_parser_create([string encoding]) 
    Create an XML parser */
@@ -1096,12 +1242,7 @@ PHP_FUNCTION(xml_parser_create)
    Create an XML parser */
 PHP_FUNCTION(xml_parser_create_ns)
 {
-#if defined(HAVE_LIBXML) && defined(HAVE_XML) && !defined(HAVE_LIBEXPAT) && LIBXML_VERSION < 20600 
-	php_error_docref(NULL TSRMLS_CC, E_WARNING, "is broken with libxml2 %s. Please upgrade to libxml2 2.6", LIBXML_DOTTED_VERSION);
-	RETURN_FALSE;
-#else
 	php_xml_parser_create_impl(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
-#endif
 }
 /* }}} */
 
@@ -1110,19 +1251,13 @@ PHP_FUNCTION(xml_parser_create_ns)
 PHP_FUNCTION(xml_set_object)
 {
 	xml_parser *parser;
-	zval **pind, **mythis;
+	zval *pind, *mythis;
 
-	if (ZEND_NUM_ARGS() != 2 ||
-		zend_get_parameters_ex(2, &pind, &mythis) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ro", &pind, &mythis) == FAILURE) {
+		return;
 	}
 
-	if (Z_TYPE_PP(mythis) != IS_OBJECT) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Argument 2 has wrong type");
-		RETURN_FALSE;
-	}
-
-	ZEND_FETCH_RESOURCE(parser,xml_parser *,pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	/* please leave this commented - or ask thies@thieso.net before doing it (again) */
 	if (parser->object) {
@@ -1135,7 +1270,7 @@ PHP_FUNCTION(xml_set_object)
 #endif */
 
 	ALLOC_ZVAL(parser->object);
-	*parser->object = **mythis;
+	*parser->object = *mythis;
 	zval_copy_ctor(parser->object);
 	INIT_PZVAL(parser->object);
 
@@ -1148,14 +1283,13 @@ PHP_FUNCTION(xml_set_object)
 PHP_FUNCTION(xml_set_element_handler)
 {
 	xml_parser *parser;
-	zval **pind, **shdl, **ehdl;
+	zval *pind, **shdl, **ehdl;
 
-	if (ZEND_NUM_ARGS() != 3 ||
-		zend_get_parameters_ex(3, &pind, &shdl, &ehdl) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZZ", &pind, &shdl, &ehdl) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *,pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->startElementHandler, shdl);
 	xml_set_handler(&parser->endElementHandler, ehdl);
@@ -1169,13 +1303,13 @@ PHP_FUNCTION(xml_set_element_handler)
 PHP_FUNCTION(xml_set_character_data_handler)
 {
 	xml_parser *parser;
-	zval **pind, **hdl;
+	zval *pind, **hdl;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->characterDataHandler, hdl);
 	XML_SetCharacterDataHandler(parser->parser, _xml_characterDataHandler);
@@ -1188,13 +1322,13 @@ PHP_FUNCTION(xml_set_character_data_handler)
 PHP_FUNCTION(xml_set_processing_instruction_handler)
 {
 	xml_parser *parser;
-	zval **pind, **hdl;
+	zval *pind, **hdl;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->processingInstructionHandler, hdl);
 	XML_SetProcessingInstructionHandler(parser->parser, _xml_processingInstructionHandler);
@@ -1207,12 +1341,12 @@ PHP_FUNCTION(xml_set_processing_instruction_handler)
 PHP_FUNCTION(xml_set_default_handler)
 {
 	xml_parser *parser;
-	zval **pind, **hdl;
+	zval *pind, **hdl;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
+		return;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->defaultHandler, hdl);
 	XML_SetDefaultHandler(parser->parser, _xml_defaultHandler);
@@ -1225,13 +1359,13 @@ PHP_FUNCTION(xml_set_default_handler)
 PHP_FUNCTION(xml_set_unparsed_entity_decl_handler)
 {
 	xml_parser *parser;
-	zval **pind, **hdl;
+	zval *pind, **hdl;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->unparsedEntityDeclHandler, hdl);
 	XML_SetUnparsedEntityDeclHandler(parser->parser, _xml_unparsedEntityDeclHandler);
@@ -1244,12 +1378,12 @@ PHP_FUNCTION(xml_set_unparsed_entity_decl_handler)
 PHP_FUNCTION(xml_set_notation_decl_handler)
 {
 	xml_parser *parser;
-	zval **pind, **hdl;
+	zval *pind, **hdl;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
+		return;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->notationDeclHandler, hdl);
 	XML_SetNotationDeclHandler(parser->parser, _xml_notationDeclHandler);
@@ -1262,12 +1396,12 @@ PHP_FUNCTION(xml_set_notation_decl_handler)
 PHP_FUNCTION(xml_set_external_entity_ref_handler)
 {
 	xml_parser *parser;
-	zval **pind, **hdl;
+	zval *pind, **hdl;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
+		return;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->externalEntityRefHandler, hdl);
 	XML_SetExternalEntityRefHandler(parser->parser, (void *) _xml_externalEntityRefHandler);
@@ -1280,13 +1414,13 @@ PHP_FUNCTION(xml_set_external_entity_ref_handler)
 PHP_FUNCTION(xml_set_start_namespace_decl_handler)
 {
 	xml_parser *parser;
-	zval **pind, **hdl;
+	zval *pind, **hdl;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->startNamespaceDeclHandler, hdl);
 	XML_SetStartNamespaceDeclHandler(parser->parser, _xml_startNamespaceDeclHandler);
@@ -1299,13 +1433,13 @@ PHP_FUNCTION(xml_set_start_namespace_decl_handler)
 PHP_FUNCTION(xml_set_end_namespace_decl_handler)
 {
 	xml_parser *parser;
-	zval **pind, **hdl;
+	zval *pind, **hdl;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &hdl) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rZ", &pind, &hdl) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	xml_set_handler(&parser->endNamespaceDeclHandler, hdl);
 	XML_SetEndNamespaceDeclHandler(parser->parser, _xml_endNamespaceDeclHandler);
@@ -1318,58 +1452,54 @@ PHP_FUNCTION(xml_set_end_namespace_decl_handler)
 PHP_FUNCTION(xml_parse)
 {
 	xml_parser *parser;
-	zval **pind, **data, **final;
-	int argc, isFinal, ret;
+	zval *pind;
+	char *data;
+	int data_len, ret;
+	long isFinal = 0;
 
-	argc = ZEND_NUM_ARGS();
-	if (argc < 2 || argc > 3 || zend_get_parameters_ex(argc, &pind, &data, &final) == FAILURE) {
-		WRONG_PARAM_COUNT;
-	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
-
-	convert_to_string_ex(data);
-
-	if (argc == 3) {
-		convert_to_long_ex(final);
-		isFinal = Z_LVAL_PP(final);
-	} else {
-		isFinal = 0;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|l", &pind, &data, &data_len, &isFinal) == FAILURE) {
+		return;
 	}
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	parser->isparsing = 1;
-	ret = XML_Parse(parser->parser, Z_STRVAL_PP(data), Z_STRLEN_PP(data), isFinal);
+	ret = XML_Parse(parser->parser, data, data_len, isFinal);
 	parser->isparsing = 0;
 	RETVAL_LONG(ret);
 }
 
 /* }}} */
 
-/* {{{ proto int xml_parse_into_struct(resource parser, string data, array &struct, array &index) 
+/* {{{ proto int xml_parse_into_struct(resource parser, string data, array &values [, array &index ])
    Parsing a XML document */
 
 PHP_FUNCTION(xml_parse_into_struct)
 {
 	xml_parser *parser;
-	zval **pind, **data, **xdata, **info = 0;
-	int argc, ret;
+	zval *pind, **xdata, **info = NULL;
+	char *data;
+	int data_len, ret;
 
-	argc = ZEND_NUM_ARGS();
-	if (zend_get_parameters_ex(4, &pind, &data, &xdata,&info) == SUCCESS) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rsZ|Z", &pind, &data, &data_len, &xdata, &info) == FAILURE) {
+		return;
+	}
+	
+	if (info) {	
 		zval_dtor(*info);
 		array_init(*info);
-	} else if (zend_get_parameters_ex(3, &pind, &data, &xdata) == FAILURE) {
-		WRONG_PARAM_COUNT;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
-	convert_to_string_ex(data);
 	zval_dtor(*xdata);
 	array_init(*xdata);
 
 	parser->data = *xdata;
-	if (info)
-	  parser->info = *info;
+	
+	if (info) {
+		parser->info = *info;
+	}
+	
 	parser->level = 0;
 	parser->ltags = safe_emalloc(XML_MAXLEVEL, sizeof(char *), 0);
 
@@ -1378,7 +1508,7 @@ PHP_FUNCTION(xml_parse_into_struct)
 	XML_SetCharacterDataHandler(parser->parser, _xml_characterDataHandler);
 
 	parser->isparsing = 1;
-	ret = XML_Parse(parser->parser, Z_STRVAL_PP(data), Z_STRLEN_PP(data), 1);
+	ret = XML_Parse(parser->parser, data, data_len, 1);
 	parser->isparsing = 0;
 
 	RETVAL_LONG(ret);
@@ -1390,12 +1520,12 @@ PHP_FUNCTION(xml_parse_into_struct)
 PHP_FUNCTION(xml_get_error_code)
 {
 	xml_parser *parser;
-	zval **pind;
+	zval *pind;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &pind) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &pind) == FAILURE) {
+		return;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	RETVAL_LONG((long)XML_GetErrorCode(parser->parser));
 }
@@ -1405,14 +1535,14 @@ PHP_FUNCTION(xml_get_error_code)
    Get XML parser error string */
 PHP_FUNCTION(xml_error_string)
 {
-	zval **code;
+	long code;
 	char *str;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &code) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &code) == FAILURE) {
+		return;
 	}
-	convert_to_long_ex(code);
-	str = (char *)XML_ErrorString((int)Z_LVAL_PP(code));
+
+	str = (char *)XML_ErrorString((int)code);
 	if (str) {
 		RETVAL_STRING(str, 1);
 	}
@@ -1424,12 +1554,12 @@ PHP_FUNCTION(xml_error_string)
 PHP_FUNCTION(xml_get_current_line_number)
 {
 	xml_parser *parser;
-	zval **pind;
+	zval *pind;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &pind) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &pind) == FAILURE) {
+		return;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	RETVAL_LONG(XML_GetCurrentLineNumber(parser->parser));
 }
@@ -1440,12 +1570,12 @@ PHP_FUNCTION(xml_get_current_line_number)
 PHP_FUNCTION(xml_get_current_column_number)
 {
 	xml_parser *parser;
-	zval **pind;
+	zval *pind;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &pind) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &pind) == FAILURE) {
+		return;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	RETVAL_LONG(XML_GetCurrentColumnNumber(parser->parser));
 }
@@ -1456,12 +1586,12 @@ PHP_FUNCTION(xml_get_current_column_number)
 PHP_FUNCTION(xml_get_current_byte_index)
 {
 	xml_parser *parser;
-	zval **pind;
+	zval *pind;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &pind) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &pind) == FAILURE) {
+		return;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	RETVAL_LONG(XML_GetCurrentByteIndex(parser->parser));
 }
@@ -1471,14 +1601,14 @@ PHP_FUNCTION(xml_get_current_byte_index)
    Free an XML parser */
 PHP_FUNCTION(xml_parser_free)
 {
-	zval **pind;
+	zval *pind;
 	xml_parser *parser;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &pind) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &pind) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
 	if (parser->isparsing == 1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Parser cannot be freed while it is parsing.");
@@ -1498,16 +1628,15 @@ PHP_FUNCTION(xml_parser_free)
 PHP_FUNCTION(xml_parser_set_option)
 {
 	xml_parser *parser;
-	zval **pind, **opt, **val;
+	zval *pind, **val;
+	long opt;
 
-	if (ZEND_NUM_ARGS() != 3 || zend_get_parameters_ex(3, &pind, &opt, &val) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rlZ", &pind, &opt, &val) == FAILURE) {
+		return;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
-	convert_to_long_ex(opt);
-
-	switch (Z_LVAL_PP(opt)) {
+	switch (opt) {
 		case PHP_XML_OPTION_CASE_FOLDING:
 			convert_to_long_ex(val);
 			parser->case_folding = Z_LVAL_PP(val);
@@ -1545,16 +1674,15 @@ PHP_FUNCTION(xml_parser_set_option)
 PHP_FUNCTION(xml_parser_get_option)
 {
 	xml_parser *parser;
-	zval **pind, **opt;
+	zval *pind;
+	long opt;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &pind, &opt) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &pind, &opt) == FAILURE) {
+		return;
 	}
-	ZEND_FETCH_RESOURCE(parser,xml_parser *, pind, -1, "XML Parser", le_xml_parser);
+	ZEND_FETCH_RESOURCE(parser,xml_parser *, &pind, -1, "XML Parser", le_xml_parser);
 
-	convert_to_long_ex(opt);
-
-	switch (Z_LVAL_PP(opt)) {
+	switch (opt) {
 		case PHP_XML_OPTION_CASE_FOLDING:
 			RETURN_LONG(parser->case_folding);
 			break;
@@ -1575,15 +1703,15 @@ PHP_FUNCTION(xml_parser_get_option)
    Encodes an ISO-8859-1 string to UTF-8 */
 PHP_FUNCTION(utf8_encode)
 {
-	zval **arg;
+	char *arg;
 	XML_Char *encoded;
-	int len;
+	int arg_len, len;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &arg) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &arg, &arg_len) == FAILURE) {
+		return;
 	}
-	convert_to_string_ex(arg);
-	encoded = xml_utf8_encode(Z_STRVAL_PP(arg), Z_STRLEN_PP(arg), &len, "ISO-8859-1");
+
+	encoded = xml_utf8_encode(arg, arg_len, &len, "ISO-8859-1");
 	if (encoded == NULL) {
 		RETURN_FALSE;
 	}
@@ -1595,15 +1723,15 @@ PHP_FUNCTION(utf8_encode)
    Converts a UTF-8 encoded string to ISO-8859-1 */
 PHP_FUNCTION(utf8_decode)
 {
-	zval **arg;
+	char *arg;
 	XML_Char *decoded;
-	int len;
+	int arg_len, len;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &arg) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &arg, &arg_len) == FAILURE) {
+		return;
 	}
-	convert_to_string_ex(arg);
-	decoded = xml_utf8_decode(Z_STRVAL_PP(arg), Z_STRLEN_PP(arg), &len, "ISO-8859-1");
+
+	decoded = xml_utf8_decode(arg, arg_len, &len, "ISO-8859-1");
 	if (decoded == NULL) {
 		RETURN_FALSE;
 	}
