@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2007 The PHP Group                                |
+  | Copyright (c) 1997-2009 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: php_xmlwriter.c,v 1.20.2.12.2.12 2007/01/06 18:03:43 rrichards Exp $ */
+/* $Id: php_xmlwriter.c,v 1.20.2.12.2.15.2.15 2009/05/26 08:10:49 pajoye Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -28,7 +28,7 @@
 #include "php_ini.h"
 #include "ext/standard/info.h"
 #include "php_xmlwriter.h"
-
+#include "ext/standard/php_string.h"
 
 #if LIBXML_VERSION >= 20605
 static PHP_FUNCTION(xmlwriter_set_indent);
@@ -168,119 +168,423 @@ static zend_object_value xmlwriter_object_new(zend_class_entry *class_type TSRML
 
 #define XMLW_NAME_CHK(__err) \
 	if (xmlValidateName((xmlChar *) name, 0) != 0) {	\
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, __err);	\
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", __err);	\
 		RETURN_FALSE;	\
 	}	\
+	
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO(arginfo_xmlwriter_void, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_resource, 0, 0, 1)
+	ZEND_ARG_INFO(0, xmlwriter)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_open_uri, 0, 0, 1)
+	ZEND_ARG_INFO(0, uri)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_set_indent, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, indent)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_set_indent, 0, 0, 1)
+	ZEND_ARG_INFO(0, indent)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_set_indent_string, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, indentString)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_set_indent_string, 0, 0, 1)
+	ZEND_ARG_INFO(0, indentString)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_start_attribute, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_start_attribute, 0, 0, 1)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_start_attribute_ns, 0, 0, 4)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, prefix)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, uri)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_start_attribute_ns, 0, 0, 3)
+	ZEND_ARG_INFO(0, prefix)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, uri)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_write_attribute_ns, 0, 0, 5)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, prefix)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, uri)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_write_attribute_ns, 0, 0, 4)
+	ZEND_ARG_INFO(0, prefix)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, uri)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_write_attribute, 0, 0, 3)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, value)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_write_attribute, 0, 0, 2)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, value)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_start_element, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_start_element, 0, 0, 1)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_start_element_ns, 0, 0, 4)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, prefix)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, uri)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_start_element_ns, 0, 0, 3)
+	ZEND_ARG_INFO(0, prefix)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, uri)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_write_element, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_write_element, 0, 0, 1)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_write_element_ns, 0, 0, 4)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, prefix)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, uri)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_write_element_ns, 0, 0, 3)
+	ZEND_ARG_INFO(0, prefix)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, uri)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_start_pi, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, target)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_start_pi, 0, 0, 1)
+	ZEND_ARG_INFO(0, target)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_write_pi, 0, 0, 3)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, target)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_write_pi, 0, 0, 2)
+	ZEND_ARG_INFO(0, target)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_write_cdata, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_write_cdata, 0, 0, 1)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_text, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_text, 0, 0, 1)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_write_raw, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_write_raw, 0, 0, 1)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_start_document, 0, 0, 1)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, version)
+	ZEND_ARG_INFO(0, encoding)
+	ZEND_ARG_INFO(0, standalone)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_start_document, 0, 0, 0)
+	ZEND_ARG_INFO(0, version)
+	ZEND_ARG_INFO(0, encoding)
+	ZEND_ARG_INFO(0, standalone)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_write_comment, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_write_comment, 0, 0, 1)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_start_dtd, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, qualifiedName)
+	ZEND_ARG_INFO(0, publicId)
+	ZEND_ARG_INFO(0, systemId)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_start_dtd, 0, 0, 1)
+	ZEND_ARG_INFO(0, qualifiedName)
+	ZEND_ARG_INFO(0, publicId)
+	ZEND_ARG_INFO(0, systemId)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_write_dtd, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, publicId)
+	ZEND_ARG_INFO(0, systemId)
+	ZEND_ARG_INFO(0, subset)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_write_dtd, 0, 0, 1)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, publicId)
+	ZEND_ARG_INFO(0, systemId)
+	ZEND_ARG_INFO(0, subset)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_start_dtd_element, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, qualifiedName)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_start_dtd_element, 0, 0, 1)
+	ZEND_ARG_INFO(0, qualifiedName)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_write_dtd_element, 0, 0, 3)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_write_dtd_element, 0, 0, 2)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_start_dtd_attlist, 0, 0, 2)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_start_dtd_attlist, 0, 0, 1)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_write_dtd_attlist, 0, 0, 3)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_write_dtd_attlist, 0, 0, 2)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_start_dtd_entity, 0, 0, 3)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, isparam)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_start_dtd_entity, 0, 0, 2)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, isparam)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_write_dtd_entity, 0, 0, 3)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_write_dtd_entity, 0, 0, 2)
+	ZEND_ARG_INFO(0, name)
+	ZEND_ARG_INFO(0, content)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_output_memory, 0, 0, 1)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, flush)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_output_memory, 0, 0, 0)
+	ZEND_ARG_INFO(0, flush)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_flush, 0, 0, 1)
+	ZEND_ARG_INFO(0, xmlwriter)
+	ZEND_ARG_INFO(0, empty)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xmlwriter_method_flush, 0, 0, 0)
+	ZEND_ARG_INFO(0, empty)
+ZEND_END_ARG_INFO()
+/* }}} */
 
 /* {{{ xmlwriter_functions */
-static zend_function_entry xmlwriter_functions[] = {
-	PHP_FE(xmlwriter_open_uri,			NULL)
-	PHP_FE(xmlwriter_open_memory,		NULL)
+static const zend_function_entry xmlwriter_functions[] = {
+	PHP_FE(xmlwriter_open_uri,			arginfo_xmlwriter_open_uri)
+	PHP_FE(xmlwriter_open_memory,		arginfo_xmlwriter_void)
 #if LIBXML_VERSION >= 20605
-	PHP_FE(xmlwriter_set_indent,		NULL)
-	PHP_FE(xmlwriter_set_indent_string, NULL)
+	PHP_FE(xmlwriter_set_indent,		arginfo_xmlwriter_set_indent)
+	PHP_FE(xmlwriter_set_indent_string, arginfo_xmlwriter_set_indent_string)
 #endif
 #if LIBXML_VERSION >= 20607
-	PHP_FE(xmlwriter_start_comment,		NULL)
-	PHP_FE(xmlwriter_end_comment,		NULL)
+	PHP_FE(xmlwriter_start_comment,		arginfo_xmlwriter_resource)
+	PHP_FE(xmlwriter_end_comment,		arginfo_xmlwriter_resource)
 #endif
-	PHP_FE(xmlwriter_start_attribute,	NULL)
-	PHP_FE(xmlwriter_end_attribute,		NULL)
-	PHP_FE(xmlwriter_write_attribute,	NULL)
+	PHP_FE(xmlwriter_start_attribute,	arginfo_xmlwriter_start_attribute)
+	PHP_FE(xmlwriter_end_attribute,		arginfo_xmlwriter_resource)
+	PHP_FE(xmlwriter_write_attribute,	arginfo_xmlwriter_write_attribute)
 #if LIBXML_VERSION > 20617
-	PHP_FE(xmlwriter_start_attribute_ns,NULL)
-	PHP_FE(xmlwriter_write_attribute_ns,NULL)
+	PHP_FE(xmlwriter_start_attribute_ns,arginfo_xmlwriter_start_attribute_ns)
+	PHP_FE(xmlwriter_write_attribute_ns,arginfo_xmlwriter_write_attribute_ns)
 #endif
-	PHP_FE(xmlwriter_start_element,		NULL)
-	PHP_FE(xmlwriter_end_element,		NULL)
-	PHP_FE(xmlwriter_full_end_element,	NULL)
-	PHP_FE(xmlwriter_start_element_ns,	NULL)
-	PHP_FE(xmlwriter_write_element,		NULL)
-	PHP_FE(xmlwriter_write_element_ns,	NULL)
-	PHP_FE(xmlwriter_start_pi,			NULL)
-	PHP_FE(xmlwriter_end_pi,			NULL)
-	PHP_FE(xmlwriter_write_pi,			NULL)
-	PHP_FE(xmlwriter_start_cdata,		NULL)
-	PHP_FE(xmlwriter_end_cdata,			NULL)
-	PHP_FE(xmlwriter_write_cdata,		NULL)
-	PHP_FE(xmlwriter_text,				NULL)
-	PHP_FE(xmlwriter_write_raw,			NULL)
-	PHP_FE(xmlwriter_start_document,	NULL)
-	PHP_FE(xmlwriter_end_document,		NULL)
-	PHP_FE(xmlwriter_write_comment,		NULL)
-	PHP_FE(xmlwriter_start_dtd,			NULL)
-	PHP_FE(xmlwriter_end_dtd,			NULL)
-	PHP_FE(xmlwriter_write_dtd,			NULL)
-	PHP_FE(xmlwriter_start_dtd_element,	NULL)
-	PHP_FE(xmlwriter_end_dtd_element,	NULL)
-	PHP_FE(xmlwriter_write_dtd_element,	NULL)
+	PHP_FE(xmlwriter_start_element,		arginfo_xmlwriter_start_element)
+	PHP_FE(xmlwriter_end_element,		arginfo_xmlwriter_resource)
+	PHP_FE(xmlwriter_full_end_element,	arginfo_xmlwriter_resource)
+	PHP_FE(xmlwriter_start_element_ns,	arginfo_xmlwriter_start_element_ns)
+	PHP_FE(xmlwriter_write_element,		arginfo_xmlwriter_write_element)
+	PHP_FE(xmlwriter_write_element_ns,	arginfo_xmlwriter_write_element_ns)
+	PHP_FE(xmlwriter_start_pi,			arginfo_xmlwriter_start_pi)
+	PHP_FE(xmlwriter_end_pi,			arginfo_xmlwriter_resource)
+	PHP_FE(xmlwriter_write_pi,			arginfo_xmlwriter_write_pi)
+	PHP_FE(xmlwriter_start_cdata,		arginfo_xmlwriter_resource)
+	PHP_FE(xmlwriter_end_cdata,			arginfo_xmlwriter_resource)
+	PHP_FE(xmlwriter_write_cdata,		arginfo_xmlwriter_write_cdata)
+	PHP_FE(xmlwriter_text,				arginfo_xmlwriter_text)
+	PHP_FE(xmlwriter_write_raw,			arginfo_xmlwriter_write_raw)
+	PHP_FE(xmlwriter_start_document,	arginfo_xmlwriter_start_document)
+	PHP_FE(xmlwriter_end_document,		arginfo_xmlwriter_resource)
+	PHP_FE(xmlwriter_write_comment,		arginfo_xmlwriter_write_comment)
+	PHP_FE(xmlwriter_start_dtd,			arginfo_xmlwriter_start_dtd)
+	PHP_FE(xmlwriter_end_dtd,			arginfo_xmlwriter_resource)
+	PHP_FE(xmlwriter_write_dtd,			arginfo_xmlwriter_write_dtd)
+	PHP_FE(xmlwriter_start_dtd_element,	arginfo_xmlwriter_start_dtd_element)
+	PHP_FE(xmlwriter_end_dtd_element,	arginfo_xmlwriter_resource)
+	PHP_FE(xmlwriter_write_dtd_element,	arginfo_xmlwriter_write_dtd_element)
 #if LIBXML_VERSION > 20608
-	PHP_FE(xmlwriter_start_dtd_attlist,	NULL)
-	PHP_FE(xmlwriter_end_dtd_attlist,	NULL)
-	PHP_FE(xmlwriter_write_dtd_attlist,	NULL)
-	PHP_FE(xmlwriter_start_dtd_entity,	NULL)
-	PHP_FE(xmlwriter_end_dtd_entity,	NULL)
-	PHP_FE(xmlwriter_write_dtd_entity,	NULL)
+	PHP_FE(xmlwriter_start_dtd_attlist,	arginfo_xmlwriter_start_dtd_attlist)
+	PHP_FE(xmlwriter_end_dtd_attlist,	arginfo_xmlwriter_resource)
+	PHP_FE(xmlwriter_write_dtd_attlist,	arginfo_xmlwriter_write_dtd_attlist)
+	PHP_FE(xmlwriter_start_dtd_entity,	arginfo_xmlwriter_start_dtd_entity)
+	PHP_FE(xmlwriter_end_dtd_entity,	arginfo_xmlwriter_resource)
+	PHP_FE(xmlwriter_write_dtd_entity,	arginfo_xmlwriter_write_dtd_entity)
 #endif
-	PHP_FE(xmlwriter_output_memory,		NULL)
-	PHP_FE(xmlwriter_flush,				NULL)
+	PHP_FE(xmlwriter_output_memory,		arginfo_xmlwriter_output_memory)
+	PHP_FE(xmlwriter_flush,				arginfo_xmlwriter_flush)
 	{NULL, NULL, NULL}
 };
 /* }}} */
 
 #ifdef ZEND_ENGINE_2
 /* {{{ xmlwriter_class_functions */
-static zend_function_entry xmlwriter_class_functions[] = {
-	PHP_ME_MAPPING(openUri,		xmlwriter_open_uri,		NULL, 0)
-	PHP_ME_MAPPING(openMemory,	xmlwriter_open_memory, 	NULL, 0)
+static const zend_function_entry xmlwriter_class_functions[] = {
+	PHP_ME_MAPPING(openUri,		xmlwriter_open_uri,		arginfo_xmlwriter_open_uri, 0)
+	PHP_ME_MAPPING(openMemory,	xmlwriter_open_memory, 	arginfo_xmlwriter_void, 0)
 #if LIBXML_VERSION >= 20605
-	PHP_ME_MAPPING(setIndent,	xmlwriter_set_indent,	NULL, 0)
-	PHP_ME_MAPPING(setIndentString,	xmlwriter_set_indent_string, NULL, 0)
+	PHP_ME_MAPPING(setIndent,	xmlwriter_set_indent,	arginfo_xmlwriter_method_set_indent, 0)
+	PHP_ME_MAPPING(setIndentString,	xmlwriter_set_indent_string, arginfo_xmlwriter_method_set_indent_string, 0)
 #endif
 #if LIBXML_VERSION >= 20607
-	PHP_ME_MAPPING(startComment,	xmlwriter_start_comment,	NULL, 0)
-	PHP_ME_MAPPING(endComment,		xmlwriter_end_comment,		NULL, 0)
+	PHP_ME_MAPPING(startComment,	xmlwriter_start_comment,	arginfo_xmlwriter_void, 0)
+	PHP_ME_MAPPING(endComment,		xmlwriter_end_comment,		arginfo_xmlwriter_void, 0)
 #endif
-	PHP_ME_MAPPING(startAttribute,	xmlwriter_start_attribute,	NULL, 0)
-	PHP_ME_MAPPING(endAttribute,	xmlwriter_end_attribute,	NULL, 0)
-	PHP_ME_MAPPING(writeAttribute,	xmlwriter_write_attribute,	NULL, 0)
+	PHP_ME_MAPPING(startAttribute,	xmlwriter_start_attribute,	arginfo_xmlwriter_method_start_attribute, 0)
+	PHP_ME_MAPPING(endAttribute,	xmlwriter_end_attribute,	arginfo_xmlwriter_void, 0)
+	PHP_ME_MAPPING(writeAttribute,	xmlwriter_write_attribute,	arginfo_xmlwriter_method_write_attribute, 0)
 #if LIBXML_VERSION > 20617
-	PHP_ME_MAPPING(startAttributeNs,	xmlwriter_start_attribute_ns,NULL, 0)
-	PHP_ME_MAPPING(writeAttributeNs,	xmlwriter_write_attribute_ns,NULL, 0)
+	PHP_ME_MAPPING(startAttributeNs,	xmlwriter_start_attribute_ns,arginfo_xmlwriter_method_start_attribute_ns, 0)
+	PHP_ME_MAPPING(writeAttributeNs,	xmlwriter_write_attribute_ns,arginfo_xmlwriter_method_write_attribute_ns, 0)
 #endif
-	PHP_ME_MAPPING(startElement,	xmlwriter_start_element,	NULL, 0)
-	PHP_ME_MAPPING(endElement,		xmlwriter_end_element,		NULL, 0)
-	PHP_ME_MAPPING(fullEndElement,	xmlwriter_full_end_element,	NULL, 0)
-	PHP_ME_MAPPING(startElementNs,	xmlwriter_start_element_ns,	NULL, 0)
-	PHP_ME_MAPPING(writeElement,	xmlwriter_write_element,	NULL, 0)
-	PHP_ME_MAPPING(writeElementNs,	xmlwriter_write_element_ns,	NULL, 0)
-	PHP_ME_MAPPING(startPi,			xmlwriter_start_pi,			NULL, 0)
-	PHP_ME_MAPPING(endPi,			xmlwriter_end_pi,			NULL, 0)
-	PHP_ME_MAPPING(writePi,			xmlwriter_write_pi,			NULL, 0)
-	PHP_ME_MAPPING(startCdata,		xmlwriter_start_cdata,		NULL, 0)
-	PHP_ME_MAPPING(endCdata,		xmlwriter_end_cdata,		NULL, 0)
-	PHP_ME_MAPPING(writeCdata,		xmlwriter_write_cdata,		NULL, 0)
-	PHP_ME_MAPPING(text,			xmlwriter_text,				NULL, 0)
-	PHP_ME_MAPPING(writeRaw,		xmlwriter_write_raw,		NULL, 0)
-	PHP_ME_MAPPING(startDocument,	xmlwriter_start_document,	NULL, 0)
-	PHP_ME_MAPPING(endDocument,		xmlwriter_end_document,		NULL, 0)
-	PHP_ME_MAPPING(writeComment,	xmlwriter_write_comment,	NULL, 0)
-	PHP_ME_MAPPING(startDtd,		xmlwriter_start_dtd,		NULL, 0)
-	PHP_ME_MAPPING(endDtd,			xmlwriter_end_dtd,			NULL, 0)
-	PHP_ME_MAPPING(writeDtd,		xmlwriter_write_dtd,		NULL, 0)
-	PHP_ME_MAPPING(startDtdElement,	xmlwriter_start_dtd_element,	NULL, 0)
-	PHP_ME_MAPPING(endDtdElement,	xmlwriter_end_dtd_element,	NULL, 0)
-	PHP_ME_MAPPING(writeDtdElement,	xmlwriter_write_dtd_element,	NULL, 0)
+	PHP_ME_MAPPING(startElement,	xmlwriter_start_element,	arginfo_xmlwriter_method_start_element, 0)
+	PHP_ME_MAPPING(endElement,		xmlwriter_end_element,		arginfo_xmlwriter_void, 0)
+	PHP_ME_MAPPING(fullEndElement,	xmlwriter_full_end_element,	arginfo_xmlwriter_void, 0)
+	PHP_ME_MAPPING(startElementNs,	xmlwriter_start_element_ns,	arginfo_xmlwriter_method_start_element_ns, 0)
+	PHP_ME_MAPPING(writeElement,	xmlwriter_write_element,	arginfo_xmlwriter_method_write_element, 0)
+	PHP_ME_MAPPING(writeElementNs,	xmlwriter_write_element_ns,	arginfo_xmlwriter_method_write_element_ns, 0)
+	PHP_ME_MAPPING(startPi,			xmlwriter_start_pi,			arginfo_xmlwriter_method_start_pi, 0)
+	PHP_ME_MAPPING(endPi,			xmlwriter_end_pi,			arginfo_xmlwriter_void, 0)
+	PHP_ME_MAPPING(writePi,			xmlwriter_write_pi,			arginfo_xmlwriter_method_write_pi, 0)
+	PHP_ME_MAPPING(startCdata,		xmlwriter_start_cdata,		arginfo_xmlwriter_void, 0)
+	PHP_ME_MAPPING(endCdata,		xmlwriter_end_cdata,		arginfo_xmlwriter_void, 0)
+	PHP_ME_MAPPING(writeCdata,		xmlwriter_write_cdata,		arginfo_xmlwriter_method_write_cdata, 0)
+	PHP_ME_MAPPING(text,			xmlwriter_text,				arginfo_xmlwriter_method_text, 0)
+	PHP_ME_MAPPING(writeRaw,		xmlwriter_write_raw,		arginfo_xmlwriter_method_write_raw, 0)
+	PHP_ME_MAPPING(startDocument,	xmlwriter_start_document,	arginfo_xmlwriter_method_start_document, 0)
+	PHP_ME_MAPPING(endDocument,		xmlwriter_end_document,		arginfo_xmlwriter_void, 0)
+	PHP_ME_MAPPING(writeComment,	xmlwriter_write_comment,	arginfo_xmlwriter_method_write_comment, 0)
+	PHP_ME_MAPPING(startDtd,		xmlwriter_start_dtd,		arginfo_xmlwriter_method_start_dtd, 0)
+	PHP_ME_MAPPING(endDtd,			xmlwriter_end_dtd,			arginfo_xmlwriter_void, 0)
+	PHP_ME_MAPPING(writeDtd,		xmlwriter_write_dtd,		arginfo_xmlwriter_method_write_dtd, 0)
+	PHP_ME_MAPPING(startDtdElement,	xmlwriter_start_dtd_element,arginfo_xmlwriter_method_start_dtd_element, 0)
+	PHP_ME_MAPPING(endDtdElement,	xmlwriter_end_dtd_element,	arginfo_xmlwriter_void, 0)
+	PHP_ME_MAPPING(writeDtdElement,	xmlwriter_write_dtd_element,	arginfo_xmlwriter_method_write_dtd_element, 0)
 #if LIBXML_VERSION > 20608
-	PHP_ME_MAPPING(startDtdAttlist,	xmlwriter_start_dtd_attlist,	NULL, 0)
-	PHP_ME_MAPPING(endDtdAttlist,	xmlwriter_end_dtd_attlist,	NULL, 0)
-	PHP_ME_MAPPING(writeDtdAttlist,	xmlwriter_write_dtd_attlist,	NULL, 0)
-	PHP_ME_MAPPING(startDtdEntity,	xmlwriter_start_dtd_entity,	NULL, 0)
-	PHP_ME_MAPPING(endDtdEntity,	xmlwriter_end_dtd_entity,	NULL, 0)
-	PHP_ME_MAPPING(writeDtdEntity,	xmlwriter_write_dtd_entity,	NULL, 0)
+	PHP_ME_MAPPING(startDtdAttlist,	xmlwriter_start_dtd_attlist,	arginfo_xmlwriter_method_start_dtd_attlist, 0)
+	PHP_ME_MAPPING(endDtdAttlist,	xmlwriter_end_dtd_attlist,	arginfo_xmlwriter_void, 0)
+	PHP_ME_MAPPING(writeDtdAttlist,	xmlwriter_write_dtd_attlist,	arginfo_xmlwriter_method_write_dtd_attlist, 0)
+	PHP_ME_MAPPING(startDtdEntity,	xmlwriter_start_dtd_entity,	arginfo_xmlwriter_method_start_dtd_entity, 0)
+	PHP_ME_MAPPING(endDtdEntity,	xmlwriter_end_dtd_entity,	arginfo_xmlwriter_void, 0)
+	PHP_ME_MAPPING(writeDtdEntity,	xmlwriter_write_dtd_entity,	arginfo_xmlwriter_method_write_dtd_entity, 0)
 #endif
-	PHP_ME_MAPPING(outputMemory,	xmlwriter_output_memory,	NULL, 0)
-	PHP_ME_MAPPING(flush,			xmlwriter_flush,			NULL, 0)
+	PHP_ME_MAPPING(outputMemory,	xmlwriter_output_memory,	arginfo_xmlwriter_method_output_memory, 0)
+	PHP_ME_MAPPING(flush,			xmlwriter_flush,			arginfo_xmlwriter_method_flush, 0)
 	{NULL, NULL, NULL}
 };
 /* }}} */
@@ -310,7 +614,10 @@ static char *_xmlwriter_get_valid_file_path(char *source, char *resolved_path, i
 
 	if (uri->scheme != NULL) {
 		/* absolute file uris - libxml only supports localhost or empty host */
-		if (strncasecmp(source, "file:///",8) == 0) {
+		if (strncasecmp(source, "file:///", 8) == 0) {
+			if (source[sizeof("file:///") - 1] == '\0') {
+				return NULL;
+			}
 			isFileUri = 1;
 #ifdef PHP_WIN32
 			source += 8;
@@ -318,6 +625,10 @@ static char *_xmlwriter_get_valid_file_path(char *source, char *resolved_path, i
 			source += 7;
 #endif
 		} else if (strncasecmp(source, "file://localhost/",17) == 0) {
+			if (source[sizeof("file://localhost/") - 1] == '\0') {
+				return NULL;
+			}
+
 			isFileUri = 1;
 #ifdef PHP_WIN32
 			source += 17;
@@ -327,14 +638,29 @@ static char *_xmlwriter_get_valid_file_path(char *source, char *resolved_path, i
 		}
 	}
 
-	file_dest = source;
-
 	if ((uri->scheme == NULL || isFileUri)) {
+		char file_dirname[MAXPATHLEN];
+		size_t dir_len;
+
 		if (!VCWD_REALPATH(source, resolved_path) && !expand_filepath(source, resolved_path TSRMLS_CC)) {
 			xmlFreeURI(uri);
 			return NULL;
 		}
+
+		memcpy(file_dirname, source, strlen(source));
+		dir_len = php_dirname(file_dirname, strlen(source));
+
+		if (dir_len > 0) {
+			struct stat buf;
+			if (php_sys_stat(file_dirname, &buf) != 0) {
+				xmlFreeURI(uri);
+				return NULL;
+			}
+		}
+
 		file_dest = resolved_path;
+	} else {
+		file_dest = source;
 	}
 
 	xmlFreeURI(uri);
@@ -464,6 +790,9 @@ static void php_xmlwriter_end(INTERNAL_FUNCTION_PARAMETERS, xmlwriter_read_int_t
 	
 	if (this) {
 		XMLWRITER_FROM_OBJECT(intern, this);
+		if (zend_parse_parameters_none() == FAILURE) {
+			return;
+		}
 	} else 
 #endif
 	{
@@ -566,7 +895,7 @@ static PHP_FUNCTION(xmlwriter_start_attribute_ns)
 	zval *this = getThis();
 	
 	if (this) {
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sss", 
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sss!", 
 			&prefix, &prefix_len, &name, &name_len, &uri, &uri_len) == FAILURE) {
 			return;
 		}
@@ -574,7 +903,7 @@ static PHP_FUNCTION(xmlwriter_start_attribute_ns)
 	} else
 #endif
 	{
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rsss", &pind, 
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rsss!", &pind, 
 			&prefix, &prefix_len, &name, &name_len, &uri, &uri_len) == FAILURE) {
 			return;
 		}
@@ -656,7 +985,7 @@ static PHP_FUNCTION(xmlwriter_write_attribute_ns)
 	zval *this = getThis();
 	
 	if (this) {
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ssss", 
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sss!s", 
 			&prefix, &prefix_len, &name, &name_len, &uri, &uri_len, &content, &content_len) == FAILURE) {
 			return;
 		}
@@ -664,7 +993,7 @@ static PHP_FUNCTION(xmlwriter_write_attribute_ns)
 	} else
 #endif
 	{
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rssss", &pind, 
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rsss!s", &pind, 
 			&prefix, &prefix_len, &name, &name_len, &uri, &uri_len, &content, &content_len) == FAILURE) {
 			return;
 		}
@@ -695,7 +1024,6 @@ static PHP_FUNCTION(xmlwriter_start_element)
 }
 /* }}} */
 
-
 /* {{{ proto bool xmlwriter_start_element_ns(resource xmlwriter, string prefix, string name, string uri)
 Create start namespaced element tag - returns FALSE on error */
 static PHP_FUNCTION(xmlwriter_start_element_ns)
@@ -709,7 +1037,7 @@ static PHP_FUNCTION(xmlwriter_start_element_ns)
 	zval *this = getThis();
 	
 	if (this) {
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s!ss",
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s!ss!",
 			&prefix, &prefix_len, &name, &name_len, &uri, &uri_len) == FAILURE) {
 			return;
 		}
@@ -717,7 +1045,7 @@ static PHP_FUNCTION(xmlwriter_start_element_ns)
 	} else
 #endif
 	{
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs!ss", &pind, 
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs!ss!", &pind, 
 			&prefix, &prefix_len, &name, &name_len, &uri, &uri_len) == FAILURE) {
 			return;
 		}
@@ -756,20 +1084,21 @@ static PHP_FUNCTION(xmlwriter_full_end_element)
 }
 /* }}} */
 
-/* {{{ proto bool xmlwriter_write_element(resource xmlwriter, string name, string content)
+/* {{{ proto bool xmlwriter_write_element(resource xmlwriter, string name[, string content])
 Write full element tag - returns FALSE on error */
 static PHP_FUNCTION(xmlwriter_write_element)
 {
 	zval *pind;
 	xmlwriter_object *intern;
 	xmlTextWriterPtr ptr;
-	char *name, *content;
+	char *name, *content = NULL;
 	int name_len, content_len, retval;
+
 #ifdef ZEND_ENGINE_2
 	zval *this = getThis();
 	
 	if (this) {
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss",
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|s!",
 			&name, &name_len, &content, &content_len) == FAILURE) {
 			return;
 		}
@@ -777,7 +1106,7 @@ static PHP_FUNCTION(xmlwriter_write_element)
 	} else
 #endif
 	{
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rss", &pind, 
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|s!", &pind, 
 			&name, &name_len, &content, &content_len) == FAILURE) {
 			return;
 		}
@@ -789,7 +1118,18 @@ static PHP_FUNCTION(xmlwriter_write_element)
 	ptr = intern->ptr;
 
 	if (ptr) {
-		retval = xmlTextWriterWriteElement(ptr, (xmlChar *)name, (xmlChar *)content);
+		if (!content) {
+			retval = xmlTextWriterStartElement(ptr, (xmlChar *)name);
+            if (retval == -1) {
+                RETURN_FALSE;
+            }
+			xmlTextWriterEndElement(ptr);
+            if (retval == -1) {
+                RETURN_FALSE;
+            }
+		} else {
+			retval = xmlTextWriterWriteElement(ptr, (xmlChar *)name, (xmlChar *)content);
+		}
 		if (retval != -1) {
 			RETURN_TRUE;
 		}
@@ -799,21 +1139,21 @@ static PHP_FUNCTION(xmlwriter_write_element)
 }
 /* }}} */
 
-/* {{{ proto bool xmlwriter_write_element_ns(resource xmlwriter, string prefix, string name, string uri, string content)
+/* {{{ proto bool xmlwriter_write_element_ns(resource xmlwriter, string prefix, string name, string uri[, string content])
 Write full namesapced element tag - returns FALSE on error */
 static PHP_FUNCTION(xmlwriter_write_element_ns)
 {
 	zval *pind;
 	xmlwriter_object *intern;
 	xmlTextWriterPtr ptr;
-	char *name, *prefix, *uri, *content;
+	char *name, *prefix, *uri, *content = NULL;
 	int name_len, prefix_len, uri_len, content_len, retval;
 
 #ifdef ZEND_ENGINE_2
 	zval *this = getThis();
 	
 	if (this) {
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s!sss", 
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s!ss!|s!", 
 			&prefix, &prefix_len, &name, &name_len, &uri, &uri_len, &content, &content_len) == FAILURE) {
 			return;
 		}
@@ -821,7 +1161,7 @@ static PHP_FUNCTION(xmlwriter_write_element_ns)
 	} else
 #endif
 	{
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs!sss", &pind, 
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs!ss!|s!", &pind, 
 			&prefix, &prefix_len, &name, &name_len, &uri, &uri_len, &content, &content_len) == FAILURE) {
 			return;
 		}
@@ -833,7 +1173,18 @@ static PHP_FUNCTION(xmlwriter_write_element_ns)
 	ptr = intern->ptr;
 
 	if (ptr) {
-		retval = xmlTextWriterWriteElementNS(ptr, (xmlChar *)prefix, (xmlChar *)name, (xmlChar *)uri, (xmlChar *)content);
+		if (!content) {
+			retval = xmlTextWriterStartElementNS(ptr,(xmlChar *)prefix, (xmlChar *)name, (xmlChar *)uri);
+            if (retval == -1) {
+                RETURN_FALSE;
+            }
+			retval = xmlTextWriterEndElement(ptr);
+            if (retval == -1) {
+                RETURN_FALSE;
+            }
+		} else {
+			retval = xmlTextWriterWriteElementNS(ptr, (xmlChar *)prefix, (xmlChar *)name, (xmlChar *)uri, (xmlChar *)content);
+		}
 		if (retval != -1) {
 			RETURN_TRUE;
 		}
@@ -1426,9 +1777,13 @@ static PHP_FUNCTION(xmlwriter_open_uri)
 
 	valid_file = _xmlwriter_get_valid_file_path(source, resolved_path, MAXPATHLEN TSRMLS_CC);
 	if (!valid_file) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to resolve file path");
 		RETURN_FALSE;
 	}
 
+	/* TODO: Fix either the PHP stream or libxml APIs: it can then detect when a given 
+		 path is valid and not report out of memory error. Once it is done, remove the
+		 directory check in _xmlwriter_get_valid_file_path */
 #ifndef ZEND_ENGINE_2
 	ioctx = php_xmlwriter_streams_IO_open_write_wrapper(valid_file TSRMLS_CC);
 	if (ioctx == NULL) {
@@ -1458,6 +1813,9 @@ static PHP_FUNCTION(xmlwriter_open_uri)
 	intern->uri_output = out_buffer;
 #else
 	if (this) {
+		if (ze_obj->xmlwriter_ptr) {
+			xmlwriter_free_resource_ptr(ze_obj->xmlwriter_ptr TSRMLS_CC);
+		}
 		ze_obj->xmlwriter_ptr = intern;
 		RETURN_TRUE;
 	} else
@@ -1508,6 +1866,9 @@ static PHP_FUNCTION(xmlwriter_open_memory)
 	intern->uri_output = NULL;
 #else
 	if (this) {
+		if (ze_obj->xmlwriter_ptr) {
+			xmlwriter_free_resource_ptr(ze_obj->xmlwriter_ptr TSRMLS_CC);
+		}
 		ze_obj->xmlwriter_ptr = intern;
 		RETURN_TRUE;
 	} else
