@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: SAPI.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: SAPI.c,v 1.202.2.7.2.15.2.7 2009/04/28 21:30:23 stas Exp $ */
 
 #include <ctype.h>
 #include <sys/stat.h>
@@ -32,6 +32,9 @@
 #if (HAVE_PCRE || HAVE_BUNDLED_PCRE) && !defined(COMPILE_DL_PCRE)
 #include "ext/pcre/php_pcre.h"
 #endif
+#if HAVE_ZLIB
+#include "ext/zlib/php_zlib.h"
+#endif
 #ifdef ZTS
 #include "TSRM.h"
 #endif
@@ -323,6 +326,9 @@ SAPI_API void sapi_activate_headers_only(TSRMLS_D)
 			sapi_module.activate(TSRMLS_C);
 		}
 	}
+	if (sapi_module.input_filter_init ) {
+		sapi_module.input_filter_init(TSRMLS_C);
+	}
 }
 
 /*
@@ -389,6 +395,9 @@ SAPI_API void sapi_activate(TSRMLS_D)
 			sapi_module.activate(TSRMLS_C);
 		}
 	}
+	if (sapi_module.input_filter_init ) {
+		sapi_module.input_filter_init(TSRMLS_C);
+	}
 }
 
 
@@ -494,7 +503,8 @@ static void sapi_update_response_code(int ncode TSRMLS_DC)
 
 static int sapi_find_matching_header(void *element1, void *element2)
 {
-	return strncasecmp(((sapi_header_struct*)element1)->header, (char*)element2, strlen((char*)element2)) == 0;
+	int len = strlen((char*)element2);
+	return strncasecmp(((sapi_header_struct*)element1)->header, (char*)element2, len) == 0 && ((sapi_header_struct*)element1)->header[len] == ':';
 }
 
 SAPI_API int sapi_add_header_ex(char *header_line, uint header_line_len, zend_bool duplicate, zend_bool replace TSRMLS_DC)
@@ -522,7 +532,6 @@ SAPI_API int sapi_header_op(sapi_header_op_enum op, void *arg TSRMLS_DC)
 	long myuid = 0L;
 	char *header_line;
 	uint header_line_len;
-	zend_bool replace;
 	int http_response_code;
 	
 	if (SG(headers_sent) && !SG(request_info).no_headers) {
@@ -543,8 +552,9 @@ SAPI_API int sapi_header_op(sapi_header_op_enum op, void *arg TSRMLS_DC)
 			sapi_update_response_code((int)(zend_intptr_t) arg TSRMLS_CC);
 			return SUCCESS;
 
+		case SAPI_HEADER_ADD:
 		case SAPI_HEADER_REPLACE:
-		case SAPI_HEADER_ADD: {
+		case SAPI_HEADER_DELETE: {
 				sapi_header_line *p = arg;
 
 				if (!p->line || !p->line_len) {
@@ -553,10 +563,16 @@ SAPI_API int sapi_header_op(sapi_header_op_enum op, void *arg TSRMLS_DC)
 				header_line = p->line;
 				header_line_len = p->line_len;
 				http_response_code = p->response_code;
-				replace = (op == SAPI_HEADER_REPLACE);
 				break;
 			}
 
+		case SAPI_HEADER_DELETE_ALL:
+			if (sapi_module.header_handler) {
+				sapi_module.header_handler(&sapi_header, op, &SG(sapi_headers) TSRMLS_CC);
+			}
+			zend_llist_clean(&SG(sapi_headers).headers);
+			return SUCCESS;
+
 		default:
 			return FAILURE;
 	}
@@ -567,8 +583,14 @@ SAPI_API int sapi_header_op(sapi_header_op_enum op, void *arg TSRMLS_DC)
 	while(header_line_len && isspace(header_line[header_line_len-1])) 
 		  header_line[--header_line_len]='\0';
 	
-	/* new line safety check */
-	{
+	if (op == SAPI_HEADER_DELETE) {
+		if (strchr(header_line, ':')) {
+			efree(header_line);
+			sapi_module.sapi_error(E_WARNING, "Header to delete may not contain colon.");
+			return FAILURE;
+		}
+	} else {
+		/* new line safety check */
 		char *s = header_line, *e = header_line + header_line_len, *p;
 		while (s < e && (p = memchr(s, '\n', (e - s)))) {
 			if (*(p + 1) == ' ' || *(p + 1) == '\t') {
@@ -583,7 +605,15 @@ SAPI_API int sapi_header_op(sapi_header_op_enum op, void *arg TSRMLS_DC)
 
 	sapi_header.header = header_line;
 	sapi_header.header_len = header_line_len;
-	sapi_header.replace = replace;
+
+	if (op == SAPI_HEADER_DELETE) {
+		if (sapi_module.header_handler) {
+			sapi_module.header_handler(&sapi_header, op, &SG(sapi_headers) TSRMLS_CC);
+		}
+		zend_llist_del_element(&SG(sapi_headers).headers, sapi_header.header, (int(*)(void*, void*))sapi_find_matching_header);
+		sapi_free_header(&sapi_header);
+		return SUCCESS;
+	}
 
 	/* Check the header for a few cases that we have special support for in SAPI */
 	if (header_line_len>=5 
@@ -607,12 +637,11 @@ SAPI_API int sapi_header_op(sapi_header_op_enum op, void *arg TSRMLS_DC)
 					ptr++;
 					len--;
 				}
-
-				/* Disable possible output compression for images */
-				if (!strncmp(ptr, "image/", sizeof("image/")-1)) {
+#if HAVE_ZLIB
+				if(!strncmp(ptr, "image/", sizeof("image/")-1)) {
 					zend_alter_ini_entry("zlib.output_compression", sizeof("zlib.output_compression"), "0", sizeof("0") - 1, PHP_INI_USER, PHP_INI_STAGE_RUNTIME);
 				}
-
+#endif
 				mimetype = estrdup(ptr);
 				newlen = sapi_apply_default_charset(&mimetype, len TSRMLS_CC);
 				if (!SG(sapi_headers).mimetype){
@@ -726,20 +755,16 @@ SAPI_API int sapi_header_op(sapi_header_op_enum op, void *arg TSRMLS_DC)
 		sapi_update_response_code(http_response_code TSRMLS_CC);
 	}
 	if (sapi_module.header_handler) {
-		retval = sapi_module.header_handler(&sapi_header, &SG(sapi_headers) TSRMLS_CC);
+		retval = sapi_module.header_handler(&sapi_header, op, &SG(sapi_headers) TSRMLS_CC);
 	} else {
 		retval = SAPI_HEADER_ADD;
 	}
-	if (retval & SAPI_HEADER_DELETE_ALL) {
-		zend_llist_clean(&SG(sapi_headers).headers);
-	}
 	if (retval & SAPI_HEADER_ADD) {
 		/* in replace mode first remove the header if it already exists in the headers llist */
-		if (replace) {
+		if (op == SAPI_HEADER_REPLACE) {
 			colon_offset = strchr(sapi_header.header, ':');
 			if (colon_offset) {
 				char sav;
-				colon_offset++;
 				sav = *colon_offset;
 				*colon_offset = 0;
 				zend_llist_del_element(&SG(sapi_headers).headers, sapi_header.header, (int(*)(void*, void*))sapi_find_matching_header);
@@ -748,6 +773,8 @@ SAPI_API int sapi_header_op(sapi_header_op_enum op, void *arg TSRMLS_DC)
 		}
 
 		zend_llist_add_element(&SG(sapi_headers).headers, (void *) &sapi_header);
+	} else {
+		sapi_free_header(&sapi_header);
 	}
 	return SUCCESS;
 }
@@ -762,6 +789,36 @@ SAPI_API int sapi_send_headers(TSRMLS_D)
 		return SUCCESS;
 	}
 
+#if HAVE_ZLIB
+	/* Add output compression headers at this late stage in order to make
+	   it possible to switch it off inside the script. */
+
+	if (zend_ini_long("zlib.output_compression", sizeof("zlib.output_compression"), 0)) {
+		zval nm_zlib_get_coding_type;
+		zval *uf_result = NULL;
+
+		ZVAL_STRINGL(&nm_zlib_get_coding_type, "zlib_get_coding_type", sizeof("zlib_get_coding_type") - 1, 0);
+
+		if (call_user_function_ex(CG(function_table), NULL, &nm_zlib_get_coding_type, &uf_result, 0, NULL, 1, NULL TSRMLS_CC) != FAILURE && uf_result != NULL && Z_TYPE_P(uf_result) == IS_STRING) {
+			char buf[128];
+			int len;
+
+			assert(Z_STRVAL_P(uf_result) != NULL);
+
+			len = slprintf(buf, sizeof(buf), "Content-Encoding: %s", Z_STRVAL_P(uf_result));
+			if (len <= 0 || sapi_add_header(buf, len, 1) == FAILURE) {
+				return FAILURE;
+			}
+			if (sapi_add_header_ex("Vary: Accept-Encoding", sizeof("Vary: Accept-Encoding") - 1, 1, 0 TSRMLS_CC) == FAILURE) {
+				return FAILURE;			
+			}
+		}
+		if (uf_result != NULL) {
+			zval_ptr_dtor(&uf_result);
+		}
+	}
+#endif
+
 	/* Success-oriented.  We set headers_sent to 1 here to avoid an infinite loop
 	 * in case of an error situation.
 	 */
@@ -874,13 +931,14 @@ SAPI_API int sapi_register_treat_data(void (*treat_data)(int arg, char *str, zva
 	return SUCCESS;
 }
 
-SAPI_API int sapi_register_input_filter(unsigned int (*input_filter)(int arg, char *var, char **val, unsigned int val_len, unsigned int *new_val_len TSRMLS_DC))
+SAPI_API int sapi_register_input_filter(unsigned int (*input_filter)(int arg, char *var, char **val, unsigned int val_len, unsigned int *new_val_len TSRMLS_DC), unsigned int (*input_filter_init)(TSRMLS_D))
 {
 	TSRMLS_FETCH();
 	if (SG(sapi_started) && EG(in_execution)) {
 		return FAILURE;
 	}
 	sapi_module.input_filter = input_filter;
+	sapi_module.input_filter_init = input_filter_init;
 	return SUCCESS;
 }
 
@@ -970,6 +1028,12 @@ SAPI_API time_t sapi_get_request_time(TSRMLS_D)
 	return SG(global_request_time);
 }
 
+SAPI_API void sapi_terminate_process(TSRMLS_D) {
+	if (sapi_module.terminate_process) {
+		sapi_module.terminate_process(TSRMLS_C);
+	}
+}
+
 /*
  * Local variables:
  * tab-width: 4
