@@ -2,7 +2,7 @@
 	+----------------------------------------------------------------------+
 	| PHP Version 5                                                        |
 	+----------------------------------------------------------------------+
-	| Copyright (c) 1997-2006 The PHP Group                                |
+	| Copyright (c) 1997-2009 The PHP Group                                |
 	+----------------------------------------------------------------------+
 	| This source file is subject to version 3.01 of the PHP license,      |
 	| that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
 	+----------------------------------------------------------------------+
 */
 
-/* $Id: internal_functions_win32.c,v 1.87.2.3 2006/06/05 08:45:50 stas Exp $ */
+/* $Id: internal_functions_win32.c,v 1.87.2.1.2.3.2.3 2008/12/31 11:15:47 sebastian Exp $ */
 
 /* {{{ includes
  */
@@ -118,7 +118,7 @@
 
 /* {{{ php_builtin_extensions[]
  */
-zend_module_entry *php_builtin_extensions[] = {
+static zend_module_entry *php_builtin_extensions[] = {
 	phpext_standard_ptr
 #if HAVE_BCMATH
 	,phpext_bcmath_ptr
@@ -193,7 +193,7 @@ zend_module_entry *php_builtin_extensions[] = {
 
 #define EXTCOUNT (sizeof(php_builtin_extensions)/sizeof(zend_module_entry *))
 	
-int php_register_internal_extensions(TSRMLS_D)
+PHPAPI int php_register_internal_extensions(TSRMLS_D)
 {
 	return php_register_extensions(php_builtin_extensions, EXTCOUNT TSRMLS_CC);
 }
