@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: output.c,v 1.164.2.2 2005/08/24 16:20:06 iliaa Exp $ */
+/* $Id: output.c,v 1.167.2.3.2.4.2.13 2009/03/25 23:59:45 cseiler Exp $ */
 
 #include "php.h"
 #include "ext/standard/head.h"
@@ -112,7 +112,7 @@ void php_output_register_constants(TSRMLS_D)
 /* }}} */
 
 
-/* {{{ php_body_wirte
+/* {{{ php_body_write
  * Write body part */
 PHPAPI int php_body_write(const char *str, uint str_length TSRMLS_DC)
 {
@@ -120,7 +120,7 @@ PHPAPI int php_body_write(const char *str, uint str_length TSRMLS_DC)
 }
 /* }}} */
 
-/* {{{ php_header_wirte
+/* {{{ php_header_write
  * Write HTTP header */
 PHPAPI int php_header_write(const char *str, uint str_length TSRMLS_DC)
 {
@@ -227,8 +227,8 @@ PHPAPI void php_end_ob_buffer(zend_bool send_buffer, zend_bool just_flush TSRMLS
 
 		ALLOC_INIT_ZVAL(orig_buffer);
 		ZVAL_STRINGL(orig_buffer, OG(active_ob_buffer).buffer, OG(active_ob_buffer).text_length, 1);
-		orig_buffer->refcount=2;	/* don't let call_user_function() destroy our buffer */
-		orig_buffer->is_ref=1;
+		Z_SET_REFCOUNT_P(orig_buffer, 2);	/* don't let call_user_function() destroy our buffer */
+		Z_SET_ISREF_P(orig_buffer);
 
 		ALLOC_INIT_ZVAL(z_status);
 		ZVAL_LONG(z_status, status);
@@ -248,8 +248,8 @@ PHPAPI void php_end_ob_buffer(zend_bool send_buffer, zend_bool just_flush TSRMLS
 		if (!just_flush) {
 			zval_ptr_dtor(&OG(active_ob_buffer).output_handler);
 		}
-		orig_buffer->refcount -=2;
-		if (orig_buffer->refcount <= 0) { /* free the zval */
+		Z_SET_REFCOUNT_P(orig_buffer, Z_REFCOUNT_P(orig_buffer) - 2);
+		if (Z_REFCOUNT_P(orig_buffer) <= 0) { /* free the zval */
 			zval_dtor(orig_buffer);
 			FREE_ZVAL(orig_buffer);
 		}
@@ -416,9 +416,25 @@ PHPAPI int php_ob_init_conflict(char *handler_new, char *handler_set TSRMLS_DC)
  */
 static int php_ob_init_named(uint initial_size, uint block_size, char *handler_name, zval *output_handler, uint chunk_size, zend_bool erase TSRMLS_DC)
 {
-	if (output_handler && !zend_is_callable(output_handler, 0, NULL)) {
+	php_ob_buffer tmp_buf;
+
+	if (output_handler && !zend_is_callable(output_handler, 0, NULL TSRMLS_CC)) {
 		return FAILURE;
 	}
+	
+	tmp_buf.block_size = block_size;
+	tmp_buf.size = initial_size;
+	tmp_buf.buffer = (char *) emalloc(initial_size+1);
+	tmp_buf.text_length = 0;
+	tmp_buf.output_handler = output_handler;
+	tmp_buf.chunk_size = chunk_size;
+	tmp_buf.status = 0;
+	tmp_buf.internal_output_handler = NULL;
+	tmp_buf.internal_output_handler_buffer = NULL;
+	tmp_buf.internal_output_handler_buffer_size = 0;
+	tmp_buf.handler_name = estrdup(handler_name&&handler_name[0]?handler_name:OB_DEFAULT_HANDLER_NAME);
+	tmp_buf.erase = erase;
+
 	if (OG(ob_nesting_level)>0) {
 #if HAVE_ZLIB && !defined(COMPILE_DL_ZLIB)
 		if (!strncmp(handler_name, "ob_gzhandler", sizeof("ob_gzhandler")) && php_ob_gzhandler_check(TSRMLS_C)) {
@@ -431,16 +447,7 @@ static int php_ob_init_named(uint initial_size, uint block_size, char *handler_n
 		zend_stack_push(&OG(ob_buffers), &OG(active_ob_buffer), sizeof(php_ob_buffer));
 	}
 	OG(ob_nesting_level)++;
-	OG(active_ob_buffer).block_size = block_size;
-	OG(active_ob_buffer).size = initial_size;
-	OG(active_ob_buffer).buffer = (char *) emalloc(initial_size+1);
-	OG(active_ob_buffer).text_length = 0;
-	OG(active_ob_buffer).output_handler = output_handler;
-	OG(active_ob_buffer).chunk_size = chunk_size;
-	OG(active_ob_buffer).status = 0;
-	OG(active_ob_buffer).internal_output_handler = NULL;
-	OG(active_ob_buffer).handler_name = estrdup(handler_name&&handler_name[0]?handler_name:OB_DEFAULT_HANDLER_NAME);
-	OG(active_ob_buffer).erase = erase;
+	OG(active_ob_buffer) = tmp_buf;
 	OG(php_body_write) = php_b_body_write;
 	return SUCCESS;
 }
@@ -501,9 +508,9 @@ static int php_ob_init(uint initial_size, uint block_size, zval *output_handler,
 		}
 	} else if (output_handler && output_handler->type == IS_ARRAY) {
 		/* do we have array(object,method) */
-		if (zend_is_callable(output_handler, 0, &handler_name)) {
+		if (zend_is_callable(output_handler, 0, &handler_name TSRMLS_CC)) {
 			SEPARATE_ZVAL(&output_handler);
-			output_handler->refcount++;
+			Z_ADDREF_P(output_handler);
 			result = php_ob_init_named(initial_size, block_size, handler_name, output_handler, chunk_size, erase TSRMLS_CC);
 			efree(handler_name);
 		} else {
@@ -519,8 +526,17 @@ static int php_ob_init(uint initial_size, uint block_size, zval *output_handler,
 			}
 		}
 	} else if (output_handler && output_handler->type == IS_OBJECT) {
-		php_error_docref(NULL TSRMLS_CC, E_ERROR, "No method name given: use ob_start(array($object,'method')) to specify instance $object and the name of a method of class %s to use as output handler", Z_OBJCE_P(output_handler)->name);
-		result = FAILURE;
+		/* do we have callable object */
+		if (zend_is_callable(output_handler, 0, &handler_name TSRMLS_CC)) {
+			SEPARATE_ZVAL(&output_handler);
+			Z_ADDREF_P(output_handler);
+			result = php_ob_init_named(initial_size, block_size, handler_name, output_handler, chunk_size, erase TSRMLS_CC);
+			efree(handler_name);
+		} else {
+			efree(handler_name);
+			php_error_docref(NULL TSRMLS_CC, E_ERROR, "No method name given: use ob_start(array($object,'method')) to specify instance $object and the name of a method of class %s to use as output handler", Z_OBJCE_P(output_handler)->name);
+			result = FAILURE;
+		}
 	} else {
 		result = php_ob_init_named(initial_size, block_size, OB_DEFAULT_HANDLER_NAME, NULL, chunk_size, erase TSRMLS_CC);
 	}
@@ -542,11 +558,10 @@ static int php_ob_list_each(php_ob_buffer *ob_buffer, zval *ob_handler_array)
  */
 PHP_FUNCTION(ob_list_handlers)
 {
-	if (ZEND_NUM_ARGS()!=0) {
-		ZEND_WRONG_PARAM_COUNT();
-		RETURN_FALSE;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
-
+	
 	array_init(return_value);
 	if (OG(ob_nesting_level)) {
 		if (OG(ob_nesting_level)>1) {
@@ -606,11 +621,7 @@ static inline void php_ob_append(const char *text, uint text_length TSRMLS_DC)
  	/* If implicit_flush is On or chunked buffering, send contents to next buffer and return. */
 	if (OG(active_ob_buffer).chunk_size
 		&& OG(active_ob_buffer).text_length >= OG(active_ob_buffer).chunk_size) {
-		zval *output_handler = OG(active_ob_buffer).output_handler;
 		
-		if (output_handler) {
-			output_handler->refcount++;
-		}
 		php_end_ob_buffer(1, 1 TSRMLS_CC);
 		return;
 	}
@@ -701,6 +712,9 @@ PHPAPI int php_ub_body_write(const char *str, uint str_length TSRMLS_DC)
 	int result = 0;
 
 	if (SG(request_info).headers_only) {
+		if(SG(headers_sent)) {
+			return 0;
+		}
 		php_header(TSRMLS_C);
 		zend_bailout();
 	}
@@ -732,15 +746,14 @@ PHP_FUNCTION(ob_start)
 	zval *output_handler=NULL;
 	long chunk_size=0;
 	zend_bool erase=1;
-	int argc = ZEND_NUM_ARGS();
-	
-	if (zend_parse_parameters(argc TSRMLS_CC, "|zlb", &output_handler, &chunk_size, &erase) == FAILURE) {
-		RETURN_FALSE;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|zlb", &output_handler, &chunk_size, &erase) == FAILURE) {
+		return;
 	}
 
 	if (chunk_size < 0)
 		chunk_size = 0;
-	
+
 	if (php_start_ob_buffer(output_handler, chunk_size, erase TSRMLS_CC)==FAILURE) {
 		RETURN_FALSE;
 	}
@@ -752,8 +765,8 @@ PHP_FUNCTION(ob_start)
    Flush (send) contents of the output buffer. The last buffer content is sent to next buffer */
 PHP_FUNCTION(ob_flush)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		ZEND_WRONG_PARAM_COUNT();
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 
 	if (!OG(ob_nesting_level)) {
@@ -761,6 +774,11 @@ PHP_FUNCTION(ob_flush)
 		RETURN_FALSE;
 	}
 	
+	if (!OG(active_ob_buffer).status && !OG(active_ob_buffer).erase) {
+		php_error_docref("ref.outcontrol" TSRMLS_CC, E_NOTICE, "failed to flush buffer %s.", OG(active_ob_buffer).handler_name);
+		RETURN_FALSE;
+	}
+	
 	php_end_ob_buffer(1, 1 TSRMLS_CC);
 	RETURN_TRUE;
 }
@@ -771,8 +789,8 @@ PHP_FUNCTION(ob_flush)
    Clean (delete) the current output buffer */
 PHP_FUNCTION(ob_clean)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		ZEND_WRONG_PARAM_COUNT();
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 	
 	if (!OG(ob_nesting_level)) {
@@ -794,8 +812,8 @@ PHP_FUNCTION(ob_clean)
    Flush (send) the output buffer, and delete current output buffer */
 PHP_FUNCTION(ob_end_flush)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		ZEND_WRONG_PARAM_COUNT();
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 	
 	if (!OG(ob_nesting_level)) {
@@ -816,8 +834,8 @@ PHP_FUNCTION(ob_end_flush)
    Clean the output buffer, and delete current output buffer */
 PHP_FUNCTION(ob_end_clean)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		ZEND_WRONG_PARAM_COUNT();
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 		
 	if (!OG(ob_nesting_level)) {
@@ -838,8 +856,8 @@ PHP_FUNCTION(ob_end_clean)
    Get current buffer contents, flush (send) the output buffer, and delete current output buffer */
 PHP_FUNCTION(ob_get_flush)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		ZEND_WRONG_PARAM_COUNT();
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 
 	/* get contents */
@@ -849,10 +867,12 @@ PHP_FUNCTION(ob_get_flush)
 	/* error checks */
 	if (!OG(ob_nesting_level)) {
 		php_error_docref("ref.outcontrol" TSRMLS_CC, E_NOTICE, "failed to delete and flush buffer. No buffer to delete or flush.");
+		zval_dtor(return_value);
 		RETURN_FALSE;
 	}
 	if (OG(ob_nesting_level) && !OG(active_ob_buffer).status && !OG(active_ob_buffer).erase) {
 		php_error_docref("ref.outcontrol" TSRMLS_CC, E_NOTICE, "failed to delete buffer %s.", OG(active_ob_buffer).handler_name);
+		zval_dtor(return_value);
 		RETURN_FALSE;
 	}
 	/* flush */
@@ -864,9 +884,9 @@ PHP_FUNCTION(ob_get_flush)
    Get current buffer contents and delete current output buffer */
 PHP_FUNCTION(ob_get_clean)
 {
-	if (ZEND_NUM_ARGS() != 0)
-			ZEND_WRONG_PARAM_COUNT();
-		
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
 	/* get contents */
 	if (php_ob_get_buffer(return_value TSRMLS_CC)==FAILURE) {
 		RETURN_FALSE;
@@ -874,10 +894,12 @@ PHP_FUNCTION(ob_get_clean)
 	/* error checks */
 	if (!OG(ob_nesting_level)) {
 		php_error_docref("ref.outcontrol" TSRMLS_CC, E_NOTICE, "failed to delete buffer. No buffer to delete.");
+		zval_dtor(return_value);
 		RETURN_FALSE;
 	}
 	if (OG(ob_nesting_level) && !OG(active_ob_buffer).status && !OG(active_ob_buffer).erase) {
 		php_error_docref("ref.outcontrol" TSRMLS_CC, E_NOTICE, "failed to delete buffer %s.", OG(active_ob_buffer).handler_name);
+		zval_dtor(return_value);
 		RETURN_FALSE;
 	}
 	/* delete buffer */
@@ -889,8 +911,8 @@ PHP_FUNCTION(ob_get_clean)
    Return the contents of the output buffer */
 PHP_FUNCTION(ob_get_contents)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		ZEND_WRONG_PARAM_COUNT();
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 		
 	if (php_ob_get_buffer(return_value TSRMLS_CC)==FAILURE) {
@@ -903,8 +925,8 @@ PHP_FUNCTION(ob_get_contents)
    Return the nesting level of the output buffer */
 PHP_FUNCTION(ob_get_level)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		ZEND_WRONG_PARAM_COUNT();
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 		
 	RETURN_LONG (OG(ob_nesting_level));
@@ -915,8 +937,8 @@ PHP_FUNCTION(ob_get_level)
    Return the length of the output buffer */
 PHP_FUNCTION(ob_get_length)
 {
-	if (ZEND_NUM_ARGS() != 0) {
-		ZEND_WRONG_PARAM_COUNT();
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 		
 	if (php_ob_get_length(return_value TSRMLS_CC)==FAILURE) {
@@ -959,12 +981,12 @@ static int php_ob_buffer_status(php_ob_buffer *ob_buffer, zval *result)
    Return the status of the active or all output buffers */
 PHP_FUNCTION(ob_get_status)
 {
-	int argc = ZEND_NUM_ARGS();
 	zend_bool full_status = 0;
-	
-	if (zend_parse_parameters(argc TSRMLS_CC, "|b", &full_status) == FAILURE )
-		RETURN_FALSE;
-	
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &full_status) == FAILURE ) {
+		return;
+	}
+
 	array_init(return_value);
 
 	if (full_status) {
@@ -993,24 +1015,12 @@ PHP_FUNCTION(ob_get_status)
    Turn implicit flush on/off and is equivalent to calling flush() after every output call */
 PHP_FUNCTION(ob_implicit_flush)
 {
-	zval **zv_flag;
-	int flag;
+	long flag = 1;
 
-	switch(ZEND_NUM_ARGS()) {
-		case 0:
-			flag = 1;
-			break;
-		case 1:
-			if (zend_get_parameters_ex(1, &zv_flag)==FAILURE) {
-				RETURN_FALSE;
-			}
-			convert_to_long_ex(zv_flag);
-			flag = Z_LVAL_PP(zv_flag);
-			break;
-		default:
-			ZEND_WRONG_PARAM_COUNT();
-			break;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &flag) == FAILURE) {
+		return;
 	}
+
 	if (flag) {
 		php_start_implicit_flush(TSRMLS_C);
 	} else {
@@ -1059,7 +1069,7 @@ PHP_FUNCTION(output_add_rewrite_var)
 	int name_len, value_len;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", &name, &name_len, &value, &value_len) == FAILURE) {
-		RETURN_FALSE;
+		return;
 	}
 
 	if (php_url_scanner_add_var(name, name_len, value, value_len, 1 TSRMLS_CC) == SUCCESS) {
