@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,15 +16,30 @@
    +----------------------------------------------------------------------+
 */
 
+/* $Id: php_ini.h,v 1.45.2.3.2.3.2.9 2008/12/31 11:15:47 sebastian Exp $ */
+
 #ifndef PHP_INI_H
 #define PHP_INI_H
 
 #include "zend_ini.h"
 
-int php_init_config();
+BEGIN_EXTERN_C()
+PHPAPI void config_zval_dtor(zval *zvalue);
+int php_init_config(TSRMLS_D);
 int php_shutdown_config(void);
-void php_ini_delayed_modules_startup(TSRMLS_D);
-zval *cfg_get_entry(char *name, uint name_length);
+void php_ini_register_extensions(TSRMLS_D);
+PHPAPI zval *cfg_get_entry(const char *name, uint name_length);
+PHPAPI int cfg_get_long(const char *varname, long *result);
+PHPAPI int cfg_get_double(const char *varname, double *result);
+PHPAPI int cfg_get_string(const char *varname, char **result);
+PHPAPI int php_parse_user_ini_file(const char *dirname, char *ini_filename, HashTable *target_hash TSRMLS_DC);
+PHPAPI void php_ini_activate_config(HashTable *source_hash, int modify_type, int stage TSRMLS_DC);
+PHPAPI int php_ini_has_per_dir_config(void);
+PHPAPI int php_ini_has_per_host_config(void);
+PHPAPI void php_ini_activate_per_dir_config(char *path, uint path_len TSRMLS_DC);
+PHPAPI void php_ini_activate_per_host_config(const char *host, uint host_len TSRMLS_DC);
+PHPAPI HashTable* php_ini_get_configuration_hash(void);
+END_EXTERN_C()
 
 #define PHP_INI_USER	ZEND_INI_USER
 #define PHP_INI_PERDIR	ZEND_INI_PERDIR
@@ -61,6 +76,7 @@ zval *cfg_get_entry(char *name, uint name_length);
 #define PHP_INI_STAGE_ACTIVATE		ZEND_INI_STAGE_ACTIVATE
 #define PHP_INI_STAGE_DEACTIVATE	ZEND_INI_STAGE_DEACTIVATE
 #define PHP_INI_STAGE_RUNTIME		ZEND_INI_STAGE_RUNTIME
+#define PHP_INI_STAGE_HTACCESS		ZEND_INI_STAGE_HTACCESS
 
 #define php_ini_boolean_displayer_cb	zend_ini_boolean_displayer_cb
 #define php_ini_color_displayer_cb		zend_ini_color_displayer_cb
