@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -17,14 +17,9 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php_scandir.c,v 1.9 2004/01/08 08:17:54 andi Exp $ */
-
-#ifdef PHP_WIN32
-#include "config.w32.h"
-#else
-#include "php_config.h"
-#endif
+/* $Id: php_scandir.c,v 1.12.2.1.2.5.2.3 2008/12/31 11:15:47 sebastian Exp $ */
 
+#include "php.h"
 #include "php_scandir.h"
 
 #ifdef HAVE_SYS_TYPES_H
@@ -38,11 +33,14 @@
 #ifndef HAVE_SCANDIR
 
 #ifdef PHP_WIN32
+#include "win32/param.h"
 #include "win32/readdir.h"
 #endif  
 
 #include <stdlib.h>
+#ifndef NETWARE
 #include <search.h>
+#endif
 
 #endif /* HAVE_SCANDIR */
 
@@ -52,20 +50,21 @@
 #include <string.h>
 #endif
 
-int php_alphasort(const struct dirent **a, const struct dirent **b)
+PHPAPI int php_alphasort(const struct dirent **a, const struct dirent **b)
 {
 	return strcoll((*a)->d_name,(*b)->d_name);
 }
 #endif /* HAVE_ALPHASORT */
 
 #ifndef HAVE_SCANDIR
-int php_scandir(const char *dirname, struct dirent **namelist[], int (*selector) (const struct dirent *entry), int (*compare) (const struct dirent **a, const struct dirent **b))
+PHPAPI int php_scandir(const char *dirname, struct dirent **namelist[], int (*selector) (const struct dirent *entry), int (*compare) (const struct dirent **a, const struct dirent **b))
 {
 	DIR *dirp = NULL;
 	struct dirent **vector = NULL;
-	struct dirent *dp = NULL;
 	int vector_size = 0;
 	int nfiles = 0;
+	char entry[sizeof(struct dirent)+MAXPATHLEN];
+	struct dirent *dp = (struct dirent *)&entry;
 
 	if (namelist == NULL) {
 		return -1;
@@ -75,7 +74,7 @@ int php_scandir(const char *dirname, struct dirent **namelist[], int (*selector)
 		return -1;
 	}
 
-	while ((dp = readdir(dirp)) != NULL) {
+	while (!php_readdir_r(dirp, (struct dirent *)entry, &dp) && dp) {
 		int dsize = 0;
 		struct dirent *newdp = NULL;
 
