@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: safe_mode.c,v 1.62.2.1 2006/01/01 12:50:17 sniper Exp $ */
+/* $Id: safe_mode.c,v 1.62.2.1.2.10.2.8 2008/12/31 11:15:48 sebastian Exp $ */
 
 #include "php.h"
 
@@ -45,16 +45,17 @@
  * 5 - only check file
  */
 
-PHPAPI int php_checkuid_ex(const char *filename, char *fopen_mode, int mode, int flags)
+PHPAPI int php_checkuid_ex(const char *filename, const char *fopen_mode, int mode, int flags)
 {
 	struct stat sb;
 	int ret, nofile=0;
 	long uid=0L, gid=0L, duid=0L, dgid=0L;
 	char path[MAXPATHLEN];
 	char *s, filenamecopy[MAXPATHLEN];
-	php_stream_wrapper *wrapper = NULL;
 	TSRMLS_FETCH();
 
+	path[0] = '\0';
+
 	if (!filename) {
 		return 0; /* path must be provided */
 	}
@@ -71,20 +72,21 @@ PHPAPI int php_checkuid_ex(const char *filename, char *fopen_mode, int mode, int
 			mode = CHECKUID_CHECK_FILE_AND_DIR;
 		}
 	}
-
-	/* 
-	 * If given filepath is a URL, allow - safe mode stuff
-	 * related to URL's is checked in individual functions
-	 */
-	wrapper = php_stream_locate_url_wrapper(filename, NULL, STREAM_LOCATE_WRAPPERS_ONLY TSRMLS_CC);
-	if (wrapper != NULL)
-		return 1;
 		
 	/* First we see if the file is owned by the same user...
 	 * If that fails, passthrough and check directory...
 	 */
 	if (mode != CHECKUID_ALLOW_ONLY_DIR) {
-		VCWD_REALPATH(filename, path);
+#if HAVE_BROKEN_GETCWD
+		char ftest[MAXPATHLEN];
+
+		strcpy(ftest, filename);
+		if (VCWD_GETCWD(ftest, sizeof(ftest)) == NULL) {
+			strcpy(path, filename);
+		} else
+#endif
+		expand_filepath(filename, path TSRMLS_CC);
+
 		ret = VCWD_STAT(path, &sb);
 		if (ret < 0) {
 			if (mode == CHECKUID_DISALLOW_FILE_NOT_EXISTS) {
@@ -111,10 +113,17 @@ PHPAPI int php_checkuid_ex(const char *filename, char *fopen_mode, int mode, int
 
 		/* Trim off filename */
 		if ((s = strrchr(path, DEFAULT_SLASH))) {
-			if (s == path)
-				path[1] = '\0';
-			else
+			if (*(s + 1) == '\0' && s != path) { /* make sure that the / is not the last character */
 				*s = '\0';
+				s = strrchr(path, DEFAULT_SLASH);
+			}
+			if (s) {
+				if (s == path) {
+					path[1] = '\0';
+				} else {
+					*s = '\0';
+				}
+			}
 		}
 	} else { /* CHECKUID_ALLOW_ONLY_DIR */
 		s = strrchr(filename, DEFAULT_SLASH);
@@ -123,7 +132,7 @@ PHPAPI int php_checkuid_ex(const char *filename, char *fopen_mode, int mode, int
 			/* root dir */
 			path[0] = DEFAULT_SLASH;
 			path[1] = '\0';
-		} else if (s) {
+		} else if (s && *(s + 1) != '\0') { /* make sure that the / is not the last character */
 			*s = '\0';
 			VCWD_REALPATH(filename, path);
 			*s = DEFAULT_SLASH;
@@ -153,8 +162,6 @@ PHPAPI int php_checkuid_ex(const char *filename, char *fopen_mode, int mode, int
  		} else if (PG(safe_mode_gid) && dgid == php_getgid()) {
  			return 1;
 		} else {
-			TSRMLS_FETCH();
-
 			if (SG(rfc1867_uploaded_files)) {
 				if (zend_hash_exists(SG(rfc1867_uploaded_files), (char *) filename, strlen(filename)+1)) {
 					return 1;
@@ -188,7 +195,8 @@ PHPAPI int php_checkuid_ex(const char *filename, char *fopen_mode, int mode, int
 	return 0;
 }
 
-PHPAPI int php_checkuid(const char *filename, char *fopen_mode, int mode) {
+PHPAPI int php_checkuid(const char *filename, const char *fopen_mode, int mode)
+{
 #ifdef NETWARE
 /* NetWare don't have uid*/
 	return 1;
@@ -197,9 +205,8 @@ PHPAPI int php_checkuid(const char *filename, char *fopen_mode, int mode) {
 #endif
 }
 
-PHPAPI char *php_get_current_user()
+PHPAPI char *php_get_current_user(void)
 {
-	struct passwd *pwd;
 	struct stat *pstat;
 	TSRMLS_FETCH();
 
@@ -215,15 +222,48 @@ PHPAPI char *php_get_current_user()
 
 	if (!pstat) {
 		return "";
-	}
+	} else {
+#ifdef PHP_WIN32
+		char name[256];
+		DWORD len = sizeof(name)-1;
 
-	if ((pwd=getpwuid(pstat->st_uid))==NULL) {
-		return "";
-	}
-	SG(request_info).current_user_length = strlen(pwd->pw_name);
-	SG(request_info).current_user = estrndup(pwd->pw_name, SG(request_info).current_user_length);
-	
-	return SG(request_info).current_user;		
+		if (!GetUserName(name, &len)) {
+			return "";
+		}
+		name[len] = '\0';
+		SG(request_info).current_user_length = len;
+		SG(request_info).current_user = estrndup(name, len);
+		return SG(request_info).current_user;		
+#else
+		struct passwd *pwd;
+#if defined(ZTS) && defined(HAVE_GETPWUID_R) && defined(_SC_GETPW_R_SIZE_MAX)
+		struct passwd _pw;
+		struct passwd *retpwptr = NULL;
+		int pwbuflen = sysconf(_SC_GETPW_R_SIZE_MAX);
+		char *pwbuf;
+
+		if (pwbuflen < 1) {
+			return "";
+		}
+		pwbuf = emalloc(pwbuflen);
+		if (getpwuid_r(pstat->st_uid, &_pw, pwbuf, pwbuflen, &retpwptr) != 0) {
+			efree(pwbuf);
+			return "";
+		}
+		pwd = &_pw;
+#else
+		if ((pwd=getpwuid(pstat->st_uid))==NULL) {
+			return "";
+		}
+#endif
+		SG(request_info).current_user_length = strlen(pwd->pw_name);
+		SG(request_info).current_user = estrndup(pwd->pw_name, SG(request_info).current_user_length);
+#if defined(ZTS) && defined(HAVE_GETPWUID_R) && defined(_SC_GETPW_R_SIZE_MAX)
+		efree(pwbuf);
+#endif
+		return SG(request_info).current_user;		
+#endif
+	}	
 }	
 
 /*
