@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -22,7 +22,7 @@
  * - CGI/1.1 conformance
  */
 
-/* $Id: aolserver.c,v 1.72.2.4.6.2 2007/01/01 09:46:51 sebastian Exp $ */
+/* $Id: aolserver.c,v 1.81.2.2.2.1.2.8 2008/12/31 11:15:48 sebastian Exp $ */
 
 /* conflict between PHP and AOLserver headers */
 #define Debug php_Debug
@@ -205,7 +205,7 @@ static void php_info_aolserver(ZEND_MODULE_INFO_FUNC_ARGS)
 	int i;
 	
 	php_info_print_table_start();
-	php_info_print_table_row(2, "SAPI module version", "$Id: aolserver.c,v 1.72.2.4.6.2 2007/01/01 09:46:51 sebastian Exp $");
+	php_info_print_table_row(2, "SAPI module version", "$Id: aolserver.c,v 1.81.2.2.2.1.2.8 2008/12/31 11:15:48 sebastian Exp $");
 	php_info_print_table_row(2, "Build date", Ns_InfoBuildDate());
 	php_info_print_table_row(2, "Config file path", Ns_InfoConfigFile());
 	php_info_print_table_row(2, "Error Log path", Ns_InfoErrorLog());
@@ -240,8 +240,13 @@ static void php_info_aolserver(ZEND_MODULE_INFO_FUNC_ARGS)
 
 PHP_FUNCTION(getallheaders);
 
-static function_entry aolserver_functions[] = {
-	PHP_FE(getallheaders, NULL)
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO(arginfo_aolserver_getallheaders, 0)
+ZEND_END_ARG_INFO()
+/* }}} */
+
+const static zend_function_entry aolserver_functions[] = {
+	PHP_FE(getallheaders, arginfo_aolserver_getallheaders)
 	{NULL, NULL, NULL}
 };
 
@@ -262,9 +267,7 @@ PHP_FUNCTION(getallheaders)
 {
 	int i;
 
-	if (array_init(return_value) == FAILURE) {
-		RETURN_FALSE;
-	}
+	array_init(return_value);
 	
 	for (i = 0; i < Ns_SetSize(NSG(conn->headers)); i++) {
 		char *key = Ns_SetKey(NSG(conn->headers), i);
@@ -387,11 +390,8 @@ static sapi_module_struct aolserver_sapi_module = {
 
 	php_ns_sapi_register_variables,
 	NULL,									/* Log message */
-
-	NULL,									/* php.ini path override */
-
-	NULL,									/* Block interruptions */
-	NULL,									/* Unblock interruptions */
+	NULL,									/* Get request time */
+	NULL,							/* child terminate */
 
 	STANDARD_SAPI_MODULE_PROPERTIES
 };
@@ -404,7 +404,7 @@ static sapi_module_struct aolserver_sapi_module = {
 static int
 php_ns_module_main(TSRMLS_D)
 {
-	zend_file_handle file_handle = {0};
+	zend_file_handle file_handle;
 
 	file_handle.type = ZEND_HANDLE_FILENAME;
 	file_handle.filename = SG(request_info).path_translated;
@@ -450,6 +450,8 @@ php_ns_request_ctor(TSRMLS_D)
 	root = Ns_PageRoot(server);
 	SG(request_info).request_uri = strdup(SG(request_info).path_translated + strlen(root));
 	SG(request_info).request_method = NSG(conn)->request->method;
+	if(NSG(conn)->request->version > 1.0) SG(request_info).proto_num = 1001;
+	else SG(request_info).proto_num = 1000;
 	SG(request_info).content_length = Ns_ConnContentLength(NSG(conn));
 	index = Ns_SetIFind(NSG(conn)->headers, "content-type");
 	SG(request_info).content_type = index == -1 ? NULL : 
@@ -556,7 +558,7 @@ php_ns_config(php_ns_context *ctx, char global)
 
 				Ns_Log(Debug, "PHP configuration option '%s=%s'", new_key, val);
 				zend_alter_ini_entry(new_key, strlen(new_key) + 1, val, 
-						strlen(val) + 1, PHP_INI_SYSTEM, PHP_INI_STAGE_RUNTIME);
+						strlen(val) + 1, PHP_INI_SYSTEM, PHP_INI_STAGE_ACTIVATE);
 				
 				efree(new_key);
 			}
