@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php_apache.h,v 1.18.2.1.8.1 2006/01/01 13:47:01 sniper Exp $ */
+/* $Id: php_apache.h,v 1.25.2.1.2.2.2.4 2008/12/31 11:15:48 sebastian Exp $ */
 
 #ifndef PHP_APACHE_H
 #define PHP_APACHE_H
@@ -26,7 +26,7 @@
 #include "http_core.h"
 
 /* Declare this so we can get to it from outside the sapi_apache2.c file */
-extern module AP_MODULE_DECLARE_DATA php4_module;
+extern module AP_MODULE_DECLARE_DATA php5_module;
 
 /* A way to specify the location of the php.ini dir in an apache directive */
 extern char *apache2_php_ini_path_override;
@@ -48,12 +48,19 @@ typedef struct php_struct {
 	int request_processed;
 } php_struct;
 
+typedef struct _php_apr_bucket_brigade {
+	apr_bucket_brigade *bb;
+} php_apr_bucket_brigade;
+
 void *merge_php_config(apr_pool_t *p, void *base_conf, void *new_conf);
 void *create_php_config(apr_pool_t *p, char *dummy);
 char *get_php_config(void *conf, char *name, size_t name_len);
 void apply_config(void *);
 extern const command_rec php_dir_cmds[];
 
+static size_t php_apache_read_stream(void *, char *, size_t TSRMLS_DC);
+static size_t php_apache_fsizer_stream(void * TSRMLS_DC);
+
 #define APR_ARRAY_FOREACH_OPEN(arr, key, val) 		\
 {													\
 	apr_table_entry_t *elts;						\
@@ -65,4 +72,10 @@ extern const command_rec php_dir_cmds[];
 
 #define APR_ARRAY_FOREACH_CLOSE() }}
 
+/* fix for gcc4 visibility patch */
+#ifndef PHP_WIN32
+# undef AP_MODULE_DECLARE_DATA
+# define AP_MODULE_DECLARE_DATA PHPAPI
+#endif
+
 #endif /* PHP_APACHE_H */
