@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php_functions.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: php_functions.c,v 1.18.2.6.2.5.2.11 2009/01/19 19:32:40 scottmac Exp $ */
 
 #define ZEND_INCLUDE_FULL_WINDOWS_HEADERS
 
@@ -71,23 +71,21 @@ static request_rec *php_apache_lookup_uri(char *filename TSRMLS_DC)
  Perform an apache sub-request */
 PHP_FUNCTION(virtual)
 {
-	zval **filename;
+	char *filename;
+	int filename_len;
 	request_rec *rr;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &filename) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) {
+		return;
 	}
 
-	convert_to_string_ex(filename);
-	
-
-	if (!(rr = php_apache_lookup_uri(Z_STRVAL_PP(filename) TSRMLS_CC))) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - URI lookup failed", Z_STRVAL_PP(filename));
+	if (!(rr = php_apache_lookup_uri(filename TSRMLS_CC))) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - URI lookup failed", filename);
 		RETURN_FALSE;
 	}
 
 	if (rr->status != HTTP_OK) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - error finding URI", Z_STRVAL_PP(filename));
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - error finding URI", filename);
 		ap_destroy_sub_req(rr);
 		RETURN_FALSE;
 	}
@@ -101,7 +99,7 @@ PHP_FUNCTION(virtual)
 	ap_rflush(rr->main);
 
 	if (ap_run_sub_req(rr)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - request execution failed", Z_STRVAL_PP(filename));
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - request execution failed", filename);
 		ap_destroy_sub_req(rr);
 		RETURN_FALSE;
 	}
@@ -120,16 +118,15 @@ PHP_FUNCTION(virtual)
 PHP_FUNCTION(apache_lookup_uri)
 {
 	request_rec *rr;
-	zval **filename;
+	char *filename;
+	int filename_len;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &filename) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) {
+		return;
 	}
 
-	convert_to_string_ex(filename);
-	
-	if (!(rr = php_apache_lookup_uri(Z_STRVAL_PP(filename) TSRMLS_CC))) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - URI lookup failed", Z_STRVAL_PP(filename));
+	if (!(rr = php_apache_lookup_uri(filename TSRMLS_CC))) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - URI lookup failed", filename);
 		RETURN_FALSE;
 	}
 	
@@ -166,7 +163,7 @@ PHP_FUNCTION(apache_lookup_uri)
 		return;
 	}
 	
-	php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - error finding URI", Z_STRVAL_PP(filename));
+	php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to include '%s' - error finding URI", filename);
 	ap_destroy_sub_req(rr);
 	RETURN_FALSE;
 }
@@ -179,8 +176,8 @@ PHP_FUNCTION(apache_request_headers)
 	const apr_array_header_t *arr;
 	char *key, *val;
 
-	if (ZEND_NUM_ARGS()) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 
 	array_init(return_value);
@@ -203,8 +200,8 @@ PHP_FUNCTION(apache_response_headers)
 	const apr_array_header_t *arr;
 	char *key, *val;
 
-	if (ZEND_NUM_ARGS()) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 
 	array_init(return_value);
@@ -224,31 +221,27 @@ PHP_FUNCTION(apache_response_headers)
 PHP_FUNCTION(apache_note)
 {
 	php_struct *ctx;
-	zval **note_name, **note_val;
+	char *note_name, *note_val = NULL;
+	int note_name_len, note_val_len;
 	char *old_note_val=NULL;
-	int arg_count = ZEND_NUM_ARGS();
 
-	if (arg_count<1 || arg_count>2 ||
-		zend_get_parameters_ex(arg_count, &note_name, &note_val) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|s", &note_name, &note_name_len, &note_val, &note_val_len) == FAILURE) {
+		return;
 	}
-	
-	ctx = SG(server_context);
-	
-	convert_to_string_ex(note_name);
 
-	old_note_val = (char *) apr_table_get(ctx->r->notes, Z_STRVAL_PP(note_name));
-	
-	if (arg_count == 2) {
-		convert_to_string_ex(note_val);
-		apr_table_set(ctx->r->notes, Z_STRVAL_PP(note_name), Z_STRVAL_PP(note_val));
+	ctx = SG(server_context);
+
+	old_note_val = (char *) apr_table_get(ctx->r->notes, note_name);
+
+	if (note_val) {
+		apr_table_set(ctx->r->notes, note_name, note_val);
 	}
 
 	if (old_note_val) {
 		RETURN_STRING(old_note_val, 1);
-	} else {
-		RETURN_FALSE;
 	}
+
+	RETURN_FALSE;
 }
 /* }}} */
 
@@ -260,31 +253,28 @@ PHP_FUNCTION(apache_note)
 PHP_FUNCTION(apache_setenv)
 {
 	php_struct *ctx;
-	zval **variable=NULL, **string_val=NULL, **walk_to_top=NULL;
+	char *variable=NULL, *string_val=NULL;
+	int variable_len, string_val_len;
+	zend_bool walk_to_top = 0;
 	int arg_count = ZEND_NUM_ARGS();
 	request_rec *r;
 
-	if (arg_count < 2 || arg_count > 3 ||
-		zend_get_parameters_ex(arg_count, &variable, &string_val, &walk_to_top) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(arg_count TSRMLS_CC, "ss|b", &variable, &variable_len, &string_val, &string_val_len, &walk_to_top) == FAILURE) {
+		return;
 	}
 
 	ctx = SG(server_context);
 
 	r = ctx->r;
 	if (arg_count == 3) {
-		convert_to_boolean_ex(walk_to_top);
-		if (Z_LVAL_PP(walk_to_top)) {
+		if (walk_to_top) {
 			while(r->prev) {
 				r = r->prev;
 			}
 		}
 	}
 
-	convert_to_string_ex(variable);
-	convert_to_string_ex(string_val);
-	
-	apr_table_set(r->subprocess_env, Z_STRVAL_PP(variable), Z_STRVAL_PP(string_val));
+	apr_table_set(r->subprocess_env, variable, string_val);
 
 	RETURN_TRUE;
 }
@@ -298,36 +288,35 @@ PHP_FUNCTION(apache_setenv)
 PHP_FUNCTION(apache_getenv)
 {
 	php_struct *ctx;
-	zval **variable=NULL, **walk_to_top=NULL;
+	char *variable=NULL;
+	int variable_len;
+	zend_bool walk_to_top = 0;
 	int arg_count = ZEND_NUM_ARGS();
 	char *env_val=NULL;
 	request_rec *r;
 
-	if (arg_count<1 || arg_count>2 ||
-		zend_get_parameters_ex(arg_count, &variable, &walk_to_top) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(arg_count TSRMLS_CC, "s|b", &variable, &variable_len, &walk_to_top) == FAILURE) {
+		return;
 	}
 
 	ctx = SG(server_context);
 
 	r = ctx->r;
 	if (arg_count == 2) {
-		convert_to_boolean_ex(walk_to_top);
-		if (Z_LVAL_PP(walk_to_top)) {
+		if (walk_to_top) {
 			while(r->prev) {
 				r = r->prev;
 			}
 		}
 	}
 
-	convert_to_string_ex(variable);
-	
-	env_val = (char*) apr_table_get(r->subprocess_env, Z_STRVAL_PP(variable));
+	env_val = (char*) apr_table_get(r->subprocess_env, variable);
+
 	if (env_val != NULL) {
 		RETURN_STRING(env_val, 1);
-	} else { 
-		RETURN_FALSE;
-	}	
+	}
+
+	RETURN_FALSE;
 }
 /* }}} */
 
@@ -383,7 +372,11 @@ PHP_MINFO_FUNCTION(apache)
 	char *p;
 	server_rec *serv = ((php_struct *) SG(server_context))->r->server;
 #if !defined(WIN32) && !defined(WINNT) && !defined(NETWARE)
+#if MODULE_MAGIC_NUMBER_MAJOR >= 20081201
+	AP_DECLARE_DATA extern unixd_config_rec ap_unixd_config;
+#else
 	AP_DECLARE_DATA extern unixd_config_rec unixd_config;
+#endif
 #endif
 	
 	for (n = 0; ap_loaded_modules[n]; ++n) {
@@ -414,7 +407,11 @@ PHP_MINFO_FUNCTION(apache)
 	php_info_print_table_row(2, "Hostname:Port", tmp);
 	
 #if !defined(WIN32) && !defined(WINNT) && !defined(NETWARE)
+#if MODULE_MAGIC_NUMBER_MAJOR >= 20081201
+	snprintf(tmp, sizeof(tmp), "%s(%d)/%d", ap_unixd_config.user_name, ap_unixd_config.user_id, ap_unixd_config.group_id);
+#else
 	snprintf(tmp, sizeof(tmp), "%s(%d)/%d", unixd_config.user_name, unixd_config.user_id, unixd_config.group_id);
+#endif
 	php_info_print_table_row(2, "User/Group", tmp);
 #endif
 
@@ -478,17 +475,55 @@ PHP_MINFO_FUNCTION(apache)
 	}
 }
 
-static zend_function_entry apache_functions[] = {
-	PHP_FE(apache_lookup_uri, NULL)
-	PHP_FE(virtual, NULL) 
-	PHP_FE(apache_request_headers, NULL)
-	PHP_FE(apache_response_headers, NULL)
-	PHP_FE(apache_setenv, NULL)
-	PHP_FE(apache_getenv, NULL)
-	PHP_FE(apache_note, NULL)
-	PHP_FE(apache_get_version, NULL)
-	PHP_FE(apache_get_modules, NULL)
-	PHP_FALIAS(getallheaders, apache_request_headers, NULL)
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO_EX(arginfo_apache2handler_lookup_uri, 0, 0, 1)
+	ZEND_ARG_INFO(0, filename)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_apache2handler_virtual, 0, 0, 1)
+	ZEND_ARG_INFO(0, uri)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_apache2handler_response_headers, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_apache2handler_getallheaders, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_apache2handler_note, 0, 0, 1)
+	ZEND_ARG_INFO(0, note_name)
+	ZEND_ARG_INFO(0, note_value)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_apache2handler_setenv, 0, 0, 2)
+	ZEND_ARG_INFO(0, variable)
+	ZEND_ARG_INFO(0, value)
+	ZEND_ARG_INFO(0, walk_to_top)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_apache2handler_getenv, 0, 0, 1)
+	ZEND_ARG_INFO(0, variable)
+	ZEND_ARG_INFO(0, walk_to_top)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_apache2handler_get_version, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_apache2handler_get_modules, 0)
+ZEND_END_ARG_INFO()
+/* }}} */
+
+static const zend_function_entry apache_functions[] = {
+	PHP_FE(apache_lookup_uri, 		arginfo_apache2handler_lookup_uri)
+	PHP_FE(virtual, 				arginfo_apache2handler_virtual) 
+	PHP_FE(apache_request_headers, 	arginfo_apache2handler_getallheaders)
+	PHP_FE(apache_response_headers, arginfo_apache2handler_response_headers)
+	PHP_FE(apache_setenv, 		arginfo_apache2handler_setenv)
+	PHP_FE(apache_getenv, 		arginfo_apache2handler_getenv)
+	PHP_FE(apache_note, 		arginfo_apache2handler_note)
+	PHP_FE(apache_get_version, 	arginfo_apache2handler_get_version)
+	PHP_FE(apache_get_modules, 	arginfo_apache2handler_get_modules)
+	PHP_FALIAS(getallheaders, 	apache_request_headers, arginfo_apache2handler_getallheaders)
 	{NULL, NULL, NULL}
 };
 
