@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -179,9 +179,8 @@ static sapi_module_struct phttpd_sapi_module = {
  
 	NULL,									/* register server variables */
 	NULL,									/* Log message */
-
-	NULL,									/* Block interruptions */
-	NULL,									/* Unblock interruptions */
+	NULL,									/* Get request time */
+	NULL,									/* Child terminate */
 
     STANDARD_SAPI_MODULE_PROPERTIES
 };
@@ -247,7 +246,7 @@ php_phttpd_request_dtor(TSRMLS_D TSRMLS_DC)
 int php_doit(TSRMLS_D TSRMLS_DC)
 {
 	struct stat sb;
-	zend_file_handle file_handle = {0};
+	zend_file_handle file_handle;
 	struct httpinfo *hip = PHG(cip)->hip;
 	TSRMLS_FETCH();
 
