@@ -102,7 +102,7 @@ static int LoadDirectory(HashTable *directories, HKEY key, char *path, int path_
 					     zend_hash_get_current_key_ex(parent_ht, &index, &index_len, &num, 0, &pos) == HASH_KEY_IS_STRING;
 					     zend_hash_move_forward_ex(parent_ht, &pos)) {
 						if (zend_hash_add(ht, index, index_len, data, sizeof(zval*), NULL) == SUCCESS) {
-						    (*data)->refcount++;
+						    Z_ADDREF_PP(data);
 						}
 					}
 				}
@@ -133,11 +133,14 @@ static int LoadDirectory(HashTable *directories, HKEY key, char *path, int path_
 						if (path_len) {
 							memcpy(new_path, path, path_len);
 							new_path[path_len] = '/';
-							path_len++;
+							memcpy(new_path+path_len+1, name, name_len+1);
+							zend_str_tolower(new_path, path_len+name_len+1);
+							name_len += path_len+1;
+						} else {
+							memcpy(new_path, name, name_len+1);
+							zend_str_tolower(new_path, name_len);
 						}
-						memcpy(new_path+path_len, name, name_len+1);
-						zend_str_tolower(new_path, path_len+name_len);
-						if (LoadDirectory(directories, subkey, new_path, path_len+name_len, ht)) {
+						if (LoadDirectory(directories, subkey, new_path, name_len, ht)) {
 							ret = 1;
 						}
 						RegCloseKey(subkey);
@@ -217,8 +220,7 @@ void UpdateIniFromRegistry(char *path TSRMLS_DC)
 			while (*cwd == '\\' || *cwd == '/') {
 				cwd++;
 			}
-			path = (char *) emalloc(2+strlen(cwd)+1+strlen(orig_path)+1);
-			sprintf(path, "%c\\%s\\%s", drive_letter, cwd, orig_path);
+			spprintf(&path, 0, "%c\\%s\\%s", drive_letter, cwd, orig_path);
 			efree(orig_path);
 			orig_path = path;
 		}
