@@ -11,7 +11,7 @@
  *
  *****************************************************************************/
 
-/* $Id: time.c,v 1.5.8.1 2002/06/24 08:21:28 edink Exp $ */
+/* $Id: time.c,v 1.10.6.3 2009/01/19 02:35:21 pajoye Exp $ */
 
  /**
   *
@@ -22,12 +22,16 @@
 
 /* Include stuff ************************************************************ */
 
+#include <config.w32.h>
+
 #include "time.h"
 #include "unistd.h"
 #include "signal.h"
+#include <windows.h>
 #include <winbase.h>
 #include <mmsystem.h>
 #include <errno.h>
+#include "php_win32_globals.h"
 
 int getfilesystemtime(struct timeval *time_Info) 
 {
@@ -46,64 +50,61 @@ __int64 ff;
 
 PHPAPI int gettimeofday(struct timeval *time_Info, struct timezone *timezone_Info)
 {
-
-	static struct timeval starttime = {0, 0};
-	static __int64 lasttime = 0;
-	static __int64 freq = 0;
 	__int64 timer;
 	LARGE_INTEGER li;
 	BOOL b;
 	double dt;
+	TSRMLS_FETCH();
 
 	/* Get the time, if they want it */
 	if (time_Info != NULL) {
-		if (starttime.tv_sec == 0) {
+		if (PW32G(starttime).tv_sec == 0) {
             b = QueryPerformanceFrequency(&li);
             if (!b) {
-                starttime.tv_sec = -1;
+                PW32G(starttime).tv_sec = -1;
             }
             else {
-                freq = li.QuadPart;
+                PW32G(freq) = li.QuadPart;
                 b = QueryPerformanceCounter(&li);
                 if (!b) {
-                    starttime.tv_sec = -1;
+                    PW32G(starttime).tv_sec = -1;
                 }
                 else {
-                    getfilesystemtime(&starttime);
+                    getfilesystemtime(&PW32G(starttime));
                     timer = li.QuadPart;
-                    dt = (double)timer/freq;
-                    starttime.tv_usec -= (int)((dt-(int)dt)*1000000);
-                    if (starttime.tv_usec < 0) {
-                        starttime.tv_usec += 1000000;
-                        --starttime.tv_sec;
+                    dt = (double)timer/PW32G(freq);
+                    PW32G(starttime).tv_usec -= (int)((dt-(int)dt)*1000000);
+                    if (PW32G(starttime).tv_usec < 0) {
+                        PW32G(starttime).tv_usec += 1000000;
+                        --PW32G(starttime).tv_sec;
                     }
-                    starttime.tv_sec -= (int)dt;
+                    PW32G(starttime).tv_sec -= (int)dt;
                 }
             }
         }
-        if (starttime.tv_sec > 0) {
+        if (PW32G(starttime).tv_sec > 0) {
             b = QueryPerformanceCounter(&li);
             if (!b) {
-                starttime.tv_sec = -1;
+                PW32G(starttime).tv_sec = -1;
             }
             else {
                 timer = li.QuadPart;
-                if (timer < lasttime) {
+                if (timer < PW32G(lasttime)) {
                     getfilesystemtime(time_Info);
-                    dt = (double)timer/freq;
-                    starttime = *time_Info;
-                    starttime.tv_usec -= (int)((dt-(int)dt)*1000000);
-                    if (starttime.tv_usec < 0) {
-                        starttime.tv_usec += 1000000;
-                        --starttime.tv_sec;
+                    dt = (double)timer/PW32G(freq);
+                    PW32G(starttime) = *time_Info;
+                    PW32G(starttime).tv_usec -= (int)((dt-(int)dt)*1000000);
+                    if (PW32G(starttime).tv_usec < 0) {
+                        PW32G(starttime).tv_usec += 1000000;
+                        --PW32G(starttime).tv_sec;
                     }
-                    starttime.tv_sec -= (int)dt;
+                    PW32G(starttime).tv_sec -= (int)dt;
                 }
                 else {
-                    lasttime = timer;
-                    dt = (double)timer/freq;
-                    time_Info->tv_sec = starttime.tv_sec + (int)dt;
-                    time_Info->tv_usec = starttime.tv_usec + (int)((dt-(int)dt)*1000000);
+                    PW32G(lasttime) = timer;
+                    dt = (double)timer/PW32G(freq);
+                    time_Info->tv_sec = PW32G(starttime).tv_sec + (int)dt;
+                    time_Info->tv_usec = PW32G(starttime).tv_usec + (int)((dt-(int)dt)*1000000);
                     if (time_Info->tv_usec > 1000000) {
                         time_Info->tv_usec -= 1000000;
                         ++time_Info->tv_sec;
@@ -111,7 +112,7 @@ PHPAPI int gettimeofday(struct timeval *time_Info, struct timezone *timezone_Inf
                 }
             }
         }
-        if (starttime.tv_sec < 0) {
+        if (PW32G(starttime).tv_sec < 0) {
             getfilesystemtime(time_Info);
         }
 
@@ -126,39 +127,31 @@ PHPAPI int gettimeofday(struct timeval *time_Info, struct timezone *timezone_Inf
 	return 0;
 }
 
-
-/* this usleep isnt exactly accurate but should do ok */
-void usleep(unsigned int useconds)
+PHPAPI int usleep(unsigned int useconds)
 {
-struct timeval tnow, tthen, t0;
+	HANDLE timer;
+	LARGE_INTEGER due;
 
-	gettimeofday(&tthen, NULL);
-    t0 = tthen;
-    tthen.tv_usec += useconds;
-    while (tthen.tv_usec > 1000000) {
-        tthen.tv_usec -= 1000000;
-        tthen.tv_sec++;
-    }
-    
-	if (useconds > 10000) {
-        useconds -= 10000;
-        Sleep(useconds/1000);
-    }
-    
-	while (1) {
-        gettimeofday(&tnow, NULL);
-        if (tnow.tv_sec > tthen.tv_sec) {
-            break;
-        }
-        if (tnow.tv_sec == tthen.tv_sec) {
-            if (tnow.tv_usec > tthen.tv_usec) {
-                break;
-            }
-        }
-    }
+	due.QuadPart = -(10 * (__int64)useconds);
+
+	timer = CreateWaitableTimer(NULL, TRUE, NULL);
+	SetWaitableTimer(timer, &due, 0, NULL, NULL, 0);
+	WaitForSingleObject(timer, INFINITE);
+	CloseHandle(timer);
+	return 0;
 }
 
+PHPAPI int nanosleep( const struct timespec * rqtp, struct timespec * rmtp )
+{
+	if (rqtp->tv_nsec > 999999999) {
+		/* The time interval specified 1,000,000 or more microseconds. */
+		errno = EINVAL;
+		return -1;
+	}
+	return usleep( rqtp->tv_sec * 1000000 + rqtp->tv_nsec / 1000  );
+}
 
+#if 0 /* looks pretty ropey in here */
 #ifdef HAVE_SETITIMER
 
 
@@ -240,3 +233,5 @@ PHPAPI int setitimer(int which, const struct itimerval *value, struct itimerval
 }
 
 #endif
+#endif
+
